package
{
	import com.adobe.portfolio.cards.ICardProvider;
	import com.adobe.portfolio.components.CardSlider;
	import com.adobe.portfolio.lists.CardContainer;
	import com.adobe.portfolio.managers.PropertyManager;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;
	import flash.geom.Matrix3D;
	
	import mx.binding.utils.BindingUtils;
	import mx.core.IButton;
	import mx.core.IVisualElement;
	import mx.core.ScrollPolicy;
	import mx.core.UIComponent;
	import mx.events.ResizeEvent;
	
	import spark.components.Group;
	import spark.components.Scroller;
	import spark.components.supportClasses.GroupBase;
	import spark.layouts.supportClasses.LayoutBase;
	
	public class AnyLayoutContainer extends CardContainer
	{
		protected static const propertyManager:PropertyManager = PropertyManager.instance;
		
		/**
		 * A button for navigating to the previous attachment, optionally provided by this instance's skin.
		 */
		[SkinPart(required="false")]
		public var prevButton:IButton;
		
		/**
		 * A button for navigating to the next attachment, optionally provided by this instance's skin.
		 */
		[SkinPart(required="false")]
		public var nextButton:IButton;
		
		[SkinPart(required="false")]
		public var cardSlider:CardSlider;
		
		private var _layoutName:String;
		private var _cardProvider:ICardProvider;
		private var _showNextPrev:Boolean;
		private var _showHScrollBar:Boolean;
		private var _showVScrollBar:Boolean;
		private var _showCardSlider:Boolean;
		private var _virtualLayout:Boolean;
		
		public function AnyLayoutContainer()
		{
			super();
		}
		
		override protected function attachSkin():void 
		{
			super.attachSkin();
			updateLayout();
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "showNextPrev", propertyManager, ["commonProperties", "showNextPrev", "value"]));
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "showHScrollBar", propertyManager, ["commonProperties", "showHScrollBar", "value"]));
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "showVScrollBar", propertyManager, ["commonProperties", "showVScrollBar", "value"]));
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "showCardSlider", propertyManager, ["commonProperties", "showCardSlider", "value"]));
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "virtualLayout", propertyManager, ["commonProperties", "virtualLayout", "value"]));
		}
		
		override protected function detachSkin():void
		{
			super.detachSkin();
			autoWatch.removeAllWatchers();
		}
		

		/**
		 * @inheritDoc
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			
			switch (instance) {
				case prevButton:
					addListener(prevButton, MouseEvent.CLICK, collectionMgr.onGotoPreviousAttachment);
					break;
				
				case nextButton:
					addListener(nextButton, MouseEvent.CLICK, collectionMgr.onGotoNextAttachment);
					break;
			}
		}

		[Bindable]
		public function get layoutName():String
		{
			return _layoutName;
		}
		
		public function set layoutName(value:String):void
		{
			_layoutName = value;
			updateLayout();
		}
		
		protected function updateLayout():void
		{
			if (!layoutName || !skin)
				return;
			
			layout = skin[layoutName] as LayoutBase;
			updateVirtualLayout();
			cardProvider = layout as ICardProvider;
			if (cardProvider) {
				cardProvider.pendingCardPosition = 0;
			}
			resetScrolling();
			resetCards();
			layout.target.invalidateDisplayList();
//			invalidateDisplayList();
			layout.target.dispatchEvent(new ResizeEvent(Event.RESIZE));
		}
		
		protected function resetCards():void
		{
			var target:GroupBase = layout.target;
			var numElements:int = target.numElements;
			var identity:Matrix3D = new Matrix3D();
			
			for (var i:int = 0; i < numElements; i++) {
				var card:IVisualElement = target.getElementAt(i);
				if (card == null) {
					continue;
				}
				card.setLayoutMatrix3D(identity, true);
				card.alpha = 1;
				var uiComponent:UIComponent = card as UIComponent;
				if (uiComponent) {
					uiComponent.visible = true;
					uiComponent.filters = null;
				}
			}
		}
		
		protected function resetScrolling():void
		{
			if (!scroller)
				return;
			
			scroller.setStyle("horizontalScrollPolicy", showHScrollBar ? ScrollPolicy.AUTO : ScrollPolicy.OFF);
			scroller.setStyle("verticalScrollPolicy", showVScrollBar ? ScrollPolicy.AUTO : ScrollPolicy.OFF);
			layout.target.horizontalScrollPosition = 0;
			layout.target.verticalScrollPosition = 0;
			
//			scroller.minViewportInset=16;
		}
		
		protected function updateVirtualLayout():void
		{
			layout.useVirtualLayout = showCardSlider ? false : virtualLayout;
//			if (propertyManager) {
//				var customProperties:CustomProperties = propertyManager.commonProperties as CustomProperties;
//				if (customProperties)
//					customProperties.virtualLayout.value = virtualLayout;
//			}
		}


		[Bindable]
		public function get showHScrollBar():Boolean
		{
			return _showHScrollBar;
		}

		public function set showHScrollBar(value:Boolean):void
		{
			_showHScrollBar = value;
			resetScrolling();
		}

		[Bindable]
		public function get showVScrollBar():Boolean
		{
			return _showVScrollBar;
		}

		public function set showVScrollBar(value:Boolean):void
		{
			_showVScrollBar = value;
			resetScrolling();
		}

		[Bindable]
		public function get cardProvider():ICardProvider
		{
			return _cardProvider;
		}

		public function set cardProvider(value:ICardProvider):void
		{
			_cardProvider = value;
		}

		[Bindable]
		public function get showCardSlider():Boolean
		{
			return _showCardSlider;
		}

		public function set showCardSlider(value:Boolean):void
		{
			_showCardSlider = value;
			updateLayout();
		}

		[Bindable]
		public function get showNextPrev():Boolean
		{
			return _showNextPrev;
		}

		public function set showNextPrev(value:Boolean):void
		{
			_showNextPrev = value;
		}

		[Bindable]
		public function get virtualLayout():Boolean
		{
			return _virtualLayout;
		}

		public function set virtualLayout(value:Boolean):void
		{
			_virtualLayout = value;
			updateVirtualLayout();
		}

		
	}
}