package components
{
	import com.adobe.portfolio.cards.CardFactory;
	import com.adobe.portfolio.lists.CardContainer;
	
	import events.PanelEvent;
	
	import flash.events.MouseEvent;
	
	import managers.PanelManager;
	
	import spark.components.Button;
	import spark.components.IItemRenderer;
	
	public class Panel extends CardContainer implements IItemRenderer
	{
		[SkinPart(required="false")]
		public var panelOrientation:Button;
		
		protected const SKIN_STATE_NORMAL:String = "normal";
		protected const SKIN_STATE_OVER:String = "over";
		
		private var _itemIndex:int;
		private var _dragging:Boolean;
		private var _label:String;
		private var _selected:Boolean;
		private var _showsCaret:Boolean;
		private var _data:Object;
		private var _mouseIsOver:Boolean = false;
		private var _panelGeometry:String;
		
		public function Panel()
		{
			super();
			var alpha:Number = 1;
			itemRenderer = new CardFactory(CardFactory.BASIC_CARD, CardFactory.STATE_SIMPLE, CardFactory.DROP_SHADOW_BOTTOM, 
				null, alpha, true, true);
			dragEnabled = false;
		}
		
		override protected function attachSkin():void {
			super.attachSkin();
			addEventListener(MouseEvent.ROLL_OVER, onRoll, false, 0, true);
			addEventListener(MouseEvent.ROLL_OUT, onRoll, false, 0, true);
		}
		
		override protected function detachSkin():void {
			super.detachSkin();
			removeEventListener(MouseEvent.ROLL_OVER, onRoll);
			removeEventListener(MouseEvent.ROLL_OUT, onRoll);
		}

		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case panelOrientation:
					addListener(panelOrientation, MouseEvent.CLICK, onHandlePanelOrientation);
					break;
			}
		}
		
		protected function onHandlePanelOrientation(event:MouseEvent):void {
			PanelManager.instance.dispatchEvent(new PanelEvent(PanelEvent.ORIENTATION_CHANGED, this));
		}
		
		public function get itemIndex():int
		{
			return _itemIndex;
		}

		public function set itemIndex(value:int):void
		{
			_itemIndex = value;
		}

		public function get dragging():Boolean
		{
			return _dragging;
		}

		public function set dragging(value:Boolean):void
		{
			_dragging = value;
		}

		public function get label():String
		{
			return _label;
		}

		public function set label(value:String):void
		{
			_label = value;
		}

		public function get selected():Boolean
		{
			return _selected;
		}

		public function set selected(value:Boolean):void
		{
			_selected = value;
		}

		public function get showsCaret():Boolean
		{
			return _showsCaret;
		}

		public function set showsCaret(value:Boolean):void
		{
			_showsCaret = value;
		}

		public function get data():Object
		{
			return _data;
		}

		public function set data(value:Object):void
		{
			_data = value;
		}

		protected function get mouseIsOver():Boolean
		{
			return _mouseIsOver;
		}
		
		protected function set mouseIsOver(value:Boolean):void
		{
			if (value != _mouseIsOver) {
				_mouseIsOver = value;
				invalidateSkinState();
			}
		}

		private function onRoll(event:MouseEvent):void
		{
			var mouseIsOver:Boolean = (event.type == MouseEvent.ROLL_OVER);
			if (!mouseIsOver) {
				//				logger.debug("event.stageX, Y = " + event.stageX + " , " + event.stageY);
				if (event.stageX < 1 && event.stageY < 1) // weird values for some rollout events which shouldn't be rollout
					mouseIsOver = true;
			}
			this.mouseIsOver = mouseIsOver;
		}
		
		override protected function getCurrentSkinState():String
		{
			return mouseIsOver ? SKIN_STATE_OVER : SKIN_STATE_NORMAL;
		}

		[Bindable]
		public function get panelGeometry():String
		{
			return _panelGeometry;
		}

		public function set panelGeometry(value:String):void
		{
			_panelGeometry = value;
		}


	}
}