/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2010 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package lists 
{
	import com.adobe.portfolio.cards.ICardProvider;
	import com.adobe.portfolio.components.supportClasses.AutoListen;
	import com.adobe.portfolio.managers.PropertyManager;
	
	import components.BorgesCardProvider;
	import components.ControlPanel;
	import components.Panel;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import layouts.BorgesLayout;
	
	import mx.binding.utils.BindingUtils;
	import mx.collections.ArrayCollection;
	import mx.collections.IList;
	import mx.events.CollectionEvent;
	import mx.managers.PopUpManager;
	
	import properties.CustomProperties;
	
	import spark.components.Button;
	import spark.components.List;
	import spark.components.RichEditableText;
	import spark.components.supportClasses.SkinnableTextBase;
	import spark.events.IndexChangeEvent;
	
	/**
	 * The BorgesContainer class extends List to provide functionality specific to the Borges navigator.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class BorgesContainer extends List
	{
		protected var borgesLayout:BorgesLayout;
		
		[SkinPart(required="false")]
		public var editButton:Button;

		protected var mouseDownX:Number;
		protected var mouseDownEvent:MouseEvent;
		protected var mouseDowncardPosition:Number;
		protected var mouseMoved:Boolean;
		protected var deltaX:Number;
		protected var autoListen:AutoListen;
		protected var panelsInvalid:Boolean;
		protected var panelCountInvalid:Boolean;
		protected var controlPanel:ControlPanel;
				
		private var _attachmentDataProvider:Object;
		private var _cardsPerPanel:int;
		private var _cardProvider:ICardProvider;
		
		public function BorgesContainer()
		{
			super();
//			useDragDropReordering = false;
			dragEnabled = false;
			useVirtualLayout = false;
			autoListen = new AutoListen();
		}
		
		override protected function attachSkin():void {
			super.attachSkin();
			borgesLayout = layout as BorgesLayout;
			cardProvider = new BorgesCardProvider();
		}
		
		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case editButton:
					autoListen.addListener(editButton, MouseEvent.CLICK, onEditButtonClick);
					break;
			}
		}
		
		override protected function partRemoved(partName:String, instance:Object):void
		{
			super.partRemoved(partName, instance);
			autoListen.removeListener(instance as IEventDispatcher);
		}

		protected function caretChangeHandler (event:IndexChangeEvent) : void
		{
			if (!borgesLayout)
				return;
			borgesLayout.scroll();
		}
		
		override protected function itemSelected(index:int, selected:Boolean):void
		{
			if (!selected)
				return;
			
			if (!borgesLayout)
				return;
			
			if (borgesLayout.focusIndex == index)
				return;
			
			borgesLayout.focusIndex = index;
			borgesLayout.scroll();
		}
		
		override protected function item_mouseDownHandler(event:MouseEvent):void 
		{
			if (event.isDefaultPrevented())
				return;
			
			// Don't handle mouseDown events if the user is selecting text
			var richTarget:RichEditableText = event.target as RichEditableText;
			if ((richTarget != null) && richTarget.enabled && richTarget.selectable) {
				return;
			}
			var textTarget:SkinnableTextBase = event.target as SkinnableTextBase;
			if ((textTarget != null) && textTarget.enabled && textTarget.selectable) {
				return;
			}
			
			mouseDownEvent = event;
			mouseDownEvent.stopImmediatePropagation();
			mouseDownX = mouseDownEvent.stageX;
			mouseMoved = false;
			mouseDowncardPosition = borgesLayout.panelPosition;		
			
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
		}
		
		protected function onMouseUp(event:MouseEvent):void 
		{
			if (!mouseDownEvent) // how can this be? it's happening...
				return;
			
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			if (!mouseMoved) {
				super.item_mouseDownHandler(mouseDownEvent);
			} else if (borgesLayout) {
				borgesLayout.nudgePanel(getDeltaX(event));		
			}
			mouseDownEvent = null;
		}
		
		protected function getDeltaX(event:MouseEvent):Number 
		{
			if (event.stageX != 0) // strange 0 input sometimes
				deltaX = mouseDownX - event.stageX;
			return deltaX;
		}
		
		protected function onMouseMove(event:MouseEvent):void 
		{
			if (!borgesLayout)
				return;
			
			mouseMoved = true;
			borgesLayout.panelPosition = mouseDowncardPosition + getDeltaX(event);
		}

		[Bindable]
		public function get attachmentDataProvider():Object
		{
			return _attachmentDataProvider;
		}

		public function set attachmentDataProvider(value:Object):void
		{
			_attachmentDataProvider = value;
			
			if (_attachmentDataProvider)
				_attachmentDataProvider.removeEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionChange);

			if (!dataGroup || !value)
				return;
			
			_attachmentDataProvider.addEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionChange);

			var borgesCardProvider:BorgesCardProvider = cardProvider as BorgesCardProvider;
			borgesCardProvider.list = _attachmentDataProvider as IList;
			
			onCollectionChange(null);
		}
		
		protected function onCollectionChange(event:Event):void 
		{
			if (!dataGroup || !attachmentDataProvider)
				return;
			
			var list:IList = attachmentDataProvider as IList;
			attachmentCount = list ? list.length : attachmentDataProvider.toArray().length;
			panelsInvalid = true;
			invalidateProperties();
		}
		
		protected function updatePanels():void 
		{
			if (!allPanelsCreated()) {
				callLater(updatePanels);
				return;
			}
			var numElements:int = dataGroup.numElements;
			for (var i:int = 0; i < numElements; i++) {
				var panel:Panel = dataGroup.getElementAt(i) as Panel;
				panel.dataProvider = new ArrayCollection(attachmentDataProvider.toArray().slice(i*cardsPerPanel, (i+1)*cardsPerPanel));
				panel.invalidateSkinState();
			}
		}
		
		protected function allPanelsCreated():Boolean 
		{
			var numElements:int = dataGroup.numElements;
			for (var i:int = 0; i < numElements; i++) {
				var panel:Panel = dataGroup.getElementAt(i) as Panel;
				if (!panel)
					return false;
			}
			
			return true;
		}

		[Bindable]
		public function get cardsPerPanel():int
		{
			return _cardsPerPanel;
		}

		public function set cardsPerPanel(value:int):void
		{
			_cardsPerPanel = value;
			onCollectionChange(null);
			panelCountInvalid = true;
		}

		private var _attachmentCount:int;
		
		[Bindable]
		public function get attachmentCount():int
		{
			return _attachmentCount;
		}

		public function set attachmentCount(value:int):void
		{
			_attachmentCount = value;
			panelCountInvalid = true;
			invalidateProperties();
		}
		
		override protected function commitProperties():void
		{
			super.commitProperties();
			if (panelCountInvalid) {
				panelCountInvalid = false;
				adjustPanelCount();
			}
			if (panelsInvalid) {
				panelsInvalid = false;
				updatePanels();
			}
		}
		
		protected function adjustPanelCount():void
		{
			var panelCount:int = (_attachmentCount + cardsPerPanel - 1) / cardsPerPanel;
			
			if (dataProvider && panelCount == dataProvider.length)
				return;
			
			var array:Array = [];
			for (var i:int = 0; i < panelCount; ++i) {
				array.push(i);
			}
			dataProvider = new ArrayCollection(array);
			
//			if (borgesLayout.focusIndex >= panelCount)
//				borgesLayout.focusIndex = panelCount -1;
			
			borgesLayout.invalidateSize();
		}
		
		protected function onEditButtonClick(e:MouseEvent):void
		{
			if (!controlPanel) {
				controlPanel = PopUpManager.createPopUp(this, ControlPanel, false) as ControlPanel;
				controlPanel.title = "Edit Layout";
				controlPanel.x = width - 20 - controlPanel.width;
				controlPanel.y = height - 60 - controlPanel.height;
				controlPanel.customProperties = PropertyManager.instance.commonProperties as CustomProperties;
				controlPanel.addEventListener(Event.CLOSE, onClosePanel);
				BindingUtils.bindProperty(controlPanel, "focusIndex", layout, "focusIndex");
			} else {
				PopUpManager.addPopUp(controlPanel, this);
			}
		}
		
		protected function onClosePanel(e:Event):void {
			PopUpManager.removePopUp(controlPanel);
		}

		[Bindable]
		public function get cardProvider():ICardProvider
		{
			return _cardProvider;
		}

		public function set cardProvider(value:ICardProvider):void
		{
			_cardProvider = value;
		}

	}
}
