package properties
{
	import com.adobe.portfolio.properties.CommonProperties;
	import com.adobe.portfolio.properties.PropertyBooleanValue;
	import com.adobe.portfolio.properties.PropertyContext;
	import com.adobe.portfolio.properties.PropertyNumberValue;
	import com.adobe.portfolio.properties.PropertyStringValue;
	
	
	[Bindable]
	public class CustomProperties extends CommonProperties
	{
		public var fieldOfView:PropertyNumberValue;
		public var zDistance:PropertyNumberValue;
		public var panelWidth:PropertyNumberValue;
		public var panelHeight:PropertyNumberValue;
		public var cardsPerPanel:PropertyNumberValue;
		public var galleryGeometry:PropertyStringValue;
		public var xAngle:PropertyNumberValue;
		public var spotlight:PropertyBooleanValue;
		public var focusIndex:PropertyNumberValue;
		
		public function CustomProperties()
		{
			super();
		}
		
		protected function initNumberField(context:PropertyContext, 
										   propertyName:String, 
										   value:Number, minValue:Number = NaN, maxValue:Number = NaN):void
		{
			var contextProperty:PropertyNumberValue = context.getProperty(propertyName) as PropertyNumberValue;
			if (contextProperty == null) {
				contextProperty = new PropertyNumberValue();
				contextProperty.name = propertyName;
				contextProperty.minValue = minValue;
				contextProperty.maxValue = maxValue;
				contextProperty.numValue = value;
				context.properties.push(contextProperty);
			}
			contextProperty.dirty = true;
			this[propertyName] = contextProperty;
		}
				
		protected function initStringField(context:PropertyContext, 
										 propertyName:String,
										 value:String,
										 visible:Boolean = true):void
		{
			var contextProperty:PropertyStringValue = context.getProperty(propertyName) as PropertyStringValue;
			if (contextProperty == null) {
				contextProperty = new PropertyStringValue();
				contextProperty.name = propertyName;
				contextProperty.value = value;
				context.properties.push(contextProperty);
			}
			contextProperty.visible = visible;
			contextProperty.dirty = true;
			this[propertyName] = contextProperty;
		}
		
		protected function initBooleanField(context:PropertyContext, 
										   propertyName:String,
										   value:Boolean,
										   visible:Boolean = true):void
		{
			var contextProperty:PropertyBooleanValue = context.getProperty(propertyName) as PropertyBooleanValue;
			if (contextProperty == null) {
				contextProperty = new PropertyBooleanValue();
				contextProperty.name = propertyName;
				contextProperty.value = value;
				context.properties.push(contextProperty);
			}
			contextProperty.visible = visible;
			contextProperty.dirty = true;
			this[propertyName] = contextProperty;
		}

		override protected function loadContext(context:PropertyContext):void
		{
			super.loadContext(context);
			if (context == null) {
				return;
			}
			
			if (context.name != "com.adobe.portfolio.Borges")
				return;
			
			initNumberField(context, "fieldOfView", 15);
			initNumberField(context, "zDistance", 400);
			initNumberField(context, "panelWidth", 900);
			initNumberField(context, "panelHeight", 600);
			initNumberField(context, "cardsPerPanel", 2);
			initNumberField(context, "xAngle", 0);
			initNumberField(context, "focusIndex", 0);
			initBooleanField(context, "spotlight", false);
			initStringField(context, "galleryGeometry", "", false);
		}
		
	}
}