package 
{
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.cards.CardFactory;
	import com.adobe.portfolio.utils.FileUtil;
	
	import mx.core.ClassFactory;
	import mx.core.IFactory;
	
	public class ExtensionCardFactory extends CardFactory
	{
		private static var imageCardFactories:Object = new Object();
		private static var textCardFactories:Object = new Object();
		private static var rendererFunctions:Object = new Object();
		
		private static var extensionMap:Object = {
			/*
			jpg: ImageCard,
			jpeg: ImageCard,
			png: ImageCard,
			gif: ImageCard,
			*/
			txt: TextCard
		}

		public function ExtensionCardFactory(cardClass:Class,
											 cardType:String = BASIC_CARD,
											 currentState:String = null,
											 dropShadowPreference:String = null,
											 rollOverPreference:String = null,
											 alpha:Number = 1,
											 enableThumbnailGeneration:Boolean = true,
											 adjustAspectRatio:Boolean = false)
		{
			super(cardType, currentState, dropShadowPreference, rollOverPreference, alpha, enableThumbnailGeneration, adjustAspectRatio);
			if (cardType == BASIC_CARD) {
				generator = cardClass;
			}
		}
		
		public static function getItemRendererFunction(defaultType:String = "BasicCard",
													   defaultState:String = null,
													   dropShadowPreference:String = null,
													   rollOverPreference:String = null,
													   alpha:Number = 1,
													   enableThumbnailGeneration:Boolean = true,
													   maintainAspectRatio:Boolean = false,
													   supportFolder:Boolean = true):Function
		{
			var factoryKey:String = defaultType +
				alpha.toString() +
				enableThumbnailGeneration.toString() +
				maintainAspectRatio.toString() + 
				((defaultState == null) ? "null" : defaultState) +
				((dropShadowPreference == null) ? "null" : dropShadowPreference) +
				((rollOverPreference == null) ? "null" : rollOverPreference);
			var functionKey:String = factoryKey + supportFolder.toString();
			
			var returnFunction:Function = rendererFunctions[functionKey] as Function;
			if (returnFunction != null) {
				return returnFunction;
			}
			
			var superFunction:Function = CardFactory.getItemRendererFunction(
				defaultType,
				defaultState,
				dropShadowPreference,
				rollOverPreference,
				alpha,
				enableThumbnailGeneration,
				maintainAspectRatio,
				supportFolder);
			
			var imageCardFactory:ExtensionCardFactory = imageCardFactories[factoryKey] as ExtensionCardFactory;
			if (imageCardFactory == null) {
				imageCardFactory = new ExtensionCardFactory(ImageCard, defaultType, defaultState, dropShadowPreference, rollOverPreference, alpha, enableThumbnailGeneration, maintainAspectRatio);
				imageCardFactories[factoryKey] = imageCardFactory;
			}
						
			var textCardFactory:ExtensionCardFactory = textCardFactories[factoryKey] as ExtensionCardFactory;
			if (textCardFactory == null) {
				textCardFactory = new ExtensionCardFactory(TextCard, defaultType, defaultState, dropShadowPreference, rollOverPreference, alpha, enableThumbnailGeneration, maintainAspectRatio);
				textCardFactories[factoryKey] = textCardFactory;
			}

			returnFunction = function(data:Object):IFactory
			{
				var attachment:IAttachment = data as IAttachment;
				if (attachment == null) {
					return superFunction(data);
				}
				if (!attachment.isFolder) {
					var extension:String = FileUtil.getExtension(attachment.fileName).toLowerCase();
					var cardClass:Class = (extension == null) ? null : extensionMap[extension]
					if (cardClass == ImageCard) {
						return imageCardFactory;
					}
					if (cardClass == TextCard) {
						return textCardFactory;
					}
				}
				var returnClass:IFactory = superFunction(data);
				return returnClass;
			};
			
			rendererFunctions[functionKey] = returnFunction;
			return returnFunction;
		}
	}
}