/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.animations
{
	import flash.events.IEventDispatcher;
	
	import mx.effects.Parallel;
	import mx.events.EffectEvent;
	
	import spark.effects.Animate;
	import spark.effects.animation.Animation;
	import spark.effects.animation.MotionPath;
	import spark.effects.animation.SimpleMotionPath;

	
	/**
	 *  The AnimateFactory class is a utility class for easily creating Animate objects.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class AnimateFactory
	{
		
		/**
		 *  Creates and plays an animation of a property of a target object
		 *  <p>Sample usage:  tweenTo(visualElement, 2000, {alpha:1});</p>
		 * 
		 *  @param target The target object.
		 *  @param duration The time duration of the animation effect in milliseconds.
		 *  @param property The property (of the target) to animate.
		 *  @param value The new value of the property.  The animation will go from 
		 *  the current value of the property to the value specified by this parameter.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static function tweenTo(target:Object, duration:Number, property:String, value:*):void 
		{
			createAnimate(target, duration, property, value).play();
		}
		
		/**
		 *  Creates an Animate object that animates a property of a target object
		 * 
		 *  <p>Sample usage:  createAnimate(visualElement, 2000, {alpha:1});</p>
		 * 
		 *  @param target The target object.
		 *  @param duration The time duration of the animation effect in milliseconds.
		 *  @param property The property (of the target) to animate.
		 *  @param value The new value of the property object.  The animation will go from 
		 *  the current value of the object property to the value specified in this parameter.
		 *  @return An Animate object that can be played.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static function createAnimate(target:Object, duration:Number, property:String, value:*):Animate 
		{
			var animate:Animate = new Animate(target);
			var motionPaths:Vector.<MotionPath> = new Vector.<MotionPath>();
			
			motionPaths.push(new SimpleMotionPath(property, target[property], value));
			animate.motionPaths = motionPaths;
			animate.duration = duration;
			
			return animate;
		}
		
		/**
		 *  Creates a parallel effect from an array of animations.
		 *  The set or animations in the array will be played at the
		 *  same time.
		 * 
		 *  @param animations An array of Animate objects that are to be played at the same time.
		 *  @return A parallel effect object that plays its child effects at the same time.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static function createParallel(animations:Array):Parallel
		{
			var parallel:Parallel = new Parallel();
			parallel.children = animations;
			
			return parallel;
		}
	}
}