/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.animations
{
	import mx.core.IVisualElement;
	
	import spark.components.supportClasses.GroupBase;

	
	/**
	 *  The SequentialFade class implements a "fade-in" effect for elements in a GroupBase.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class SequentialFade
	{
		private var fadeAnimations:AnimationSequence;
		private var _onElementDone:Function;
		
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function SequentialFade()
		{
		}
		
		/**
		 *  Fade in all of the elements of a GroupBase.
		 * 
		 *  @param duration The time duration of the fade (per element) in milliseconds.
		 *  @param delay The time duration in milliseconds of the delay before starting the fade.
		 *  @param onDone An optional callback function of the form: callback(target:Object):void, 
		 *  where target is fade in animation target object. The callback function is called
		 *  when the fade-in completes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function fadeInElements(target:GroupBase, duration:Number, delay:Number, onDone:Function = null):void 
		{
			const alpha:Number = 1;
			var numElements:uint = target.numElements;
			
			if (!fadeAnimations) {
				fadeAnimations = new AnimationSequence();
				fadeAnimations.onElementDone = onElementDone;
			}
			for (var i:int = 0; i < numElements; i++ ) {
				var visualElement:IVisualElement = target.getElementAt(i);
				if (visualElement && visualElement.alpha == 0) {
					fadeAnimations.addAnimation(visualElement, duration, "alpha", alpha, delay);
				}
			}
			fadeAnimations.onDone = onDone;
		}
		
		
		/**
		 *  Returns true if the fade-in animation sequence is running.
		 * 
		 *  @return <code>true</code> if the fade-in animation sequence is running; <code>false</code> otherwise.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function get fading():Boolean 
		{
			return fadeAnimations ? (fadeAnimations.length > 0) : 0;
		}

		/**
		 *  The callback function to run when an element of the fade-in sequence has completed.
		 * 
		 *  @param value a callback function of the form: callback(element:Object):void, 
		 *  where element is an element of the target for which fade-in has been completed. 
		 *  The callback function is called when an element of the fade-in sequence has completed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function set onElementDone(value:Function):void
		{
			_onElementDone = value;
		}

		/**
		 *  @private
		 */
		public function get onElementDone():Function
		{
			return _onElementDone;
		}
	}
}