/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.cards
{
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.managers.CommandManager;
	import com.adobe.portfolio.managers.StringManager;
	import com.adobe.portfolio.managers.ThumbnailManager;
	import com.adobe.portfolio.previews.Preview;
	import com.adobe.portfolio.previews.PreviewFactory;
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.utils.Dictionary;
	
	import mx.effects.IEffect;
	import mx.events.EffectEvent;
	import mx.events.FlexEvent;
	import mx.events.PropertyChangeEvent;
	import mx.states.State;
	
	import spark.components.Button;
	
	[SkinState("normal")]
	[SkinState("over")]
	[SkinState("selected")]
	[SkinState("overAndSelected")]
	
	[SkinState("smallNormal")]
	[SkinState("mediumNormal")]
	[SkinState("largeNormal")]
	[SkinState("infoNormal")]
	
	[SkinState("smallOver")]
	[SkinState("mediumOver")]
	[SkinState("largeOver")]
	[SkinState("infoOver")]
	
	[SkinState("smallSelected")]
	[SkinState("mediumSelected")]
	[SkinState("largeSelected")]
	[SkinState("infoSelected")]
	
	[SkinState("smallOverAndSelected")]
	[SkinState("mediumOverAndSelected")]
	[SkinState("largeOverAndSelected")]
	[SkinState("infoOverAndSelected")]
	
	/**
	 * The BasicCard class extends SimpleCard and implements a card with additional controls and three different layouts.
	 * 
	 * It implements IListItemRenderer,
	 * so it can be used as an item renderer for relevant containers. The skin is not required
	 * to provide all the relevant controls, which allows this object to act as the core for
	 * both simple (mini-nav) and complex (basic click-through card) rendering.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class BasicCard extends SimpleCard implements IBasicCard
	{
		// These match the names of the SkinPart variables.
		
		/**
		 *  String literal used to represent the previewButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_PREVIEW_BUTTON:String = "previewButton";
		
		/**
		 *  String literal used to represent the extractButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_EXTRACT_BUTTON:String = "extractButton";
		
		/**
		 *  String literal used to represent the infoButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_INFO_BUTTON:String = "infoButton";
		
		/**
		 *  String literal used to represent the infoCloseButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_INFO_CLOSE_BUTTON:String = "infoCloseButton";
		
		/**
		 *  String literal used to represent the deleteButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_DELETE_BUTTON:String = "deleteButton";
		
		/**
		 *  String literal used to represent the extractOrDeleteButton skin part 
		 *  exposed by BasicCard to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_EXTRACT_OR_DELETE_BUTTON:String = "extractOrDeleteButton";
		
		/**
		 *  String literal used to represent the "small" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const STATE_SMALL:String = "small";
		
		/**
		 *  String literal used to represent the "medium" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const STATE_MEDIUM:String = "medium";
		
  	    /**
	 	 *  String literal used to represent the "large" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const STATE_LARGE:String = "large";
		
		/**
		 *  String literal used to represent the "info" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const STATE_INFO:String = "info";
		
		/**
		 * String literal used to represent the small card normal state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_SMALL_NORMAL:String = "smallNormal";
		
		/**
		 * String literal used to represent the small card over state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_SMALL_OVER:String = "smallOver";
		
		/**
		 * String literal used to represent the small card selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_SMALL_SELECTED:String = "smallSelected";
		
		/**
		 * String literal used to represent the small card over and selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_SMALL_OVER_AND_SELECTED:String = "smallOverAndSelected";
		
		/**
		 * String literal used to represent the medium card normal state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_MEDIUM_NORMAL:String = "mediumNormal";
		
		/**
		 * String literal used to represent the medium card over state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_MEDIUM_OVER:String = "mediumOver";
		
		/**
		 * String literal used to represent the medium card selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_MEDIUM_SELECTED:String = "mediumSelected";
		
		/**
		 * String literal used to represent the medium card over and selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_MEDIUM_OVER_AND_SELECTED:String = "mediumOverAndSelected";
		
		/**
		 * String literal used to represent the large card normal state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_LARGE_NORMAL:String = "largeNormal";
		
		/**
		 * String literal used to represent the large card over state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_LARGE_OVER:String = "largeOver";
		
		/**
		 * String literal used to represent the large card selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_LARGE_SELECTED:String = "largeSelected";
		
		/**
		 * String literal used to represent the large card over and selected state of a BasicCard skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_LARGE_OVER_AND_SELECTED:String = "largeOverAndSelected";
		
		/**
		 *  String literals used to represent the normal "info" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_INFO_NORMAL:String = "infoNormal";
		
		/**
		 *  String literals used to represent the over "info" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_INFO_OVER:String = "infoOver";
		
		/**
		 *  String literals used to represent the selected "info" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_INFO_SELECTED:String = "infoSelected";
		
		/**
		 *  String literals used to represent the over and selected "info" state of BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const SKIN_STATE_INFO_OVER_AND_SELECTED:String = "infoOverAndSelected";
		
		/**
		 *  String literal used to represent the default dimensions for the small state of BasicCard.
		 *  The value is chosen so that the preview size of the small card does not exceed 128x128 pixels
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public static const DEFAULT_SMALL_DIMENSION:Number = 152;
		
		/**
		 *  Boolean switch indicating whether to run a large or normal zoom effect.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var zoomToLarge:Boolean = false;
		
		/**
		 *  The Flex effect to run when zooming to a medium card size
		 * 
		 * 	@see IBasicCardSkin#zoomNormalEffect
		 * 	
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var zoomNormal:IEffect;
		
		/**
		 *  The Flex effect to run when zooming to a large card size
		 * 
		 * 	@see IBasicCardSkin#zoomLargeEffect
		 * 	
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var zoomLarge:IEffect;
		
		/**
		 *  A dictionary used to track cards that are in their info state.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected static var infoMap:Dictionary = new Dictionary();
		
		/** Default constructor for BasicCard.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function BasicCard()
		{
			super();
			// Setup the states for this instance. They should match the skin states.
			states = [	new State({name: STATE_SMALL}),
						new State({name: STATE_MEDIUM}),
						new State({name: STATE_LARGE}),
						new State({name: STATE_INFO})];
			
			// Default to the small state, since that will give us the best chance of
			// using cached thumbnails.
			currentState = STATE_SMALL;
			downSampleOK = true;
			addEventListener(FlexEvent.CREATION_COMPLETE,
				function (e:Event):void {
					e.target.removeEventListener(FlexEvent.CREATION_COMPLETE, arguments.callee);
					initZoom();
				}
			);
		}
		
		/**
		 *  Initialize zoom effects and set up event listener for such events.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function initZoom():void
		{
			if (rollOverPreference != CardFactory.ROLL_OVER_SCALE)
				return;
			
			var basicCardSkin:IBasicCardSkin = skin as IBasicCardSkin;
			if (!basicCardSkin)
				return;
			
			if (!zoomNormal) {
				zoomNormal = basicCardSkin.zoomNormalEffect;
				if (zoomNormal) {
					zoomNormal.addEventListener(EffectEvent.EFFECT_END, 
						function (e:EffectEvent):void {
//							logger.debug("zoomNormal effect end");
							zoomCard(mouseIsOver);
						}
					);
				}
			}
			if (!zoomLarge) {
				zoomLarge = basicCardSkin.zoomLargeEffect;
				if (zoomLarge) {
					zoomLarge.addEventListener(EffectEvent.EFFECT_END, 
						function (e:EffectEvent):void {
//							logger.debug("zoomLarge effect end");
							zoomCard(mouseIsOver);
						}
					);
				};
			}
		}
		
		[SkinPart(required="false")]
		/**
		 *  previewButton is the button control to initiate a full preview in the viewing application.
		 *  It is an optional skin part, so not necessarily implemented for all skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var previewButton:Button;
		
		[SkinPart(required="false")] 
		/**
		 *  extractButton is the button control to initiate save an attachment to a user-specified location.
		 *  It is an optional skin part, so not necessarily implemented for all skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		public var extractButton:Button;
		
		[SkinPart(required="false")] 
		/**
		 *  infoButton is the button control to toggle the info state of a card. When in
		 *  the info state, the thumbnail is replaced with additional metadata.
		 *  It is an optional skin part, so not necessarily implemented for all skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var infoButton:Button;
		
		[SkinPart(required="false")] 
		/**
		 *  infoCloseButton is the button control for closing the Info state
		 *  (information on the card) for a card.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var infoCloseButton:Button;
		
		[SkinPart(required="false")] 
		/**
		 *  deleteButton is the button control to delete the attachment represented by a card.
		 *  It is an optional skin part, so not necessarily implemented for all skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var deleteButton:Button;
		
		[SkinPart(required="false")]
		/**
		 *  extractOrDeleteButton behave like deleteButton when in authoring mode, and extractButton
		 *  otherwise.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var extractOrDeleteButton:Button;
		
		/**
		 *  The event handler for clicking the preview button events. By default, this invokes SimpleCard.handlePreview.
		 * 
		 * 	@see SimpleCard#handlePreview
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onPreviewClick(evt:Event):void
		{
			handlePreview();
		}
		
		/**
		 *  The event handler for clicking the info button events. By default, this toggles the current state
		 * 	between info and large.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onInfoClick(evt:Event):void
		{
			if (currentState == BasicCard.STATE_INFO) {
				currentState = BasicCard.STATE_LARGE;
				delete infoMap[data];
			} else {
				currentState = BasicCard.STATE_INFO;
				infoMap[data] = true;
			}
		}
		
		/**
		 *  The event handler for closing the BasicCard's info state. By default, this returns the current state
		 * 	to large from info.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onInfoCloseClick(evt:Event):void
		{
			if (currentState == BasicCard.STATE_INFO) {
				currentState = BasicCard.STATE_LARGE;
				delete infoMap[data];
			}
		}
		
		/**
		 *  The event handler for clicking the extract button events. By default, this invokes the command
		 * 	to allow the user to save the attachment data as a local file.
		 * 
		 * 	@see CommandManager#extractAttachment()
		 * 	@see ICommand
		 * 	@see CommandType#EXTRACT_FILES
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onExtractClick(evt:Event):void
		{
			if (attachment != null) {
				CommandManager.instance.extractAttachment(attachment);
			}
		}
		
		/**
		 *  The event handler for the clicking the delete button events. By default, this deletes the attachment
		 * 	data from the portfolio.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onDeleteClick(evt:Event):void
		{
			if (attachment != null) {
				CommandManager.instance.deleteAttachment(attachment);
			}
		}
		/**
		 *  The event handler for the handling either clicking the delete button events.
		 *  or clicking the extract button events.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function onExtractOrDeleteClick(event:Event):void
		{
			if (collectionManager.authoringMode) {
				onDeleteClick(event);
			} else {
				onExtractClick(event);
			}
		}
		/**
		 *  For an extractOrDelete button control set the label to either delete (file or folder,
		 *  depending on the attachment type) or extract, depending on the authoring mode.
		 *  An extractOrDelete button control behaves like a deleteButton when in authoring mode
		 *  and like an extractButton when not in authoring mode.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function setExtractOrDeleteInfo():void
		{
			if (extractOrDeleteButton == null) {
				return;
			}
			if ((attachment != null) && attachment.isFolder) {
				extractOrDeleteButton.label = extractOrDeleteButton.toolTip = 
					collectionManager.authoringMode ? StringManager.instance.deleteFolder : StringManager.instance.extractFolder;
			} else {
				extractOrDeleteButton.label = extractOrDeleteButton.toolTip = 
					collectionManager.authoringMode ? StringManager.instance.deleteFile : StringManager.instance.extractFile;
			}
		}
		
		/** 
		 *  @inheritDoc
		 *
		 *	@see #STATE_SMALL
		 * 	@see #STATE_MEDIUM
		 * 	@see #STATE_LARGE
		 * 	@see #STATE_INFO
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void
		{
			if (stateName != currentState) {
				super.setCurrentState(stateName, playTransition);
				
				// We used to call skin.currentState = stateName; here, but simply invalidating
				// plays nicer with the Flex model.
				invalidateSkinState();

				if (preview != null) {
					if (preview.skinState != preferredPreviewState) {
						invalidatePreview(true);
						preview.skinState = preferredPreviewState;
					} 
				}
				
				if (skin)
					skin.validateNow();
				
				hasFocusableChildren = ((stateName == STATE_LARGE) || (stateName == STATE_INFO));
			}
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 * 
		 */
		override protected function getCurrentSkinState():String
		{
			switch (currentState) {
				case STATE_SMALL:
					if (mouseIsOver) {
						return selected ? SKIN_STATE_SMALL_OVER_AND_SELECTED : SKIN_STATE_SMALL_OVER;
					} else {
						return selected ?  SKIN_STATE_SMALL_SELECTED : SKIN_STATE_SMALL_NORMAL;
					}
				case STATE_MEDIUM:
					if (mouseIsOver) {
						return selected ? SKIN_STATE_MEDIUM_OVER_AND_SELECTED : SKIN_STATE_MEDIUM_OVER;
					} else {
						return selected ?  SKIN_STATE_MEDIUM_SELECTED : SKIN_STATE_MEDIUM_NORMAL;
					}
				case STATE_LARGE:
					if (mouseIsOver) {
						return selected ? SKIN_STATE_LARGE_OVER_AND_SELECTED : SKIN_STATE_LARGE_OVER;
					} else {
						return selected ?  SKIN_STATE_LARGE_SELECTED : SKIN_STATE_LARGE_NORMAL;
					}
				case STATE_INFO:
					if (mouseIsOver) {
						return selected ? SKIN_STATE_INFO_OVER_AND_SELECTED : SKIN_STATE_INFO_OVER;
					} else {
						return selected ?  SKIN_STATE_INFO_SELECTED : SKIN_STATE_INFO_NORMAL;
					}
				default:
					CONFIG::logging{
						LogHelper.logger.warn("Unexpected component state: "+currentState);
					}
					if (mouseIsOver) {
						return selected ? SKIN_STATE_MEDIUM_OVER_AND_SELECTED : SKIN_STATE_MEDIUM_OVER;
					} else {
						return selected ?  SKIN_STATE_MEDIUM_SELECTED : SKIN_STATE_MEDIUM_NORMAL;
					}
			}
		}
		
		/** 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partAdded
			handlePartChange(partName, instance, true);
		}
		/** 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partRemoved
			handlePartChange(partName, instance, false);
		}

		/** 
		 *  @private
		 *  Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if ((instance != null) && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case PART_PREVIEW_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onPreviewClick ];
						previewButton.label = previewButton.toolTip = StringManager.instance.fullPreview;
						break;
					
					case PART_EXTRACT_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onExtractClick ];
						extractButton.label = extractButton.toolTip = StringManager.instance.extractFile;
						break;

					case PART_INFO_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onInfoClick ];
						infoButton.label = infoButton.toolTip = StringManager.instance.showInfo;
						break;

					case PART_INFO_CLOSE_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onInfoCloseClick ];
						infoCloseButton.label = infoCloseButton.toolTip = StringManager.instance.closeInfo;
						break;
					
					case PART_DELETE_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onDeleteClick ];
						deleteButton.label = deleteButton.toolTip = StringManager.instance.deleteFile;
						break;
					
					case PART_EXTRACT_OR_DELETE_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onExtractOrDeleteClick ];
						setExtractOrDeleteInfo();
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = (listeners == null) ? 0 : listeners.length;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function get allowMultiPage():Boolean
		{
			// Calculating the page count is expensive, so only allow multi-page at all if our attachment supports it
			var previewState:String = collectionManager.apiIsSupported(CollectionManager.API_VERSION_10) ?
				PreviewFactory.instance.previewStateFromMimeType(mimeTypeFromAttachment(), extensionFromAttachment()) :
				Preview.THUMBNAIL;
			return (previewState == Preview.PAGING_THUMBNAIL);
		}
		
		/**
		 * 	@inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function get prefersMediaPreview():Boolean
		{
			var preferredState:String = preferredPreviewState;
			return ((preferredState == Preview.AUDIO) || (preferredState == Preview.VIDEO));
		}
		
		/**
		 *  Returns the preferred preview state based on the MIME type or file
		 * 	extension of the current attachment.
		 * 
		 * 	@inheritDoc
		 * 
		 * 	@see Preview#AUDIO
		 * 	@see Preview#PAGING_THUMBNAIL
		 * 	@see Preview#THUMBNAIL
		 * 	@see Preview#VIDEO
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function get preferredPreviewState():String
		{
			if (!attachment)
				return Preview.THUMBNAIL;
			
			if ((currentState != STATE_LARGE) && (currentState != STATE_INFO)) {
				return Preview.THUMBNAIL;
			}
			if (!collectionManager.apiIsSupported(CollectionManager.API_VERSION_10)) {
				return Preview.THUMBNAIL;
			}
			var previewState:String = PreviewFactory.instance.previewStateFromMimeType(mimeTypeFromAttachment(), extensionFromAttachment());
			if (previewState == Preview.PAGING_THUMBNAIL && (!allowMultiPage || !ThumbnailManager.instance.enableDynamicThumbnails || (attachment.pageCount <= 1))) {
				return Preview.THUMBNAIL;
			}
			return previewState;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function invalidateSkinState():void
		{
			super.invalidateSkinState();
			zoomCard(mouseIsOver);
		}
		
		/**
		 *  Set the zoomToLarge Boolean switch for the BasicCard and
		 *  play the relevant zoom effect (zoomLarge or zoomNormal)
		 * 
		 *  @param zoomToLarge a Boolean, <code>true</code> if the zoomLarge
		 *  effect is to be used, <code>false</code> if the zoomNormal effect is to be used.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		protected function zoomCard(zoomToLarge:Boolean):void
		{
//			logger.debug("zoomCard: zoomToLarge= " + zoomToLarge);
			if (this.zoomToLarge == zoomToLarge)
				return;
			
			this.zoomToLarge = zoomToLarge;
			
			var effect:IEffect = zoomToLarge ? zoomLarge : zoomNormal;
			if (effect && !effect.isPlaying) {
				effect.play();
			}
		}

		/**
		 *  @inheritDoc
		 * 
		 *	@see CollectionManager
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function onHostPropertyChange(event:PropertyChangeEvent):void
		{
			if (event.property == "authoringMode") {
				setExtractOrDeleteInfo();
			}
			super.onHostPropertyChange(event);
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function updateRenderer(oldRenderer:ISimpleCard):void 
		{
			super.updateRenderer(oldRenderer);
			var oldBasicCard:IBasicCard = oldRenderer as IBasicCard;
			if (oldBasicCard) {
				setCurrentState(oldBasicCard.currentState);
			}
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function set data(val:Object):void
		{
			if (data == val) {
				return;
			}
			super.data = val;
			if ((val != null) && (currentState == STATE_LARGE) && infoMap[val]) {
				// We left this card in its info state, so take it back
				setCurrentState(STATE_INFO, false);
			}
		}
	}
}
