/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.cards
{
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.states.State;
	
	import spark.components.Button;

	
	/**
	 *  The FolderCard class extends BasicCard and displays attachments that are folders.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public class FolderCard extends BasicCard
	{
		/**
		 *  Dispatched when the user clicks the close button for this component.
		 */
		[Event(name="closeButton", type="flash.events.Event")]
		
		[SkinPart(required="false")]
		/**
		 *  closeButton is the button control to close the folder.
		 *  It is an optional skin part, so not necessarily implemented for all skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.1
		 *
		 */
		public var closeButton:Button;

		/**
		 *  The string literal used to specify a simple state.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public static const STATE_SIMPLE:String = "simple";

		private var _showCloseButton:Boolean;
		
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function FolderCard()
		{
			super();
			if (states != null) {
				states.push(new State({name: STATE_SIMPLE}));
			}
			_showCloseButton = false;
		}
		
		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			if (instance != closeButton)
				return;
			
			closeButton.addEventListener(MouseEvent.CLICK, onCloseButton);
		}
		
		override protected function partRemoved(partName:String, instance:Object):void
		{
			super.partRemoved(partName, instance);
			if (instance != closeButton)
				return;
			
			closeButton.removeEventListener(MouseEvent.CLICK, onCloseButton);
		}
		
		protected function onCloseButton(event:MouseEvent):void
		{
			dispatchEvent(new Event("closeButton"));
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override protected function getCurrentSkinState():String
		{
			if (currentState == STATE_SIMPLE) {
				if (mouseIsOver) {
					return selected ? SimpleCard.SKIN_STATE_OVER_AND_SELECTED : SimpleCard.SKIN_STATE_OVER;
				} else {
					return selected ?  SimpleCard.SKIN_STATE_SELECTED : SimpleCard.SKIN_STATE_NORMAL;
				}
			}
			return super.getCurrentSkinState();
		}
		
		
		/**
		 * 	<p>If maintainAspectRatio is <code>true</code>, then this call will attempt to prevent
		 * 	the card dimensions from exceeding its preferred (measured) dimensions.</p>
		 * 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function setLayoutBoundsSize(width:Number, height:Number, postLayoutTransform:Boolean=true):void
		{
			if (!maintainAspectRatio) {
				super.setLayoutBoundsSize(width, height);
				return;
			}
			// We really don't want to scale up the folder card
			super.setLayoutBoundsSize(Math.min(measuredWidth, width), Math.min(measuredHeight, height));
		}

		/**
		 * 	Property that controls the visibility of the closeButton (if present)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.1
		 *
		 */	
		[Bindable]
		public function get showCloseButton():Boolean
		{
			return _showCloseButton;
		}

		public function set showCloseButton(value:Boolean):void
		{
			_showCloseButton = value;
		}

	}
}