/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.cards
{
	
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	
	import mx.core.IButton;
	import mx.events.FlexEvent;
	
	/**
	 *  
	 * The GuideCard class extends SimpleCard to support next/previous card navigation and is used by the Linear navigator.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public class GuideCard extends SimpleCard
	{
		/**
		 *  A string literal used to represent the previous attachment button
		 *  skin parts exposed by OneUp to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		static public const PART_PREV_BUTTON:String = "prevButton";
		/**
		 *  A string literal used to represent the next attachment button
		 *  skin parts exposed by OneUp to its skins.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		static public const PART_NEXT_BUTTON:String = "nextButton";
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function GuideCard()
		{
			super();
			focusEnabled = false;
			hasFocusableChildren = true;
		}
		
		[SkinPart(required="false")]
		/**
		 *  A button for navigating to the previous attachment,
		 *  optionally provided by this instance's skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public var prevButton:IButton;
		
		[SkinPart(required="false")]
		/**
		 *  A button for navigating to the next attachment,
		 *  optionally provided by this instance's skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public var nextButton:IButton;

		/** 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override protected function partAdded(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partAdded
			handlePartChange(partName, instance, true);
		}

		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override protected function partRemoved(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partRemoved
			handlePartChange(partName, instance, false);
		}
		/** 
		 *  @private
		 * 
		 *  Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if ((instance != null) && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case PART_NEXT_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ collectionManager.onGotoNextAttachment ];
						break;
					case PART_PREV_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ collectionManager.onGotoPreviousAttachment ];
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = (listeners == null) ? 0 : listeners.length;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
	}
}