/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.cards
{
	
	/**
	 *  ICardProvider is an interface that should be implemented
	 *  by Layouts that want to use CardSlider to scroll to a Card.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public interface ICardProvider
	{
		[Bindable (name="maxCardPositionChanged")]
		/**
		 *  Returns the maximum card position of the layout 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get maxCardPosition():Number;

		[Bindable (name="cardPositionChanged")]
		/**
		 *  Returns the current card position
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get cardPosition():Number
		
		
		/**
		 *  Sets a pending card position 
		 * 
		 *  @param value The position number to set.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function set pendingCardPosition(value:Number):void;
		
		
		/**
		 *  Returns the card at the given position.
		 * 
		 *  @param cardPosition The position number of the card to return.
		 *  @return The ISimpleCard that has the specified card position.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function getCard(cardPosition:Number):ISimpleCard
	}
}