/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.cards
{
	import com.adobe.portfolio.components.supportClasses.ColorPalette;
	import com.adobe.portfolio.previews.IPreview;
	
	import flash.events.IEventDispatcher;
	
	import mx.core.IVisualElement;
	

	[SkinState("normal")]
	[SkinState("over")]
	[SkinState("selected")]
	[SkinState("overAndSelected")]
	/**
	 *  ISimpleCard is an interface that describes the minimum capabilities of a SimpleCard.  
	 * 
	 *  Typically layouts and skins will use this interface instead of referencing SimpleCard directly.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public interface ISimpleCard extends IEventDispatcher, IVisualElement
	{
		[Bindable (event="propertyChange")]
		/**
		 *  The file name of this card's underlying attachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get fileName():String;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The description of this card's underlying attachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get description():String;
		/**
		 *  @private
		 *
		 */	
		function set description(val:String):void;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The modification date of this card's underlying attachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get modDate():Date;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The creation date of this card's underlying attachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get creationDate():Date;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The size of this card's underlying attachment, if it were to be uncompressed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get size():uint;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The size of this card's underlying attachment as it is currently compressed
		 *  in the Navigator file.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get compressedSize():uint;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The color palette being used by this object.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get colorPalette():ColorPalette;
		
		[Bindable (event="propertyChange")]
		/**
		 *  The data object to be renderered. Only IAttachment objects are supported.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get data():Object;
		/**
		 *  @private
		 *
		 */	
		function set data(val:Object):void;
		
		[Bindable (event="propertyChange")]
		/**
		 *  @copy IItemRenderer#selected
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get selected():Boolean;
		/**
		 *  @private
		 *
		 */	
		function set selected(value:Boolean):void;
		
		/**
		 * <code>true</code> if it is OK to scale down the image;
		 * otherwise <code>false</code>.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get downSampleOK():Boolean;
		/**
		 *  @private
		 *
		 */	
		function set downSampleOK(value:Boolean):void;
				
		/**
		 *  Delay the creation of the thumbnail image for this card.
		 * 
		 *  @param value If <code>true</code> then do not generate the thumbnail image;
		 *  if <code>false</code> then it is OK to generate the thumbnail image. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function set delayThumbnailGeneration(value:Boolean):void;
		
		/**
		 *  Allow the creation of the thumbnail image for this card.
		 * 
		 *  @param value If <code>true</code> then allow the generation of the thumbnail image;
		 *  if <code>false</code> then do not generate the thumbnail image. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function set enableThumbnailGeneration(value:Boolean):void;
		
		/**
		 *  @private
		 *
		 */	
		function get enableThumbnailGeneration():Boolean;

		/**
		 *  The drop shadow effect preference.  This is a string specifying
		 *  the preferred location for a drop shadow effect on this component.
		 *  This is used as a hint to skins when specifying filters.  The
		 *  possible values are "none", "bottom", and "rear".
		 * 
		 *  @see CardFactory
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get dropShadowPreference():String;
		
		/**
		 *  A string specifying the preferred type of rollover effect on this component.
		 *  This is used as a hint to skins when specifying filters.
		 *  Possible values are "none", "glow", and "scale".
		 * 
		 * 	@see CardFactory
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get rollOverPreference():String;

		/**
		 *  Update the renderer to use for the card
		 * 
		 *  @param oldRenderer The renderer to use. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function updateRenderer(oldRenderer:ISimpleCard):void;
			
		/**
		 *  Returns the preview component for this card.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function get previewComponent():IPreview;
	
		/**
		 *  invalidates the current preview
		 * 
		 *  @param invalidateModel If <code>true</code> then nulify the preview model.
		 *  @param generateThumb If <code>true</code> generate a new thumbnail image.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function invalidatePreview(invalidateModel:Boolean = false, generateThumb:Boolean = true):void;
		
		/**
		 *  Enable double clicking events.
		 * 
		 *  @param value If <code>true</code> then double click events are enabled
		 *  for this card; if <code>false</code> then double click events are disabled.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function set doubleClickEnabled(value:Boolean):void;
		
		/**
		 *  Give this simpleCard the focus.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		function setFocus():void;
	}
}