/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.cards.ICardProvider;
	import com.adobe.portfolio.components.supportClasses.AutoListen;
	import com.adobe.portfolio.components.supportClasses.AutoWatch;
	import com.adobe.portfolio.components.supportClasses.IListenAndWatch;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import mx.binding.utils.BindingUtils;
	import mx.events.FlexEvent;
	
	import spark.components.HSlider;
	
	/**
	 * The CardSlider class extends HSlider and displays card thumbnails above the slider thumb.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class CardSlider extends HSlider implements IListenAndWatch
	{

		protected var autoWatch:AutoWatch;
		protected var autoListen:AutoListen;
		protected var mouseDown:Boolean;
	
		private var _cardProvider:ICardProvider;

		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function CardSlider()
		{
			super();
			minimum = 0;
			dataTipFormatFunction = dataTipFormat;
			autoWatch = new AutoWatch();
			autoListen = new AutoListen();
			addListenAndWatch();  // remove once addListenAndWatch is called externally
		}
		
		/**
		 * Adds listeners and watchers that are unwatched and unlistened by <code>removeListenAndWatch</code>.  
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function addListenAndWatch():void
		{
			autoListen.addListener(this, FlexEvent.CHANGE_END, onChangeEnd);
			autoListen.addListener(this, Event.CHANGE, onChange);
			autoListen.addListener(this, MouseEvent.MOUSE_DOWN, onMouseDown);
			autoListen.addListener(this,  MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Removes all listeners and watchers.  
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function removeListenAndWatch():void
		{
			autoWatch.removeAllWatchers();
			autoWatch = null;
			autoListen.removeAllListeners();
			autoListen = null;
		}
		
		protected function onChangeEnd(e:Event):void 
		{
			if (!cardProvider)
				return;
			
			cardProvider.pendingCardPosition = value;			
		}

		protected function onChange(e:Event):void 
		{
			if (mouseDown)
				return;
			
			if (!cardProvider)
				return;
			
			cardProvider.pendingCardPosition = value;			
		}
		
		protected function onMouseDown(e:MouseEvent):void 
		{
			mouseDown = true;
		}
		
		protected function onMouseUp(e:MouseEvent):void 
		{
			mouseDown = false;
		}

		protected function dataTipFormat(value:Number):Object {
			return "" + (value as int);
		}
		
		[Bindable]
		/**
		 * The ICardProvider that CardSlider uses for displaying the card thumbnail.  Typically set from MXML.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get cardProvider():ICardProvider
		{
			return _cardProvider;
		}

		/**
		 *  @private
		 */
		public function set cardProvider(value:ICardProvider):void
		{
			if (_cardProvider == value)
				return;
			
			autoWatch.removeAllWatchers();
			_cardProvider = value;
			
			if (!value)
				return;
			
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "value", cardProvider, "cardPosition"));
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "maximum", cardProvider, "maxCardPosition"));
		}
		
		override protected function thumb_mouseDownHandler(event:MouseEvent):void
		{
			// workaround for bug in SliderBase that doesn't account for missed mouseUp events
			// calling the mouseUpHandler ensures that any existing dataTip instance 
			// is removed before creating a new one - Bruce
			super.system_mouseUpHandler(event);  
			super.thumb_mouseDownHandler(event);
			
		}
		
		
		// for some reason mouseWheelEvent have a sign-reversed delta - this fixes that
		override protected function system_mouseWheelHandler(event:MouseEvent):void
		{
			if (!event.isDefaultPrevented())
			{
				var newValue:Number = nearestValidValue(value - event.delta * stepSize, stepSize);
				setValue(newValue);
				dispatchEvent(new Event(Event.CHANGE));
				event.preventDefault();
			}
		}

		
	}
}
