/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.cards.ICardProvider;
	import com.adobe.portfolio.cards.ISimpleCard;
	
	import mx.core.IDataRenderer;
	
	/**
	 * The CardTip class extends NavSkinnableComponent and is used in conjunction with CardSlider to display a card thumbnail.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class CardTip extends NavSkinnableComponent implements IDataRenderer
	{
		private var _data:Object;
		private var _cardProvider:ICardProvider;
		private var _cardName:String;
		
		/**
		 * A required skin part that is used to display a card thumbnail.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="true")]
		public var card:ISimpleCard;
		
		protected var pendingCard:ISimpleCard;

		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function CardTip()
		{
			super();
		}
		
		override protected function partAdded(partName:String, instance:Object):void 
		{
			super.partAdded(partName, instance);
			if (instance == card) {
				card.enableThumbnailGeneration = true;
				card.downSampleOK = true;
				updateCard(pendingCard);
			}
		}
		
		override protected function attachSkin():void
		{
			super.attachSkin();
		}
		
		[Bindable]
		/**
		 * The index of the card whose thumb should be displayed (formatted as a String).  
		 * The use of Object and String instead of just an int is dictated by the HSlider component.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get data():Object
		{
			return _data;
		}

		public function set data(value:Object):void
		{
			_data = value;
			
			if (!cardProvider)
				return;

			var s:String = value as String;
			if (!s)
				return;
			
			var layoutCard:ISimpleCard = cardProvider.getCard(parseInt(s));
			updateCard(layoutCard);
		}
		
		private function updateCard(layoutCard:ISimpleCard):void
		{
			if (!layoutCard)
				return;
			
			if (!card) {
				pendingCard = layoutCard;
				return;
			}
			
			pendingCard = null;
			card.data = layoutCard.data;
			// Set to default card bounds
			card.setLayoutBoundsSize(Number.NaN, Number.NaN);
			cardName = card.fileName;
		}

		[Bindable]
		/**
		 * A <code>CardProvider</code> used to obtain information about the thumbnail.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get cardProvider():ICardProvider
		{
			return _cardProvider;
		}

		public function set cardProvider(value:ICardProvider):void
		{
			_cardProvider = value;
		}

		[Bindable]
		/**
		 * The name of the card displayed by the <code>CardTip</code>.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get cardName():String
		{
			return _cardName;
		}

		public function set cardName(value:String):void
		{
			_cardName = value;
		}
	}
}