/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.components.supportClasses.IconButton;
	import com.adobe.portfolio.utils.LogHelper;
	
	/**
	 * The FileNavigationButton class extends IconButton and is used for next/previous file navigation.
	 */
	public class FileNavigationButton extends IconButton
	{
		/**
		 * A string literal representing the file navigation behavior of
		 * proceeding to the next file.
		 * 
		 * @see #navigationType
		 * 
		 */
		public static const FILE_NAVIGATION_NEXT:String = "next";

		/**
		 * A string literal representing the file navigation behavior of
		 * returning to the previous file.
		 * 
		 * @see #navigationType
		 * 
		 */
		public static const FILE_NAVIGATION_PREVIOUS:String = "previous";

		private var _navigationType:String = FILE_NAVIGATION_PREVIOUS;
		
		/**
		 * Default constructor.
		 */
		public function FileNavigationButton()
		{
			super();
			defaultIcon = LeftArrow;
		}
		
		[Bindable]
		/**
		 * A string indicating the navigational behavior that should occur when
		 * this button is clicked.
		 * 
		 * @see #FILE_NAVIGATION_NEXT
		 * @see #FILE_NAVIGATION_PREVIOUS
		 */
		public function get navigationType():String
		{
			return _navigationType;
		}
		
		/**
		 * @private
		 */
		public function set navigationType(value:String):void
		{
			if (value == _navigationType) {
				return;
			}
			_navigationType = value;
			if (_navigationType == FILE_NAVIGATION_PREVIOUS) {
				iconRotation -= 180;
			} else if (_navigationType == FILE_NAVIGATION_NEXT) {
				iconRotation += 180;
			} 
			else {
				CONFIG::logging{
					LogHelper.logger.warn("Unknown navigation type: "+_navigationType);
				}
			}
		}
	}
}
