/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.components.supportClasses.ColorPalette;
	import com.adobe.portfolio.components.supportClasses.IListenAndWatch;
	import com.adobe.portfolio.events.CollectionManagerEvent;
	import com.adobe.portfolio.managers.CollectionManager;
	
	import flash.system.Capabilities;
	
	import mx.collections.IList;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	
	import spark.components.supportClasses.SkinnableComponent;
	
	/**
	 * The Filecount class extends SkinnableComponent and displays the total number of attachments in the current folder, as well
	 * as the index of the attachment that is currently in focus.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class Filecount extends SkinnableComponent implements IListenAndWatch
	{
		/**
		 * Default constructor for Filecount.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function Filecount()
		{
			super();
			addListenAndWatch();
		}
		
		/**
		 * A boolean indicating whether to display only the number of files, or both the current
		 * index and the number of files.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public var showCurrentIndex:Boolean = false;
		
		/**
		 * The navigator's CollectionManager instance. All host interaction should go through this
		 * instance, rather than the INavigatorHost directly.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private static const  collectionManager:CollectionManager = CollectionManager.instance;
		
		/**
		 * Function to setup the event listeners required by this component.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function addListenAndWatch():void
		{
			// Don't forget to remove all these events if our part gets removed...
			collectionManager.addEventListener(CollectionManagerEvent.COLLECTION_CHANGE, onHostEvent);
			if (currentItems != null) {
				currentItems.addEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionEvent);
			}
		}

		/**
		 * Function to cleanup the event listeners required by this component. This should be
		 * called when the component is no longer needed, to prevent memory leaks.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function removeListenAndWatch():void
		{
			collectionManager.removeEventListener(CollectionManagerEvent.COLLECTION_CHANGE, onHostEvent);
			if (currentItems != null) {
				currentItems.removeEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionEvent);
				currentItems = null;
			}

		}
		
		/**
		 * Internal representation of the int referenced by the fileCount getter and setter.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private var _fileCount:int = -1;
		
		/**
		 * The total number of attachments at the in the collection's current folder.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get fileCount():int
		{
			return _fileCount;
		}
		protected function set fileCount(count:int):void
		{
			if (count != _fileCount) {
				_fileCount = count;
			}
		}
		
		private var _fontSize:int = 14;
		/**
		 * Setting the font size on the label
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get fontSize():int
		{
			return _fontSize;
		}
		protected function set fontSize(size:int):void
		{
			if (size != _fontSize) {
				_fontSize = size;
			}
		}
		
		private var _fileIndex:int = -1;
		/**
		 * The index of the current focus attachment in the collection's current folder.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get fileIndex():int
		{
			return _fileIndex;
		}
		protected function set fileIndex(index:int):void
		{
			if (index != _fileIndex) {
				_fileIndex = index;
			}
		}
	
		private var _localColorPalette:ColorPalette = null;
		/**
		 * The color palette being used by this object.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get colorPalette():ColorPalette
		{
			return (_localColorPalette != null) ? _localColorPalette : collectionManager.colorPalette;
		}
		protected function set colorPalette(value:ColorPalette):void
		{
			_localColorPalette = value;
		}
		
		private var _currentItems:IList;
		/**
		 * @return A pointer to the collection's current items.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get currentItems():IList
		{
			return _currentItems;
		}
		public function set currentItems(value:IList):void
		{
			if (value == _currentItems) {
				return;
			}
			if (_currentItems != null) {
				_currentItems.removeEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionEvent);
			}
			_currentItems = value;
			if (_currentItems != null) {
				_currentItems.addEventListener(CollectionEvent.COLLECTION_CHANGE, onCollectionEvent);
			}
			fileInfoDirty = true;
			invalidateProperties();
		}
		
		/**
		 * A boolean indicating that the cached information concerning the fileCount and fileIndex
		 * needs to be synchronized.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected var fileInfoDirty:Boolean = true;
		
		/**
		 * Synchronizes the fileCount and fileIndex information, if necessary.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if (fileInfoDirty) {
				fileCount = (currentItems == null) ? 0 : currentItems.length;
				if ((currentItems == null) || (currentItems.length <= 0)) {
					fileIndex = -1;
				} else {
					var newIndex:int = currentItems.getItemIndex(collectionManager.focusAttachment);
					if (newIndex >= 0) {
						fileIndex = newIndex;
					}
				}
				fileInfoDirty = false;
			}
		}
		
		/**
		 * A listener to respond to <code>CollectionMangagerEvent</code>s on the <code>CollectionMgr</code>.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onHostEvent(event:CollectionManagerEvent):void
		{
			switch (event.kind) {
				case CollectionManagerEvent.PART_REMOVED_KIND:
					if (event.oldVal == this) {
						removeListenAndWatch();
					}
					break;
				case CollectionManagerEvent.ITEMS_DID_CHANGE_KIND:
				case CollectionManagerEvent.FOCUS_DID_CHANGE_KIND:
					fileInfoDirty = true;
					invalidateProperties();
					break;
				default:
					// An event we don't care about
					break;
			}
		}

		/**
		 * A listener to respond to <code>CollectionEvent</code>s on the current items.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onCollectionEvent(event:CollectionEvent):void
		{
			if (event.kind == CollectionEventKind.UPDATE) {
				// don't bother doing anything on an update
			} else {
				fileInfoDirty = true;
				invalidateProperties();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function initializeAccessibility():void
		{
			var runningOnWindows:Boolean = Capabilities.version.indexOf('WIN') != -1;
			
			if ( runningOnWindows)
				accessibilityImplementation = new FileCountAccImpl(this);
		}
	}
}
