/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.events.CollectionManagerEvent;
	import com.adobe.portfolio.layouts.FreeFormLayout;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.managers.StringManager;
	
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.events.DragEvent;
	import mx.managers.DragManager;
	
	import spark.components.Button;
	
	/**
	 * The FolderBack class extends NavSkinnableComponent and shows the current folder and which can be clicked to go to the parent folder.
	 * It also supports drag and drop.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class FolderBack extends NavSkinnableComponent
	{
		/**
		 * String literals used to represent the skin parts exposed by FolderBack
		 *  to its skins.
		 * 
		 *  @langversion 3.0
	 	 *  @playerversion Flash 10.1
	 	 *  @playerversion AIR 2.0
	 	 *  @productversion Portfolio 10.0
	 	 */
		public static const PART_BACK_BUTTON:String = "backButton";
		[SkinPart (required="false")]
		public var backButton:Button;
		
		/**
		 * Format that defines the set of Acrobat drag and drop attachments. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const DRAG_DROP_ATTACHMENTS:String = "attachments";
		
		/** The navigator's CollectionManager instance. All host interaction should go through this
		 *  instance, rather than the INavigatorHost directly.
		 * 
		 *  @langversion 3.0
	 	 *  @playerversion Flash 10.1
	 	 *  @playerversion AIR 2.0
	 	 *  @productversion Portfolio 10.0
	 	 */
		protected const collectionManager:CollectionManager = CollectionManager.instance;

		private var _currentFolderName:String = "";
		private var _currentFolderIsRoot:Boolean = true;
		private var dragging:Boolean = false;
		private var dragTimer:Timer = new Timer(500, 1);
		
		/** 
		 * Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function FolderBack()
		{
			super();
			collectionManager.addEventListener(CollectionManagerEvent.COLLECTION_CHANGE, onHostEvent, false, 0, true);
		}
		
		[Bindable]
		/**
		 * @return The name of the current folder as a String
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get currentFolderName():String
		{
			return _currentFolderName;
		}
		protected function set currentFolderName(value:String):void
		{
			if (value == _currentFolderName) {
				return;
			}
			_currentFolderName = value;
		}
		
		[Bindable]
		/**
		 * @return True if the current folder is the root folder.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get currentFolderIsRoot():Boolean
		{
			return _currentFolderIsRoot;
		}
		protected function set currentFolderIsRoot(value:Boolean):void
		{
			if (value == _currentFolderIsRoot) {
				return;
			}
			_currentFolderIsRoot = value;
		}

		/**
		 * Listener that responds to events from the Collection Manager.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onHostEvent(event:CollectionManagerEvent):void
		{
			if ((collectionManager == null) || (event.kind != CollectionManagerEvent.ITEMS_DID_CHANGE_KIND)) {
				return;
			}
			currentFolderIsRoot = (collectionManager.currentFolder == null);
			currentFolderName = currentFolderIsRoot ? collectionManager.fileName : collectionManager.currentFolder.fileName;
		}
		/**
		 * Listener that responds to a click event on the back button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onBackButtonClick(event:MouseEvent):void
		{
			upOneFolder();
		}
		
		/**
		 * Listener that responds to a drag over the back button. 
		 * If hovered over the back button, the folder will be changed to its parent during the drag.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onDragOver(event:DragEvent):void
		{
			// Don't exit to root in FreeForm. Bug #2733597. 
			// This is a temporary fix. The design needs to be rethought for a permanent fix
			if (collectionManager.currentFolder && collectionManager.currentFolder.parent == null)
			{
				var navPreview:NavPreview = event.dragInitiator as NavPreview;
				if (navPreview && navPreview.rootDataGroup && navPreview.rootDataGroup.layout is FreeFormLayout)
					return;
			}
			
			if (event.isDefaultPrevented())
				return;
			
			if (!(collectionManager.authoringMode) || !(event.dragSource.hasFormat(DRAG_DROP_ATTACHMENTS)))
				return;
			
			if (event.type == DragEvent.DRAG_ENTER)
				DragManager.acceptDragDrop(this);
			
			dragging = true;
			
			dragTimer.addEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
			dragTimer.start();
		}
		
		/**
		 * Listener that responds to the timer complete event of <code>dragTimer</code>.
		 * At timer complete, the folder is changed to its parent.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onTimerComplete(event:TimerEvent):void
		{
			if (!dragging)
				return;
			
			dragTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
			dragging = false;
			upOneFolder();
		}
		
		/**
		 * Listener that responds to a drop event on the back button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */ 
		protected function onDragDrop(event:DragEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			if (!(collectionManager.authoringMode) || !(event.dragSource.hasFormat(DRAG_DROP_ATTACHMENTS)))
				return;
			
			dragging = false;
			
			var dragItems:Vector.<Object> = event.dragSource.dataForFormat(DRAG_DROP_ATTACHMENTS) as Vector.<Object>;
			for (var i:int = 0; i < dragItems.length; ++i)
			{
				var att:IAttachment = dragItems[i] as IAttachment;	
				if (collectionManager.currentFolder != null)
					att.parent = collectionManager.currentFolder.parent;
			}
			upOneFolder();
		}
		
		/**
		 * Listener that responds to a drag exit or a mouse out event on the back button
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onDragExit(event:DragEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			dragging = false;
			if (dragTimer.running)
			{
				dragTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
				dragTimer.reset();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function keyDownHandler(event:KeyboardEvent):void
		{
			// Trap and ignore left and right arrows so they don't propagate to nav and change the row selection.
			// Make an enter on the button equivalent to pushing it i.e. change the folder.
			if ( (event.keyCode == Keyboard.LEFT) || (event.keyCode == Keyboard.RIGHT) || (event.keyCode == Keyboard.ENTER))
			{
				if ( event.keyCode == Keyboard.ENTER)
					upOneFolder();
				
				event.preventDefault();
				return;
			}
		}
		
		/**
		 * Changes folder to its parent.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function upOneFolder():void
		{
			if (collectionManager.currentFolder != null) {
				collectionManager.collectionDropManager.targetPath = collectionManager.currentFolder.parent;
				collectionManager.currentFolder = collectionManager.currentFolder.parent;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, true);
		}
		/**
		 * @inheritDoc
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, false);
		}
		
		/** 
		 *  @private
		 *  Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 *  
		 *  @param partName The name of the part being added.
		 *  @param instance The instance of the object to which the part is added or removed.
		 *  @param adding Whether the part is being added or removed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if ((instance != null) && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case PART_BACK_BUTTON:
						types = [ MouseEvent.CLICK, DragEvent.DRAG_ENTER, DragEvent.DRAG_DROP, DragEvent.DRAG_EXIT, KeyboardEvent.KEY_DOWN  ];
						listeners = [ onBackButtonClick, onDragOver, onDragDrop, onDragExit, keyDownHandler ];
						
						addWatcher(BindingUtils.bindProperty(backButton, "label", StringManager.instance, "backFolder"));
						addWatcher(BindingUtils.bindProperty(backButton, "toolTip", StringManager.instance, "backFolder"));
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = (listeners == null) ? 0 : listeners.length;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
	}
}
