/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.components.supportClasses.CanvasList;
	import com.adobe.portfolio.managers.PropertyManager;
	import com.adobe.portfolio.properties.CommonProperties;
	import com.adobe.portfolio.properties.PropertyContext;
	import com.adobe.portfolio.properties.PropertyListValue;
	import com.adobe.portfolio.properties.PropertyValue;
	
	import mx.binding.utils.BindingUtils;
	
	/**
	 * The Header class extends CanvasList and specifies a header-specific xml resource file and designer skin.
	 */
	public class Header extends CanvasList
	{
		private var presetIsBlank:Boolean = true;

		/**
		 * Default constructor
		 */
		public function Header()
		{			
			super();
			
			// Initialize dataURL to the standard header location
			dataURL = "navigator/header.xml";
			swfClass = "HeaderBackground";
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function setCanvasTemplate(value:PropertyValue):void
		{
			super.setCanvasTemplate(value);
			
			if (value == null) {
				return;
			}
			var shouldBeBlank:Boolean = (value.name == CanvasList.BLANK_PRESET_NAME);
			if (presetIsBlank != shouldBeBlank) {
				presetIsBlank = shouldBeBlank;
				updateActualWidth();
				updateActualHeight();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function shouldDisplayCanvas():Boolean
		{
			return (collectionManager.authoringMode || !presetIsBlank);
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function getTemplateProperty():PropertyListValue
		{
			if (PropertyManager.instance.commonProperties == null) {
				return null;
			}
			return PropertyManager.instance.commonProperties.headerTemplateList;
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function addPropertyBindings():void
		{
			if (addedPropertyBindings) {
				return;
			}
			templateWatcher = BindingUtils.bindSetter(setCanvasTemplate, PropertyManager.instance, ["commonProperties", "headerTemplateList", "activeValue"]);
			backgroundColorWatcher = BindingUtils.bindSetter(setBackgroundColor, PropertyManager.instance, ["commonProperties", "headerBackgroundColor", "numValue"]);
			backgroundOpacityWatcher = BindingUtils.bindSetter(setBackgroundOpacity, PropertyManager.instance, ["commonProperties", "headerBackgroundOpacity", "numValue"]);
			backgroundGradientTypeWatcher = BindingUtils.bindSetter(setBackgroundGradientType, PropertyManager.instance, ["commonProperties", "headerBackgroundGradientType", "activeValue"]);
			backgroundGradientTypeWatcher = BindingUtils.bindSetter(setBackgroundGradientIntensity, PropertyManager.instance, ["commonProperties", "headerBackgroundGradientIntensity", "numValue"]);
			addTextWatcher = BindingUtils.bindSetter(addTextAction, PropertyManager.instance, ["commonProperties", "headerAddTextAction", "value"]);
			addImageWatcher = BindingUtils.bindSetter(addImageAction, PropertyManager.instance, ["commonProperties", "headerAddImageAction", "value"]);
			addedPropertyBindings = true;
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function setupContext():PropertyContext
		{
			addPropertyBindings();
			if (PropertyManager.instance.commonProperties != null) {
				return PropertyManager.instance.commonProperties.getCurrentContextByName(CommonProperties.CONTEXT_HEADER);
			}
			return null;
		}
	}
}