package com.adobe.portfolio.components
{
	import com.adobe.portfolio.managers.StringManager;
	
	import flash.display.DisplayObject;
	import flash.events.MouseEvent;
	
	import mx.core.IFlexDisplayObject;
	import mx.core.IVisualElementContainer;
	import mx.managers.PopUpManager;
	import mx.states.State;
	
	import spark.components.Button;
	import spark.components.Label;
	
	[SkinState("normal")]
	[SkinState("disabled")]
	[SkinState("popup")]

	/**
	 * The NavAlert class extends NavSkinnableComponent to display user alerts.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class NavAlert extends NavSkinnableComponent
	{
		public static const STATE_POPUP:String = "popup";
		
		[SkinPart(required="true")]
		/**
		 * The close button for the Alert.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var closeButton:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional Label for displaying a message.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var messageDisplay:Label;
		
		private var container:IVisualElementContainer;
		private var _message:String;
		private var messageUpdated:Boolean = false;

		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function NavAlert()
		{
			super();
			currentState = STATE_DISABLED;
			horizontalCenter = 0;
			verticalCenter = 0;
		}
		
		/**
		 * Creats an alert with the given message as a popup with the given parent.
		 */
		public static function showPopup(parent:DisplayObject, message:String, modal:Boolean = true):NavAlert
		{
			var popup:IFlexDisplayObject = PopUpManager.createPopUp(parent, NavAlert, modal);
			var navAlert:NavAlert = popup as NavAlert;
			if (navAlert != null) {
				navAlert.maxWidth = parent.width * 0.8;
				navAlert.maxHeight = parent.height * 0.8;
				navAlert.currentState = STATE_POPUP;
				navAlert.message = message;
			} else if (popup != null) {
				PopUpManager.removePopUp(popup);
			}
			return navAlert;
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function measure():void
		{
			super.measure();
			
			if (!messageUpdated) {
				return;
			}
			if (currentState == STATE_POPUP) {
				// Popups aren't handled with a normal Flex layout,
				// so explicitly center us with the default width/height
				width = measuredWidth;
				height = measuredHeight + 40;
				PopUpManager.centerPopUp(this);
			}
			messageUpdated = false;
		}
		
		/**
		 * Shows the alert.
		 * 
		 * @param container The parent container
		 * @param msg An optional String to be displayed in the alert
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function show(container:IVisualElementContainer, msg:String = null):void
		{
			this.container = container;
			if (msg)
				this.message = msg;
			if (container)
				container.addElement(this);
			currentState = STATE_NORMAL;
		}
		
		override protected function initStates():void 
		{
			states.push(new State({name: STATE_NORMAL}));
			states.push(new State({name: STATE_DISABLED}));
			states.push(new State({name: STATE_POPUP}));
		}

		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case closeButton:
					addListener(closeButton, MouseEvent.CLICK, onCloseButton);
					break;
				default:
					break;
			}
		}
		
		protected function onCloseButton(event:MouseEvent):void {
			if (currentState == STATE_POPUP) {
				PopUpManager.removePopUp(this);
			}
			currentState = STATE_DISABLED;
			if (container)
				container.removeElement(this);
		}

		[Bindable]
		/**
		 * The String to be displayed in the alert.  This can also be set using the msg parameter of the show method.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get message():String
		{
			return _message;
		}

		/**
		 * @private
		 */
		public function set message(value:String):void
		{
			_message = value;
			
			messageUpdated = true;
			invalidateSize()
		}
	}
}