/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.components.supportClasses.IListenAndWatch;
	
	import flash.events.MouseEvent;
	
	import mx.events.PropertyChangeEvent;
	
	import spark.components.SkinnableContainer;
	
	[SkinState("over")]
	/**
	 * A container that can be used to show/hide other UI components for manipulating
	 * a portfolio.
	 */
	public class NavHUD extends SkinnableContainer implements IListenAndWatch
	{
		/**
		 * Default constructor.
		 */
		public function NavHUD()
		{
			super();
			addListenAndWatch();
		}
		
		[Bindable]
		/**
		 * The desired value of the <code>left</code> property to be used by the skin
		 * when positioning the background and contentGroup.
		 */
		public var contentLeft:Number = 0;
		
		[Bindable]
		/**
		 * The desired value of the <code>right</code> property to be used by the skin
		 * when positioning the background and contentGroup.
		 */
		public var contentRight:Number = 0;
		
		[Bindable]
		/**
		 * The desired value of the <code>top</code> property to be used by the skin
		 * when positioning the background and contentGroup.
		 */
		public var contentTop:Number = 0;
		
		[Bindable]
		/**
		 * The desired value of the <code>bottom</code> property to be used by the skin
		 * when positioning the background and contentGroup.
		 */
		public var contentBottom:Number;
		
		private var _mouseIsOver:Boolean;
		/**
		 * True if the cursor is currently over this component; false otherwise.
		 */
		protected function get mouseIsOver():Boolean
		{
			return _mouseIsOver;
		}
		protected function set mouseIsOver(value:Boolean):void
		{
			if (value == _mouseIsOver) {
				return;
			}
			_mouseIsOver = value;
			invalidateSkinState();
		}
		
		/**
		 * @inheritDoc
		 */
		public function addListenAndWatch():void
		{
			addEventListener(MouseEvent.MOUSE_OVER, onMouseEvent, false, 0, true);
			addEventListener(MouseEvent.MOUSE_OUT, onMouseEvent, false, 0, true);
		}
		
		/**
		 * @inheritDoc
		 */
		public function removeListenAndWatch():void
		{
			removeEventListener(MouseEvent.MOUSE_OVER, onMouseEvent, false);
			removeEventListener(MouseEvent.MOUSE_OUT, onMouseEvent, false);
		}
		
		/**
		 * An event listener used to determine whether or not the cursor is currently
		 * over this component.
		 * 
		 * @see #mouseIsOver
		 */
		protected function onMouseEvent(event:MouseEvent):void
		{
			if (event == null) {
				return;
			}
			if (event.type == MouseEvent.MOUSE_OVER) {
				mouseIsOver = true;
			} else if (event.type == MouseEvent.MOUSE_OUT) {
				var hitCounter:uint = 0;
				if (hitTestPoint(event.stageX-1, event.stageY-1)) {
					++hitCounter;
				}
				if (hitTestPoint(event.stageX-1, event.stageY+1)) {
					++hitCounter;
				}
				if (hitTestPoint(event.stageX+1, event.stageY-1)) {
					++hitCounter;
				}
				if (hitTestPoint(event.stageX+1, event.stageY+1)) {
					++hitCounter;
				}
				if (hitCounter > 2) {
					// We've moved over a subcomponent, so stay in the over state
					return;
				}
				mouseIsOver = false;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function getCurrentSkinState():String
		{
			if (!enabled) {
				return "disabled";
			}
			return mouseIsOver ? "over" : "normal";
		}
		
		/**
		 * @inheritDoc
		 */
		override public function set enabled(value:Boolean):void
		{
			super.enabled = value;
			if (!value) {
				// Reset the mouseIsOver state if we're being disabled
				mouseIsOver = false;
			}
		}
	}
}