/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import flash.display.Graphics;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import spark.skins.spark.ListDropIndicator;
	
	/**
	 * The NavListDropIndicator class extends ListDropIndicator to draw a custom drop indicator.
	 * The drop indicator used by all navigators is an I-beam.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class NavListDropIndicator extends ListDropIndicator
	{
		private static const iBeamHeadWidth:Number = 2;
		private static const iBeamHeadHeight:Number = 12;
		private static const iBeamOffsetWidth:Number = 5;
		private static const iBeamOffsetHeight:Number = 2;
	
		/**
		 * The function which describes the actual position (bounding rect) of the ibeam on screen. 
		 * It is invoked by updateDisplayList. Set this in your layout class by overriding the setter for dropIndicator.
		 * 
		 * @See com.adobe.portfolio.layouts.BasicTileLayout for an example.
	     * 
	  	 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var getGraphicPosition:Function = getGraphicPositionDefault;
		
		/** 
		 * The maximum height of the drop indicator. Used by layouts that need a vertical i-beam.
		 * 
	  	 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private var maxHeight:Number = 150;
		public function setMaxHeight(height:Number):void
		{
			maxHeight = height;
		}
		
		/** 
		 * The maximum width of the drop indicator. Used by layouts that need a horizontal i-beam.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private var maxWidth:Number = 150;
		public function setMaxWidth(width:Number):void
		{
			maxWidth = width;
		}
			
		/** The default function that is used to determine the position of the dropIndicator.
		 * To use your own function, set <code>getGraphicPosition</code> to point to your local function.
		 * 
	  	 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function getGraphicPositionDefault(w:Number, h:Number, x:Number, y:Number, end:Boolean):Rectangle
		{
			// If the gap is 0 or negative, the layout would size us as 0 width/height,
			// we need some minimum to ensure drawing.
			var width:Number = Math.max(iBeamHeadWidth, w);
			var height:Number = Math.max(iBeamHeadWidth, h);
			
			// Make the shorter side 2 pixels so that the drop indicator is always 
			// a 2 pixel thick line regardless of vertical/horizontal orientation.
			if (width < height)
				width = iBeamHeadWidth;
			else
				height = iBeamHeadWidth;
			
			// Center the drawing within the bounds
			var draw:Rectangle = new Rectangle;
			draw.x = Math.round((w - width) / 2);
			if (end)
			{
				draw.y = Math.round((h - height) / 2);
			}
			else
			{
				var p:Point = new Point(x, y);
				draw.y = Math.round(this.globalToLocal(p).y);
			}
			
			draw.width = width;
			draw.height = height;
			
			return draw;
		}
		
		
		//--------------------------------------------------------------------------
		//
		//  Overridden methods
		//
		//--------------------------------------------------------------------------
		
		/**
		 * @inheritDoc
		 */
		override protected function updateDisplayList(w:Number, h:Number):void
		{	
			var end:Boolean = false;
			if (h == 0)
			{
				h = maxHeight;
				end = true;
			}
			if (w == 0)
			{
				w = maxWidth;
				end = true;
			}
			super.updateDisplayList(w, h);
			
			var draw:Rectangle = getGraphicPosition(w, h, x, y, end);
			var x:Number = draw.x;
			var y:Number = draw.y;
			var width:Number = draw.width;
			var height:Number = draw.height;
			
			// This is specific to horizontal layouts. 
			var g:Graphics = graphics;
			g.clear();
			g.beginFill(0xFFFFFF);
			if (width < height)
			{
				g.drawRect(x-iBeamOffsetWidth, y-iBeamOffsetHeight, iBeamHeadHeight, iBeamHeadWidth);
				g.drawRoundRect(x, y, width, height, iBeamHeadWidth, iBeamHeadWidth);
				g.drawRect(x-iBeamOffsetWidth, y + height, iBeamHeadHeight, iBeamHeadWidth);
			}
			else
			{
				// Do some different drawing for vertical layouts.
				g.drawRect(x-iBeamOffsetHeight, y-iBeamOffsetWidth, iBeamHeadWidth, iBeamHeadHeight);
				g.drawRoundRect(x, y, width, height, iBeamHeadWidth, iBeamHeadWidth);
				g.drawRect(x + width, y-iBeamOffsetWidth, iBeamHeadWidth, iBeamHeadHeight);
			}
			g.endFill();
		}
	}
}
