/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import acrobat.proxies.Page;
	
	import com.adobe.portfolio.api.FieldValueChangedEvent;
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.cards.BasicCard;
	import com.adobe.portfolio.cards.FolderCard;
	import com.adobe.portfolio.cards.IPreviewCard;
	import com.adobe.portfolio.cards.ISimpleCard;
	import com.adobe.portfolio.cards.PreviewCard;
	import com.adobe.portfolio.cards.SimpleCard;
	import com.adobe.portfolio.components.supportClasses.INavPreviewSkin;
	import com.adobe.portfolio.events.CollectionManagerEvent;
	import com.adobe.portfolio.events.ThumbnailManagerEvent;
	import com.adobe.portfolio.layouts.FreeFormLayout;
	import com.adobe.portfolio.lists.CardContainer;
	import com.adobe.portfolio.lists.CardList;
	import com.adobe.portfolio.managers.CommandManager;
	import com.adobe.portfolio.managers.StringManager;
	import com.adobe.portfolio.managers.ThumbnailManager;
	import com.adobe.portfolio.previews.IPreview;
	import com.adobe.portfolio.previews.PreviewHUD;
	import com.adobe.portfolio.utils.FileUtil;
	
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.collections.ArrayCollection;
	import mx.collections.IList;
	import mx.core.IVisualElement;
	import mx.effects.Effect;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.events.DragEvent;
	import mx.events.EffectEvent;
	import mx.events.PropertyChangeEvent;
	import mx.events.ResizeEvent;
	import mx.logging.ILogger;
	import mx.logging.Log;
	import mx.managers.DragManager;
	import mx.states.State;
	
	import spark.components.Button;
	import spark.components.DataGroup;
	import spark.components.Group;
	import spark.components.Label;
	
	[SkinState("disabled")]
	[SkinState("cardPreview")]
	[SkinState("folderPreview")]
	[SkinState("folderCardPreview")]
	
	

	/**
	 * The NavPreview class extends CardContainer to preview the 
	 * current focus card, scroll to next and previous cards, and to view the contents of folders. 
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 * */
	public class NavPreview extends CardContainer
	{
		/**
		 * The "disabled" state for the NavPreview and its skin. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_DISABLED:String = "disabled";

		/**
		 * The "cardPreview" state for the NavPreview and its skin.  
		 * This state is active when a single card at the root folder level is being previewed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_CARD_PREVIEW:String = "cardPreview";

		/**
		 * The "folderPreview" state for the NavPreview and its skin.  
		 * This state is active when a non-root folder is being displayed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_FOLDER_PREVIEW:String = "folderPreview";

		/**
		 * The "folderCardPreview" state for the NavPreview and its skin.  
		 * This state is active when a card from a non-root folder is being previewed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_FOLDER_CARD_PREVIEW:String = "folderCardPreview";
		
		/*
		 * Format that defines the set of Acrobat drag and drop attachments. 
		 */
		protected static const DRAG_DROP_ATTACHMENTS:String = "attachments";
		
		private var dragOnX:int = 0;
		private var dragTimer:Timer = new Timer(300, 1);
		
		CONFIG::logging {
			protected static const logger:ILogger = Log.getLogger("NavPreview");
		}
		
		// internal meta states
		private static const PREVIEW:String = "preview";
		private static const NOT_PREVIEW:String = "notPreview";
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that dims back underlying components while the preview card is displayed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var cardShield:Group;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that dims back underlying components while the folder preview is displayed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var folderShield:Group;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display a full-sized preview.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var previewCard:IPreviewCard;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the card preview transition (next, prev).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var tempPreviewCard:IPreviewCard;

		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display a full-sized preview when not at the root folder level.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var previewFolderCard:FolderCard;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the card preview transition (next, prev) 
		 * when not at the root folder level.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var tempPreviewFolderCard:FolderCard;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the contents of a folder. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var previewFolder:Group;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the close button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var closeButton:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the next button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var nextButton:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the prev button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var prevButton:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the open file button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var openFileButton:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is used to display the current file (attachement) name.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var fileNameLabel:Label;
		
		[SkinPart(required="false")] 
		/**
		 * An optional skin part that is used to display a floating HUD w/ navigation controls.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var previewHUD:PreviewHUD;
		
		private var focusEffectCanceled:Boolean = false;
		
		// If a focusEffect gets canceled before it's done, use this to track what the focusAttachment is supposed to be 
		private var tempFocusAttachment:IAttachment;
		
		protected var focusChangeEffect:Effect;
		protected var previewCardInvalid:Boolean;
		protected var thumbnailManager:ThumbnailManager;
		protected var showInfoString:String;
		protected var deleteFolderString:String;
		protected var extractFolderString:String;
		
		private var _rootDataGroup:DataGroup;
		private var _focusAttachment:IAttachment; // necessary because of the delay in CollectionManager.focusAttachment
		private var _enableDataProvider:Boolean;
		private var __dataProvider:IList;
		private var _previewFolderSize:Point;
		private var _previewCardSize:Point;
		
		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function NavPreview()
		{
			super();
	
			// Setup the states for this instance. They should match the skin states.
			states.push(new State({name: STATE_DISABLED}));
			states.push(new State({name: STATE_CARD_PREVIEW}));
			states.push(new State({name: STATE_FOLDER_PREVIEW}));
			states.push(new State({name: STATE_FOLDER_CARD_PREVIEW}));

			currentState = STATE_DISABLED;
			
			_enableDataProvider = false;
			previewCardInvalid = false;
			thumbnailManager = ThumbnailManager.instance;
		}
		
		public function set rootDataGroup(value:DataGroup):void
		{
			if (!value)
				return;
			
			_rootDataGroup = value;
		}
		
		/**
		 * The dataGroup associated with the root collection.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get rootDataGroup():DataGroup
		{
			return _rootDataGroup;
		}
		
		/**
		 * Adds listeners and watchers at creation time.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function addListenAndWatch():void
		{
			super.addListenAndWatch();
			addListener(this, DragEvent.DRAG_OVER, onButtonDragExit);
		}
		
		/**
		 * Overrides set dataProvider to capture a private copy.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function set dataProvider(value:IList):void
		{
			__dataProvider = value;
			super.dataProvider = dataProvider;
		}
		 
		[Bindable("dataProviderChanged")]
		/**
		 * Overrides get dataProvider to provide an empty collection if the enableDataProvider property is false.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function get dataProvider():IList
		{
			return enableDataProvider ? __dataProvider : new ArrayCollection();
		}

		/**
		 * Enables the dataProvider property.
		 * 
		 *  @private
		 */
		protected function get enableDataProvider():Boolean
		{
			return _enableDataProvider;
		}

		protected function set enableDataProvider(value:Boolean):void
		{
			_enableDataProvider = value;
			super.dataProvider = dataProvider;
		}

		override protected function onResize(event:ResizeEvent):void 
		{
			super.onResize(event);
			invalidatePreviewCard();
		}
		
		override protected function resizeCachedThumbs():void 
		{
			// do nothing by design- let the parent container deal with thumbs
		}
			
		override protected function keyDownHandler(event:KeyboardEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			switch (event.keyCode) {
				case Keyboard.ESCAPE:
					transitionCurrentState(NOT_PREVIEW);
					event.stopImmediatePropagation();
					return;
				case Keyboard.ENTER:
					transitionCurrentState(currentStateIsCardPreview() ? NOT_PREVIEW : PREVIEW);
					event.stopImmediatePropagation();
					return;
			}
			
			if (isInPreviewState)
				return
			
			super.keyDownHandler( event);
		}		

		protected function onPreviewCardDataChanged(val:Object):void 
		{
			invalidatePreviewCard();
		}
		
		protected function invalidatePreviewCard():void 
		{
			invalidateProperties();
			previewCardInvalid = true;
			if (previewCard)
				previewCard.invalidatePreview(true, true);
		}
		
		override protected function commitProperties():void 
		{
			super.commitProperties();
			
			if (width == 0 || height == 0)
				return;
			
			if (previewCardInvalid) {
//				setPreviewFrom();
				setPreviewTo();
				previewCardInvalid = false;
			}
		}
		
		protected function setPreviewTo(force:Boolean = false):void 
		{
			if (!force && isSkinEffectPlaying) {
				return;
			}
			
			if (previewFolder) { // setElementBounds doesn't work properly for previewFolder
				previewFolder.width = previewFolderSize.x;
				previewFolder.height = previewFolderSize.y;
				previewFolder.x = (parent.width - previewFolder.width) / 2;
				previewFolder.y = (parent.height - previewFolder.height) / 2;
			}
			
			var cardSize:Point = new Point(previewCardSize.x, previewCardSize.y);
			if (focusAttachment && thumbnailManager.isSmallPreview(focusAttachment.mimeType)) {
				cardSize.x = Math.min(ThumbnailManager.SMALL_PREVIEW_THUMBSIZE, cardSize.x);
				cardSize.y = Math.min(ThumbnailManager.SMALL_PREVIEW_THUMBSIZE, cardSize.y);
			}
			if (previewCard) {
				setElementBounds(previewCard, cardSize);
			}
			if (tempPreviewCard) {
				setElementBounds(tempPreviewCard, cardSize);
			}
			if (previewFolderCard) {
				setElementBounds(previewFolderCard, previewFolderSize);
			}
			if (tempPreviewFolderCard) {
				setElementBounds(tempPreviewFolderCard, previewFolderSize);
			}
		}
		
		protected function setElementBounds(element:IVisualElement, elementSize:Point):void {
			element.setLayoutBoundsSize(elementSize.x, elementSize.y);
			element.x = (parent.width - element.getLayoutBoundsWidth()) / 2;
			element.y = (parent.height - element.getLayoutBoundsHeight()) / 2;
		}
		
		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case previewCard:
					previewCard.visible = false;
					addListener(previewCard, ResizeEvent.RESIZE, onPreviewCardResize);
					addListener(previewCard, "closeButton", onPreviewCardClose);
					addWatcher(BindingUtils.bindSetter(onPreviewCardDataChanged, previewCard, "data"));
					addListener(this, ResizeEvent.RESIZE, onPreviewCardResize);
					addListener(previewCard, EffectEvent.EFFECT_END, onPreviewCardEffectEnd);
					addListener(thumbnailManager, ThumbnailManagerEvent.ASPECT_RATIO_CHANGED, onAspectRatioChanged);
					break;
				case tempPreviewCard:
					tempPreviewCard.visible = false;
					break;
				case previewFolderCard:
					addListener(previewFolderCard, "closeButton", onPreviewCardClose);
					addWatcher(BindingUtils.bindSetter(onShowInfoString, StringManager.instance, "showInfo"));
					addWatcher(BindingUtils.bindSetter(onExtractFolderString, StringManager.instance, "extractFolder"));
					addWatcher(BindingUtils.bindSetter(onDeleteFolderString, StringManager.instance, "deleteFolder"));
					break;
				case closeButton:
					addListener(closeButton, MouseEvent.CLICK, onCloseButton);
					addListener(closeButton, DragEvent.DRAG_ENTER, onDragOver);
					addListener(closeButton, DragEvent.DRAG_OVER, onDragOver);
					break;
				case nextButton:
					addListener(nextButton, MouseEvent.CLICK, collectionMgr.onGotoNextAttachment);
					break;
				case prevButton:
					addListener(prevButton, MouseEvent.CLICK, collectionMgr.onGotoPreviousAttachment);
					break;
				case openFileButton:
					addListener(openFileButton, MouseEvent.CLICK, onOpenFile);
					break;
				default:
					break;
			}
		}
		
		protected function onShowInfoString(value:String):void {
			showInfoString = value;
		}
		
		protected function onExtractFolderString(value:String):void {
			extractFolderString = value;
		}
		protected function onDeleteFolderString(value:String):void {
			deleteFolderString = value;
		}

		protected function onAspectRatioChanged(event:PropertyChangeEvent):void
		{
			if ((event != null) &&
				(previewCard != null) &&
				(event.source == previewCard.data))
			{
				invalidatePreviewCard();
			}
		}
		
		protected function onOpenFile(event:MouseEvent):void 
		{
			CommandManager.instance.editAttachment(focusAttachment);
		}
		
		private static const toPreview:Object = {
			disabled: {notFolder:"cardPreview", folder:"folderPreview"},
			folderPreview: {notFolder:"folderCardPreview", folder:"folderPreview"},
			cardPreview: {notFolder:"cardPreview", folder:"folderPreview"},
			folderCardPreview: {notFolder:"folderCardPreview", folder:"folderCardPreview"}
		}

		private static const fromPreview:Object = {
			disabled: {rootFolder:"disabled", childFolder:"disabled"},
			folderPreview: {rootFolder:"disabled", childFolder:"folderPreview"},
			cardPreview: {rootFolder:"disabled", childFolder:"folderPreview"},
			folderCardPreview: {rootFolder:"folderPreview", childFolder:"folderPreview"}
		}
		
		protected function transitionCurrentState(metaState:String):void {
			
			var isFolder:Boolean = focusAttachment && focusAttachment.isFolder;
			var newFolder:IAttachment  = collectionMgr.currentFolder ? collectionMgr.currentFolder.parent : null;
			var state:String;
			
			switch (metaState) {
				case PREVIEW:
					state = toPreview[currentState][isFolder ? "folder" : "notFolder"];
					newFolder = isFolder ? focusAttachment : collectionMgr.currentFolder;
					setFocus();
					break;
				case NOT_PREVIEW:
					var parentFolder:IAttachment  = collectionMgr.currentFolder ? collectionMgr.currentFolder.parent : null;
					state = fromPreview[currentState][parentFolder ? "childFolder" : "rootFolder"];
					newFolder = (currentState == STATE_CARD_PREVIEW || currentState == STATE_FOLDER_CARD_PREVIEW) ?
						collectionMgr.currentFolder : parentFolder;
					if (newFolder == null && rootDataGroup) {
						rootDataGroup.setFocus();
					} else if (dataGroup) {
						dataGroup.setFocus();
					}
					break;
				default:
					return;
			}
			
			if (metaState == NOT_PREVIEW && previewCard.currentState == PreviewCard.STATE_INFO)
				previewCard.setCurrentState(PreviewCard.STATE_PREVIEW);
			
			currentState = state;
			collectionMgr.currentFolder = newFolder;
		}

		override protected function onNavHostEvent(event:CollectionManagerEvent):void
		{
			super.onNavHostEvent(event);
			switch (event.kind) {
				case CollectionManagerEvent.FOCUS_DID_CHANGE_KIND:
					focusChanged(event.newVal as IAttachment);
					break;
				case CollectionManagerEvent.ITEMS_DID_CHANGE_KIND:
					if ((currentState == STATE_FOLDER_PREVIEW) && (collectionMgr.currentFolder == null)) {
						transitionCurrentState(NOT_PREVIEW);
					} else if (collectionMgr.currentFolder != null) {
						focusAttachment = collectionMgr.currentFolder;
						previewFolderCard.visible = false;
						transitionCurrentState(PREVIEW);
					}
					enableDataProvider = collectionMgr.currentFolder != null;
					break;
				case CollectionManagerEvent.NAVIGATOR_PREVIEW_KIND:
					focusAttachment = event.newVal as IAttachment;
//					CONFIG::logging {
//						logger.info("onNavHostEvent: NAVIGATOR_PREVIEW_KIND: focusAttachment= " + focusAttachment.fileName);
//					}
					if (isInPreviewState) {
						CommandManager.instance.editAttachment(focusAttachment);
					} else {
						var preview:IPreview = getFocusPreview();
						var extension:String = FileUtil.getExtension(focusAttachment.fileName);
						extension = (extension == null) ? "" : extension.toLowerCase();
						var platformPreviewExtension:Boolean = ((extension == "htm") || (extension == "html") || (extension == "swf"));
						
						if (platformPreviewExtension)
						{
							CommandManager.instance.previewAttachment(focusAttachment);
							break;
						}
						transitionCurrentState(PREVIEW);
					}
					break;
			}
		}
		
		protected function get isInPreviewState():Boolean {
			return currentState == STATE_CARD_PREVIEW || currentState == STATE_FOLDER_CARD_PREVIEW;
		}
		
		protected function focusChanged(newFocusAttachment:IAttachment):void {
			if (focusAttachment == newFocusAttachment)
				return;

			if (focusAttachment)
				focusAttachment.removeEventListener("fieldValueChanged", onFieldValueChanged);

			if (isInPreviewState) {
				focusChangedInPreview(newFocusAttachment);
			} else {
//				CONFIG::logging {
//					logger.info("focusChanged: focusAttachment= " + (focusAttachment ? focusAttachment.fileName : "null"));
//				}
				focusAttachment = newFocusAttachment;
				if (!focusAttachment)
					return;
				
				focusAttachment.addEventListener("fieldValueChanged", onFieldValueChanged, false, 0, true);
			}
		}
		
		protected function onFieldValueChanged (e:FieldValueChangedEvent):void {
			if (e.fieldName == SimpleCard.ASPECT_RATIO_FIELD) {
				e.target.removeEventListener(e.type, arguments.callee);
				invalidatePreviewCard();
			}
		}

		protected function focusChangedInPreview(newFocusAttachment:IAttachment):void {
			
			if (!newFocusAttachment)
				return;
			
//			CONFIG::logging {
//				logger.debug("focusChangedInPreview: newFocusAttachment= " + newFocusAttachment);
//			}

			// Fix bug 2721899: Stop any audio/video playback when switching between navigator previews
			if (previewCard && previewCard.previewComponent)
				previewCard.previewComponent.stopPlayback();
			
			var currentFocusAttachment:IAttachment = focusAttachment;
			if (focusChangeEffect) {
				focusEffectCanceled = true;
				focusChangeEffect.end();
				focusEffectCanceled = false;
				
				// If we canceled the effect prematurely, tempFocusAttachment will have been set
				// instead of focusAttachment, so use it instead
				currentFocusAttachment = tempFocusAttachment;
			}
			if (currentFocusAttachment == newFocusAttachment)
				return;
			
			var newAttachmentIsFolder:Boolean = newFocusAttachment.isFolder;
			if (newAttachmentIsFolder) {
				tempPreviewFolderCard.data = newFocusAttachment;

				// As part of the mitigation for bug 2729623, we reset the state to large for the transition
				tempPreviewFolderCard.currentState = BasicCard.STATE_LARGE;
			}
			else 
				tempPreviewCard.data = newFocusAttachment;
			
			setPreviewTo(true); 
			
			if (!newAttachmentIsFolder) {
				tempPreviewCard.delayThumbnailGeneration = false;
				tempPreviewCard.invalidatePreview(true, true);
			}
			tempPreviewFolderCard.visible = newAttachmentIsFolder;
			tempPreviewCard.visible = !newAttachmentIsFolder;
			
			focusChangeEffect = (currentFocusAttachment == collectionMgr.prevAttachment) ? nextCardPreviewEffect : prevCardPreviewEffect;
			
			if (!focusChangeEffect)
				return;	// TODO Shouldn't we set focusAttachment to newFocusAttachment and re-hide the tempPreviewCards in this case?
			
			focusChangeEffect.addEventListener(EffectEvent.EFFECT_END,
				function (e:Event):void {
					
					focusChangeEffect.removeEventListener(e.type, arguments.callee);
					focusChangeEffect = null;
					
					tempPreviewFolderCard.visible = false;
					tempPreviewFolderCard.x = previewFolderCard.x;						

					// Don't set focusAttachment if we're ending this effect prematurely
					if (focusEffectCanceled) {
						tempFocusAttachment = newFocusAttachment;
					} else if (focusAttachment == newFocusAttachment) {
						// If the user has gone back-and-forth quickly, the focusAttachment might be back where we started
						invalidatePreviewCard();
					} else {
						focusAttachment = newFocusAttachment;
					}
					
					previewCard.visible = focusAttachment && !focusAttachment.isFolder;
					previewCard.delayThumbnailGeneration = false;
					previewCard.invalidatePreview(true, true);
					
					// Give the previewCard some time to get it's preview ready before hiding the tempPreviewCard
					//	this eliminates some flashing when flipping between attachments in NavPreview mode
					var t:Timer = new Timer(200);
					t.addEventListener(TimerEvent.TIMER,
						function (event:Event):void {
							t.stop();
							tempPreviewCard.visible = false;
							tempPreviewCard.x = previewCard.x;
						}
					);
					t.start();

					previewFolderCard.visible = focusAttachment && focusAttachment.isFolder;
					if (previewFolderCard.visible) {
						// this is necessary because StringManager properties aren't initialized at the time the previewFolderCard is instantiated
						// it only needs to happen once, but the card buttons aren't initialized when the previewFolderCard part is added,
						// so we're using this as the trigger point
						previewFolderCard.infoButton.toolTip = showInfoString;
						previewFolderCard.infoButton.label = showInfoString;
						previewFolderCard.extractOrDeleteButton.toolTip = collectionMgr.authoringMode ? deleteFolderString : extractFolderString;
						previewFolderCard.extractOrDeleteButton.label = collectionMgr.authoringMode ? deleteFolderString : extractFolderString;
			}
				}
			);
			focusChangeEffect.play();
			
			// As part of the mitigation for bug 2729623, we reset the state to large for the transition
			if (previewFolderCard.currentState != BasicCard.STATE_LARGE) {
				previewFolderCard.currentState = BasicCard.STATE_LARGE;
			}
		}
		
		protected function get nextCardPreviewEffect():Effect
		{
			var navPreviewSkin:INavPreviewSkin = skin as INavPreviewSkin;
			return navPreviewSkin ? navPreviewSkin.nextCardPreviewEffect : null;
		}

		protected function get prevCardPreviewEffect():Effect
		{
			var navPreviewSkin:INavPreviewSkin = skin as INavPreviewSkin;
			return navPreviewSkin ? navPreviewSkin.prevCardPreviewEffect : null;
		}

		protected function get isSkinEffectPlaying():Boolean
		{
			var navPreviewSkin:INavPreviewSkin = skin as INavPreviewSkin;
			return navPreviewSkin ? navPreviewSkin.isEffectPlaying : false;
		}
		
		protected function onCloseButton(event:MouseEvent):void 
		{
			if (event.isDefaultPrevented())
				return;
			
			collectionMgr.currentFolder = null;
			transitionCurrentState(NOT_PREVIEW);
			event.preventDefault();
		}
		
		protected function onPreviewCardClose(event:Event):void {
			transitionCurrentState(NOT_PREVIEW);
		}
		
		
		protected function onPreviewCardResize(event:ResizeEvent):void 
		{
			if (isSkinEffectPlaying) {
				previewCard.delayThumbnailGeneration = true;
				return;
			}
			
			previewCard.delayThumbnailGeneration = false;
			if (currentState != STATE_CARD_PREVIEW && currentState != STATE_DISABLED)
				return;
			
			invalidatePreviewCard();
		}
		
		/** Returns the current state of this instance, since its skins should have a matching
		 *  set of states.
		 */
		override protected function getCurrentSkinState():String
		{
			return currentState;
		}
		
		/**
		 * Sets the state of the skin as well as the state of this instance.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void
		{
			if (stateName == STATE_INTRO || stateName == STATE_NORMAL)
				return;
			
			if (stateName == currentState) {
				return;
			}
			
//			logger.info("setting currentState: old= " + currentState + " new= " + stateName);
			super.setCurrentState(stateName, playTransition);
			
			// Set enabled based on the current state
			// NavPreview must be disabled when its state is disabled, otherwise
			// the client using NavPreview won't get called for drag/drop events.
			enabled = currentState != STATE_DISABLED;
			
			if (enabled) {
				stopPlayback();
			}
			
			var isCardPreview:Boolean = currentState == STATE_CARD_PREVIEW || currentState == STATE_FOLDER_CARD_PREVIEW;
			
			// throttle ThumbnailManager to only generate static and preview priority thumbs while card preview is active
			if (thumbnailManager)
				thumbnailManager.minThumbQueuePriority = isCardPreview ? ThumbnailManager.STATIC_THUMB_PRIORITY : ThumbnailManager.ANY_THUMB_PRIORITY;
			
			collectionMgr.isNavPreviewCardPreview = isCardPreview;
			
			invalidateSkinState();
			if (skin) {
				if (stateName != STATE_DISABLED) // validate for disabled state slows things down, isn't needed
					validateProperties();  // not sure why this is necessary, but otherwise the skin state transition doesn't fire reliably -
											// TODO: find cleaner solution
			}
		}
		
		protected function getFocusPreview():IPreview
		{
			var card:ISimpleCard =  CardList.cardFromAttachment(rootDataGroup, collectionMgr.focusAttachment);
			if (!card)
				return null;
			
			return card.previewComponent;
		}
		
		protected function stopPlayback():void 
		{
			var preview:IPreview = getFocusPreview();
			if (preview)
				preview.stopPlayback();
		}
		
		protected function currentStateIsCardPreview():Boolean
		{
			return currentState == STATE_CARD_PREVIEW || currentState == STATE_FOLDER_CARD_PREVIEW;
		}

		[Bindable(name="focusAttachmentChanged")]
		/**
		 * The attachment that currently has the focus.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get focusAttachment():IAttachment
		{
			return _focusAttachment;
		}
		
		public function set focusAttachment(value:IAttachment):void
		{
//			CONFIG::logging {
//				logger.info("set focusAttachment: " + (value ? value.fileName : "null"));
//			}
			if (_focusAttachment == value)
				return;
			
			_focusAttachment = value;
			collectionMgr.focusAttachment = value;
			dispatchEvent(new Event("focusAttachmentChanged"));
			invalidatePreviewCard();
		}
		
		// works around a bug in ListBase.dataProvider_collectionChangeHandler, which assumes dataProvider is non-null
		override protected function dataProvider_collectionChangeHandler(event:Event):void
		{
			var ce:CollectionEvent = event as CollectionEvent;
			if (ce.kind == CollectionEventKind.RESET && !dataProvider)
				return;
			super.dataProvider_collectionChangeHandler(event);
		}
		
		protected function onPreviewCardEffectEnd(event:EffectEvent):void 
		{
			if (!previewHUD)
				return;
			
			previewHUD.updateCurrentState();
		}

		[Bindable]
		/**
		 * The dimensions of the preview folder (used to display the content non-root folders) in pixels.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get previewFolderSize():Point
		{
			return _previewFolderSize;
		}

		public function set previewFolderSize(value:Point):void
		{
//			logger.debug("previewFolderSize= " + value);
			_previewFolderSize = value;
		}

		[Bindable]
		/**
		 * The dimensions of the preview card in pixels.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get previewCardSize():Point
		{
			return _previewCardSize;
		}

		public function set previewCardSize(value:Point):void
		{
//			logger.debug("previewCardSize= " + value);
			_previewCardSize = value;
		}
		
		protected function onDragOver(event:DragEvent):void
		{
			// Don't exit to root in FreeForm. Bug #2733597. 
			// This is a temporary fix. The design needs to be rethought for a permanent fix
			if (rootDataGroup && rootDataGroup.layout is FreeFormLayout)
				return;
			
			if (dragTimer.running)
			{
				dragOnX += 2;
				return;
			}
				
			if (event.isDefaultPrevented())
				return;
			
			if (!(collectionMgr.authoringMode) || !(event.dragSource.hasFormat(DRAG_DROP_ATTACHMENTS)))
				return;
			
			if (event.type == DragEvent.DRAG_ENTER)
				DragManager.acceptDragDrop(this);
			
			dragOnX += 2;
			
			dragTimer.addEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
			dragTimer.start();

		}
		
		protected function onTimerComplete(event:TimerEvent):void
		{
			if (dragOnX < 0)
				return;
			
			dragTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
			collectionMgr.currentFolder = null;
			collectionMgr.collectionDropManager.targetPath = null;
			transitionCurrentState(NOT_PREVIEW);
			event.preventDefault();
		}
		
		/** 
		 * Close button is in the child hierarchy of the NavPreview. It does not receive a dragExit until we exit from NavPreview.
		 * The only way we can know if we exited from the close button is when we didnt receive a dragOver event on the button, but received
		 * a dragOverEvent on NavPreview. Use the below logic to reset the timer.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */ 
		protected function onButtonDragExit(event:DragEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			if (dragTimer.running && dragOnX <= 0)
			{
				dragOnX = 0;
				dragTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, onTimerComplete);
				dragTimer.reset();
			}
			if (dragTimer.running)
				dragOnX--;

			super.dragOverHandler(event);
		}
	}
}
