package com.adobe.portfolio.components
{
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	import spark.components.Scroller;
	
	/**
	 * The NavScroller class extends Scroller to provide extra navigator specific functionality.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class NavScroller extends Scroller
	{
		public function NavScroller()
		{
			super();
		}
		
		/**
		 * Set to true to have the scroller ignore pageUp and pageDown keyboard events.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var ignorePageUpAndPageDown:Boolean = false;
		
		/**
		 * Overrides the default behavior of the scroller so that these events are not
		 * processed by the scroller.  The <code>PAGE_DOWN</code> and <code>PAGE_UP</code> events are ignored
		 * only if <code>ignorePageUpAndPageDown</code> is true, which defaults to false.
		 * 
		 * <table class="innertable">
		 *		<tr><th>Event</th></tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.UP</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.DOWN</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.DOWN</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.LEFT</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.RIGHT</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.HOME</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.END</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.PAGE_DOWN</code></td>
		 *		</tr>
		 *		<tr>
		 *			<td><code>flash.ui.Keyboard.PAGE_UP</code></td>
		 *		</tr>
		 *  </table>
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override protected function keyDownHandler(event:KeyboardEvent):void
		{
			switch (event.keyCode) { // don't grab these keyboard events!
				case Keyboard.UP:
				case Keyboard.DOWN:
				case Keyboard.LEFT:
				case Keyboard.RIGHT:
				case Keyboard.HOME:
				case Keyboard.END:
					return;

				case Keyboard.PAGE_DOWN:
				case Keyboard.PAGE_UP:
					if (ignorePageUpAndPageDown)
						return;
					
				default:
					break;
			}
			super.keyDownHandler(event);
		}
	}
}