/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	
	import com.adobe.portfolio.components.supportClasses.AutoListen;
	import com.adobe.portfolio.components.supportClasses.AutoWatch;
	import com.adobe.portfolio.components.supportClasses.IListenAndWatch;
	
	import flash.events.IEventDispatcher;
	
	import mx.binding.utils.ChangeWatcher;
	import mx.utils.UIDUtil;
	
	import spark.components.supportClasses.SkinnableComponent;
	
	/**
	 * The NavSkinnableComponent class extends SkinnableComponent to provide extra navigator specific functionality. It implements
	 * the IListenAndWatch interface and synchronizes the currentState with the skin state.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class NavSkinnableComponent extends SkinnableComponent implements IListenAndWatch
	{
		public static const STATE_NORMAL:String = "normal";
		public static const STATE_DISABLED:String = "disabled";

		private var _skinState:String;
		
		protected var autoListen:AutoListen;
		protected var autoWatch:AutoWatch;

		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function NavSkinnableComponent()
		{
			super();
			uid = UIDUtil.createUID();
			initStates();
			addListenAndWatch();
		}
		
		/**
		 * Creates autoListen and autoWatch for automatically tracking listeners and watchers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function addListenAndWatch():void {
			autoListen = new AutoListen();
			autoWatch = new AutoWatch();
		}
		
		
		protected function initStates():void {
			// override as necessary
		}
		
		/**
		 * Uses autoListen and autoWatch to remove all listeners and watchers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function removeListenAndWatch():void {
			if (autoListen)
				autoListen.removeAllListeners();
			autoListen = null;
			if (autoWatch)
				autoWatch.removeAllWatchers();
			autoWatch = null;
		}
		
		override protected function detachSkin():void {
			super.detachSkin();
			removeListenAndWatch();
		}

		override protected function partRemoved(partName:String, instance:Object):void {
			if (!autoListen)
				return;
			autoListen.removeListener(instance as IEventDispatcher);
		}

		protected function addListener(listener:IEventDispatcher, type:String, handler:Function,
									   useCapture:Boolean = false, priority:int = 0):void {
			if (!autoListen)
				return;
			autoListen.addListener(listener, type, handler, useCapture, priority);
		}
		
		protected function addWatcher(watcher:ChangeWatcher):void {
			if (!autoWatch)
				return;
			autoWatch.addWatcher(watcher);
		}
		
		/**
		 * The current skin state
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get skinState():String
		{
			return _skinState;
		}
		
		public function set skinState(value:String):void
		{
			if (_skinState == value)
				return;
			
			_skinState = value;
			invalidateSkinState();
		}
		
		override protected function getCurrentSkinState():String
		{
			return skinState;
		}
		
		/**
		 * Sets the current component state and skin state.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void {
			super.setCurrentState(stateName, playTransition);
			skinState = stateName;
		}
		
	}
}
