/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.components.supportClasses.AutoListen;
	import com.adobe.portfolio.components.supportClasses.AutoWatch;
	import com.adobe.portfolio.components.supportClasses.IListenAndWatch;
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.events.IEventDispatcher;
	
	import mx.binding.utils.ChangeWatcher;
	import mx.utils.UIDUtil;
	
	import spark.components.SkinnableContainer;
	
	/**
	 * The NavSkinnableContainer class extends SkinnableContainer to provide extra navigator specific functionality. It implements
	 * the IListenAndWatch interface and synchronizes the currentState with the skin state.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class NavSkinnableContainer extends SkinnableContainer implements IListenAndWatch
	{
		
		protected static const STATE_NORMAL:String = "normal";
		
		private var _skinState:String;
		
		private var autoListen:AutoListen;
		private var autoWatch:AutoWatch;
		
		public function NavSkinnableContainer()
		{
			super();
			uid = UIDUtil.createUID();
			initStates();
			addListenAndWatch();
		}
		
		/**
		 * Creates autoListen and autoWatch for automatically tracking listeners and watchers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function addListenAndWatch():void {
			autoListen = new AutoListen();
			autoWatch = new AutoWatch();
		}
		
		/**
		 * Uses autoListen and autoWatch to remove all listeners and watchers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function removeListenAndWatch():void {
			var nChildren:int = this.numChildren;
			autoListen.removeAllListeners();
			autoListen = null;
			autoWatch.removeAllWatchers();
			autoWatch = null;
		}
		
		protected function initStates():void {
			// override as necessary
		}
		
		override protected function detachSkin():void {
			super.detachSkin();
			autoListen.removeAllListeners();
			autoWatch.removeAllWatchers();
			CONFIG::logging{
				LogHelper.logger.info(""+this+" detachSkin stage");
			}
		}
		
		override protected function partRemoved(partName:String, instance:Object):void {
			super.partRemoved(partName, instance);
			autoListen.removeListener(instance as IEventDispatcher);
			CONFIG::logging{
				LogHelper.logger.info(""+instance+" partRemoved stage");
			}
		}
		
		protected function addListener(listener:IEventDispatcher, type:String, handler:Function):void {
			autoListen.addListener(listener, type, handler);
		}
		
		protected function addWatcher(watcher:ChangeWatcher):void {
			autoWatch.addWatcher(watcher);
		}
		
		/**
		 * The current skin state
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get skinState():String
		{
			return _skinState;
		}
		
		public function set skinState(value:String):void
		{
			if (_skinState == value)
				return;
			
			_skinState = value;
			invalidateSkinState();
		}
		
		override protected function getCurrentSkinState():String
		{
			return skinState;
		}
		
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void {
			super.setCurrentState(stateName, playTransition);
			skinState = stateName;
		}
		
	}
}