/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2010 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.nav.host.NavigatorHost;
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.api.ICollection;
	import com.adobe.portfolio.api.INavigator;
	import com.adobe.portfolio.api.INavigatorHost;
	import com.adobe.portfolio.api.RightClickEvent;
	import com.adobe.portfolio.components.supportClasses.ColorPalette;
	import com.adobe.portfolio.events.PropertyManagerEvent;
	import com.adobe.portfolio.managers.PropertyManager;
	import com.adobe.portfolio.managers.ResourceManager;
	import com.adobe.portfolio.managers.StringManager;
	import com.adobe.portfolio.properties.CommonProperties;
	import com.adobe.portfolio.properties.PropertyValue;
	import com.adobe.portfolio.utils.AttachmentCollection;
	
	import flash.display.Bitmap;
	import flash.display.DisplayObject;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.events.TransformGestureEvent;
	import flash.geom.Point;
	import flash.system.Capabilities;
	import flash.text.Font;
	import flash.ui.Keyboard;
	import flash.ui.Multitouch;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.binding.utils.ChangeWatcher;
	import mx.core.ClassFactory;
	import mx.core.EventPriority;
	import mx.core.UIComponent;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.events.FlexEvent;
	import mx.managers.IFocusManagerComponent;
	import mx.states.State;
	import mx.styles.CSSStyleDeclaration;
	import mx.styles.StyleManager;
	
	import spark.components.Application;
	import spark.components.TextArea;
	
	// This is needed to support Acrobat drag-drop. This replaces the 
	// Flex system manager with the Acrobat system manager
	[Frame(factoryClass="acrobat.managers.AcrobatSystemManager")]

	/** The Navigator class extends Application and is the base class for Acrobat X Navigators. 
	 * 
	 *  New navigators should extend this class rather than Application
	 *  to handle common initialization procedures.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class Navigator extends Application implements INavigator
	{
		import com.adobe.portfolio.managers.CollectionManager;
		import com.adobe.portfolio.managers.CommandManager;
		
		/**
		 * The default duration for effects when the focus attachment changes.
		 */
		public static const FOCUS_DURATION:Number = 1000;
	
		/**
		 * String literal representing the loading state.
		 */
		protected const STATE_LOADING:String = "loading";
		
		/**
		 * String literal representing the empty state.
		 */
		protected const STATE_EMPTY:String = "empty";
		
		/**
		 * String literal representing the normal state.
		 */
		protected const STATE_NORMAL:String = "normal";
		
		private const LANGUAGE_PREFIX_FOR_JAPAN:String = "ja";
		private const LANGUAGE_PREFIX_FOR_CHINESE:String = "zh";

		// Chinese GB18030 fonts we don't want to ever override
		private static const SIM_SUN_GB18030:String = "simsun-18030";
		private static const HKSCS_FONT_NAME:String = "mingliu_hkscs";
		private static const HKSCS_EXTB_FONT_NAME:String = "mingliu_hkscs-extb";

		private const FONT_NAME_SEPARATOR:String = ",";
		// Japanese fonts
		private static const MONA_FONT_NAME:String = "Mona";
		private static const MEIRYO_FONT_NAME:String = "Meiryo";
		private static const MS_GOTHIC_FONT_NAME:String = "\uFF2D\uFF33 \u30B4\u30B7\u30C3\u30AF";						// MS ????
		private static const MS_UI_GOTHIC_FONT_NAME:String = "MS UI Gothic";
		private static const ADOBE_KOZUKA_GOTHIC_FONT_NAME:String = "\u5C0F\u585A\u30B4\u30B7\u30C3\u30AF Pr6N M";			// ?????? Pr6N M
		private static const HIRAGINO_KAKU_GOTHIC_FONT_NAME:String = "\u30D2\u30E9\u30AE\u30CE\u89D2\u30B4 ProN W3";	// ?????? ProN W3
		
		// Chinese Traditional fonts
		private static const MICROSOFT_JHENG_HEI:String = "\u5FAE\u8EDF\u6B63\u9ED1\u9AD4";		// ?????
		private static const MICROSOFT_YA_HEI:String = "\u5FAE\u8F6F\u96C5\u9ED1"; 				// ????
		private static const SIM_SUN: String = "\u5B8B\u4F53";									// ??
		private static const HEITI_TC_LIGHT:String = "Heiti TC Light";							// Default Traditional Chinese fallback font on Mac OS 10.6		
		private static const LI_HEI_PRO_MEDIUM: String = "\u5137\u9ED1 Pro";					// ?? Pro - Default Traditional Chinese fallback font on Mac OS 10.5
		private static const ADOBE_MING_STD_LIGHT: String = "Adobe \u660e\u9ad4 Std L";			// AdobeMingStd-Light 
		
		// Chinese Simplified fonts
		private static const HEITI_SC_LIGHT: String = "Heiti SC Light";							// Default Simplified Chinese fallback font on Mac OS 10.6
		private static const ST_HEITI_LIGHT: String = "\u534E\u6587\u7EC6\u9ED1";				// ???? - Default Simplified Chinese fallback font on Mac OS 10.5
		private static const ADOBE_SONG_STD_LIGHT: String = "Adobe \u5b8b\u4f53 Std L";			// AdobeSongStd-Light
		
		
//		// Korean fonts
//		// Korean Adobe font is AdobeSongStd-Light, same as for simplified Chinese
//		
//		// Arabic fonts
//		private static const ADOBE_ARABIC: String = "Adobe Arabic";								// AdobeArabic-Regular
//				
//		// Thai fonts
//		private static const ADOBE_THAI: String = "Adobe Thai";									// AdobeThai-Regular
//		
//		// Hebrew fonts
//		private static const ADOBE_HEBREW: String = "Adobe Hebrew";								// AdobeHebrew-Regular

		/**
		 * The navigator's CollectionManager instance. All host interaction should go through this
		 *  instance, rather than the INavigatorHost directly.
		 */
		protected const collectionManager:CollectionManager = CollectionManager.instance;

		/**
		 * Returns true if this application is loaded under Mac OS X.
		 */
		protected const runningOnMacintosh:Boolean = Capabilities.version.indexOf('MAC') != -1;

		// Make sure we don't set the host until we're actually done creating everything.
		// Otherwise we're not yet ready for host events, and we'll miss them.
		
		/**
		 * True if this application has been initialized with a navigator host.
		 * 
		 * @see INavigatorHost
		 */
		protected var initializedHost:Boolean = false;
		
		/**
		 * A host object that has been set, but not yet initialized.
		 */
		protected var pendingHost:INavigatorHost = null;
		
		private var rootItemsWatcher:ChangeWatcher;
		private var _rootItemsDidInit:Boolean;
		
		/**
		 * A boolean indicating whether the <code>rootItems</code> of the <code>CollectionManager</code>
		 * have been initialized.
		 */
		protected function get rootItemsDidInit():Boolean
		{
			return _rootItemsDidInit;
		}
		/**
		 * @private
		 */
		protected function set rootItemsDidInit(value:Boolean):void
		{
			if (_rootItemsDidInit == value) {
				return;
			}
			_rootItemsDidInit = value;
			setupState();
		}

		private var _propertiesDidInit:Boolean;

		/**
		 * A boolean indicating whether the navigator properties have been initialized.
		 */
		protected function get propertiesDidInit():Boolean
		{
			return _propertiesDidInit;
		}
		/**
		 * @private
		 */
		protected function set propertiesDidInit(value:Boolean):void
		{
			if (_propertiesDidInit == value) {
				return;
			}
			_propertiesDidInit = value;
			setupState();
		}
		private var resourceLoadingCountWatcher:ChangeWatcher;
		private var _resourceLoadingDidInit:Boolean;

		/**
		 * A boolean indicating whether the navigator resources have been initialized.
		 */
		protected function get resourceLoadingDidInit():Boolean
		{
			return _resourceLoadingDidInit;
		}
		/**
		 * @private
		 */
		protected function set resourceLoadingDidInit(value:Boolean):void
		{
			if (_resourceLoadingDidInit == value) {
				return;
			}
			_resourceLoadingDidInit = value;
			setupState();
		}
		
		/**
		 * Sets the current state of the navigator to the desired state based on
		 * the conditions of initialization.
		 * 
		 * @see getDesiredState
		 */
		protected function setupState():void
		{
			var newState:String = getDesiredState();
			if (newState != currentState) {
				currentState = newState;
			}
		}
		
		/**
		 * Returns the current desired state of the navigator based on the
		 * conditions of initialization.
		 */
		protected function getDesiredState():String
		{
			if (currentState == STATE_LOADING) {
				// Stay in loading state if we're not ready to get out of it yet
				// TODO We probably want to add some sort of thumbnail generation check here as well
				if (!rootItemsDidInit || !propertiesDidInit || !resourceLoadingDidInit) {
					return currentState;
				}
			}
			if ((collectionManager.rootItems == null) || (collectionManager.rootItems.length <= 0)) {
				return STATE_EMPTY;
			} else {
				return STATE_NORMAL;
			}
		}
		
		/**
		 * Default constructor
		 */
		public function Navigator()
		{
			CONFIG::performance {
				CollectionManager.instance.logPerformance("Navigator constructor");
			}
			super();
			
			// Setup the states for this instance. They should match the skin states.
			states = [
				new State({name: STATE_LOADING}),
				new State({name: STATE_NORMAL}),
				new State({name: STATE_EMPTY})];
			
			addEventListener(FlexEvent.APPLICATION_COMPLETE, onApplicationComplete);
			addEventListener(flash.events.Event.CLEAR, onClearEvent);
		}
		
		protected function onApplicationComplete(event:FlexEvent):void 
		{
			event.target.removeEventListener(event.type, arguments.callee);
			
			if (!CollectionManager.instance.queryAdobeHost(this)) {
				showExperienceAlert();
			}
			initNavigatorHost();
		}
		
		private function deleteSelectionIfAllowed():Boolean
		{
			if (collectionManager.authoringMode &&
				(collectionManager.selectedItems.length > 0) &&
				collectionManager.permissionCheck("EmbeddedFile", "Delete"))
			{
				CommandManager.instance.deleteSelection();
				return true;
			}
			else
				return false;
		}

		/**
		 * Called when we get a clear event from flash, such as the Edit->Delete menu item in Acrobat.
		 */
		private function onClearEvent(event:Event):void
		{
			if (event.isDefaultPrevented()) {
				return;
			}
			var currentFocus:IFocusManagerComponent = (focusManager != null) ? focusManager.getFocus() : null;
			var textArea:TextArea = currentFocus as TextArea;
			if ((textArea != null) && textArea.enabled && textArea.selectable) {
				// Don't delete any attachments if the user is trying to delete text
				return;
			}
			deleteSelectionIfAllowed();
		}
		
		/**
		 * Displays an alert to the user when this navigator is loaded in an unsupported host application,
		 * informing that functionality will be limited.
		 */
		protected function showExperienceAlert():void
		{
			var alert:BestExperienceAlert = new BestExperienceAlert();
			BindingUtils.bindProperty(alert, "message", StringManager.instance, "bestExperience");
			
			alert.show(this);
		}
		
		
		/////////////////// getters support for automation //////////////////////////
		
		/**
		 * The number of pages in the focus attachment.
		 */
		public function get currAttNumPages():int
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return att.pageCount;
			}
			else
				return -1;
		}
		
		/**
		 * The current page of the focus attachment.
		 */
		public function get currAttCurrentPageNum():int
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return att.pageNumber;
			}
			else
				return 0;
		}
		
		/**
		 * The filename of the focus attachment.
		 */
		public function get currAttFilename():String
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return att.fileName;
			}
			else
				return "";
		}
		
		/**
		 * The index of the focus attachment.
		 */
		public function get currAttIndex():int
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return collectionManager.getCurrentIndex(att);
			}
			else
				return -1;
		}
		
		/**
		 * True when the focus attachment is a folder; false otherwise.
		 */
		public function get currAttIsFolder():Boolean
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return att.isFolder;
			}
			else
				return false;
		}
		
		/**
		 * The name of the focus attachment's parent folder.
		 */
		public function get currAttFolderName():String
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att && att.parent)
			{
				return att.parent.fileName;
			}
			else
				return "";
		}
		
		/**
		 * The mime type of the current attachment.
		 */
		public function get currAttMIMEType():String
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				return att.mimeType;
			}
			else
				return "";
		}
		
		/**
		 * The thumbnail of the current attachment.
		 */
		private var aThumb:Bitmap = null;
		private var requestState:int;   // 0 => ready for request ; 1 => waiting for delivery;  2 => thumbnail is available
		private var forAttachment:IAttachment = null;
		/* 
			The request for properties (of which the thumbnail is one) is synchronous, but the production of the thumbnail
			is asynchonous. The Viewer may need cycles to generate it. To get around this when the automation client first asks for
			the thumbnail (as part of a larger request for other properties perhaps), we make the requestCachedThumbnail
			call to initiate the creation process, but to the requestor of the properties we say 'not ready; ask again later'. The Viewer
			will eventually cycle back around and retry the request.
		*/
		private function heresTheThumb( id:Object, thumb:Bitmap):void
		{
			aThumb = thumb;
			requestState = 2;
		}
		
		private static var padding:Array = [ "", "0", "00", "000", "0000", "00000"];
		private function uintToSixHexChar( num:uint):String
		{
			var val:String = num.toString(16);
			
			return padding[6 - val.length] + val;
		}
		
		public function get currAttThumb():String
		{
			var att:IAttachment = collectionManager.focusAttachment;
			if ( att)
			{
				if ( (requestState == 0) ||
					 ( (requestState == 2) && ( att != forAttachment)) ) // client changed mind on which attachment they are interested in
				{
					att.requestCachedThumbnail( this, heresTheThumb);
					requestState = 1;
					forAttachment = att;
					throw new Error( "prop not ready", 4);
				}
				
				if ( requestState == 1)
				{	// still waiting
					throw new Error( "prop not ready", 4);
				}
				
				if ( requestState == 2)
				{
					// make return value
					var retVal:String = "";
					
					if ( aThumb)
					{
						retVal += aThumb.bitmapData.width.toString() + " " + aThumb.bitmapData.height.toString() + " ";
						
						for (var y:int=0; y<aThumb.bitmapData.height; y++)
							for (var x:int=0; x<aThumb.bitmapData.width; x++)
							{
								var pixelVal:uint = aThumb.bitmapData.getPixel( x, y);
								retVal += uintToSixHexChar( pixelVal);
							} 
					}
					
					requestState = 0;
					aThumb = null;
					
					return retVal;
				}
			}

			return "";
		}
		
		
		/**
		 * The number of non-folder attachments in the current folder.
		 */
		public function get portFolderNumFiles():int
		{	 	
			if ( collectionManager.currentItems)
			{
				var folderCount:int = 0;
				for (var i:int=0; i<collectionManager.currentItems.length; i++)
				{
					var att:IAttachment = collectionManager.currentItems.getItemAt(i) as IAttachment;
					if ( att && att.isFolder)
						folderCount++;
				}
							
				return collectionManager.currentItems.length - folderCount;
			}
			else
				return 0;
		}
		
		
		/**
		 * The number of folder attachments in the current folder.
		 */
		public function get portFolderNumSubfolders():int
		{	 
			if ( collectionManager.currentItems)
			{
				var folderCount:int = 0;
				for (var i:int=0; i<collectionManager.currentItems.length; i++)
				{
					var att:IAttachment = collectionManager.currentItems.getItemAt(i) as IAttachment;
					if ( att && att.isFolder)
						folderCount++;
				}
				
				return folderCount;
			}
			else
				return 0;
		}
		
		/**
		 * The localized name of this navigator.
		 */
		public function get portLayoutName():String
		{
			var hostCollection:ICollection = (collectionManager.host != null) ? collectionManager.host.collection : null;
			var navName:String = (hostCollection == null) ? "unknownNav" : hostCollection.getLocalizedString( "navName", "unknownNav");
			return navName;
		}
		
		/**
		 * A string listing the non-folder attachment filenames in the current folder.
		 */
		public function get portFolderFileListing():String
		{
			var retVal:String = "";
			if ( collectionManager.currentItems)
			{
				for (var i:int=0; i<collectionManager.currentItems.length; i++)
				{
					var att:IAttachment = collectionManager.currentItems.getItemAt(i) as IAttachment;
					if ( att && !att.isFolder)
					retVal = retVal + " '" + att.fileName + "'";
				}
			}
		
			return retVal;
		}
			
		/**
		 * A string listing the folder attachment names in the current folder.
		 */
		public function get portFolderSubfolderListing():String
		{
			var retVal:String = "";
			if ( collectionManager.currentItems)
			{
				for (var i:int=0; i<collectionManager.currentItems.length; i++)
				{
					var att:IAttachment = collectionManager.currentItems.getItemAt(i) as IAttachment;
					if ( att && att.isFolder)
						retVal = retVal + " '" + att.fileName + "'";
				}
			}
			
			return retVal;
		}
		
			// Color palette values
		public function get portBackgroundColor():int
		{
			var colorPalette:ColorPalette = collectionManager.colorPalette;
			if ( colorPalette)
				return colorPalette.background;
			else
				return 0;
		}
		
		public function get portBorderColor():int
		{
			var colorPalette:ColorPalette = collectionManager.colorPalette;
			if ( colorPalette)
				return colorPalette.cardBorder;
			else
				return 0;
		}
		
		public function get portPrimaryColor():int
		{
			var colorPalette:ColorPalette = collectionManager.colorPalette;
			if ( colorPalette)
				return colorPalette.cardBackground;
			else
				return 0;
		}
		
		public function get portAccentColor():int
		{
			var colorPalette:ColorPalette = collectionManager.colorPalette;
			if ( colorPalette)
				return colorPalette.secondaryText;
			else
				return 0;
		}
		
		public function get portTextColor():int
		{
			var colorPalette:ColorPalette = collectionManager.colorPalette;
			if ( colorPalette)
				return colorPalette.primaryText;
			else
				return 0;
		}
		
		public function get portFontName():String
		{
			var cp:CommonProperties = PropertyManager.instance.commonProperties;
			
			if ( cp.portfolioFontName)
				return cp.portfolioFontName.name;
			else
				return "";
		}
		
		public function get portCardSize():String
		{
			var cp:CommonProperties = PropertyManager.instance.commonProperties;
			
			if ( cp.cardSize)
				return cp.cardSize.name;
			else
				return "";
		}
		
		public function get portTheme():String
		{
			var cp:CommonProperties = PropertyManager.instance.commonProperties;
			
			if ( cp.skin)
				return cp.skin.label;
			else
				return "";
		}
	
		//////////////// end of getter support for automation /////////////////////////
		
		private var _useApplicationFontOnly:Boolean = false;

		/**
		 * True if the navigator should always use the provided application font; false if the
		 * navigator should set its font according to the navigator properties.
		 * 
		 * @see CollectionManager#applicationFont
		 * @see CommonProperties#portfolioFontName
		 */
		public function get useApplicationFontOnly():Boolean
		{
			return _useApplicationFontOnly;
		}
		/**
		 * @private
		 */
		public function set useApplicationFontOnly(value:Boolean):void
		{
			if (value == _useApplicationFontOnly)
				return;
			_useApplicationFontOnly = value;
			setChromeFont(null);
		}
		
		private var _propertiesFactory:ClassFactory;
		/**
		 * Returns a factory that can be used to create a CommonProperties instance for this navigator.
		 * Subclasses can override this function to allow for custom properties.
		 * 
		 * @see commonProperties
		 * @see CommonProperties
		 * @see PropertyManager#commonProperties
		 */
		public function get propertiesFactory():ClassFactory
		{
			if (_propertiesFactory == null) {
				_propertiesFactory = new ClassFactory(CommonProperties);
			}
			return _propertiesFactory;
		}
		
		/**
		 * An object for loading and accessing customizable properties for this navigator.
		 */
		protected var commonProperties:CommonProperties;
		
		/**
		 * A function to initialize commonProperties. Subclasses can override this method to
		 *  support custom properties.
		 */
		protected function initCommonProperties():void
		{
			var considerPropertiesInitialized:Boolean = true;
			if ((commonProperties == null) && (propertiesFactory != null)) {
				commonProperties = propertiesFactory.newInstance() as CommonProperties;
				if (commonProperties != null) {
					fontWatcher = BindingUtils.bindSetter(setChromeFont, commonProperties, "portfolioFontName");
					commonProperties.navigator = this;
					considerPropertiesInitialized = false;
				}
			} 
			if (considerPropertiesInitialized) {
				// Either we don't have properties, or they're already setup. Either way make sure
				// we treat them as initialized
				propertiesDidInit = true;
			}
		}
		
		private var fontWatcher:ChangeWatcher;
		private function setChromeFont(value:PropertyValue):void
		{
			var toolTipStyles:CSSStyleDeclaration = StyleManager.getStyleManager(null).getStyleDeclaration("mx.controls.ToolTip");
			var useAppFont:Boolean = (useApplicationFontOnly || ((value != null) && (value.name == "applicationFont")));
			if (useAppFont) {
				// Use the host's application font
				var appFont:Font = collectionManager.applicationFont;
				if ((appFont != null) && (appFont.fontName != null)) {
					var fontFamily:String;
					var overrideFonts:String = applicationOverrideFonts(appFont.fontName);
					if (overrideFonts != null)
						fontFamily = overrideFonts + FONT_NAME_SEPARATOR + appFont.fontName;
					else
						fontFamily = appFont.fontName;
					// Ideally the two lines below should just be:
					// StyleManager.getStyleManager(null).getStyleDeclaration("global").setStyle("fontFamily", fontFamily);
					// But it is too close to RC to do that, so fixing this bug (2726747) specifically for the tooltip component.
					if (getStyle("fontFamily") != fontFamily) {
						setStyle("fontFamily", fontFamily);
					}
					if (toolTipStyles.getStyle("fontFamily") != fontFamily) {
						toolTipStyles.setStyle("fontFamily", fontFamily);
					}
				}
			} else if (value == null) {
				// We used to load the applicationFont if value is null, but this can be a big performance hit if we're just going
				// to load a different font when the real property is loaded. Instead, navs that want the applicationFont behavior
				// should specify that in the relevant property or use the useApplicationFontOnly flag.
			} else {
				// Ideally the two lines below should just be:
				// StyleManager.getStyleManager(null).getStyleDeclaration("global").setStyle("fontFamily", value.name);
				// But it is too close to RC to do that, so fixing this bug (2726747) specifically for the tooltip component.
				if (getStyle("fontFamily") != value.name) {
					setStyle("fontFamily", value.name);
				}
				if (toolTipStyles.getStyle("fontFamily") != value.name) {
					toolTipStyles.setStyle("fontFamily", value.name);
				}
			}
		}
		
		private function applicationOverrideFonts(fontFamily:String):String
		{
			// We override the application font with these
			// "override" fonts, if the locale of Acrobat is Japan
			// and the current application font is not either of
			// the GB-18030 compatible or HKSCS compatible fonts.
			
			// Only for Japanese or Chinese version of Acrobat
			if ((collectionManager.localeLanguagePrefix != LANGUAGE_PREFIX_FOR_JAPAN) &&
				(collectionManager.localeLanguagePrefix != LANGUAGE_PREFIX_FOR_CHINESE))
				return null;
			
			// Shouldn't be called with null fontFamily, but check
			if (fontFamily == null)
				return null;
			// Don't override the application font if user has chosen
			// to use the GB18030 or HKSCS compatibility fonts
			var fontFamilyLC:String = fontFamily.toLowerCase();
			if ((fontFamilyLC == SIM_SUN_GB18030) ||
				(fontFamilyLC == HKSCS_FONT_NAME) ||
				(fontFamilyLC == HKSCS_EXTB_FONT_NAME))
				return null;

			if ((collectionManager.host == null) ||
				(collectionManager.host.locale == null))
				return null;
			var hostLocale:String = collectionManager.host.locale.toLowerCase();
			
			var overrideFonts:String = "";
			var os:String = Capabilities.os;
			
			//Would use a switch, but some locales are reported differently and have varying lengths
			if (collectionManager.localeLanguagePrefix == LANGUAGE_PREFIX_FOR_JAPAN)
			{
				// This is only correct for WinXP, WinVista and Win7,
				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
				if (os.indexOf("Windows") >= 0) {
					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
					overrideFonts = MS_UI_GOTHIC_FONT_NAME + FONT_NAME_SEPARATOR + MS_GOTHIC_FONT_NAME;
					if ((os.indexOf("Windows 7") >= 0) ||
						(os.indexOf("Windows Vista") >= 0) ||
						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
						overrideFonts = MEIRYO_FONT_NAME + FONT_NAME_SEPARATOR + overrideFonts;
					}
					// Here is the fallback order for pre-WinXP version of Windows, if anyone needs this info:
					// Win 2K, Win NT, Win 95 - MS Gothic, MS UI Gothic
					// Win 98, Win ME - MS UI Gothic, MS Gothic
				}
				else if (os.indexOf("Mac") >= 0) {
					overrideFonts = HIRAGINO_KAKU_GOTHIC_FONT_NAME;
				}
				else if (os.indexOf("Linux") >= 0) {
					overrideFonts = MONA_FONT_NAME;
				}
				
				// Always include Adobe-supplied font as the last fallback
				if (overrideFonts.length > 0)
					overrideFonts += FONT_NAME_SEPARATOR;
				overrideFonts += ADOBE_KOZUKA_GOTHIC_FONT_NAME;
			}
			else if (hostLocale == "zh_cn")  // Simplified Chinese
			{
				// This is only correct for WinXP, WinVista and Win7,
				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
				if (os.indexOf("Windows") >= 0) {
					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
					overrideFonts = SIM_SUN_GB18030 + FONT_NAME_SEPARATOR + HKSCS_FONT_NAME + FONT_NAME_SEPARATOR + HKSCS_EXTB_FONT_NAME + FONT_NAME_SEPARATOR + SIM_SUN;
					if ((os.indexOf("Windows 7") >= 0) ||
						(os.indexOf("Windows Vista") >= 0) ||
						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
						overrideFonts = MICROSOFT_YA_HEI + FONT_NAME_SEPARATOR + overrideFonts;
					}
				}
				else if (os.indexOf("Mac") >= 0) {
					overrideFonts = HEITI_SC_LIGHT + FONT_NAME_SEPARATOR + ST_HEITI_LIGHT;
				}
				
				// Always include Adobe-supplied font as the last fallback
				if (overrideFonts.length > 0)
					overrideFonts += FONT_NAME_SEPARATOR;
				overrideFonts += ADOBE_SONG_STD_LIGHT;
			}
			else if (hostLocale == "zh_tw")	// Traditional Chinese
			{
				// This is only correct for WinXP, WinVista and Win7,
				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
				if (os.indexOf("Windows") >= 0) {
					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
					overrideFonts = SIM_SUN;
					if ((os.indexOf("Windows 7") >= 0) ||
						(os.indexOf("Windows Vista") >= 0) ||
						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
						overrideFonts = MICROSOFT_JHENG_HEI + FONT_NAME_SEPARATOR + overrideFonts;
					}
				}
				else if (os.indexOf("Mac") >= 0) {
					overrideFonts = HEITI_TC_LIGHT + FONT_NAME_SEPARATOR +  LI_HEI_PRO_MEDIUM;
				}
				
				// Always include Adobe-supplied font as the last fallback
				if (overrideFonts.length > 0)
					overrideFonts += FONT_NAME_SEPARATOR;
				overrideFonts += ADOBE_MING_STD_LIGHT;
			}
					
//			else if (hostLocale == "th_th")	// Thai
//			{			
//				// This is only correct for WinXP, WinVista and Win7,
//				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
//				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
//				if (os.indexOf("Windows") >= 0) {
//					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
//					overrideFonts = SIM_SUN;
//					if ((os.indexOf("Windows 7") >= 0) ||
//						(os.indexOf("Windows Vista") >= 0) ||
//						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
//						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
//						overrideFonts = MICROSOFT_JHENG_HEI + FONT_NAME_SEPARATOR + overrideFonts;
//					}
//				}
//				else if (os.indexOf("Mac") >= 0) {
//				//	overrideFonts = ST_HEITI + FONT_NAME_SEPARATOR + ST_HEITI_LIGHT;
//				}
//						
//				// Always include Adobe-supplied font as the last fallback
//				if (overrideFonts.length > 0)
//					overrideFonts += FONT_NAME_SEPARATOR;
//				overrideFonts += ADOBE_THAI;
//			}
//					
//			else if (hostLocale == "he_il")	// Hebrew
//			{
//				// This is only correct for WinXP, WinVista and Win7,
//				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
//				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
//				if (os.indexOf("Windows") >= 0) {
//					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
//					overrideFonts = MS_UI_GOTHIC_FONT_NAME + FONT_NAME_SEPARATOR + MS_GOTHIC_FONT_NAME;
//					if ((os.indexOf("Windows 7") >= 0) ||
//						(os.indexOf("Windows Vista") >= 0) ||
//						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
//						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
//						overrideFonts = MEIRYO_FONT_NAME + FONT_NAME_SEPARATOR + overrideFonts;
//					}
//				}
//				else if (os.indexOf("Mac") >= 0) {
//					overrideFonts = HIRAGINO_KAKU_GOTHIC_FONT_NAME;
//				}
//						
//				// Always include Adobe-supplied font as the last fallback
//				if (overrideFonts.length > 0)
//					overrideFonts += FONT_NAME_SEPARATOR;
//				overrideFonts += ADOBE_HEBREW;
//			}
//					
//			else if (hostLocale == "ar_ae") Arabic
//			{
//				// This is only correct for WinXP, WinVista and Win7,
//				// Windows Server 2003, Windows Server 2008 and Windows Server 2008 R2
//				// Reader/Acrobat X is not supported on earlier versions of the Windows OS
//				if (os.indexOf("Windows") >= 0) {
//					// This covers WinXP, Windows Server 2003 and any pre-WinXP Windows versions
//					overrideFonts = MS_UI_GOTHIC_FONT_NAME + FONT_NAME_SEPARATOR + MS_GOTHIC_FONT_NAME;
//					if ((os.indexOf("Windows 7") >= 0) ||
//						(os.indexOf("Windows Vista") >= 0) ||
//						(os.indexOf("Windows Server 2008") >= 0)) {	// Includes Windows Server 2008 R2, too
//						// These fonts are preferred before the MS UI Gothic and MS Gothic fonts for Win7, Vista and Server 2008 versions.
//						overrideFonts = MEIRYO_FONT_NAME + FONT_NAME_SEPARATOR + overrideFonts;
//					}
//				}
//				else if (os.indexOf("Mac") >= 0) {
//					overrideFonts = HIRAGINO_KAKU_GOTHIC_FONT_NAME;
//				}
//						
//				// Always include Adobe-supplied font as the last fallback
//				if (overrideFonts.length > 0)
//					overrideFonts += FONT_NAME_SEPARATOR;
//				overrideFonts += ADOBE_ARABIC;
//			}
			else
			{//Return null for unsupported languages.
				overrideFonts = null;
			}

			return overrideFonts;
		}
		
		/**
		 * A component that's suitable for receiving focus by default.
		 *  Navigators can override this getter to provide an appropriate default.
		 */
		public function get defaultFocusItem():IFocusManagerComponent
		{
			return null;
		}
		
		/**
		 * A NavigatorHost object that manages the interaction with host application.
		 */
		private var navigatorHost:NavigatorHost;
		
		/** A function to initialize navigatorHost.
		 */
		private function initNavigatorHost():void
		{
			navigatorHost = new NavigatorHost();
			navigatorHost.navigator = this;
		}
		
		/**
		 * Initializes and later shuts down an <code>INavigator</code> instance.
		 * 
		 * <p>Called early with a non-<code>null</code> <code>host</code> instance, at which
		 * point the PDF portfolio navigator can initialize itself and add event listeners.  
		 * This is likely to occur shortly after construction and may occur before any 
		 * children have been added (in the case of MXML components).</p>
		 * 
		 * <p>When the host shuts down, the PDF portfolio navigator calls this method once again with
		 * a value of <code>null</code>.  At this point the <code>INavigator</code> instance 
		 * is expected to shut itself down.</p>
		 * 
		 * @param host	 The <code>INavigator</code> instance that hosts the PDF portfolio navigator.
		 * @productversion Acrobat 9.0
		 */
		public function set host(host:INavigatorHost):void
		{
			if (host) {
				collectionManager.setHostFromApplication(host, this);
				initHost();
			} else if (commonProperties) {
				commonProperties.unloadSkin(false);
			}

			if (initializedHost) {
				collectionManager.setHostFromApplication(host, this);
				initCommonProperties();
			} else {
				pendingHost = host;
			}
		}
		
		/**
		 * A setup function to initialize host specific data
		 */
		protected function initHost():void
		{
			initializedHost = true;
			addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown, false, EventPriority.DEFAULT_HANDLER);
			collectionManager.addEventListener(RightClickEvent.RIGHT_CLICK, onRightClick);
			
			// Listen for events to indicate when we're done "loading"
			if (rootItemsWatcher != null) {
				rootItemsWatcher.unwatch();
			}
			rootItemsWatcher = BindingUtils.bindSetter(setRootItems, collectionManager, "rootItems");
			PropertyManager.instance.addEventListener(PropertyManagerEvent.UPDATED_CONTEXTS, onUpdatedPropertyContexts);
			
			if (pendingHost != null) {
				host = pendingHost;
				pendingHost = null;
			}
			initCommonProperties();
			var flexFocus:IFocusManagerComponent = defaultFocusItem;
			if (flexFocus != null) {
				collectionManager.defaultFocusItem = flexFocus;
				if (focusManager)
					focusManager.setFocus(flexFocus);
			}
		}
		
		private var lastRootItems:AttachmentCollection;
		private function setRootItems(value:AttachmentCollection):void
		{
			if (lastRootItems != null) {
				lastRootItems.removeEventListener(CollectionEvent.COLLECTION_CHANGE, rootItemsListener, false);
			}
			lastRootItems = value;
			if (value != null) {
				// We use a lower-than-default priority to let any dataGroups using this as a dataProvider
				// take care of their renderers/layouts first (see Watson bug #2711223).
				value.addEventListener(CollectionEvent.COLLECTION_CHANGE, rootItemsListener, false, EventPriority.DEFAULT_HANDLER, true);
				if (value.length > 0) {
					rootItemsDidInit = true;
				} else {
					// Let's wait to see if items get added soon
					var rootItemsTimer:Timer = new Timer(200, 1);
					rootItemsTimer.addEventListener(TimerEvent.TIMER_COMPLETE,
						function (event:TimerEvent):void
						{
							rootItemsTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, arguments.callee);
							rootItemsDidInit = true;
						}
					);
					rootItemsTimer.start();
				}
			}
		}
		
		private function rootItemsListener(event:CollectionEvent):void
		{
			if ((event.kind == CollectionEventKind.ADD) ||
				(event.kind == CollectionEventKind.REMOVE) ||
				(event.kind == CollectionEventKind.RESET))
			{
				setupState();
			}
		}
		
		private function onUpdatedPropertyContexts(event:PropertyManagerEvent):void
		{
			PropertyManager.instance.removeEventListener(PropertyManagerEvent.UPDATED_CONTEXTS, onUpdatedPropertyContexts);
			resourceLoadingCountWatcher = BindingUtils.bindSetter(setLoadingCount, ResourceManager.instance, "loadingCount");
			propertiesDidInit = true;
		}
		
		private function setLoadingCount(value:uint):void
		{
			if (value == 0) {
				if (resourceLoadingCountWatcher != null) {
					resourceLoadingCountWatcher.unwatch();
					resourceLoadingCountWatcher = null;
				}
				resourceLoadingDidInit = true;
			}
		}
		
		/**
		 * Default handling of keyboard events for a navigator. Components handling their
		 * own keyboard events who want to avoid this behavior should call preventDefault
		 * to avoid handling the event again.
		 */
		protected function onKeyDown(event:KeyboardEvent):void
		{
			if (event.isDefaultPrevented()) {
				// Something already handled this event and called preventDefault, so ignore
				return;
			}

			if (event.altKey) {
				// We don't handle any alt events from here
				return;
			}
			
			var shouldPreventDefault:Boolean = false;
			
			if (event.ctrlKey) {
				if (event.charCode != 0) {
					if (String.fromCharCode(event.charCode).toLowerCase() == "a") {
						if (event.shiftKey) {
							// Deselect all
							collectionManager.setSelectedItems([]);
						} else {
							// Select all
							collectionManager.setSelectedItems(collectionManager.currentItems.toArray());
						}
						shouldPreventDefault = true;
					}
				}
			} else if (event.shiftKey) {
				if (event.keyCode == Keyboard.F10) {
					// Context menu
					var menuPoint:Point = new Point();
					if (event.target is UIComponent) {
						menuPoint.x = (event.target as UIComponent).x;
						menuPoint.y = (event.target as UIComponent).y;
					}
					CommandManager.instance.showContextMenu(menuPoint.x, menuPoint.y);
					shouldPreventDefault = true;
				}
			} else {
				// No modifiers; proceed with normal handling
				switch(event.keyCode)
				{
					case Keyboard.LEFT:
						collectionManager.onGotoPreviousAttachment(event);
						shouldPreventDefault = true;
						break;
					case Keyboard.RIGHT:
						collectionManager.onGotoNextAttachment(event);
						shouldPreventDefault = true;
						break;
					case Keyboard.ENTER:
						collectionManager.handlePreview(collectionManager.focusAttachment);
						shouldPreventDefault = true;
						break;
					case Keyboard.ESCAPE: 
						if(collectionManager.currentFolder != null) {
							collectionManager.currentFolder = collectionManager.currentFolder.parent;
						}
						shouldPreventDefault = true;
						break;
					case Keyboard.HOME:
						if (collectionManager.currentItems.length > 0) {
							collectionManager.setSelectedItems([collectionManager.currentItems[0]]);
						}
						shouldPreventDefault = true;
						break;
					case Keyboard.END:
						if (collectionManager.currentItems.length > 0) {
							collectionManager.setSelectedItems([collectionManager.currentItems[collectionManager.currentItems.length-1]]);
						}
						shouldPreventDefault = true;
						break;
					case Keyboard.DELETE:
					case Keyboard.BACKSPACE:
						// Backspace (Delete key) is only allowed on Macintosh
						if ((event.keyCode == Keyboard.BACKSPACE) && !runningOnMacintosh)
							break;
						
						if ( deleteSelectionIfAllowed())
							shouldPreventDefault = true;
						
						break;
					default:
						// TODO no default handling yet, but maybe we should funnel the event to the miniNav?
						break;
				}
			}
			if (shouldPreventDefault) {
				event.preventDefault();
			}
		}
		
		/**
		 * Default handling of right-click events for a navigator. Components handling their
		 * own events who want to avoid this behavior should call preventDefault
		 * to avoid handling the event again.
		 */
		protected function onRightClick(event:RightClickEvent):void
		{
			if (event.isDefaultPrevented()) {
				// Something already handled this event and called preventDefault, so ignore
				return;
			}
			
			var where:Point = new Point(event.stageX, event.stageY);
			var theAttachment:IAttachment = getAttachmentUnderPoint(where);
			if(theAttachment)
			{
				// if the attachment is not already part of the selection
				if (collectionManager.selectedItems.getItemIndex(theAttachment) == -1)
					collectionManager.setSelectedItems([theAttachment]);  // then make it the selection
			}
			
			// If we ever need to worry about modifier keys on a right-click,
			// this is the time. However, be warned that a ctrl+click on the mac
			// will have ctrlKey set to true, so we must show a context menu in that case.
			CommandManager.instance.showContextMenu(event.stageX, event.stageY);
			event.preventDefault();
		}
			
		private function getAttachmentUnderPoint(where:Point):IAttachment
		{
			// this code is somewhat problematic... ideally it should call the layout to do the hit-test
			var theAttachment:IAttachment;
			var moduleRoot:UIComponent = this as UIComponent;
			
			// check all the objects under the right-click point
			const stage:Stage = moduleRoot.stage;
			var hits:Array = stage ? stage.getObjectsUnderPoint(where) : null;
			if(!hits)
				return null;
			
			var currentItems:AttachmentCollection = collectionManager.currentItems;
			hits = hits.reverse();
			hits.some(
				function(hit:*, a:*,b:*):Boolean {
					if(!hit || !hit is DisplayObject) return false;
					var rents:Array = parentage(hit);
					return rents.some(
						function(comp:*,aa:*,bb:*):Boolean {
							if(comp && 'data' in comp)
								theAttachment = comp.data as IAttachment;
							if (currentItems && currentItems.getItemIndex(theAttachment) == -1)
								theAttachment = null;
							return theAttachment != null;
						}
					); // find one good ancestor
				}
			); // find one hit
			
			return theAttachment;
		}
		
		private function parentage(obj:DisplayObject):Array
		{
			var rval:Array = [];
			while(obj){
				rval.push(obj);
				obj = obj.parent;
			}
			return rval;
		}
		
		private var _handleGestures:Boolean = false;
		protected function get handleGestures():Boolean
		{
			return _handleGestures;
		}
		protected function set handleGestures(value:Boolean):void
		{
			if (value == _handleGestures) {
				return;
			}
			var gestureType:String;
			if (value) {
				if (!Multitouch.supportsGestureEvents) {
					// Gestures aren't supported
					return;
				}
				_handleGestures = value;
				for each (gestureType in Multitouch.supportedGestures) {
					addEventListener(gestureType, onGestureEvent, false, 0, true);
				}
			} else {
				_handleGestures = value;
				for each (gestureType in Multitouch.supportedGestures) {
					removeEventListener(gestureType, onGestureEvent, false);
				}
			}
		}
		
		protected function onGestureEvent(event:TransformGestureEvent):void
		{
			if ((event == null) || event.isDefaultPrevented()) {
				return;
			}
			if (event.type != TransformGestureEvent.GESTURE_SWIPE) {
				// Only swipe events are supported
				return;
			}
			if (event.offsetX > 0) {
				// swipe right
				collectionManager.onGotoPreviousAttachment(event);
				return;
			} else if (event.offsetX < 0) {
				// swipe left
				collectionManager.onGotoNextAttachment(event);
				return;
			}
			var pageNumber:uint = (collectionManager.focusAttachment == null) ? 0 : collectionManager.focusAttachment.pageNumber;
			if (event.offsetY > 0) {
				// swipe down
				if (pageNumber > 1) {
					collectionManager.focusAttachment.pageNumber = (pageNumber-1);
				}
			} else if (event.offsetY < 0) {
				// swipe up
				var pageCount:uint = (collectionManager.focusAttachment == null) ? 0 : collectionManager.focusAttachment.pageCount;
				if ((pageCount > 0) && (pageNumber < pageCount))
				{
					collectionManager.focusAttachment.pageNumber = (pageNumber+1);
				}
			}
		}
	}
}
