/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components
{
	import com.adobe.portfolio.events.PreviewEvent;
	import com.adobe.portfolio.lists.CardList;
	import com.adobe.portfolio.previews.IPreviewMultiPageModel;
	
	import flash.display.Bitmap;
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.collections.IList;
	import mx.core.IVisualElement;
	import mx.effects.Effect;
	import mx.events.EffectEvent;
	import mx.logging.ILogger;
	import mx.logging.Log;
	import mx.utils.UIDUtil;
	
	import spark.components.Button;
	import spark.components.DataGroup;
	import spark.components.Group;
	
	/**
	 * The ScrollPad class extends NavSkinnableComponent to implements "touch-like" scrolling
	 * of multipage documents.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class ScrollPad extends NavSkinnableComponent  
	{
		protected static const INIT_SCROLL_DURATION:int = 200;

		[SkinPart(required="false")]
		/**
		 * An optional skin part that displays a list of pages.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var imageDisplay:DataGroup;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that displays a next button for scrolling to the next page.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var nextBtn:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that displays a previous button for scrolling to the next page.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var prevBtn:Button;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part that is normally transparent and which listens to mouse and keyboard events.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var touchPad:Group;

		[Bindable]
		/**
		 * The index of the page that should be scrolled to.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var scrollToValue:int;
		
		[Bindable]
		/**
		 * The duration in msec of the scrolling action (effect).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var scrollDuration:Number;
		
		[Bindable]
		/**
		 * The model of the multipage document being scrolled.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var model:IPreviewMultiPageModel;

		protected static const logger:ILogger = Log.getLogger("ScrollPad");
		
		protected const MIN_SCROLL_DURATION:int = 200;

		protected var initY:int;
		protected var finalY:int;
		protected var initScrollPosition:int;
		protected var scrollTimer:Timer;
		protected var delayScrollTimer:Timer;
		protected var delayScrollCount:int = 0;
		protected var scrollEffect:Effect;
		protected var wheelDelta:int = 0;
		
		private var _scrollPosition:int;
		private var _showButtons:Boolean = true;
		private var _pageNumber:uint;
		private var _dataProvider:IList;
		private var _pageNavigationEnabled:Boolean;
		
		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function ScrollPad()
		{
			super();
			scrollTimer = new Timer(100);
			scrollPosition = 0;
			_pageNumber = 0;
			_pageNavigationEnabled = false;
			scrollDuration = INIT_SCROLL_DURATION;
			initY = -1;
		}
		
		override protected function partAdded(partName:String, instance:Object):void {
			super.partAdded(partName, instance);
			switch (instance) {
				case touchPad:
					addListener(touchPad, MouseEvent.MOUSE_DOWN, touchPad_mouseDownHandler);
					addListener(touchPad, MouseEvent.MOUSE_WHEEL, touchPad_mouseWheelHandler);
					break;

				case prevBtn:
					addListener(prevBtn, MouseEvent.CLICK, button_clickHandler);
					break;
				
				case nextBtn:
					addListener(nextBtn, MouseEvent.CLICK, button_clickHandler);
					break;
				
				default:
					break;
			}
		}
		
		override protected function attachSkin():void 
		{
			super.attachSkin();
			
			addListener(this, Event.ADDED_TO_STAGE, onAddedToStage);
			addListener(this, Event.REMOVED_FROM_STAGE, onRemovedFromStage);
			addListener(this, Event.RESIZE, onResize);
			addListener(this, MouseEvent.ROLL_OUT, touchPad_rollOutHandler);
			addListener(this, MouseEvent.ROLL_OVER, touchPad_rollOverHandler);
			addListener(systemManager.getSandboxRoot(), MouseEvent.MOUSE_UP, touchPad_mouseUpHandler);
			addListener(parentApplication as IEventDispatcher, KeyboardEvent.KEY_DOWN, touchPad_keyDownHandler);
			
			scrollEffect = skin["buttonScroll"];
			addListener(scrollEffect, EffectEvent.EFFECT_END, onEndScroll);
		}
		
		protected function onAddedToStage(e:Event):void
		{
			addWatcher(BindingUtils.bindSetter(onModelPageNumberChanged, this, ["model", "pageNumber"]));
		}
		
		protected function onRemovedFromStage(e:Event):void
		{
			autoWatch.removeAllWatchers();
		}
		
		protected function onResize(e:Event):void
		{
			invalidateProperties();
		}
		
		protected function onEndScroll(e:Event):void
		{
			dispatchEvent(new Event(Event.COMPLETE));
		}
		
		
		protected function onModelPageNumberChanged(value:uint):void
		{
			setPageNumber(value);
		}
		
		protected function setModelPageNumber(pageNumber:uint):void
		{
			if (!enabled)
				return;
			if (!model)
				return;
			model.pageNumber = pageNumber;
		}
		
		protected function touchPad_mouseDownHandler(event:MouseEvent):void
		{
			if (!doubleClickEnabled)
				return;
			if (_pageNumber == 0)
				return;
			
			initY = event.localY;
			initScrollPosition = imageDisplay.verticalScrollPosition;
			scrollTimer.start();
			touchPad.addEventListener(MouseEvent.MOUSE_MOVE, watchMovement);

			// Disable drag-and-drop in CardLists to avoid unwanted reordering
			CardList.disableDragAndDrop();
		}
		
		protected function touchPad_rollOutHandler(event:MouseEvent):void
		{
			prevBtn.visible = false;
			nextBtn.visible = false;
		}
		
		protected function touchPad_rollOverHandler(event:MouseEvent):void
		{
			if (showButtons)
			{
				prevBtn.visible = true;
				nextBtn.visible = true;
			}
		}
		
		protected function touchPad_mouseUpHandler(event:MouseEvent):void
		{
			if (initY == -1 || _pageNumber == 0)
				return;
			
			CardList.enableDragAndDrop();
			touchPad.removeEventListener(MouseEvent.MOUSE_MOVE, watchMovement);
			finalY = event.localY;
			
			scrollDuration = Math.max( (scrollTimer.currentCount * 100), MIN_SCROLL_DURATION);
			scrollTimer.reset();
			
			var currentItemInView:Vector.<int> = imageDisplay.getItemIndicesInView();
			if (currentItemInView.length >= 2) {
				chooseActivePage(currentItemInView[0], currentItemInView[1]);
			}
		}
		
		protected function chooseActivePage( index1:int, index2:int ):void
		{	
			if (index2 < index1) {
				var temp:int = index2;
				index2 = index1;
				index1 = temp;
			}
			var elementIndex:int = finalY < initY ? index2 : index1;
			
//			skin["resetScroll"].play();
			pageNumber = elementIndex + 1;
		}
		
		protected function watchMovement(event:MouseEvent):void
		{
			var lastElementY:int = imageDisplay.getElementAt(imageDisplay.dataProvider.length - 1).getLayoutBoundsY();
			
			var deltaY:int = event.localY - initY;
			var newPosition:int = initScrollPosition - deltaY;
			if (newPosition < 0)
				newPosition = 0;
			else if (newPosition > lastElementY) {
				newPosition = lastElementY;
			}
			scrollPosition = newPosition;
		}
		
		protected function touchPad_mouseWheelHandler(event:MouseEvent):void
		{
			if (!pageNavigationEnabled || !enabled || !visible)
				return;
						
			if (event.isDefaultPrevented())
				return;

			wheelDelta += event.delta;
			
			if (Math.abs(wheelDelta) < 2)
				return;
			
			pageByDelta(wheelDelta > 0 ? -1 : 1)
			wheelDelta = 0;
			
			event.preventDefault();
		}
		
		protected function touchPad_keyDownHandler(event:KeyboardEvent):void
		{
			if (!pageNavigationEnabled || !enabled || !visible)
				return;
			
			if (event.isDefaultPrevented())
				return;
			
			switch (event.keyCode) { // don't grab these keyboard events!
				case Keyboard.HOME:
					setModelPageNumber(1);
					break;
				
				case Keyboard.END:
					if (imageDisplay && imageDisplay.dataProvider)
						setModelPageNumber(imageDisplay.dataProvider.length);
					break;
					
				case Keyboard.PAGE_DOWN:
				case Keyboard.DOWN:
					pageByDelta(1);
					break;
					
				case Keyboard.PAGE_UP:
				case Keyboard.UP:
					pageByDelta(-1);
					break;
				
				default:
					return;
			}
			
			event.preventDefault();
		}

		[Bindable]
		/**
		 * The dataProvider for the list of pages.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get dataProvider():IList
		{
			return _dataProvider;
		}
		
		public function set dataProvider(value:IList):void
		{
			_dataProvider = value;
			if (!value) {
				_pageNumber = 0;
			}
		}
		
		protected function button_clickHandler(event:MouseEvent):void
		{
			pageByDelta(event.currentTarget.id == "nextBtn" ? 1 : -1);
		}
		
		protected function pageByDelta(delta:int):void
		{
			if (delta != -1 && delta != 1)
				return;
						
			var indices:Vector.<int> = imageDisplay.getItemIndicesInView()
			if (indices.length == 0) 
				return;
			
			var newIndex:int = indices[0] + delta;
			var newPageNumber:uint = newIndex + 1; 
			if (newPageNumber < 1 || newPageNumber > model.pageCount)
				return;
			
			setModelPageNumber(newPageNumber);
			pageNumber = newPageNumber;
		}

		[Bindable (name="pageNumberChanged")]
		/**
		 * The current page number.  Setting it will cause ScrollPad to scroll to the indicated page.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get pageNumber():uint
		{
			return _pageNumber;
		}

		public function set pageNumber(value:uint):void
		{
			if (value == 0)
				return;
			
			if (_pageNumber == value)
				return;
			
			setPageNumber(value);
			dispatchEvent(new PreviewEvent(PreviewEvent.PAGE_NUMBER_CHANGED));
		}
		
		protected function setPageNumber(value:uint):void
		{
			_pageNumber = value;
			playButtonScroll();
		}
		
		override protected function commitProperties():void 
		{
			super.commitProperties();
			playButtonScroll();
		}
		
		protected function playButtonScroll():void {
			if (!imageDisplay || !dataProvider || !enabled || _pageNumber <= 0)
				return;
			
			if (dataPending()) {
				delayedButtonScroll();
				return;
			}
			
			var newElement:IVisualElement = imageDisplay.getElementAt(_pageNumber-1);
			if (!newElement)
				return;
			
			var newScrollToValue:Number = newElement.getLayoutBoundsY();
			if (newScrollToValue == 0 && _pageNumber > 1)
				return;
			
			if (newScrollToValue == scrollToValue)
				return;
			
			scrollToValue = newScrollToValue;
			
//			logger.debug(UID + " ScrollPad: playButtonScroll pageNumber= " + _pageNumber + " scrollToValue= " + scrollToValue);
			if (scrollEffect) {
				if (scrollEffect.isPlaying)
					scrollEffect.end();
				scrollEffect.play();
			}
			
		}
		
		public function invalidateScrolling():void
		{
			playButtonScroll();
		}
		
		protected function dataPending():Boolean {
			var index:int = _pageNumber-1;
			var element:IVisualElement = imageDisplay.getElementAt(index);
			if (!element) {
//				logger.debug(UID + " ScrollPad: dataPending TRUE NO ELEMENT pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
				return true;
			}
			var bitmap:Bitmap = dataProvider.getItemAt(index) as Bitmap;
			if (bitmap == null) {
//				logger.debug(UID + " ScrollPad: dataPending TRUE NO DATA pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
				return true;
			}
			if (bitmap.height == 0) {
				logger.debug(UID + " ScrollPad: dataPending TRUE NO VALID DATA pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
				return true;
			}
			
//			logger.debug(UID + " ScrollPad: dataPending FALSE pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
			return false;
		}
		
		protected function delayedButtonScroll():void {
			if (!dataProvider)
				return;
			
			if (!delayScrollTimer) {
				delayScrollTimer = new Timer(200, 1);
				delayScrollTimer.addEventListener(TimerEvent.TIMER,
					handleDelayScroll, false, 0, true);
			}
			delayScrollTimer.reset();
			delayScrollTimer.start();
			logger.debug(UID + " ScrollPad: delayedButtonScroll START TIMER pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
		}
		
		protected function handleDelayScroll(e:Event):void {
			delayScrollTimer.stop();
				
			if (dataPending()) {
//				logger.debug(UID + " ScrollPad: delayedButtonScroll CALL LATER pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
				delayScrollCount++;
				if (delayScrollCount > 10) { // too many retries - give up
					delayScrollCount = 0;
					logger.warn(UID + " too many retries for delayed scroll");
					return;
				}
				callLater(delayedButtonScroll);
				return;
			}
			
			delayScrollCount = 0;
			jumpToPage();
			
//			logger.debug(UID + " ScrollPad: delayedButtonScroll pageNumber= " + _pageNumber + " scrollPosition= " + scrollPosition);
		}
		
		protected function jumpToPage():void 
		{
			if (!imageDisplay)
				return;
			
			var newElement:IVisualElement = imageDisplay.getElementAt(_pageNumber-1);
			if (!newElement)
				return;
			
			scrollPosition = 0; // this is necessary to trigger a new scroll position in some cases
			scrollPosition = newElement.getLayoutBoundsY();
			dispatchEvent(new Event(Event.COMPLETE));
		}
		
		private var _uid:String;

		private function get UID():String {
			
			if (_uid == null) {
				var s:String = UIDUtil.getUID(this);
				_uid = s.slice(s.length-40);
			}
			return _uid;
		}

		[Bindable]
		/**
		 * Whether to show the next / previous buttons on a rollOver.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get showButtons():Boolean
		{
			return _showButtons;
		}

		public function set showButtons(value:Boolean):void
		{
			_showButtons = value;
		}

		[Bindable]
		/**
		 * The current scroll position.  Different from the page index, it basically specifies
		 * the vertical edge of pages.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get scrollPosition():int
		{
			return _scrollPosition;
		}

		public function set scrollPosition(value:int):void
		{
			_scrollPosition = value;
		}
		
		override public function set enabled(value:Boolean):void
		{
			super.enabled = value;
			if (value)
				jumpToPage();
		}

		[Bindable]
		/**
		 *  Whether page navigation is enabled (via keyboard, mousewheel)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.1
		 */	
		public function get pageNavigationEnabled():Boolean
		{
			return _pageNavigationEnabled;
		}

		public function set pageNavigationEnabled(value:Boolean):void
		{
			_pageNavigationEnabled = value;
		}

	}
}
