/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.details
{
	import com.adobe.portfolio.api.IField;

	/**
	 *  The DetailsField class represents the IField of an IAttachment and adds Details specific information.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */	
	public class DetailsField
	{
		/**
		 *  Constructor.
		 * 
		 *  @param field A field of an IAttachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function DetailsField(field:IField)
		{
			this._field = field;
		}
		
		private var _field:IField;

		/**
		 *  The IAttachment field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get field():IField
		{
			return _field;
		}
		
		/**
		 *  The type of the field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get type():String
		{
			return field ? field.type : "";
		}
		
		/**
		 *  The name of the field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get name():String
		{
			return field ? field.name : "";
		}

		/**
		 *  The display name of the field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get displayName():String
		{
			return field ? field.displayName : "";
		}

		/**
		 *  The visibility of the field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get visible():Boolean
		{
			return field ? field.visible : false;
		}

		/**
		 *  The editability of the field.
		 *  @see com.adobe.portfolio.api.IField
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get editable():Boolean
		{
			return field ? field.editable : false;
		}

		// Additional info other than info provided by IField
		
		// All fields are resizable except for the last one
		private var _resizable:Boolean = true;
		
		/**
		 *  Whether a field can be resized or not.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get resizable():Boolean
		{
			return _resizable;
		}
		
		/**
		 *  @private
		 */ 
		public function set resizable(value:Boolean):void
		{
			if (value == _resizable)
				return;
			_resizable = value;
		}

		// The width set by Details.as when assigning a default width
		private var _calculatedWidth:Number = 0;

		/**
		 *  The calculated width of the field.
		 *  This is usually set by the Details class, based on the width of the header and first few items in a column.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get calculatedWidth():Number
		{
			return _calculatedWidth;
		}

		/**
		 *  @private
		 */ 
		public function set calculatedWidth(value:Number):void
		{
			if (value == _calculatedWidth)
				return;
			_calculatedWidth = value;
		}

		// The width set by the user if they explicitly resize the field column width
		private var _explicitWidth:Number = Number.NaN;

		/**
		 *  The explicit width of the field.
		 *  This is usually set by the Details class when the user resizes a column.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get explicitWidth():Number
		{
			return _explicitWidth;
		}

		/**
		 *  @private
		 */ 
		public function set explicitWidth(value:Number):void
		{
			if (value == _explicitWidth)
				return;
			_explicitWidth = value;
		}

		/**
		 *  The width, in pixels, of the field.
		 *  If the field is not visible the width is zero.
		 *  If the explicitWidth has been set, that is returned, otherwise the calculatedWidth is returned.
		 *  @see #explicitWidth
		 *  @see #calculatedWidth
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get width():Number
		{
			if (!visible)
				return 0;
			if (!isNaN(explicitWidth))
				return explicitWidth;
			return calculatedWidth;
		}

	}
}