/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.details
{
	import com.adobe.portfolio.components.supportClasses.CustomizableTextArea;
	import com.adobe.portfolio.managers.AttachmentDropManager;
	import com.adobe.portfolio.managers.CollectionManager;
	
	import flash.events.FocusEvent;
	import flash.events.IEventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	import mx.collections.ArrayList;
	import mx.events.DataGridEvent;
	import mx.events.DragEvent;
	
	import spark.components.List;
	import spark.events.RendererExistenceEvent;
	
	/**
	 *  The DetailsHeader class extends List to implement the header row in the Details navigator. 
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */	
	public class DetailsHeader extends List
	{
		/**
		 *  The navigator's CollectionManager instance.
		 *  All host interaction should go through this instance, rather than the INavigatorHost directly.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected const collectionMgr:CollectionManager = CollectionManager.instance;
		
		private static const DATA_GROUP:String = "dataGroup";
		
		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function DetailsHeader()
		{
			super();
			
			dragEnabled = true;
			dragMoveEnabled = true;
			dropEnabled = true;
			
			hasFocusableChildren = true;
			tabFocusEnabled = true;

			useVirtualLayout = false;
		}
		
		private var _details:Details;

		/**
		 *  The main Details class.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function get details():Details
		{
			return _details;
		}

		/**
		 *  @private
		 */
		public function set details(value:Details):void
		{
			if (value == _details)
				return;
			_details = value;
			var dhLayout:DetailsHeaderLayout = layout as DetailsHeaderLayout;
			if (dhLayout)
				dhLayout.details = details;
		}
		
		private var _fields:ArrayList = null;

		/**
		 *  An array of DetailFields that corresponds to each of the fields in the IAttachment.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get fields():ArrayList
		{
			return _fields;
		}

		/**
		 *  @private
		 */
		public function set fields(value:ArrayList):void
		{
			if (value == _fields)
				return;
			_fields = value;
			dataProvider = fields;
		}
		
		/** Called when a skin has added a skin part for this instance.
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, true);
		}
		
		/** Called when a skin has removed a skin part for this instance.
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, false);
		}
		
		/** Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			var activePart:IEventDispatcher = instance as IEventDispatcher;
			if (activePart) {
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case DATA_GROUP:
						types = [ RendererExistenceEvent.RENDERER_ADD, RendererExistenceEvent.RENDERER_REMOVE ];
						listeners = [ rendererAddHandler, rendererRemoveHandler ];
						break;
					
					default:
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = listeners ? listeners.length : 0;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
		
		/**
		 *  @private
		 *  Called when an item has been added to this component.
		 */
		private function rendererAddHandler(event:RendererExistenceEvent):void
		{
			var headerItem:DetailsHeaderItem = event.renderer as DetailsHeaderItem;
			if (!headerItem)
				return;
			headerItem.addEventListener(DataGridEvent.HEADER_RELEASE, onHeaderClicked);
			headerItem.addEventListener(DataGridEvent.COLUMN_STRETCH, onColumnResizeStart);
		}
		
		/**
		 *  @private
		 *  Called when an item has been removed from this component.
		 */
		private function rendererRemoveHandler(event:RendererExistenceEvent):void
		{
			var headerItem:DetailsHeaderItem = event.renderer as DetailsHeaderItem;
			if (!headerItem)
				return;
			headerItem.removeEventListener(DataGridEvent.HEADER_RELEASE, onHeaderClicked);
			headerItem.removeEventListener(DataGridEvent.COLUMN_STRETCH, onColumnResizeStart);
		}
		
		private function onHeaderClicked(event:DataGridEvent):void
		{
			var detailsHeaderItem:DetailsHeaderItem = event.itemRenderer as DetailsHeaderItem;
			if (!details || !detailsHeaderItem)
				return;

			// Update the sort order
			details.updateSortOrder(detailsHeaderItem.detailsField.name, !detailsHeaderItem.sortAscending);

			// Pass the event up the chain (to Details)
			dispatchEvent(event);
		}
		
		override protected function keyDownHandler(event:KeyboardEvent):void
		{
			if (!dataGroup)
				return;

			// Don't change the sort order/column if the user is trying to edit the header name
			if (CustomizableTextArea.textAreaHasFocus(focusManager))
				return;
			
			if ( (event.keyCode == Keyboard.LEFT) || (event.keyCode == Keyboard.RIGHT) ||
				 (event.keyCode == Keyboard.SPACE) || (event.keyCode == Keyboard.ENTER))
			{
				var changed:Boolean = false;
				
				var indx:int;
				for (indx=0; indx<dataGroup.numElements; indx++)
				{
					var anItem:DetailsHeaderItem = dataGroup.getElementAt(indx) as DetailsHeaderItem;
					if ( anItem && anItem.sortedField)
						break;
				}
				
				if ( (event.keyCode == Keyboard.LEFT) && (indx > 0))
				{
					indx = indx - 1;
					changed = true;
				}
			
				if ( (event.keyCode == Keyboard.RIGHT) && (selectedIndex < dataGroup.numElements-1))
				{
					indx = indx + 1;
					changed = true;
				}
							
				event.preventDefault();
				
				var headerItem:DetailsHeaderItem = dataGroup.getElementAt(indx) as DetailsHeaderItem;
				if ( !headerItem)
					return;
				
				if ( (event.keyCode == Keyboard.SPACE) || (event.keyCode == Keyboard.ENTER))
					headerItem.sortAscending = !headerItem.sortAscending;								
				
				// Update the sort order
				if (details)
				{
					details.updateSortOrder(headerItem.detailsField.name, !headerItem.sortAscending);
					focusManager.setFocus(this);
				}
				
				if (changed)
				{	// This will notify the accessibility code that it needs to send a focus event to clients. 
					var focusEvent:FocusEvent = new FocusEvent(FocusEvent.FOCUS_IN);
					headerItem.dispatchEvent( focusEvent);
				}

				return;
			}
			
			super.keyDownHandler( event);
		}
		
		/**
		 *  Called when the sort order is changed by clicking in the header of a column.
		 *  This method calls updateSortState for each DetailsHeaderItem for this header.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function updateSortIndicators():void
		{
			if (!dataGroup)
				return;
			for (var index:int = 0; index < dataGroup.numElements; ++index) {
				var detailsHeaderItem:DetailsHeaderItem = dataGroup.getElementAt(index) as DetailsHeaderItem;
				if (!detailsHeaderItem)
					continue;
				detailsHeaderItem.updateSortState();
			}
		}
		
		private var columnResizeInProgress:Boolean = false;
		
		private function onColumnResizeStart(event:DataGridEvent):void
		{
			// This keeps drag and drop from moving the column while we're resizing columns
			columnResizeInProgress = true;
			// Pass the event up the chain (to Details)
			dispatchEvent(event);
		}
		
		/**
		 *  Called when the column resizing has completed.
		 *  Used by dragStart to avoid confusing drag and resize actions.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function columnResizingFinished():void
		{
			columnResizeInProgress = false;
		}
		
		override protected function dragStartHandler(event:DragEvent):void
		{
			// We're dragging the column "edge" for resizing, so tell drag/drop to take a hike
			if (columnResizeInProgress) {
				event.preventDefault();
				return;
			}

			super.dragStartHandler(event);
			if (event.isDefaultPrevented())
				return;
		}
		
		override protected function dragEnterHandler(event:DragEvent):void
		{
			// Don't accept (dragging/dropping) attachments in the header
			if (event.dragSource.hasFormat(AttachmentDropManager.DRAG_DROP_ATTACHMENTS)) {
				event.preventDefault();
				return;
			}
			
			super.dragEnterHandler(event);
		}
		
		override protected function dragDropHandler(event:DragEvent):void
		{
			super.dragDropHandler(event);
			if (event.isDefaultPrevented())
				return;
			
			var movedField:DetailsField = getMovedField(event);
			if (!movedField)
				return;
			
			var nextFieldName:String = getFieldNameAfter(movedField);
			if (!details || nextFieldName == null)
				return;
			
			// Update the field order
			details.updateFieldOrder(movedField.field, nextFieldName);
		}
		
		private function getMovedField(event:DragEvent):DetailsField
		{
			var movedFields:Vector.<Object> = event.dragSource.dataForFormat(AttachmentDropManager.DRAG_DROP_ITEMS) as Vector.<Object>;
			if (!movedFields || !movedFields.length)
				return null;
			return movedFields[0] as DetailsField;
		}
		
		// Find the field that follows the 'moved' field (to determine its new location)
		private function getFieldNameAfter(movedField:DetailsField):String
		{
			if (!fields || !fields.source || !fields.source.length)
				return null;
			for (var fieldIndex:int = 0; fieldIndex < fields.source.length; ++fieldIndex) {
				var field:DetailsField = fields.source[fieldIndex] as DetailsField;
				if (!field)
					continue;
				if (field.name == movedField.name) {
					if (fieldIndex < fields.source.length - 1) {
						field = fields.source[fieldIndex + 1] as DetailsField;
						if (!field)
							return null;
						return field.name;
					}
					// Field repositioned to the end is indicated by the empty string
					return "";
				}
			}
			// The drag/dropped field was not found (which shouldn't happen)
			return null;
		}
	}
}
