/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.details
{
	import com.adobe.portfolio.api.FieldType;
	import com.adobe.portfolio.api.IField;
	import com.adobe.portfolio.api.ISort;
	import com.adobe.portfolio.components.NavSkinnableComponent;
	import com.adobe.portfolio.components.supportClasses.FieldTextInput;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.utils.FieldUtils;
	import com.adobe.portfolio.utils.GlobalizationUtil;
	
	import flash.events.MouseEvent;
	
	import mx.controls.listClasses.IListItemRenderer;
	import mx.core.UIFTETextField;
	import mx.events.DataGridEvent;
	import mx.managers.CursorManager;
	import mx.managers.CursorManagerPriority;
	import mx.managers.IFocusManagerComponent;
	
	import spark.components.HGroup;
	import spark.components.IItemRenderer;
	
	/**
	 *  The DetailsHeaderItem class extends NavSkinnableComponent to implement a cell of the header row in the Details navigator. 
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */	
	public class DetailsHeaderItem extends NavSkinnableComponent implements IFocusManagerComponent, IListItemRenderer, IItemRenderer
	{
		/**
		 *  The navigator's CollectionManager instance.
		 *  All host interaction should go through this instance, rather than the INavigatorHost directly.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected const collectionMgr:CollectionManager = CollectionManager.instance;

		// ToDo: Seems like it would be better if this came from the skin (or total width of skin parts)
		private static const DETAILS_HEADER_ITEM_PADDING:Number = 31;
		
		/**
		 * Used to provide minimum widths for field values
		 */
		private static const TYPICAL_SIZE:Number = 1024;
		private static const TYPICAL_NUMBER:Number = 1000;
		private static const TYPICAL_STRING_FIELD:String = "Example";
		
		private var typicalDateEntry:String = null;
		private var typicalSizeEntry:String = null;
		private var typicalNumberEntry:String = null;
		
		/**
		 * Use this to measure a header which has no name
		 */
		private static const MINIMUM_HEADER_NAME:String = "X";
		
		/**
		 *  The group containing the header items so clicks around the items can be detected.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="true")]
		public var headerItems:HGroup;
		
		/**
		 *  The (editable if user-defined) header text.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="true")]
		public var headerName:FieldTextInput;
		
		/**
		 *  An icon to show that this column is being sorted with ascending/descending sort.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="true")]
		public var sortIndicator:HGroup;
		
		/**
		 *  A button/region that identifies where columns can be resized from.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="true")]
		public var resizeSeparator:HGroup;
		
		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function DetailsHeaderItem()
		{
			super();
			
			tabFocusEnabled = false;
			hasFocusableChildren = false;
		}
		
		private var _field:IField;

		/**
		 *  The IAttachment IField for this header item.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get field():IField
		{
			return _field;
		}

		/**
		 *  @private
		 */	
		public function set field(value:IField):void
		{
			_field = value;
		}
		
		private var _resizable:Boolean = true;

		/**
		 *  Indicates if this header item is a resizable field.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get resizable():Boolean
		{
			return _resizable;
		}

		/**
		 *  @private
		 */	
		public function set resizable(value:Boolean):void
		{
			_resizable = value;
		}

		private var _sortedField:Boolean = false;
		
		/**
		 *  Indicates if this header item is the sort field.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get sortedField():Boolean
		{
			return _sortedField;
		}
		
		/**
		 *  @private
		 */	
		public function set sortedField(value:Boolean):void
		{
			_sortedField = value;
		}
		
		private var _sortAscending:Boolean = true;
		
		/**
		 *  Indicates if this header item is sorted with an ascending sort.
		 *  This is used by the skin to draw an up (vs down) arrow icon.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public function get sortAscending():Boolean
		{
			return _sortAscending;
		}
		
		/**
		 *  @private
		 */	
		public function set sortAscending(value:Boolean):void
		{
			_sortAscending = value;
		}
		
		/**
		 *  Adds listeners and watchers that are automatically unwatched and unlistened.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		override public function addListenAndWatch():void
		{
			super.addListenAndWatch();
			addListener(this, MouseEvent.CLICK, onMouseClick);
		}
		
		private var detailsFieldChanged:Boolean = false;
		private var sortStateChanged:Boolean = false;
		override protected function commitProperties():void
		{
			if (detailsFieldChanged) {
				// Initialize some things if the field is not null
				if (detailsField) {
					enabled = detailsField.visible;
					resizable = detailsField.resizable;
					includeInLayout = detailsField.visible;
					field = detailsField.field;
					if (headerName)
						headerName.visible = detailsField.visible;
				}
					
				// Anything to be reset if the new field value is null?
				else {
					enabled = false;
					resizable = false;
					includeInLayout = false;
					field = null;
					if (headerName)
						headerName.visible = false;
				}
				detailsFieldChanged = false;
				sortStateChanged = true;
			}
			if (sortStateChanged) {
				if (collectionMgr.sort && collectionMgr.sort.length) {
					var iSort:ISort = collectionMgr.sort[0] as ISort;
					// If sort exists, field exists and this is the primary sort field,
					// show the icon and as ascending/descending, whichever it is now
					if (iSort && field && field.visible && field.name == iSort.name) {
						sortedField = true;
						sortAscending = iSort.ascending;
					}
					else {
						sortedField = false;
					}
				}
				sortStateChanged = false;
			}
			super.commitProperties();
		}
		
		/** Called when a skin has added a skin part for this instance.
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case headerName:
					addListener(headerName, MouseEvent.CLICK, onHeaderNameClicked);
					break;
				
				case sortIndicator:
					addListener(sortIndicator, MouseEvent.CLICK, onHeaderClicked);
					break;
				
				case resizeSeparator:
					if (resizable) {
						addListener(resizeSeparator, MouseEvent.MOUSE_OVER, onResizeAreaMouseOver);
						addListener(resizeSeparator, MouseEvent.MOUSE_OUT, onResizeAreaMouseOut);
						addListener(resizeSeparator, MouseEvent.MOUSE_DOWN, onResizeAreaMouseDown);
					}
					break;
				
				default:
					break;
			}
		}
				
		// Catch mouse clicks around the skin parts and use them to toggle the sort state, too
		private function onMouseClick(event:MouseEvent):void 
		{
			if (event.isDefaultPrevented())
				return;
			event.preventDefault();
			if (event.target == headerItems || event.target == skin) {
				onHeaderClicked(event);
			} 
		}
		
		private function onHeaderNameClicked(event:MouseEvent):void
		{
			// If user is editing a user-defined name, ignore the
			// header click that gets passed through to us.
			// Otherwise when the user tries to edit the field
			// they end up sorting that field, too (which they didn't want to do).
			if (collectionMgr.authoringMode && FieldUtils.isUserDefinedField(field))
				return;
			onHeaderClicked(event);
		}
		
		// note that event is not used here and that it could be null if called from the routine 'selected' below (which is used by automation)
		private function onHeaderClicked(event:MouseEvent):void
		{
			// If the sort icon is showing, change ascending/descending
			if (sortedField)
				sortAscending = !sortAscending;
			var headerReleaseEvent:DataGridEvent = new DataGridEvent(DataGridEvent.HEADER_RELEASE);
			headerReleaseEvent.itemRenderer = this;
			dispatchEvent(headerReleaseEvent);
		}
		
		private var resizeCursorID:int = -1;
		
		private function onResizeAreaMouseOver(event:MouseEvent):void
		{
			if (event.buttonDown)
				return;
			resizeCursorID = CursorManager.setCursor(
				DetailsResizeCursor, CursorManagerPriority.HIGH,
				-DetailsResizeCursor.bitmap.width/2, -DetailsResizeCursor.bitmap.height/2);
		}
		
		private function onResizeAreaMouseOut(event:MouseEvent):void
		{
			restoreCursor();
		}
		
		private function onResizeAreaMouseDown(event:MouseEvent):void
		{
			restoreCursor();
			var resizeStartEvent:DataGridEvent = new DataGridEvent(DataGridEvent.COLUMN_STRETCH);
			resizeStartEvent.itemRenderer = this;
			dispatchEvent(resizeStartEvent);
		}
		
		private function restoreCursor():void
		{
			if (resizeCursorID == -1)
				return;
			CursorManager.removeCursor(resizeCursorID);
			resizeCursorID = -1;
		}
		
		/**
		 *  Get the ideal width for this card item. Usually this is the measured width of the text plus some padding.
		 * 
		 *  @param field The DetailsField for this card item. This method can be called before the DetailsField is set,
		 *  which is why this is passed as an argument.
		 *  @param measureField A field on the stage appropriate for measuring text.
		 *  @return Width of the text, sort indicator, resize indicator plus some padding.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function getCalculatedWidth(field:DetailsField, measureField:UIFTETextField = null):Number
		{
			if (!field)
				return 0;
			var text:String = field.displayName;
			// I don't think a user can define an empty field,
			// but in case there is one, use minimum header name
			// to guarantee that the width is not zero.
			if (!text || !text.length)
				text = MINIMUM_HEADER_NAME;
			var calcWidth:Number;
			if (measureField != null) {
				measureField.multiline = false;
				calcWidth = measureField.getUITextFormat().measureText(text).width;
			} else {
				calcWidth = measureText(text).width;
			}
			calcWidth = Math.max(calcWidth, minimumItemWidth(field.field));
//			trace("Calculated header width for '" + text + "' is: " + calcWidth);
			if (calcWidth == 0)
				return Number.NaN;
			// The sort indicator must be available to measure properly
			if (!sortIndicator || !sortIndicator.width)
				return Number.NaN;
			// The resize separator must be available to measure properly
			if (!resizeSeparator || !resizeSeparator.width)
				return Number.NaN;
			return calcWidth + sortIndicator.width + resizeSeparator.width + DETAILS_HEADER_ITEM_PADDING;
		}
		
		// Some attachments don't have any entries for fields, one example being folders
		// that do not have any "size" or "compressed size" fields. If the first 10 entries in
		// a portfolio are all folders, then the auto-sizing code in Details (which calls
		// getCalculatedWidth above) will size the "size" column to a width of zero
		// (or near zero) and then the entries that follow will be truncated.
		// Another example is when the details navigator is first created or is empty.
		// The code below provides some "typical" values for fields that do not have any
		// values to avoid that problem.
		private function minimumItemWidth(field:IField):Number
		{
			if (!field)
				return 0;
			
			var text:String;
			switch(field.type) {
				case FieldType.DATE:
				case FieldType.MOD_DATE:
				case FieldType.CREATION_DATE:
					if (!typicalDateEntry) {
						// Use a date which has 2 digits in each component
						// to try to get a maximum width date. May not work
						// out for some locales, but a good estimate.
						var date:Date = new Date(2010, 11, 24, 11, 59, 59);
						typicalDateEntry = GlobalizationUtil.formatDateTime(date);
					}
					text = typicalDateEntry;
					break;
				
				case FieldType.SIZE:
				case FieldType.COMPRESSED_SIZE:
					if (!typicalSizeEntry)
						typicalSizeEntry = GlobalizationUtil.formatSize(TYPICAL_SIZE);
					text = typicalSizeEntry;
					break;
				
				case FieldType.NUMBER:
					if (!typicalNumberEntry)
						typicalNumberEntry = GlobalizationUtil.formatNumber(TYPICAL_NUMBER);
					text = typicalNumberEntry;
					break;
				
				case FieldType.STRING:
				case FieldType.FILE_NAME:
				case FieldType.DESCRIPTION:
				default:
					text = TYPICAL_STRING_FIELD;
					break;
			}
			
			return measureText(text).width;
		}
		
		/**
		 *  Called when the sort order of the IAttachment fields change.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function updateSortState():void
		{
			sortStateChanged = true;
			invalidateProperties();
		}

		// IListItemRenderer(IDataRenderer) implementation
		
		/**
		 *  The DetailsField data object being rendered by this component, set through the public
		 *  data setter.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var detailsField:DetailsField = null;
		
		/**
		 *  The data object to be rendered. Only DetailsField objects are supported.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get data():Object
		{
			return detailsField;
		}
		
		/**
		 *  @private
		 */	
		public function set data(val:Object):void
		{
			if (val == detailsField)
				return;
			
			// Anything to clean up for the old field (before it changes?)
			if (detailsField) {
				
			}
			
			// New field value
			detailsField = val as DetailsField;
			
			detailsFieldChanged = true;
			invalidateProperties();
		}
		
		// IItemRenderer implementation
		/** Internal representation of the int referenced by the itemIndex getter and setter.
		 */
		protected var _itemIndex:int = -1;
		
		/** Internal representation of the string referenced by the label getter and setter. 
		 */
		protected var _label:String = ""; 
		
		/**
		 *  The index of the item in the data set 
		 *  that this renderer is visualizing.
		 */
		public function get itemIndex():int
		{
			return _itemIndex;
		}
		public function set itemIndex(val:int):void
		{
			if(_itemIndex != val)
				_itemIndex = val;
		}
		
		/** Internal representation of the Boolean referenced by the selected getter and setter. 
		 */
		protected var _selected:Boolean = false;
		
		/**  
		 * Contains <code>true</code> if the item renderer 
		 * can show itself as selected.
		 */
		[Bindable]
		public function get selected():Boolean
		{
			return _selected;
		}
		
		public function set selected(value:Boolean):void
		{
			if (value != _selected)
			{
				sortedField = value;
				if ( sortedField)
					onHeaderClicked( null);
				_selected = value;
				invalidateSkinState();
			}
		}
		
		
		/** Internal representation of the Boolean referenced by the showsCaret getter and setter. 
		 */
		protected var _showsCaret:Boolean = false;
		
		/**
		 *  Contains <code>true</code> if the item renderer 
		 *  can show itself as focused. 
		 */
		public function get showsCaret():Boolean
		{
			return _showsCaret;
		}
		public function set showsCaret(value:Boolean):void
		{
			if(value != _showsCaret)
				_showsCaret = value;
		}
		
		/**
		 *  True if the item renderer is being dragged. 
		 */
		public function get dragging():Boolean
		{
			return false;
		}
		public function set dragging(value:Boolean):void
		{
			
		}
		
		public function get label():String
		{
			return _label;
		}
		public function set label(val:String):void
		{
			if(_label != val)
				_label = val;
		}
		
	}
}
