/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.details
{
	import flash.geom.Rectangle;
	
	import mx.binding.utils.BindingUtils;
	import mx.core.ILayoutElement;
	
	import spark.layouts.HorizontalLayout;
	
	/**
	 *  The DetailsHeaderLayout class extends HorizontalLayout to implement the layout of the header row in the Details navigator. 
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */	
	public class DetailsHeaderLayout extends HorizontalLayout
	{
		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function DetailsHeaderLayout()
		{
			super();
		}
		
		private var _details:Details;

		/**
		 *  The main Details class.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function get details():Details
		{
			return _details;
		}

		/**
		 *  @private
		 */ 
		public function set details(value:Details):void
		{
			if (value == _details)
				return;
			_details = value;
			if (details)
				BindingUtils.bindSetter(detailsHorizontalScrollPositionChanged, details, "horizontalScrollPosition");
		}
		
		protected function detailsHorizontalScrollPositionChanged(value:Number):void
		{
			// Keep the header's horizontal position in sync with the horizontal position of the cards below it
			horizontalScrollPosition = details.horizontalScrollPosition;
		}
		
		/**
		 *  Override measure and adjust the targets measured width to match the total widths of the detail fields.
		 *  @see com.adobe.portfolio.components.details.Details#fieldColumnTotalWidth
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		override public function measure():void
		{
			super.measure();
			if ((details != null) && !isNaN(details.fieldColumnTotalWidth) && (details.fieldColumnTotalWidth > 0)) {
				target.measuredWidth = details.fieldColumnTotalWidth;
			}
		}
		
		/**
		 *  Update the display list
		 * 
		 *  @param width The new width of the target, in pixels, in the targets's coordinates.
		 *  @param height The new height of the target, in pixels, in the targets's coordinates.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function updateDisplayList(width:Number, height:Number):void
		{
			super.updateDisplayList(width, height);
			
			if (!target)
				return;
			
			var widths:Array = details ? details.getFieldColumnWidths() : null;
			var widthsValid:Boolean = widths && (widths.length == target.numElements);

			var thisWidth:Number;
			var xOffset:Number = Details.PREVIEW_ICON_WIDTH + 1;
			for (var index:int = 0; index < target.numElements; ++index) {
				// Note that classes derived from list are virtual and need to use getVirtualElementAt
				var layoutElement:ILayoutElement = getLayoutElementAt(index);
				if (!layoutElement || !(layoutElement is DetailsHeaderItem))
					continue;
				
				thisWidth = widthsValid ? widths[index] : 0;
				(layoutElement as DetailsHeaderItem).visible = thisWidth > 0;
				layoutElement.setLayoutBoundsSize(thisWidth, height);
				layoutElement.setLayoutBoundsPosition(xOffset, 0);
				xOffset += thisWidth;
			}
			target.setContentSize(xOffset-1, target.contentHeight);
		}
		
		/**
		 * 
		 *  Returns the specified element's layout bounds.
		 * 
		 *  @param index Index of the layout element.
		 *  @return The specified element's layout bounds.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function getElementBounds(elemIndex:int):Rectangle
		{
			// Call super and let it do some checks (which may end up returning a null rect)
			// or if the rect is valid we want super to set y and height.
			// Super also checks for indices that are out of range.
			var rect:Rectangle = super.getElementBounds(index);
			if (!rect || !details || !target)
				return rect;
			
			var widths:Array = details.getFieldColumnWidths();
			if (!widths || widths.length != target.numElements)
				return rect;
			
			var thisWidth:Number;
			var xOffset:Number = Details.PREVIEW_ICON_WIDTH + 1;
			for (var index:int = 0; index < target.numElements; ++index) {
				// Note that classes derived from list are virtual and need to use getVirtualElementAt
				var layoutElement:ILayoutElement = getLayoutElementAt(index);
				if (!layoutElement || !(layoutElement is DetailsHeaderItem))
					continue;
				
				thisWidth = widths[index];
				if (index == elemIndex)
					break;
				xOffset += thisWidth;
			}
			
			rect.x = xOffset;
			rect.width = thisWidth;
			return rect;
		}
		
		private function getLayoutElementAt(index:int):ILayoutElement
		{
			// Validate that the index is in range
			if (!target || index < 0 || index >= target.numElements)
				return null;
			// Note that classes derived from list are virtual and need to use getVirtualElementAt
			var layoutElement:ILayoutElement = useVirtualLayout ? target.getVirtualElementAt(index) : target.getElementAt(index);
			return layoutElement;
		}
	}
}