/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.details
{
	import com.adobe.portfolio.components.NavListDropIndicator;
	import com.adobe.portfolio.layouts.supportClasses.CardLayoutHelper;
	import com.adobe.portfolio.layouts.supportClasses.ICardLayout;
	
	import flash.display.DisplayObject;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import spark.layouts.VerticalLayout;
	
	/**
	 * 
	 *  The DetailsLayout class extends VerticalLayout to implement the layout of the Details navigator.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class DetailsLayout extends VerticalLayout implements ICardLayout
	{
		private var _cardLayoutHelper:CardLayoutHelper;
		private var _thumbFitSize:Point;
		private static const iBeam:Number = 2;

		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function DetailsLayout()
		{
			super();
			_cardLayoutHelper = new CardLayoutHelper(this);
		}

		private var _details:Details;
		
		/**
		 *  The main Details class.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get details():Details
		{
			return _details;
		}
		
		/**
		 *  @private
		 */ 
		public function set details(value:Details):void
		{
			if (value == _details)
				return;
			_details = value;
		}
		
		/**
		 *  Returns the instance of the CardLayoutHelper to use.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get cardLayoutHelper():CardLayoutHelper
		{
			return _cardLayoutHelper;
		}
		
		/**
		 *  Find the list of cards that overlap the specified rectangle. Always returns null for Details.
		 * 
		 *  @param rect A bounding region for which you want to find cards that intersect it.
		 *  @return Vector A vector of integers that represent the indexes of the cards in the layout that intersect
		 *  the specified rectangle.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function findCardsThatIntersect(rect:Rectangle):Vector.<int>
		{
			return null;
		}
		
		/**
		 *  The size to use for thumbnail images in the layout
		 * 
		 *  @param value The size, in pixels, for thumbnails to fit in the tile layout
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get thumbFitSize():Point
		{
			return _thumbFitSize;
		}
		
		/**
		 *  @private
		 */
		public function set thumbFitSize(value:Point):void
		{
			_thumbFitSize = value;
		}
		
		/**
		 *  Override measure and adjust the targets measured width to match the total widths of the detail fields.
		 *  @see com.adobe.portfolio.components.details.Details#fieldColumnTotalWidth
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		override public function measure():void
		{
			super.measure();
			if ((details != null) && !isNaN(details.fieldColumnTotalWidth) && (details.fieldColumnTotalWidth > 0)) {
				target.measuredWidth = details.fieldColumnTotalWidth;
			}
		}
		
		/**
		 *  The DisplayObject to use as the dropIndicator that provides visual feedback 
		 *  to the user of where the items will be inserted at the end of a drag and drop gesture.
		 * 
		 *  @param value An instance of DisplayObject to use as the dropIndicator
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function set dropIndicator(value:DisplayObject):void
		{
			super.dropIndicator = value;
			customizeDropIndicator(value);
		}

		/**
		 *  Customize the dropIndicator icon to look like an I-beam.
		 * 
		 *  @param dropIndicator The displayObject instance to use as the dropIndicator icon
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function customizeDropIndicator(dropIndicator:DisplayObject):void
		{
			if (!dropIndicator)
				return;
			
			var listDropIndicator:NavListDropIndicator = dropIndicator as NavListDropIndicator;
			listDropIndicator.getGraphicPosition = getDropIndicatorPosition;
			
			if (!typicalLayoutElement)
				return;
			
			var elementWidth:Number = typicalLayoutElement.getLayoutBoundsWidth();
			var elementHeight:Number = typicalLayoutElement.getLayoutBoundsHeight();
			listDropIndicator.setMaxWidth(elementWidth);
			listDropIndicator.setMaxHeight(elementHeight);	
		}
		
		/**
		 *  Given a location, width, and height for the dropIndicator icon,
		 *  this method returns the drawing rectangle for the dropIndicator icon.
		 * 
		 *  @param w The width, in pixels of the drop indicator icon
		 *  @param h The height, in pixels of the drop indicator icon
		 *  @param x The horizontal position of the drop indicator, in pixels
		 *  @param y The vertical position of the drop indicator, in pixels
		 *  @param end If this is the last element
		 *  @return A rectangle that is the bounding box of the dropIndicator icon as it will be drawn.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function getDropIndicatorPosition(w:Number, h:Number, x:Number, y:Number, end:Boolean):Rectangle
		{
			// If the gap is 0 or negative, the layout would size us as 0 width/height,
			// we need some minimum to ensure drawing.
			var iBeamWidth:Number = Math.max(iBeam, w);
			var iBeamHeight:Number = Math.max(iBeam, h);
			
			// Make the shorter side iBeam pixels so that the drop indicator is always 
			// a iBeam pixel thick line regardless of vertical/horizontal orientation.
			if (iBeamWidth < iBeamHeight)
				iBeamWidth = iBeam;
			else
				iBeamHeight = iBeam;
			
			// Center the drawing within the bounds
			var draw:Rectangle = new Rectangle;
			draw.x = iBeam;
			draw.y = Math.round(iBeamHeight/2);
			
			draw.width = iBeamWidth;
			draw.height = iBeamHeight;
			return draw;
		}
		
		/**
		 *  Updates the layout of the display list
		 * 
		 *  @param unscaledWidth The unscaled target width
		 *  @param unscaledHeight The unscaled target height
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			if (useVirtualLayout && (target != null)) {
				// We use the first items in the list to calculate column widths, so make sure
				// we keep them around
				var numCheckItems:Number = Math.min(Details.MAXIMUM_DATA_ITEMS_TO_CHECK, target.numElements);
				for (var i:Number = 0; i < numCheckItems; ++i) {
					target.getVirtualElementAt(i);
				}
			}
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			cardLayoutHelper.enableThumbnailGeneration();
			if (details != null) {
				var contentWidth:Number = details.fieldColumnTotalWidth + Details.PREVIEW_ICON_WIDTH;
				if (isNaN(contentWidth) || (contentWidth <= 0)) {
					return;
				}
				
				// This block will eliminate the horizontal scrollbar if possible, but results
				// in the edge of the list being clipped (see Watson bug 2686607).
				if ((contentWidth > unscaledWidth) &&
					(unscaledWidth > (details.fieldColumnMinWidth + Details.PREVIEW_ICON_WIDTH)))
				{
					contentWidth = unscaledWidth;
				}
				
				target.setContentSize(contentWidth, target.contentHeight);
			}
		}
	}
}
