/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import com.adobe.portfolio.api.FieldType;
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.api.IField;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.utils.FieldUtils;
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.globalization.Collator;
	import flash.globalization.LocaleID;
	
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.utils.ObjectUtil;

	/**
	 * The AttachmentFieldSort class extends Sort to provide a custom sort function for attachments.
	 * The attachments are sorted on their in-built fields. 
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class AttachmentFieldSort extends Sort
	{
		
		private var collator:Collator;

		/**
		 * Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function AttachmentFieldSort()
		{
			super();
			collator = new Collator(LocaleID.DEFAULT);
			collator.ignoreCase = true;
			compareFunction = acrobatCompareFunction;
		}
		
		/**
		 * @inheritDoc
		 */
		override public function findItem(items:Array,
								 values:Object,
								 mode:String,
								 returnInsertionIndex:Boolean = false,
								 compareFunction:Function = null):int
		{
			// Always pass in our compare function
			var rVal:int = super.findItem(items, values, mode, returnInsertionIndex, this.compareFunction);
			
			return rVal;
		}
		
		private function acrobatCompareFunction(a:IAttachment, b:IAttachment, callFields:Array = null):int
		{
			if(callFields == null)
			{
				if (a == b)
					return 0;
				callFields = fields;
			}
			
			if(callFields == null || callFields.length == 0)
			{
				return 0;
			}
			
			if(a == null && b == null)
				return 0;
			if(a == null)
				return 1;
			if(b == null)
				return -1;
			
			// Input is valid, now sort
			var result:int = 0;
			var i:int = 0;
			var len:int = callFields.length;
			
			while(result == 0 && i < len)
			{
				var sortField:SortField = callFields[i] as SortField;
				var fieldName:String = sortField ? sortField.name : callFields[i] as String;
				if(fieldName)
				{
					// Compare attachments
					result = compareAttachmentField(a, b, fieldName, sortField);
					if(sortField && sortField.descending)
						result *= -1;
				}
				++i;
			}
			
			// It's a tie, resort to name
			if (result == 0 && i == len)
				result = compareAttachmentField(a, b, FieldType.FILE_NAME, null);
			
			return result;
		}
		
		
		/**
		 * @param a
		 * @param b
		 * @param fieldName
		 * @param sortField
		 * @return 0 if a == b, 1 if b > a and -1 if a > b
		 * 
		 */
		private function compareAttachmentField(a:IAttachment, b:IAttachment, fieldName:String, sortField:SortField):int
		{
			var field:IField = FieldUtils.getFieldByFieldName(CollectionManager.instance.fields, fieldName);
			
			var aValue:Object = FieldUtils.getFieldOrPropertyValue(a, field, fieldName);
			var bValue:Object = FieldUtils.getFieldOrPropertyValue(b, field, fieldName);
			
			return compareFieldValueFunction(aValue, bValue, sortField ? sortField.caseInsensitive : true);
		}
		
		private function compareFieldValueFunction(a:Object, b:Object, caseInsensitive:Boolean):int
		{
			try
			{
				if(a == null && b == null)
					return 0;
				if(a == null)
					return 1;
				if(b == null)
					return -1;
				
				if(a is Number && b is Number)
					return ObjectUtil.numericCompare(a as Number, b as Number);
				if(a is String && b is String)
					return collator.compare(a as String, b as String);
				if(a is Date && b is Date)
					return ObjectUtil.dateCompare(a as Date, b as Date);
			}
			catch(e:Object)
			{
			}
			
			//Can't sort any other type.
			LogHelper.logger.error("Error:AcrobatSort.compareFieldValueFunction - input values of unknown or different types.");
			return 0;				
		}
		
	}
}
