/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import flash.utils.Dictionary;
	
	import mx.binding.utils.ChangeWatcher;

	/**
	 * The AutoWatch class both adds and tracks change watchers and provides the ability to remove all watchers with a single call.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class AutoWatch
	{
		private var watchers:Dictionary;

		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function AutoWatch()
		{
			watchers = new Dictionary(true);
		}
		
		/**
		 *  Adds a change watcher to the list that <code>removeAllWatchers</code> will reference when called.
		 * 
		 *  @param watcher The change watcher to add.
		 * 
		 *  @see ChangeWatcher
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function addWatcher(watcher:ChangeWatcher):void {
			if (!watcher)
				return;
			watchers[watcher] = watcher;
		}
		
		/**
		 *  Remove all watchers that were added through addWatcher.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function removeAllWatchers():void {
			for each (var watcher:ChangeWatcher in watchers) {
				watcher.unwatch();
			}
			watchers = new Dictionary();
		}
	}
}