/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.ui.Multitouch;
	
	import mx.core.IButton;
	import mx.core.UIComponent;
	
	import spark.components.supportClasses.ToggleButtonBase;
	
	/**
	 * The ButtonBuffer class provides a transparent component that captures clicks and propagates
	 * them through a separate button instance. This allows an enlarged area to act as a button
	 * click without affecting additional button states.
	 * 
	 */
	public class ButtonBuffer extends UIComponent
	{
		public static const DEFAULT_BUFFER_EXTENSION:Number = 10;
		
		/**
		 * @inheritDoc
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			// Draw a transparent rect for hit detection
			graphics.clear();
			graphics.beginFill(0, 0);
			graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			graphics.endFill();
			
			super.updateDisplayList(unscaledWidth, unscaledHeight);
		}
		
		private var _button:IEventDispatcher;
		/**
		 * The button that should dispatch a click event when this component is clicked.
		 */
		public function get button():IEventDispatcher
		{
			return _button;
		}
		public function set button(value:IEventDispatcher):void
		{
			if (value == _button) {
				return;
			}
			if (_button == null) {
				handleClickListener(!touchOnly || isTouchSupported);
			} else if (value == null) {
				handleClickListener(false);
			}
			_button = value;
		}
		
		/**
		 * Returns true if touch interaction is supported in the current environment; false otherwise.
		 */
		static protected function get isTouchSupported():Boolean
		{
			return Multitouch.supportsGestureEvents || Multitouch.supportsTouchEvents;
		}
		
		private var _touchOnly:Boolean = true;
		/**
		 * True if this component should only propagate click events in environments supporting
		 * touch interaction; false if this component should always propagate click events.
		 * 
		 * @see #isTouchSupported
		 */
		public function get touchOnly():Boolean
		{
			return _touchOnly;
		}
		public function set touchOnly(value:Boolean):void
		{
			if (value == _touchOnly) {
				return;
			}
			_touchOnly = value;
			
			if (button == null) {
				// If we don't even have a button yet, then no need for any action
				return;
			}
			handleClickListener(!value || isTouchSupported);
		}
		
		private var listeningForClick:Boolean;
		/**
		 * A helper function to add or remove the default click event listener.
		 * 
		 * @see #onBufferClick()
		 */
		protected function handleClickListener(shouldListen:Boolean):void
		{
			if (listeningForClick == shouldListen) {
				return;
			}
			if (shouldListen) {
				addEventListener(MouseEvent.CLICK, onBufferClick, false, 0, true);
			} else {
				removeEventListener(MouseEvent.CLICK, onBufferClick, false);
			}
			listeningForClick = shouldListen;
		}
		
		/**
		 * An event listener that responds to click events on this component and propagates
		 * them to <code>button</code>.
		 */
		protected function onBufferClick(event:MouseEvent):void
		{
			var toggleButton:ToggleButtonBase = button as ToggleButtonBase;
			if (toggleButton != null) {
				// Toggle-buttons handle mouse events at a lower-level, so simulate that on a click
				toggleButton.selected = !toggleButton.selected;
				toggleButton.dispatchEvent(new Event(Event.CHANGE));
			}
			
			button.dispatchEvent(event.clone());
		}
	}
}