/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import com.adobe.portfolio.api.ICollection;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.managers.PropertyManager;
	import com.adobe.portfolio.managers.ResourceManager;
	import com.adobe.portfolio.properties.CommonProperties;
	import com.adobe.portfolio.properties.PropertyContext;
	import com.adobe.portfolio.properties.PropertyValue;
	import com.adobe.portfolio.utils.LogHelper;
	import com.adobe.portfolio.utils.NavURLLoader;
	
	import flash.events.Event;
	import flash.events.FocusEvent;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.ui.Keyboard;
	
	import flashx.textLayout.elements.TextFlow;
	
	import mx.binding.utils.BindingUtils;
	import mx.binding.utils.ChangeWatcher;
	import mx.controls.Image;
	import mx.controls.SWFLoader;
	import mx.controls.listClasses.IListItemRenderer;
	import mx.core.IButton;
	import mx.events.FlexEvent;
	import mx.events.PropertyChangeEvent;
	import mx.managers.IFocusManagerComponent;
	import mx.utils.UIDUtil;
	
	import spark.components.IItemRenderer;
	import spark.components.TextArea;
	import spark.components.supportClasses.SkinnableComponent;
	import spark.events.TextOperationEvent;
	import spark.primitives.Rect;
	import spark.primitives.supportClasses.GraphicElement;
	import spark.utils.TextFlowUtil;
	
	/**
	 *  Dispatched when the user requests that this item be removed from its parent.
	 */
	[Event(name="removeItem", type="flash.events.Event")]

	[SkinState("imageNormal")]
	[SkinState("imageOver")]
	[SkinState("imageSelected")]
	[SkinState("imageOverAndSelected")]
	[SkinState("imageAuthoringNormal")]
	[SkinState("imageAuthoringOver")]
	[SkinState("imageAuthoringSelected")]
	[SkinState("imageAuthoringOverAndSelected")]
	[SkinState("textNormal")]
	[SkinState("textOver")]
	[SkinState("textSelected")]
	[SkinState("textOverAndSelected")]
	[SkinState("textAuthoringNormal")]
	[SkinState("textAuthoringOver")]
	[SkinState("textAuthoringSelected")]
	[SkinState("textAuthoringOverAndSelected")]
	[SkinState("swfNormal")]
	[SkinState("swfOver")]
	[SkinState("swfSelected")]
	[SkinState("swfOverAndSelected")]
	[SkinState("swfAuthoringNormal")]
	[SkinState("swfAuthoringOver")]
	[SkinState("swfAuthoringSelected")]
	[SkinState("swfAuthoringOverAndSelected")]
	/**
	 * An item renderer used to render CanvasDataItemData objects in a CanvasList.
	 */
	public class CanvasItem extends SkinnableComponent implements IFocusManagerComponent, IItemRenderer, IListItemRenderer, IListenAndWatch
	{
		
		/**
		 * A string literal representing the name of the normal image consumption state.
		 */
		public static const SKIN_STATE_IMAGE_NORMAL:String = "imageNormal";
		
		/**
		 * A string literal representing the name of the over image consumption state.
		 */
		public static const SKIN_STATE_IMAGE_OVER:String = "imageOver";
		
		/**
		 * A string literal representing the name of the selected image comsumption state.
		 */
		public static const SKIN_STATE_IMAGE_SELECTED:String = "imageSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected image consumption state.
		 */
		public static const SKIN_STATE_IMAGE_OVER_AND_SELECTED:String = "imageOverAndSelected";
		
		/**
		 * A string literal representing the name of the normal image authoring state.
		 */
		public static const SKIN_STATE_IMAGE_AUTHORING_NORMAL:String = "imageAuthoringNormal";
		
		/**
		 * A string literal representing the name of the over image authoring state.
		 */
		public static const SKIN_STATE_IMAGE_AUTHORING_OVER:String = "imageAuthoringOver";
		
		/**
		 * A string literal representing the name of the selected image authoring state.
		 */
		public static const SKIN_STATE_IMAGE_AUTHORING_SELECTED:String = "imageAuthoringSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected image authoring state.
		 */
		public static const SKIN_STATE_IMAGE_AUTHORING_OVER_AND_SELECTED:String = "imageAuthoringOverAndSelected";
		
		/**
		 * A string literal representing the name of the normal text consumption state.
		 */
		public static const SKIN_STATE_TEXT_NORMAL:String = "textNormal";
		
		/**
		 * A string literal representing the name of the over text consumption state.
		 */
		public static const SKIN_STATE_TEXT_OVER:String = "textOver";
		
		/**
		 * A string literal representing the name of the selected text consumption state.
		 */
		public static const SKIN_STATE_TEXT_SELECTED:String = "textSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected text consumption state.
		 */
		public static const SKIN_STATE_TEXT_OVER_AND_SELECTED:String = "textOverAndSelected";
		
		/**
		 * A string literal representing the name of the normal text authoring state.
		 */
		public static const SKIN_STATE_TEXT_AUTHORING_NORMAL:String = "textAuthoringNormal";
		
		/**
		 * A string literal representing the name of the over text authoring state.
		 */
		public static const SKIN_STATE_TEXT_AUTHORING_OVER:String = "textAuthoringOver";
		
		/**
		 * A string literal representing the name of the selected text authoring state.
		 */
		public static const SKIN_STATE_TEXT_AUTHORING_SELECTED:String = "textAuthoringSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected text authoring state.
		 */
		public static const SKIN_STATE_TEXT_AUTHORING_OVER_AND_SELECTED:String = "textAuthoringOverAndSelected";
		
		/**
		 * A string literal representing the name of the normal swf consumption state.
		 */
		public static const SKIN_STATE_SWF_NORMAL:String = "swfNormal";
		
		/**
		 * A string literal representing the name of the over swf consumption state.
		 */
		public static const SKIN_STATE_SWF_OVER:String = "swfOver";
		
		/**
		 * A string literal representing the name of the selected swf consumption state.
		 */
		public static const SKIN_STATE_SWF_SELECTED:String = "swfSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected swf consumption state.
		 */
		public static const SKIN_STATE_SWF_OVER_AND_SELECTED:String = "swfOverAndSelected";
		
		/**
		 * A string literal representing the name of the normal swf authoring state.
		 */
		public static const SKIN_STATE_SWF_AUTHORING_NORMAL:String = "swfAuthoringNormal";
		
		/**
		 * A string literal representing the name of the over swf authoring state.
		 */
		public static const SKIN_STATE_SWF_AUTHORING_OVER:String = "swfAuthoringOver";
		
		/**
		 * A string literal representing the name of the selected swf authoring state.
		 */
		public static const SKIN_STATE_SWF_AUTHORING_SELECTED:String = "swfAuthoringSelected";
		
		/**
		 * A string literal representing the name of the overAndSelected swf authoring state.
		 */
		public static const SKIN_STATE_SWF_AUTHORING_OVER_AND_SELECTED:String = "swfAuthoringOverAndSelected";
		
		/**
		 * A string literal representing the name of the text area skin part.
		 */
		public static const SKIN_PART_TEXT_AREA:String = "textArea";
		
		/**
		 * A string literal representing the name of the image component skin part.
		 */
		public static const SKIN_PART_IMAGE_COMPONENT:String = "imageComponent";
		
		/**
		 * A string literal representing the name of the swf component skin part.
		 */
		public static const SKIN_PART_SWF_COMPONENT:String = "swfComponent";
		
		/**
		 * A string literal representing the name of the set image button skin part.
		 */
		public static const SKIN_PART_SET_IMAGE_BUTTON:String = "setImageButton";
		
		/**
		 * A string literal representing the name of the handle move skin part.
		 */
		public static const SKIN_PART_HANDLE_MOVE:String = "handleMove";
		
		/**
		 * A string literal representing the name of the handle border skin part.
		 */
		public static const SKIN_PART_HANDLE_BORDER:String = "handleBorder";		

		/**
		 * A string literal representing the name of the top-left handle skin part.
		 */
		public static const SKIN_PART_HANDLE_TOP_LEFT:String = "handleTL";
		
		/**
		 * A string literal representing the name of the top-center handle skin part.
		 */
		public static const SKIN_PART_HANDLE_TOP_CENTER:String = "handleTC";
		
		/**
		 * A string literal representing the name of the top-right handle skin part.
		 */
		public static const SKIN_PART_HANDLE_TOP_RIGHT:String = "handleTR";
		
		/**
		 * A string literal representing the name of the middle-left handle skin part.
		 */
		public static const SKIN_PART_HANDLE_MIDDLE_LEFT:String = "handleML";
		
		/**
		 * A string literal representing the name of the middle-right handle skin part.
		 */
		public static const SKIN_PART_HANDLE_MIDDLE_RIGHT:String = "handleMR";
		
		/**
		 * A string literal representing the name of the bottom-left handle skin part.
		 */
		public static const SKIN_PART_HANDLE_BOTTOM_LEFT:String = "handleBL";
		
		/**
		 * A string literal representing the name of the bottom-center handle skin part.
		 */
		public static const SKIN_PART_HANDLE_BOTTOM_CENTER:String = "handleBC";
		
		/**
		 * A string literal representing the name of the bottom-right handle skin part.
		 */
		public static const SKIN_PART_HANDLE_BOTTOM_RIGHT:String = "handleBR";
		
		/**
		 * A string literal representing the name of the removeItem event.
		 */
		public static const EVENT_REMOVE:String = "removeItem";
		
		/**
		 * Default constructor
		 */
		public function CanvasItem()
		{
			super();
			addEventListener(FlexEvent.CREATION_COMPLETE,
				function (event:FlexEvent):void
				{
					removeEventListener(FlexEvent.CREATION_COMPLETE, arguments.callee);
					addListenAndWatch();
				}
			);
		}
		
		/**
		 * The navigator's CollectionManager instance. All host interaction should go through this
		 * instance, rather than the INavigatorHost directly.
		 */
		protected static const collectionManager:CollectionManager = CollectionManager.instance;
		
		[SkinPart(required="true")]
		/**
		 * The text area skin part used for text items.
		 */
		public var textArea:TextArea;
		
		[SkinPart(required="true")]
		/**
		 * The image skin part used for image items.
		 */
		public var imageComponent:Image;
		
		[SkinPart(required="true")]
		/**
		 * The swf skin part used for image items.
		 */
		public var swfComponent:SWFLoader;
		
		[SkinPart(required="false")]
		/**
		 * An optional button skin part used to set the source of the image component for image items.
		 */
		public var setImageButton:IButton;
		
		[SkinPart(required="false")]
		/**
		 * An optional handle border skin part for moving.
		 */
		public var handleBorder:Rect;
		
		[SkinPart(required="false")]
		/**
		 * An optional skin part representing a drag region for moving.
		 */
		public var handleMove:Rect;
		
		[SkinPart(required="false")]
		/**
		 * An optional top-left handle skin part for resizing.
		 */
		public var handleTL:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional top-center handle skin part for resizing.
		 */
		public var handleTC:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional top-right handle skin part for resizing.
		 */
		public var handleTR:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional middle-left handle skin part for resizing.
		 */
		public var handleML:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional middle-right handle skin part for resizing.
		 */
		public var handleMR:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional bottom-left handle skin part for resizing.
		 */
		public var handleBL:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional bottom-center handle skin part for resizing.
		 */
		public var handleBC:GraphicElement;

		[SkinPart(required="false")]
		/**
		 * An optional bottom-right handle skin part for resizing.
		 */
		public var handleBR:GraphicElement;
		
		private var draggingHandle:GraphicElement;
		private var draggingProperties:Object;
		private var draggingStagePoint:Point;
		
		private var _itemIndex:int;
		/**
		 * @inheritDoc
		 */
		public function get itemIndex():int
		{
			return _itemIndex;
		}
		/**
		 * @private
		 */
		public function set itemIndex(value:int):void
		{
			_itemIndex = value;
		}
		
		private var _dragging:Boolean = false;
		/**
		 * @inheritDoc
		 */
		public function get dragging():Boolean
		{
			return _dragging;
		}
		/**
		 * @private
		 */
		public function set dragging(value:Boolean):void
		{
			_dragging = value;
		}
		
		private var _label:String;
		/**
		 * @inheritDoc
		 */
		public function get label():String
		{
			// For now, just pass through a label if a client decides to set one
			return _label;
		}
		/**
		 * @private
		 */
		public function set label(value:String):void
		{
			_label = value;
		}
		
		private var _selected:Boolean = false;
		/**
		 * @inheritDoc
		 */
		public function get selected():Boolean
		{
			return _selected;
		}
		/**
		 * @private
		 */
		public function set selected(value:Boolean):void
		{
			_selected = value;
			invalidateSkinState();
		}
		
		private var _showsCaret:Boolean = false;
		/**
		 * @inheritDoc
		 */
		public function get showsCaret():Boolean
		{
			return _showsCaret;
		}
		/**
		 * @private
		 */
		public function set showsCaret(value:Boolean):void
		{
			_showsCaret = value;
		}
		
		private var canvasDataChanged:Boolean = false;
		private var ignoreStyledText:Boolean = false;
		
		private var _canvasData:CanvasItemData;
		/**
		 * The CanvasItemData object to be rendered.
		 */
		[Bindable]
		public function get canvasData():CanvasItemData
		{
			return _canvasData;
		}
		/**
		 * @private
		 */
		public function set canvasData(value:CanvasItemData):void
		{
			if (value != _canvasData) {
				if (_canvasData != null) {
					_canvasData.removeEventListener(PropertyChangeEvent.PROPERTY_CHANGE, onDataPropertyChanged);
				}
				_canvasData = value;
				if (_canvasData != null) {
					_canvasData.addEventListener(PropertyChangeEvent.PROPERTY_CHANGE, onDataPropertyChanged);
					canvasDataChanged = true;
					ignoreStyledText = false;
					invalidateProperties();
				}
				invalidateSkinState();
			}
		}
		
		/**
		 *  The data object to be renderered. Only CanvasItemData objects are supported.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get data():Object
		{
			return canvasData;
		}
		
		public function set data(value:Object):void
		{
			canvasData = value as CanvasItemData;
		}
		
		private static var _imageContext:PropertyContext;
		/**
		 * The property context used to represent the image properties for this item.
		 */
		protected function get imageContext():PropertyContext
		{
			if (_imageContext == null) {
				// Auto-generate the context for text fields
				_imageContext = new PropertyContext();
				_imageContext.loadFromXML(
					<PropertyContext labelKey="imageProperties" label="IMAGE_PROPERTIES" uid="com.adobe.navigator.image" global="false">
						<PropertyNumber name="imageOpacity" labelKey="imageOpacity" label="IMAGE_OPACITY" minValue="0" value="100" maxValue="100"/>
						<PropertyAction name="setImage" labelKey="setImage" label="SET_IMAGE"/>
						<PropertyList name="horizontalAnchor" labelKey="horizontalAnchor" label="HORIZONTAL_ANCHOR">
							<PropertyValue name="left" labelKey="positionMiddleLeft" label="LEFT"/>
							<PropertyValue name="center" labelKey="positionMiddleCenter" label="CENTER"/>
							<PropertyValue name="right" labelKey="positionMiddleRight" label="RIGHT"/>
						</PropertyList>
						<PropertyAction name="removeItem" labelKey="removeItem" label="REMOVE_ITEM"/>
					</PropertyContext>
				);
			}
			return _imageContext;
		}
		
		private static var _swfContext:PropertyContext;
		/**
		 * The property context used to represent the swf properties for this item.
		 */
		protected function get swfContext():PropertyContext
		{
			if (_swfContext == null) {
				// Auto-generate the context for text fields
				_swfContext = new PropertyContext();
				_swfContext.loadFromXML(
					<PropertyContext labelKey="swfProperties" label="SWF_PROPERTIES" uid="com.adobe.navigator.swf" global="false">
						<PropertyNumber name="swfOpacity" labelKey="swfOpacity" label="SWF_OPACITY" minValue="0" value="100" maxValue="100"/>
						<PropertyAction name="setSwf" labelKey="setSwf" label="SET_SWF"/>
						<PropertyList name="horizontalAnchor" labelKey="horizontalAnchor" label="HORIZONTAL_ANCHOR">
							<PropertyValue name="left" labelKey="positionMiddleLeft" label="LEFT"/>
							<PropertyValue name="center" labelKey="positionMiddleCenter" label="CENTER"/>
							<PropertyValue name="right" labelKey="positionMiddleRight" label="RIGHT"/>
						</PropertyList>
						<PropertyAction name="removeItem" labelKey="removeItem" label="REMOVE_ITEM"/>
					</PropertyContext>
				);
			}
			return _swfContext;
		}
		
		private static var _textContext:PropertyContext;
		/**
		 * The property context used to represent the text properties for this item.
		 */
		protected function get textContext():PropertyContext
		{
			if (_textContext == null) {
				// Auto-generate the context for text fields
				_textContext = new PropertyContext();
				_textContext.loadFromXML(
					<PropertyContext labelKey="textProperties" label="TEXT_PROPERTIES" uid="com.adobe.navigator.text" global="false">
						<PropertyList name="horizontalAnchor" labelKey="horizontalAnchor" label="HORIZONTAL_ANCHOR">
							<PropertyValue name="left" labelKey="positionMiddleLeft" label="LEFT"/>
							<PropertyValue name="center" labelKey="positionMiddleCenter" label="CENTER"/>
							<PropertyValue name="right" labelKey="positionMiddleRight" label="RIGHT"/>
						</PropertyList>
						<PropertyAction name="removeItem" labelKey="removeItem" label="REMOVE_ITEM"/>
					</PropertyContext>
				);
			}
			return _textContext;
		}
		
		[Bindable (name="propertyChange")]
		/**
		 * The opacity of the image component for this item.
		 */
		public function get imageOpacity():Number
		{
			return (_canvasData == null) ? 1.0 : _canvasData.imageOpacity;
		}
		
		private function setImageOpacity(value:Number):void
		{
			if (_canvasData != null) {
				// The imageOpacity property is between 0 and 100, not 0 and 1
				_canvasData.imageOpacity = (value / 100);
			}
		}
		
		[Bindable (name="propertyChange")]
		/**
		 * The opacity of the swf component for this item.
		 */
		public function get swfOpacity():Number
		{
			return (_canvasData == null) ? 1.0 : _canvasData.swfOpacity;
		}
		
		private function setSwfOpacity(value:Number):void
		{
			if (_canvasData != null) {
				// The imageOpacity property is between 0 and 100, not 0 and 1
				_canvasData.swfOpacity = (value / 100);
			}
		}
		
		private function setImageAction(value:uint):void
		{
			if (value == 0) {
				return;
			}
			onSetImageClick(null);
		}
		
		private function setSwfAction(value:uint):void
		{
			if (value == 0) {
				return;
			}
			onSetSwfClick(null);
		}
		
		private function getHorizontalAnchor():String
		{
			if (_canvasData == null) {
				return "left";
			}
			if (!isNaN(_canvasData.left)) {
				return "left";
			}
			if (!isNaN(_canvasData.right)) {
				return "right";
			}
			if (!isNaN(_canvasData.horizontalCenter)) {
				return "center";
			}
			return "left";
		}
		
		private function setHorizontalAnchor(value:PropertyValue):void
		{
			if ((value == null) || (_canvasData == null) || (parent == null)) {
				return;
			}
			if (getHorizontalAnchor() == value.name) {
				return;
			}
			var bounds:Rectangle = getBounds(parent);
			if (value.name == "left") {
				if (isNaN(_canvasData.left) && (isNaN(_canvasData.x))) {
					_canvasData.left = bounds.left;
				}
				_canvasData.horizontalCenter = NaN;
				_canvasData.right = NaN;
			} else if (value.name == "right") {
				if (isNaN(_canvasData.right)) {
					
					_canvasData.right = parent.width - bounds.right;
				}
				_canvasData.horizontalCenter = NaN;
				_canvasData.left = NaN;
			} else if (value.name == "center") {
				if (isNaN(_canvasData.horizontalCenter)) {
					
					_canvasData.horizontalCenter = ((bounds.left + bounds.right) - parent.width) / 2;
				}
				_canvasData.left = NaN;
				_canvasData.right = NaN;
			}
			if (isNaN(_canvasData.width)) {
				_canvasData.width = bounds.width;
			}
		}
		
		private function removeItemAction(value:uint):void
		{
			if (value != 0) {
				dispatchEvent(new Event(EVENT_REMOVE));
			}
		}
		
		private var authoringModeWatcher:ChangeWatcher;
		private var imageOpacityWatcher:ChangeWatcher;
		private var imageSetWatcher:ChangeWatcher;
		private var swfOpacityWatcher:ChangeWatcher;
		private var swfSetWatcher:ChangeWatcher;
		private var anchorWatcher:ChangeWatcher;
		private var removeWatcher:ChangeWatcher;
		private var mouseIsOver:Boolean = false;
		private var takeFocusOnCreation:Boolean = false;
		
		private var didAddListenAndWatch:Boolean = false;
		/**
		 * A function to add event listeners and change watchers required by this item.
		 */
		public function addListenAndWatch():void
		{
			if (didAddListenAndWatch) {
				CONFIG::logging{
					LogHelper.logger.warn("Listeners are already setup");
				}
				return;
			}
			authoringModeWatcher = BindingUtils.bindSetter(setAuthoringMode, collectionManager, "authoringMode");
			addEventListener(MouseEvent.ROLL_OVER, onMouseRoll, false, 0, true);
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown, false, 0, true);
			addEventListener(FocusEvent.FOCUS_IN, onFocusInOrOut, false, 0, true);
			addEventListener(FocusEvent.FOCUS_OUT, onFocusInOrOut, false, 0, true);
			addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown, false, 0, true);
			
			if (takeFocusOnCreation && (focusManager != null)) {
				takeFocusOnCreation = false;
				focusManager.setFocus(this);
			}
			didAddListenAndWatch = true;
		}
		
		/**
		 * A function to clean up event listeners and change watchers required by this item.
		 */
		public function removeListenAndWatch():void
		{
			if (!didAddListenAndWatch) {
				CONFIG::logging{
					LogHelper.logger.warn("Listeners were never setup");
				}
				return;
			}
			if (authoringModeWatcher != null) {
				authoringModeWatcher.unwatch();
			}
			removeEventListener(MouseEvent.ROLL_OVER, onMouseRoll);
			removeEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			removeEventListener(FocusEvent.FOCUS_IN, onFocusInOrOut);
			removeEventListener(FocusEvent.FOCUS_OUT, onFocusInOrOut);
			removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			didAddListenAndWatch = false;
			data = null;
		}
		
		private function onKeyDown(event:KeyboardEvent):void
		{
			if (event.keyCode != Keyboard.DELETE) {
				return;
			}
			if (event.altKey || event.ctrlKey || event.shiftKey) {
				return;
			}
			if (!collectionManager.authoringMode || !collectionManager.permissionCheck("Document", "Modify")) {
				return;
			}
			if ((textArea != null) && textArea.enabled) {
				// If the textArea is enabled, then the user is actively editing text, so don't delete the item
				return;
			}
			
			// Do the same thing as the delete button; any non-zero param should do it
			removeItemAction(1)
			
			if (event.cancelable) {
				event.preventDefault();
			}
			if (event.bubbles) {
				event.stopImmediatePropagation();
			}
		}
		
		private function onFocusInOrOut(event:FocusEvent):void
		{
			if (!collectionManager.authoringMode) {
				// We're only concerned with item focus in authoring mode
				return;
			}
			if (event.type == FocusEvent.FOCUS_IN) {
				takeContext();
			} else if ((focusManager != null) && (focusManager.getFocus() == null) && (stage != null)) {
				// If nothing else actually got focus, then wait until something does
				stage.addEventListener(FocusEvent.FOCUS_IN, onStageFocusIn, false, 0, true);
			} else {
				// We lost focus, so clear the current property context
				loseContext();
			}
		}
		
		private function onStageFocusIn(event:FocusEvent):void
		{
			var currentFocus:IFocusManagerComponent = (focusManager == null) ? null : focusManager.getFocus();
			if (currentFocus == null) {
				return;
			}
			if (event.currentTarget is IEventDispatcher) {
				(event.currentTarget as IEventDispatcher).removeEventListener(FocusEvent.FOCUS_IN, onStageFocusIn);
			}
			if ((currentFocus != this) && (currentFocus != skin)) {
				loseContext();
			}
		}
		
		/**
		 * Gives focus to this item.
		 */
		public function takeFocus():void
		{
			if (!didAddListenAndWatch) {
				takeFocusOnCreation = true;
				return;
			}
			if (focusManager == null) {
				return;
			}
			focusManager.setFocus(this);
		}
		
		/**
		 * Sets the property context for this item as the current property context.
		 * 
		 * @see PropertyContext
		 * @see PropertyManager#setCurrentContext
		 */
		public function takeContext():Boolean
		{
			if (PropertyManager.instance.commonProperties == null) {
				return false;
			}
			if (removeWatcher != null) {
				// We've already taken context;
				return true;
			}
			var canvasProperty:PropertyValue;
			var subvalue:PropertyValue;
			var currentAnchor:String = getHorizontalAnchor();
			if (_canvasData.type == CanvasItemData.IMAGE_TYPE) {
				if (imageContext == null) {
					return false;
				}
				canvasProperty = imageContext.getProperty("imageOpacity");
				if (canvasProperty != null) {
					canvasProperty.value = _canvasData.imageOpacity * 100;
					imageOpacityWatcher = BindingUtils.bindSetter(setImageOpacity, canvasProperty, "numValue");
				}
				canvasProperty = imageContext.getProperty("setImage");
				if (canvasProperty != null) {
					canvasProperty.value = 0;
					imageSetWatcher = BindingUtils.bindSetter(setImageAction, canvasProperty, "value");
				}
				canvasProperty = imageContext.getProperty("horizontalAnchor");
				if (canvasProperty != null) {
					for each (subvalue in canvasProperty.subvalues) {
						if ((subvalue != null) && (subvalue.name == currentAnchor)) {
							canvasProperty.value = subvalue;
							break;
						}
					}
					anchorWatcher = BindingUtils.bindSetter(setHorizontalAnchor, canvasProperty, "value");
				}
				canvasProperty = imageContext.getProperty("removeItem");
				if (canvasProperty != null) {
					canvasProperty.value = 0;
					removeWatcher = BindingUtils.bindSetter(removeItemAction, canvasProperty, "value");
				}
				PropertyManager.instance.commonProperties.setCurrentContext(imageContext, this);
				return true;
			}
			if (_canvasData.type == CanvasItemData.SWF_TYPE) {
				if (swfContext == null) {
					return false;
				}
				canvasProperty = swfContext.getProperty("swfOpacity");
				if (canvasProperty != null) {
					canvasProperty.value = _canvasData.swfOpacity * 100;
					swfOpacityWatcher = BindingUtils.bindSetter(setSwfOpacity, canvasProperty, "numValue");
				}
				canvasProperty = swfContext.getProperty("setSwf");
				if (canvasProperty != null) {
					canvasProperty.value = 0;
					swfSetWatcher = BindingUtils.bindSetter(setSwfAction, canvasProperty, "value");
				}
				canvasProperty = swfContext.getProperty("horizontalAnchor");
				if (canvasProperty != null) {
					for each (subvalue in canvasProperty.subvalues) {
						if ((subvalue != null) && (subvalue.name == currentAnchor)) {
							canvasProperty.value = subvalue;
							break;
						}
					}
					anchorWatcher = BindingUtils.bindSetter(setHorizontalAnchor, canvasProperty, "value");
				}
				canvasProperty = swfContext.getProperty("removeItem");
				if (canvasProperty != null) {
					canvasProperty.value = 0;
					removeWatcher = BindingUtils.bindSetter(removeItemAction, canvasProperty, "value");
				}
				PropertyManager.instance.commonProperties.setCurrentContext(swfContext, this);
				return true;
			}
			if (_canvasData.type == CanvasItemData.TEXT_TYPE) {
				if (textContext == null) {
					return false;
				}
				if ((textArea != null) && textArea.enabled) {
					// If the textArea is enabled, then the user is actively editing text, so don't use the item context
					return false;
				}
				canvasProperty = textContext.getProperty("horizontalAnchor");
				if (canvasProperty != null) {
					for each (subvalue in canvasProperty.subvalues) {
						if ((subvalue != null) && (subvalue.name == currentAnchor)) {
							canvasProperty.value = subvalue;
							break;
						}
					}
					anchorWatcher = BindingUtils.bindSetter(setHorizontalAnchor, canvasProperty, "value");
				}
				canvasProperty = textContext.getProperty("removeItem");
				if (canvasProperty != null) {
					canvasProperty.value = 0;
					removeWatcher = BindingUtils.bindSetter(removeItemAction, canvasProperty, "value");
				}
				PropertyManager.instance.commonProperties.setCurrentContext(textContext, this);
				return true;
			}
			return false;
		}
		
		/**
		 * Clears the property context for this item as the current property context.
		 * 
		 * @see PropertyContext
		 * @see PropertyManager#setCurrentContext
		 */
		public function loseContext():Boolean
		{
			if ((textArea != null) &&
				(focusManager != null) &&
				(focusManager.getFocus() != textArea))
			{
				textArea.enabled = !collectionManager.authoringMode;
			}
			if (imageOpacityWatcher != null) {
				imageOpacityWatcher.unwatch();
				imageOpacityWatcher = null;
			}
			if (imageSetWatcher != null) {
				imageSetWatcher.unwatch();
				imageSetWatcher = null;
			}
			if (swfOpacityWatcher != null) {
				swfOpacityWatcher.unwatch();
				swfOpacityWatcher = null;
			}
			if (swfSetWatcher != null) {
				swfSetWatcher.unwatch();
				swfSetWatcher = null;
			}
			if (anchorWatcher != null) {
				anchorWatcher.unwatch();
				anchorWatcher = null;
			}
			if (removeWatcher != null) {
				removeWatcher.unwatch();
				removeWatcher = null;
			}
			if (PropertyManager.instance.commonProperties != null) {
				PropertyManager.instance.commonProperties.setCurrentContextByName(CommonProperties.CONTEXT_CLEAR, this);
			}
			return true;
		}
		
		private var imageLoader:NavURLLoader;
		private var imageBinding:ChangeWatcher;
		private function releaseImageBinding():void
		{
			if (imageBinding == null) {
				return;
			}
			imageBinding.unwatch();
			imageBinding = null;
		}
		
		private var swfLoader:NavURLLoader;
		private var swfBinding:ChangeWatcher;
		private function releaseSwfBinding():void
		{
			if (swfBinding == null) {
				return;
			}
			swfBinding.unwatch();
			swfBinding = null;
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function commitProperties():void
		{
			if (canvasDataChanged) {
				if (_canvasData != null) {
					visible = enabled = _canvasData.visible;
					_canvasData.applyPositionAndSize(this);
					if (!ignoreStyledText && textArea != null) {
						if (_canvasData.styledText != null) {
							textArea.textFlow = TextFlowUtil.importFromXML(_canvasData.styledText);
						} else {
							textArea.textFlow = new TextFlow();
						}
					}
					if ((imageComponent != null) && (_canvasData.imageURL != null) && (_canvasData.imageURL != "")) {
						if (imageLoader == null) {
							imageLoader = new NavURLLoader(_canvasData.imageURL, true);
						} else {
							imageLoader.url = _canvasData.imageURL;
						}
						if (imageBinding == null) {
							imageBinding = BindingUtils.bindSetter(setImageSource, imageLoader, "dataBytes");
						}
					}
					if ((swfComponent != null) && (_canvasData.swfURL != null) && (_canvasData.swfURL != "")) {
						if (swfLoader == null) {
							swfLoader = new NavURLLoader(_canvasData.swfURL, true);
						} else {
							swfLoader.url = _canvasData.swfURL;
						}
						if (swfBinding == null) {
							swfBinding = BindingUtils.bindSetter(setSwfSource, swfLoader, "dataBytes");
						}
					}
				} else {
					visible = enabled = false;
					if (textArea != null) {
						textArea.textFlow = new TextFlow();
					}
					releaseImageBinding();
					if (imageComponent != null) {
						// TODO blank out image?
					}
				}
				canvasDataChanged = false;
				ignoreStyledText = false;
			}
			
			// Because the above code manipulates the positional properties of this item, it's
			// important that commitProperties be called after those changes have been made.
			super.commitProperties();
		}
		private function setImageSource(value:Object):void
		{
			if (imageComponent == null) {
				CONFIG::logging{
					LogHelper.logger.warn("setImageSource invoked with a null imageComponent");
				}
				releaseImageBinding();
				return;
			}
			imageComponent.source = value;
		}

		private function setSwfSource(value:Object):void
		{
			if (swfComponent == null) {
				CONFIG::logging{
					LogHelper.logger.warn("setSwfSource invoked with a null swfComponent");
				}
				releaseSwfBinding();
				return;
			}
			swfComponent.source = playSWF ? value : null;
		}
		
		private var _playSWF:Boolean = true;
		/**
		 * True if SWF content within this component should be played; false otherwise.
		 */
		public function get playSWF():Boolean
		{
			return _playSWF;
		}
		
		public function set playSWF(value:Boolean):void
		{
			if (value == _playSWF) {
				return;
			}
			_playSWF = value;
			
			if (swfComponent == null) {
				return;
			}
			if (value) {
				if ((swfComponent.source == null) &&
					(swfLoader != null) ||
					(swfLoader.dataBytes != null) ||
					(swfLoader.dataBytes.length > 0))
				{
					swfComponent.source = swfLoader.dataBytes;
				}
			} else {
				swfComponent.unloadAndStop();
				swfComponent.source = null;
			}
		}

		/**
		 * A function invoked when authoring mode changes.
		 */
		protected function setAuthoringMode(value:Boolean):void
		{
			// TODO Turn off selection
			
			// Trigger a non-editing state
			if (textArea != null) {
				textArea.enabled = !value;
			}
			invalidateSkinState();
		}
		
		/**
		 * Event listener for rollover events on this item.
		 */
		protected function onMouseRoll(event:MouseEvent):void
		{
			mouseIsOver = (event.type == MouseEvent.ROLL_OVER);
			if (collectionManager.authoringMode) {
				// We don't care about rollover unless we're in authoring mode
				invalidateSkinState();
			}
		}
		
		/**
		 * Event listener for mouseDown events on this item.
		 */
		protected function onMouseDown(event:MouseEvent):void
		{
			if ((stage == null) || !collectionManager.authoringMode) {
				return;
			}
			setClickedHandle(event);
			if (draggingHandle != null) {
				stage.addEventListener(MouseEvent.MOUSE_UP, onMouseHandles, false, 0, true);
				stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseHandles, false, 0, true);
			}
		}
		
		/**
		 * Event listener for double-click events on this item.
		 */
		protected function onMouseDoubleClick(event:MouseEvent):void
		{
			if (!collectionManager.authoringMode) {
				// We're only concerned with item focus in authoring mode
				return;
			}
			if (textArea != null) {
				textArea.enabled = true;
				invalidateSkinState();
				if (focusManager != null) {
					focusManager.setFocus(textArea);
				}
			}
		}
		
		private function onMouseHandles(event:MouseEvent):void
		{
			var property:String;
			if (event.type == MouseEvent.MOUSE_UP) {
				if (draggingProperties != null) {
					for (property in draggingProperties) {
						if ((property == "minX") || (property == "maxX") ||
							(property == "minY") || (property == "maxY"))
						{
							continue;
						}
						canvasData[property] = this[property];
					}
					draggingProperties = null;
				}
				draggingHandle = null;
				stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseHandles);
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseHandles);
				if (canvasData.parent != null) {
					canvasData.parent.setCustomTemplate();
				}
			} else if (event.type == MouseEvent.MOUSE_MOVE) {
				var xDiff:Number = (event.stageX - draggingStagePoint.x);
				var yDiff:Number = (event.stageY - draggingStagePoint.y);
				if (!isNaN(draggingProperties.minX)) {
					xDiff = Math.max(xDiff, draggingProperties.minX);
				}
				if (!isNaN(draggingProperties.maxX)) {
					xDiff = Math.min(xDiff, draggingProperties.maxX);
				}
				if (!isNaN(draggingProperties.minY)) {
					yDiff = Math.max(yDiff, draggingProperties.minY);
				}
				if (!isNaN(draggingProperties.maxY)) {
					yDiff = Math.min(yDiff, draggingProperties.maxY);
				}
				for (property in draggingProperties) {
					var factor:Number = draggingProperties[property];
					switch (property) {
						case "left":
						case "right":
						case "x":
						case "width":
						case "horizontalCenter":
							this[property] = canvasData[property] + (factor * xDiff);
							break;
						case "top":
						case "bottom":
						case "y":
						case "height":
						case "verticalCenter":
							this[property] = canvasData[property] + (factor * yDiff);
							break;
					}
				}
			}
		}
		
		/**
		 * Returns true if the given point is contained in the bounds of the given handle.
		 */
		protected function handleContainsPoint(handle:GraphicElement, point:Point):Boolean
		{
			if ((handle == null) || (point == null)) {
				return false;
			}
			if ((point.x >= handle.x) && (point.x <= (handle.x + handle.width))) {
				if ((point.y >= handle.y) && (point.y <= (handle.y + handle.height))) {
					return true;
				}
			}
			return false;
		}
		
		/**
		 * setClickedHandle determines which (if any) drag handle is under the provided event,
		 * and sets up the current dragging properties to manipulate this object based on the result.
		 */
		private function setClickedHandle(event:MouseEvent):void
		{
			draggingHandle = null;
			draggingProperties = null;
			draggingStagePoint = new Point(event.stageX, event.stageY);
			var globalOrigin:Point = localToGlobal(new Point(0, 0));
			var localPoint:Point = new Point(event.stageX - globalOrigin.x, event.stageY - globalOrigin.y);
			
			var draggingLeft:Boolean = false;
			var draggingRight:Boolean = false;
			var draggingTop:Boolean = false;
			var draggingBottom:Boolean = false;
			var draggingMove:Boolean = false;
			
			if ((draggingHandle == null) && handleContainsPoint(handleTL, localPoint)) {
				draggingHandle = handleTL;
				draggingTop = true;
				draggingLeft = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleTC, localPoint)) {
				draggingHandle = handleTC;
				draggingTop = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleTR, localPoint)) {
				draggingHandle = handleTR;
				draggingTop = true;
				draggingRight = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleML, localPoint)) {
				draggingHandle = handleML;
				draggingLeft = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleMR, localPoint)) {
				draggingHandle = handleMR;
				draggingRight = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleBL, localPoint)) {
				draggingHandle = handleBL;
				draggingBottom = true;
				draggingLeft = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleBC, localPoint)) {
				draggingHandle = handleBC;
				draggingBottom = true;
			}
			if ((draggingHandle == null) && handleContainsPoint(handleBR, localPoint)) {
				draggingHandle = handleBR;
				draggingBottom = true;
				draggingRight = true;
			}
			if ((draggingHandle == null) && (handleBorder != null)) {
				// Check to see if we're on the border by checking the distance from the left, right,
				// top, or bottom of this item. If the stroke SkinPart has been defined, use its weight;
				// otherwise, pick some relatively small tolerance.
				var tolerance:Number = (handleBorder.stroke != null) ? handleBorder.stroke.weight : 3;
				var leftX:Number = (handleBorder.left is Number) ? handleBorder.left as Number : 0;
				var rightX:Number = (handleBorder.width - ((handleBorder.right is Number) ? handleBorder.right as Number : 0));
				var topY:Number = (handleBorder.top is Number) ? handleBorder.top as Number : 0;
				var bottomY:Number = (handleBorder.height - ((handleBorder.bottom is Number) ? handleBorder.bottom as Number : 0));
				if ((Math.abs(localPoint.x - leftX) <= tolerance) ||
					(Math.abs(localPoint.x - rightX) <= tolerance) ||
					(Math.abs(localPoint.y - topY) <= tolerance) ||
					(Math.abs(localPoint.y - bottomY) <= tolerance))
				{
					draggingHandle = handleBorder;
					draggingMove = true;
				}
			}
			if (draggingHandle == null) {
				if (handleMove == null) {
					return;
				}
				if ((textArea != null) && textArea.enabled) {
					// Don't use handleMove if the user is editing text
					return;
				}
				draggingHandle = handleMove;
				draggingMove = true;
			}
			
			if (draggingMove) {
				setupDraggingMove();
			} else {
				setupDraggingHandle(draggingLeft, draggingRight, draggingTop, draggingBottom);
			}
		}
		
		private function setupDraggingHandle(draggingLeft:Boolean, draggingRight:Boolean, draggingTop:Boolean, draggingBottom:Boolean):void
		{
			draggingProperties = new Object();
			if (draggingLeft) {
				// Keep width >= minWidth
				draggingProperties.maxX = width - minWidth;
				// Keep us from going off the left edge
				draggingProperties.minX = -x;
				
				if (isNaN(canvasData.left)) {
					draggingProperties.width = -1;
					if (!isNaN(canvasData.horizontalCenter)) {
						draggingProperties.horizontalCenter = 0.5;
					} else if (isNaN(canvasData.right)) {
						draggingProperties.x = 1;
					}
				} else {
					draggingProperties.left = 1;
					if (isNaN(canvasData.right)) {
						draggingProperties.width = -1;
					}
				}
			} else if (draggingRight) {
				// Keep width >= minWidth
				draggingProperties.minX = minWidth - width;
				// Keep us from going off the right edge
				draggingProperties.maxX = (parent.width - width - x);
				
				if (isNaN(canvasData.right)) {
					draggingProperties.width = 1;
					if (!isNaN(canvasData.horizontalCenter)) {
						draggingProperties.horizontalCenter = 0.5;
					}
				} else {
					draggingProperties.right = -1;
					if (isNaN(canvasData.left)) {
						draggingProperties.width = 1;
					}
				}
			}
			
			if (draggingTop) {
				// Keep height >= minHeight
				draggingProperties.maxY = height - minHeight;
				// Keep us from going off the top edge
				draggingProperties.minY = -y;
			
				if (isNaN(canvasData.top)) {
					draggingProperties.height = -1;
					if (isNaN(canvasData.bottom)) {
						draggingProperties.y = 1;
					}
				} else {
					draggingProperties.top = 1;
					if (isNaN(canvasData.bottom)) {
						draggingProperties.height = -1;
					}
				}
			} else if (draggingBottom) {
				// Keep height >= minHeight
				draggingProperties.minY = minHeight - height;
				// Keep us from going off the bottom edge
				draggingProperties.maxY = (parent.height - height - y);
				
				if (isNaN(canvasData.bottom)) {
					draggingProperties.height = 1;
				} else {
					draggingProperties.bottom = -1;
					if (isNaN(canvasData.top)) {
						draggingProperties.height = 1;
					}
				}
			}
		}
		
		private function setupDraggingMove():void
		{
			draggingProperties = new Object();
			
			draggingProperties.minX = -x;
			draggingProperties.maxX = parent.width - width - x;
			draggingProperties.minY = -y;
			draggingProperties.maxY = parent.height - height - y;
			
			if (isNaN(canvasData.left) && isNaN(canvasData.right) && isNaN(canvasData.horizontalCenter)) {
				draggingProperties.x = 1;
			} else if (!isNaN(canvasData.horizontalCenter)) {
				draggingProperties.horizontalCenter = 1;
			} else if (isNaN(canvasData.right)) {
				draggingProperties.left = 1;
			} else if (isNaN(canvasData.left)) {
				draggingProperties.right = -1;
			} else {
				draggingProperties.left = 1;
				draggingProperties.right = -1;
			}
			if (isNaN(canvasData.top) && isNaN(canvasData.bottom) && isNaN(canvasData.verticalCenter)) {
				draggingProperties.y = 1;
			} else if (!isNaN(canvasData.verticalCenter)) {
				draggingProperties.verticalCenter = 1;
			} else if (isNaN(canvasData.bottom)) {
				draggingProperties.top = 1;
			} else if (isNaN(canvasData.top)) {
				draggingProperties.bottom = -1;
			} else {
				draggingProperties.top = 1;
				draggingProperties.bottom = -1;
			}
		}
		
		/**
		 * Event listener called when this item's data changes.
		 */
		protected function onDataPropertyChanged(event:PropertyChangeEvent):void
		{
			if (event.property == "imageOpacity") {
				dispatchEvent(PropertyChangeEvent.createUpdateEvent(this, "imageOpacity", event.oldValue, event.newValue));
			} else if (event.property == "swfOpacity") {
				dispatchEvent(PropertyChangeEvent.createUpdateEvent(this, "swfOpacity", event.oldValue, event.newValue));
			}
			if (canvasDataChanged) {
				// If canvasDataChanged is already true, then we've already called invalidateProperties()
				// and we're waiting for commitProperties to be called (so just return).
				return;
			}
			canvasDataChanged = true;
			invalidateProperties();
		}
		
		/**
		 * Event listener for text change events on the text area skin part.
		 */
		protected function onTextChange(event:TextOperationEvent):void
		{
			if ((textArea != null) && (_canvasData != null) && (_canvasData.type == CanvasItemData.TEXT_TYPE)) {
				ignoreStyledText = true;
				if (textArea.textFlow)
					_canvasData.styledText = TextFlowUtil.export(textArea.textFlow);
				else
					_canvasData.styledText = null;
			}
		}
		
		/**
		 * Event listener for click events on the image component skin part.
		 */
		protected function onSetImageClick(event:MouseEvent):void
		{
			if ((_canvasData == null) || !collectionManager.permissionCheck("Document", "Modify")) {
				return;
			}
			
			var hostCollection:ICollection = (collectionManager.host != null) ? collectionManager.host.collection : null;
			var titleString:String = (hostCollection == null) ? "CANVAS_SELECT_IMAGE_TITLE" : hostCollection.getLocalizedString("canvasSelectImageTitle", "CANVAS_SELECT_IMAGE_TITLE");
			ResourceManager.instance.addResource(
				titleString,
				[ ResourceManager.instance.imageFilter ],
				"navigator/canvas_images/"+UIDUtil.createUID(),
				function (response:Object):void
				{
					var originalURL:String = response["originalURL"];
					if (!response["success"] as Boolean) {
						LogHelper.logger.error("addResource failed: "+originalURL);
						return;
					}
					var relativeURL:String = response["relativeURL"];
					
					// We need to dereference the old imageURL resource now that we're not using
					// it anymore, and reference the new one so that it doesn't get purged.
					ResourceManager.instance.dereferenceResource(_canvasData.imageURL, true);
					_canvasData.imageURL = relativeURL;
					ResourceManager.instance.referenceResource(_canvasData.imageURL);
				}
			);
		}
		
		/**
		 * Event listener for click events on the swf component skin part.
		 */
		protected function onSetSwfClick(event:MouseEvent):void
		{
			if ((_canvasData == null) || !collectionManager.permissionCheck("Document", "Modify")) {
				return;
			}
			
			var hostCollection:ICollection = (collectionManager.host != null) ? collectionManager.host.collection : null;
			var titleString:String = (hostCollection == null) ? "CANVAS_SELECT_SWF_TITLE" : hostCollection.getLocalizedString("canvasSelectSwfTitle", "CANVAS_SELECT_SWF_TITLE");
			ResourceManager.instance.addResource(
				titleString,
				[ ResourceManager.instance.swfFilter ],
				"navigator/canvas_swfs/"+UIDUtil.createUID(),
				function (response:Object):void
				{
					var originalURL:String = response["originalURL"];
					if (!response["success"] as Boolean) {
						LogHelper.logger.error("addResource failed: "+originalURL);
						return;
					}
					var relativeURL:String = response["relativeURL"];
					
					// We need to dereference the old imageURL resource now that we're not using
					// it anymore, and reference the new one so that it doesn't get purged.
					ResourceManager.instance.dereferenceResource(_canvasData.swfURL, true);
					_canvasData.swfURL = relativeURL;
					ResourceManager.instance.referenceResource(_canvasData.swfURL);
				}
			);
		}

		/**
		 * @inheritDoc
		 */
		override protected function getCurrentSkinState():String
		{
			if ((_canvasData == null) || (_canvasData.type == CanvasItemData.TEXT_TYPE)) {
				// text item
				if (collectionManager.authoringMode) {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_TEXT_AUTHORING_OVER_AND_SELECTED : SKIN_STATE_TEXT_AUTHORING_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_TEXT_AUTHORING_OVER : SKIN_STATE_TEXT_AUTHORING_NORMAL;
					}
				} else {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_TEXT_OVER_AND_SELECTED : SKIN_STATE_TEXT_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_TEXT_OVER : SKIN_STATE_TEXT_NORMAL;
					}
				}
			} else if (_canvasData.type == CanvasItemData.IMAGE_TYPE) {
				// image item
				if (collectionManager.authoringMode) {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_IMAGE_AUTHORING_OVER_AND_SELECTED : SKIN_STATE_IMAGE_AUTHORING_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_IMAGE_AUTHORING_OVER : SKIN_STATE_IMAGE_AUTHORING_NORMAL;
					}
				} else {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_IMAGE_OVER_AND_SELECTED : SKIN_STATE_IMAGE_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_IMAGE_OVER : SKIN_STATE_IMAGE_NORMAL;
					}
				}
			} else if (_canvasData.type == CanvasItemData.SWF_TYPE) {
				// image item
				if (collectionManager.authoringMode) {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_SWF_AUTHORING_OVER_AND_SELECTED : SKIN_STATE_SWF_AUTHORING_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_SWF_AUTHORING_OVER : SKIN_STATE_SWF_AUTHORING_NORMAL;
					}
				} else {
					if (selected) {
						return mouseIsOver ? SKIN_STATE_SWF_OVER_AND_SELECTED : SKIN_STATE_SWF_SELECTED;
					} else {
						return mouseIsOver ? SKIN_STATE_SWF_OVER : SKIN_STATE_SWF_NORMAL;
					}
				}
			} else {
				CONFIG::logging{
					LogHelper.logger.warn("Unknown item type");
				}
				return SKIN_STATE_TEXT_NORMAL;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partAdded
			handlePartChange(partName, instance, true);
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			//handlePartChange will call super.partRemoved
			handlePartChange(partName, instance, false);
		}
		
		/**
		 * @inheritDoc
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if ((instance != null) && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case SKIN_PART_TEXT_AREA:
						if (adding && (canvasData != null)) {
							canvasDataChanged = true;
							invalidateProperties();
						}
						types = [ TextOperationEvent.CHANGE ];
						listeners = [ onTextChange ];
						
						if (adding) {
							// In authoring mode, disable the textArea until the user double-clicks on the item
							textArea.enabled = !collectionManager.authoringMode;
							doubleClickEnabled = true;
							addEventListener(MouseEvent.DOUBLE_CLICK, onMouseDoubleClick, false, 0, true);
						} else {
							removeEventListener(MouseEvent.DOUBLE_CLICK, onMouseDoubleClick);
						}
						break;
					case SKIN_PART_IMAGE_COMPONENT:
					case SKIN_PART_SWF_COMPONENT:
						if (adding && (canvasData != null)) {
							canvasDataChanged = true;
							invalidateProperties();
						}
						break;
					case SKIN_PART_SET_IMAGE_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ onSetImageClick ];
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = (listeners == null) ? 0 : listeners.length;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
	}
}
