/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import flash.events.EventDispatcher;
	
	import mx.core.IVisualElement;

	/**
	 * A class representing text or image data to be rendered by a CanvasItem
	 * in a CanvasList.
	 * 
	 * @see CanvasItem
	 * @see CanvasList
	 */
	public class CanvasItemData extends EventDispatcher
	{
		/**
		 * A string literal representing the element name used in the XML
		 * representation of a CanvasList.
		 */
		public static const XML_ELEMENT_NAME:String = "CanvasItem";

		/**
		 * A string literal representing a text data type.
		 */
		public static const TEXT_TYPE:String = "text";
		
		/**
		 * A string literal representing an image data type.
		 */
		public static const IMAGE_TYPE:String = "image";
		
		/**
		 * A string literal representing an swf data type.
		 */
		public static const SWF_TYPE:String = "swf";
		
		private var loadingFromXML:Boolean = false;
		
		/**
		 * Default constructor
		 */
		public function CanvasItemData(parent:CanvasList = null)
		{
			super();
			_parent = parent as CanvasList;
		}
		
		private var _parent:CanvasList;
		/**
		 * This CanvasList containing the CanvasItem rendering this data.
		 */
		public function get parent():CanvasList
		{
			return _parent;
		}

		/**
		 * The type of this canvas item data.
		 * 
		 * @see #TEXT_TYPE
		 * @see #IMAGE_TYPE
		 */
		[Bindable]
		public var type:String;
		
		[Bindable]
		/**
		 * True if the item rendering this data should be visible.
		 */
		public var visible:Boolean = true;
		
		private var _left:Number;
		/**
		 * @copy IVisualElement#left
		 */
		[Bindable]
		public function get left():Number
		{
			return _left;
		}
		/**
		 * @private
		 */
		public function set left(value:Number):void
		{
			_left = value;
			dirtyParent()
		}
		
		private var _right:Number;
		/**
		 * @copy IVisualElement#right
		 */
		[Bindable]
		public function get right():Number
		{
			return _right;
		}
		/**
		 * @private
		 */
		public function set right(value:Number):void
		{
			_right = value;
			dirtyParent()
		}
		
		private var _top:Number;
		/**
		 * @copy IVisualElement#top
		 */
		[Bindable]
		public function get top():Number
		{
			return _top;
		}
		/**
		 * @private
		 */
		public function set top(value:Number):void
		{
			_top = value;
			dirtyParent()
		}
		
		private var _bottom:Number;
		/**
		 * @copy IVisualElement#bottom
		 */
		[Bindable]
		public function get bottom():Number
		{
			return _bottom;
		}
		/**
		 * @private
		 */
		public function set bottom(value:Number):void
		{
			_bottom = value;
			dirtyParent()
		}

		private var _horizontalCenter:Number;
		/**
		 * @copy IVisualElement#horizontalCenter
		 */
		[Bindable]
		public function get horizontalCenter():Number
		{
			return _horizontalCenter;
		}
		/**
		 * @private
		 */
		public function set horizontalCenter(value:Number):void
		{
			_horizontalCenter = value;
			dirtyParent();
		}
		
		private var _verticalCenter:Number;
		/**
		 * @copy IVisualElement#verticalCenter
		 */

		[Bindable]
		public function get verticalCenter():Number
		{
			return _verticalCenter;
		}
		/**
		 * @private
		 */
		public function set verticalCenter(value:Number):void
		{
			_verticalCenter = value;
			dirtyParent();
		}

		private var _x:Number;
		/**
		 * @copy IVisualElement#x
		 */
		[Bindable]
		public function get x():Number
		{
			return _x;
		}
		/**
		 * @private
		 */
		public function set x(value:Number):void
		{
			_x = value;
			dirtyParent()
		}
		
		private var _y:Number;
		[Bindable]
		/**
		 * @copy IVisualElement#y
		 */
		public function get y():Number
		{
			return _y;
		}
		/**
		 * @private
		 */
		public function set y(value:Number):void
		{
			_y = value;
			dirtyParent()
		}
		
		private var _width:Number;
		/**
		 * The desired width of the item rendering this data.
		 */
		[Bindable]
		public function get width():Number
		{
			return _width;
		}
		/**
		 * @private
		 */
		public function set width(value:Number):void
		{
			_width = value;
			dirtyParent()
		}

		private var _height:Number;
		/**
		 * The desired heigth of the item rendering this data.
		 */
		[Bindable]
		public function get height():Number
		{
			return _height;
		}
		/**
		 * @private
		 */
		public function set height(value:Number):void
		{
			_height = value;
			dirtyParent()
		}
		
		private var _imageURL:String;
		/**
		 * The url of the image to be used by the item rendering this data.
		 */
		[Bindable]
		public function get imageURL():String
		{
			return _imageURL;
		}
		/**
		 * @private
		 */
		public function set imageURL(value:String):void
		{
			_imageURL = value;
			dirtyParent()
		}
		
		private var _swfURL:String;
		/**
		 * The url of the swf to be used by the item rendering this data.
		 */
		[Bindable]
		public function get swfURL():String
		{
			return _swfURL;
		}
		/**
		 * @private
		 */
		public function set swfURL(value:String):void
		{
			_swfURL = value;
			dirtyParent()
		}
		
		private var _imageOpacity:Number = 1.0;
		/**
		 * The opacity of the image to be used by the item rendering this data.
		 */
		[Bindable]
		public function get imageOpacity():Number
		{
			return _imageOpacity;
		}
		public function set imageOpacity(value:Number):void
		{
			_imageOpacity = value;
			dirtyParent();
		}

		private var _swfOpacity:Number = 1.0;
		/**
		 * The opacity of the swf to be used by the item rendering this data.
		 */
		[Bindable]
		public function get swfOpacity():Number
		{
			return _swfOpacity;
		}
		public function set swfOpacity(value:Number):void
		{
			_swfOpacity = value;
			dirtyParent();
		}
		
		private var _styledText:XML;
		/**
		 * The styled text to be used by the item rendering this data.
		 */
		[Bindable]
		public function get styledText():XML
		{
			return _styledText;
		}
		public function set styledText(value:XML):void
		{
			_styledText = value;
			dirtyParent()
		}
		
		private function dirtyParent():void
		{
			if (!loadingFromXML && (_parent != null)) {
				_parent.dirty = true;
			}
		}
		
		/**
		 * A function to copy the layout information from another CanvasItemData into this object.
		 */
		public function setValuesFromData(itemData:CanvasItemData):Boolean
		{
			if (itemData.type != type) {
				return false;
			}
			visible = itemData.visible;
			left = itemData.left;
			right = itemData.right;
			top = itemData.top;
			bottom = itemData.bottom;
			horizontalCenter = itemData.horizontalCenter;
			verticalCenter = itemData.verticalCenter;
			x = itemData.x;
			y = itemData.y;
			width = itemData.width;
			height = itemData.height;
			
			return true;
		}
		
		/**
		 * A function to copy the layout information this object into a visual element.
		 */
		public function applyPositionAndSize(visual:IVisualElement):void
		{
			if (visual == null) {
				return;
			}
			visual.x = x;
			visual.y = y;
			visual.left = left;
			visual.right = right;
			visual.top = top;
			visual.bottom = bottom;
			visual.horizontalCenter = horizontalCenter;
			visual.verticalCenter = verticalCenter;
			visual.width = width;
			visual.height = height;
		}
		
		/**
		 * A function to populate this data based on an XML representation.
		 * 
		 * @see #generateXML
		 */
		public function loadFromXML(valueXML:XML):Boolean
		{
			loadingFromXML = true;
			
			type = valueXML.@type;
			var numberString:String = valueXML.@opacity;
			if (type == IMAGE_TYPE) {
				imageURL = valueXML.@url;
				imageOpacity = (numberString == "") ? 1.0 : parseFloat(numberString);
			} else if (type == SWF_TYPE) {
				swfURL = valueXML.@url;
				swfOpacity = (numberString == "") ? 1.0 : parseFloat(numberString);
			}
			
			var elements:XMLList = valueXML.elements();
			for each (var element:XML in elements) {
				var localName:Object = element.localName();
				if (localName as String == "TextFlow") {
					styledText = element;
					break;
				}
			}
			var booleanString:String = valueXML.@visible;
			visible = (booleanString.toLowerCase() != "false");
			
			numberString = valueXML.@x;
			x = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@y;
			y = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@width;
			width = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@height;
			height = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@left;
			left = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@right;
			right = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@top;
			top = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@bottom;
			bottom = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@horizontalCenter;
			horizontalCenter = (numberString == "") ? Number.NaN : parseFloat(numberString);
			numberString = valueXML.@verticalCenter;
			verticalCenter = (numberString == "") ? Number.NaN : parseFloat(numberString);
			
			loadingFromXML = false;
			
			return true;
		}
		
		/**
		 * Creates an XML representation of this data.
		 * 
		 * @see #loadFromXML
		 */
		public function generateXML():XML
		{
			var newXML:XML = new XML("<"+XML_ELEMENT_NAME+"/>");
			
			newXML.@type = type;
			if (type == IMAGE_TYPE) {
				newXML.@url = imageURL;
				if (!isNaN(imageOpacity)) {
					newXML.@opacity = imageOpacity;
				}
			} else if (type == SWF_TYPE) {
				newXML.@url = swfURL;
				if (!isNaN(swfOpacity)) {
					newXML.@opacity = swfOpacity;
				}
			}
			
			newXML.@visible = visible;

			if (!isNaN(x))
				newXML.@x = x;
			if (!isNaN(y))
				newXML.@y = y;
			if (!isNaN(width))
				newXML.@width = width;
			if (!isNaN(height))
				newXML.@height = height;
			if (!isNaN(left))
				newXML.@left = left;
			if (!isNaN(right))
				newXML.@right = right;
			if (!isNaN(top))
				newXML.@top = top;
			if (!isNaN(bottom))
				newXML.@bottom = bottom;
			if (!isNaN(horizontalCenter))
				newXML.@horizontalCenter = horizontalCenter;
			if (!isNaN(verticalCenter))
				newXML.@verticalCenter = verticalCenter;
			
			if ((styledText != null) && (String(styledText.localName()) == "TextFlow")) {
				newXML.appendChild(styledText);
			}
			
			return newXML;
		}

	}
}