/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import spark.components.supportClasses.SkinnableComponent;
	
	/**
	 * A component for drawing rectangular and elliptical shapes with gradient fills
	 * and strokes, as well as potential rotation and an inset stroke.
	 */
	public class CustomShape extends SkinnableComponent
	{
		
		// Variables used by the gradient parts of the background skin
		/**
		 * The first color used by the skin's background fill gradient.
		 */
		[Bindable]
		public var backgroundColorDark:uint;
		
		/**
		 * The second color used by the skin's background fill gradient.
		 */
		[Bindable]
		public var backgroundColorLight:uint;
		
		/**
		 * The first color used by the skin's background stroke gradient.
		 */
		[Bindable]
		public var borderColorDark:uint;
		
		/**
		 * The second color used by the skin's background stroke gradient.
		 */
		[Bindable]
		public var borderColorlight:uint;
		
		/**
		 * The color use by the skin's inset stroke. The inset stroke appears
		 * between the background fill and stroke in the z-order of the skin.
		 */
		[Bindable]
		public var insetStrokeColor:uint;
		
		/**
		 * True if the skin's inset stroke should be visible; false otherwise.
		 */
		[Bindable]
		public var insetStrokeColorVisible:Boolean;
		
		/**
		 * The stroke weight of the skin's background stroke gradient.
		 */
		[Bindable]
		public var strokeWeight:uint;
		
		/**
		 * The corner radius of a rectangular custom shape.
		 * 
		 * @see #isRectangle
		 */
		[Bindable]
		public var cornerRadius:uint;
		
		/**
		 * The rotation of the skin's background fill gradient.
		 */
		[Bindable]
		public var backgroundRotation:int = 0;
		
		/**
		 * The rotation of the skin's background stroke gradient.
		 */
		[Bindable]
		public var borderRotation:int = 0;
		
		/**
		 * The opacity of the skin's background fill gradient.
		 */
		[Bindable]
		public var backgroundAlpha:Number = 1;
		
		/**
		 * The opacity of the skin's background stroke gradient.
		 */
		[Bindable]
		public var borderAlpha:Number = 1;
		
		/**
		 * True if the data in this shape should be used to draw a rectangle; false otherwise.
		 */
		[Bindable]
		public var isRectangle:Boolean = false;
		
		/**
		 * True if the data in this shape should be used to draw an ellipse; false otherwise.
		 */
		[Bindable]
		public var isEllipse:Boolean = false;
		
		/**
		 * Default constructor.
		 */
		public function CustomShape()
		{
			super();
		}
		
		/**
		 * @inheritDoc
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void
		{
			super.setCurrentState(stateName, playTransition);
			
			// We used to call skin.currentState = stateName; here, but simply invalidating
			// plays nicer with the Flex model.
			invalidateSkinState();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function getCurrentSkinState():String
		{
			return currentState;
		}
		
	}
}