/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import com.adobe.portfolio.managers.CollectionManager;
	
	import mx.binding.utils.BindingUtils;
	
	import spark.components.Button;
	
	/**
	 * A button whose skin incorporates an icon. The use of the properties within this class
	 * are highly dependent on the skin implementation, and may change from skin to skin.
	 */
	public class IconButton extends Button
	{
		/**
		 * True when the button is in authoring mode; false otherwise.
		 */
		[Bindable]
		public var authoringMode:Boolean;
		
		/**
		 * An icon image that is appropriate to display in authoring mode.
		 */
		[Bindable]
		public var editIcon:Class;
		
		/**
		 * An icon image that is appropriate to display in consumption mode.
		 */
		[Bindable]
		public var defaultIcon:Class;
		
		/**
		 * An icon image that is appropriate to display while the cursor is over the button.
		 */
		[Bindable]
		public var overIcon:Class;
		
		/**
		 * An icon image that is appropriate to display while the button is disabled.
		 */
		[Bindable]
		public var disabledIcon:Class;

		/**
		 * An appropriate base color for the button.
		 */
		[Bindable]
		public var baseColor:uint;
		
		/**
		 * An appropriate alternate color for the button.
		 */
		[Bindable]
		public var altColor:uint;
		
		/**
		 * An appropriate degree of rotation for the displayed icon.
		 */
		[Bindable]
		public var iconRotation:int;
		
		/**
		 * Default constructor.
		 */
		public function IconButton()
		{
			super();
			BindingUtils.bindProperty(this, "authoringMode", CollectionManager.instance, "authoringMode");
		}
	}
}