/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import com.adobe.portfolio.api.FieldType;
	import com.adobe.portfolio.api.IField;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.utils.FieldUtils;
	
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.binding.utils.ChangeWatcher;
	import mx.core.IDataRenderer;
	
	import spark.components.supportClasses.SkinnableComponent;
	
	[Style(name="color", type="uint", format="Color", inherit="yes")]

	[SkinState("disabled")]
	[SkinState("normal")]
	
	/**
	 * A component used to display a standard set of fields for an IAttachment.
	 */
	public class InfoFields extends SkinnableComponent implements IDataRenderer
	{
		private static var fieldsWatcher:ChangeWatcher;
		private static const DEFAULT_TIMER_DELAY:Number = 1000;
		
		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the filename of an attachment.
		 */
		public static var fileNameField:IField;
		
		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the size of an attachment.
		 */
		public static var sizeField:IField;

		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the creation date of an attachment.
		 */
		public static var creationDateField:IField;

		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the modification date of an attachment.
		 */
		public static var modifiedDateField:IField;

		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the description of an attachment.
		 */
		public static var descriptionField:IField;

		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the display name of an attachment.
		 */
		public static var displayNameField:IField;

		[Bindable]
		/**
		 * The field object from the CollectionManager's fields array
		 * representing the tags of an attachment.
		 */
		public static var tagsField:IField;
		
		/**
		 * Default constructor.
		 */
		public function InfoFields()
		{
			super();
			if (fieldsWatcher == null) {
				fieldsWatcher = BindingUtils.bindSetter(onFieldsSet, CollectionManager.instance, "fields", false, true);
			}
		}
		
		private var _data:Object;
		
		[Bindable]
		/**
		 *  @inheritDoc
		 */
		public function get data():Object
		{
			return _data;
		}
		
		/**
		 * @private
		 */
		public function set data(value:Object):void
		{
			_data = value;
		}
		
		private var _enableFields:Boolean;
		
		/**
		 * True if the field controls for this component should be enabled;
		 * false otherwise. Application performance may be impacted when field
		 * controls are enabled.
		 */
		public function get enableFields():Boolean
		{
			return _enableFields;
		}
		/**
		 * @private
		 */
		public function set enableFields(value:Boolean):void
		{
			if (value == _enableFields) {
				return;
			}
			_enableFields = value;
			invalidateSkinState();
		}
		
		private var delayTimer:Timer;
		/**
		 * Sets enableFields to true after the specified delay, or reasonable
		 * default delay if <code>delay</code> is less than or equal to zero.
		 */
		public function delayEnableFields(delay:Number = 0):void
		{
			if (delayTimer != null) {
				return;
			}
			if (delay <= 0) {
				delay = DEFAULT_TIMER_DELAY;
			}
			delayTimer = new Timer(delay, 1);
			delayTimer.addEventListener(TimerEvent.TIMER,
				function (event:TimerEvent):void
				{
					enableFields = true;
					delayTimer.removeEventListener(TimerEvent.TIMER, arguments.callee);
					delayTimer.stop();
					delayTimer = null;
				});
			delayTimer.start()
		}
		
		/**
		 *  @inheritDoc
		 */
		override protected function getCurrentSkinState():String
		{
			return enableFields ? "normal" : "disabled";
		}
		
		private static function onFieldsSet(fields:Array):void
		{
			for each (var field:IField in fields) {
				if (field == null) {
					continue;
				}
				if (field.name == FieldUtils.DISPLAY_NAME_FIELD_NAME) {
					displayNameField = field;
				} else if (field.name == FieldUtils.TAGS_FIELD_NAME) {
					tagsField = field;
				} else if (field.type == FieldType.FILE_NAME) {
					fileNameField = field;
				} else if (field.type == FieldType.SIZE) {
					sizeField = field;
				} else if (field.type == FieldType.CREATION_DATE) {
					creationDateField = field;
				} else if (field.type == FieldType.MOD_DATE) {
					modifiedDateField = field;
				} else if (field.type == FieldType.DESCRIPTION) {
					descriptionField = field;
				}
			}
		}
		
	}
}