/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2010 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.components.supportClasses
{
	import com.adobe.portfolio.api.ICollection;
	import com.adobe.portfolio.api.SessionDataChangedEvent;
	import com.adobe.portfolio.events.CollectionManagerEvent;


	/**
	 * A text component whose value is loaded and stored in a portfolio's session data,
	 * similar to a cookie.
	 * 
	 * @see ICollection#getSessionData
	 * @see ICollection#getSessionData
	 */
	public class SessionTextArea extends CustomizableTextArea
	{
		/**
		 * Default constructor.
		 */
		public function SessionTextArea()
		{
			super();
			
			// We don't currently recover/store styled text
			allowStyling = false;
			
			// We don't currently support multiline session text
			multiline = false;

			hostCollection = (collectionManager.host == null) ? null : collectionManager.host.collection;
		}
		
		private var _sessionKey:String;
		
		[Bindable]
		/**
		 * The key used to load and store session data for this component.
		 */
		public function get sessionKey():String
		{
			return _sessionKey;
		}
		/**
		 * @private
		 */
		public function set sessionKey(value:String):void
		{
			_sessionKey = value;
			setTextFromData();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function shouldBeEditable():Boolean
		{
			// SessionData doesn't dirty the document, so assume it can always be edited
			return true;
		}
		
		private var _hostCollection:ICollection;
		private function get hostCollection():ICollection
		{
			return _hostCollection;
		}
		private function set hostCollection(value:ICollection):void
		{
			if (value == _hostCollection) {
				return;
			}
			if (_hostCollection != null) {
				_hostCollection.removeEventListener("sessionDataChanged", onSessionDataChanged);
			}
			_hostCollection = value;
			if (_hostCollection != null) {
				_hostCollection.addEventListener("sessionDataChanged", onSessionDataChanged);
			}
			setTextFromData();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function onNavHostEvent(event:CollectionManagerEvent):void
		{
			super.onNavHostEvent(event);
			if (event.kind == CollectionManagerEvent.HOST_CHANGED_KIND) {
				hostCollection = (collectionManager.host == null) ? null : collectionManager.host.collection;
			}
		}
		
		/**
		 * An event listener that is called when session data for a portfolio is changed.
		 * 
		 * @see ICollection#setSessionData
		 * @see SessionDataChangedEvent
		 */
		protected function onSessionDataChanged(event:SessionDataChangedEvent):void
		{
			if ((event == null) ||
				pendingChanges ||
				(event.key != sessionKey) ||
				(event.sessionData == text))
			{
				return;
			}
			setTextFromData();
		}

		/**
		 * Updates the current session value based on the displayed text.
		 * 
		 * @inheritDoc
		 */
		override protected function setDataFromText():void
		{
			if (!pendingChanges)
				return;
			
			if ((sessionKey != null) && (sessionKey != "") && (hostCollection != null)) {
				hostCollection.setSessionData(sessionKey, text);
			}
		}
		
		/**
		 * Updates the displayed text based on the current session value.
		 * 
		 * @inheritDoc
		 */
		override protected function setTextFromData():void
		{
			if ((sessionKey != null) && (sessionKey != "") && (hostCollection != null)) {
				hostCollection.getSessionData(sessionKey,
					function (key:String, sessionData:*):void
					{
						if (sessionData != null) {
							text = sessionData;
						} else {
							text = "";
						}
					});
			} else {
				text = "";
			}
		}
	}
}