/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.events
{
	import com.adobe.portfolio.managers.CollectionManager;
	
	import flash.events.Event;
	
	
	/**
	 *  The CollectionManagerEvent class extends Event and is dispatched in response to changes to the Navigator host or its collection of attachments.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class CollectionManagerEvent extends Event
	{
		/**
		 *  A string representing the event type when a navigator host has changed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const HOST_CHANGED_KIND:String = "hostChanged";
		
		/**
		 *  A string representing the event type when the host informs the navigator that
		 *  one or more items will change.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const ITEMS_WILL_CHANGE_KIND:String = "itemsWillChange";
		/**
		 *  A string representing the event type when the host informs the navigator that
		 *  one or more items have changed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const ITEMS_DID_CHANGE_KIND:String = "itemsDidChange";
		
		/**
		 *  A string representing the event type when the host informs the navigator that
		 *  the selection has changed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const SELECTION_DID_CHANGE_KIND:String = "selectionDidChange";
		
		/**
		 *  A string representing the event type when the current focus attachment has changed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const FOCUS_DID_CHANGE_KIND:String = "focusDidChange";
		
		/**
		 *  A string representing the event type when a skin part has been removed from
		 * 	a navigator component.
		 * 
		 * 	@see CollectionManager#dispatchPartRemoved
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PART_REMOVED_KIND:String = "partRemoved";
		
		/**
		 *  A string representing the event type when the host informs the navigator
		 *  that an attachment has changed and needs to be saved.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const NEEDS_SAVE_KIND:String = "needsSave";
		
		/**
		 *  A string representing the event type when the host informs the navigator
		 *  that permissions have changed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const PERMISSIONS_CHANGED_KIND:String = "permissionsChanged";
		
		/**
		 *  A string representing the event type when the the navigator should present
		 * 	a modal preview mode.
		 * 
		 * 	@see CollectionManager#dispatchNavigatorPreview
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const NAVIGATOR_PREVIEW_KIND:String = "navigatorPreview";
		
		/**
		 *  An event type used to indicate the initialization of the root items queue in CollectionManager
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const ROOT_ITEMS_QUEUE_BEGIN_KIND:String = "rootItemsQueueBegin";
		
		/**
		 *  An event type used to indicate the termination of the root items queue in CollectionManager
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const ROOT_ITEMS_QUEUE_END_KIND:String = "rootItemsQueueEnd";
		
		/**
		 *  A string representing the kind of events triggered by the the Navigator host
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const COLLECTION_CHANGE:String = "collectionChange";
		public var oldVal:Object = null;
		public var newVal:Object = null;
		public var kind:String = "";
		
		
		/**
		 *  Constructor
		 * 
		 *  @param kind The kind of event as a string, usually one of values in <code>CollectionManagerEvent</code>.
		 *  @param oldVal The old value of the relevant changed object
		 *  @param newVal The new value of the relevant changed object
		 *  @param bubbles Determines whether the Event object participates
		 *  in the bubbling stage of the event flow. The default value is <code>false</code>.
		 *  @param cancelable Determines whether the Event object can be canceled. 
		 *  The default values is <code>false</code>.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function CollectionManagerEvent(kind:String, oldVal:Object = null, newVal:Object = null, bubbles:Boolean=false, cancelable:Boolean=false)
		{
			super(COLLECTION_CHANGE, bubbles, cancelable);
			this.kind = kind;
			this.oldVal = oldVal;
			this.newVal = newVal;
		}
	}
}