/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.events
{
	import flash.events.Event;
	
	/**
	 *  The MarqueeSelectionEvent class extends Event and is dispatched by the MarqueeSelection class during a marquee drag gesture.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class MarqueeSelectionEvent extends Event
	{
		
		/**
		 *  Type of MarqueeSelectionEvent this is dispatched at the beginning of a marquee drag.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const BEGIN:String = "marqueeSelectionBegin";
		
		/**
		 *  Type of MarqueeSelectionEvent this is dispatched during a marquee drag when the mouse moves.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const MOVE:String = "marqueeSelectionMove";
		
		/**
		 *  Type of MarqueeSelectionEvent this is dispatched at the end of a marquee drag.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const END:String = "marqueeSelectionEnd";
		
		/**
		 *  Type of MarqueeSelectionEvent when a marquee drag is cancelled (i.e. a single click instead of a drag).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const CANCELLED:String = "marqueeSelectionCancelled";
		
		
		/**
		 *  x location of marquee (top left corner)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var x:Number;

		/**
		 *  y location of marquee (top left corner)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var y:Number;

		/**
		 *  width of marquee in pixels
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var width:Number;

		/**
		 *  height of marquee in pixels
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var height:Number;

		/**
		 *  True if control key is down
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var ctrlKey:Boolean;

		/**
		 *  True if shift key is down
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var shiftKey:Boolean;
		
		/**
		 *  Constructor
		 * 
		 *  @param type Type of event.
		 *  @param bubbles Whether event bubbles. 
		 *  @param cancelable Whether event is cancelable 
		 * 
		 *  @see flash.events.Event
		 *  @see #BEGIN
		 *  @see #MOVE
		 *  @see #END
		 *  @see #CANCELLED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function MarqueeSelectionEvent(type:String, bubbles:Boolean=false, cancelable:Boolean=false)
		{
			super(type, bubbles, cancelable);
		}
	}
}