/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.events
{
	import flash.events.Event;
	
	/**
	 *  The PreviewEvent class extends Event and is dispatched by some components in the com.adobe.portfolio.previews package.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PreviewEvent extends Event
	{
		/**
		 *  Type of PreviewEvent that is dispatched when the thumbnail changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const THUMBNAIL_CHANGED:String = "thumbnailChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when the list of thumbnails for a multipage preview changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const THUMBNAILS_CHANGED:String = "thumbnailsChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when the page number changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const PAGE_NUMBER_CHANGED:String = "pageNumberChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when the page count changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const PAGE_COUNT_CHANGED:String = "pageCountChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when the playHeadTime changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const PLAYHEAD_TIME_CHANGED:String = "playHeadTimeChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when the source (of IPreviewModel) changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const SOURCE_CHANGED:String = "sourceChanged";

		/**
		 *  Type of PreviewEvent that is dispatched when extract file is requested.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const HANDLE_FILE_EXTRACT:String = "handleFileExtract";

		/**
		 *  Type of PreviewEvent that is dispatched when file info is requested.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const HANDLE_FILE_INFO:String = "handleFileInfo";
		
		/**
		 *  Type of PreviewEvent that is dispatched when the preview model changes.
		 *  @see IPreviewModel
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const MODEL_CHANGED:String = "modelChanged";
		
		/**
		 *  Type of PreviewEvent that is dispatched when the aspect ratio of a preview changes.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const ASPECT_RATIO_CHANGED:String = "aspectRatioChanged";
		
		/**
		 *  Constructor
		 * 
		 *  @param type Type of event
		 *  @param bubbles Whether event bubbles or not.
		 *  @param cancelable Whether event is cancelable
		 * 
		 *  @see flash.events.Event
		 *  @see #THUMBNAIL_CHANGED
		 *  @see #THUMBNAILS_CHANGED
		 *  @see #PAGE_NUMBER_CHANGED
		 *  @see #PAGE_COUNT_CHANGED
		 *  @see #PLAYHEAD_TIME_CHANGED
		 *  @see #SOURCE_CHANGED
		 *  @see #HANDLE_FILE_EXTRACT
		 *  @see #HANDLE_FILE_INFO
		 *  @see #MODEL_CHANGED
		 *  @see #ASPECT_RATIO_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function PreviewEvent(type:String, bubbles:Boolean=false, cancelable:Boolean=false)
		{
			super(type, bubbles, cancelable);
		}
	}
}