/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.events
{
	import com.adobe.portfolio.properties.PropertyContext;
	
	import flash.events.Event;
	
	
	/**
	 *  The PropertyManagerEvent class extends Event and is dispatched when Navigator properties
	 *  have changed.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PropertyManagerEvent extends Event
	{
		
		/**
		 *  A string representing the event type when a property
		 *  context has been loaded
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const CONTEXT_LOADED:String = "navPropertyContextLoaded";
		
		/**
		 *  A string representing the event type when a property context
		 *  has been updated
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		static public const UPDATED_CONTEXTS:String = "navPropertyUpdatedContexts";
		
		public var context:PropertyContext;
		
		/**
		 *  Constructor
		 * 
		 *  @param type The type of the event as a string, accessible as <code>PropertyManagerEvent.type</code>.
		 *  @param bubbles Determines whether the Event object participates
		 *  in the bubbling stage of the event flow. The default value is <code>false</code>.
		 *  @param cancelable Determines whether the Event object can be canceled. 
		 *  The default values is <code>false</code>.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function PropertyManagerEvent(type:String, bubbles:Boolean=false, cancelable:Boolean=false)
		{
			super(type, bubbles, cancelable);
		}
	}
}