/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.layouts.supportClasses
{	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import spark.components.supportClasses.GroupBase;

	/** 
	 *  The ICardLayout interface defines a subset of Layout capabilites that are referenced by other components (such as CardList).
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public interface ICardLayout
	{
		/**
		 *  Find the list of cards that overlap the specified rectangle
		 * 
		 *  @param rect A bounding region for which you want to find card that intersect it
		 *  @return A vector of integers that represent the indexes
		 *  of the cards in the layout that intersect the specified rectangle.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function findCardsThatIntersect(rect:Rectangle):Vector.<int>
			
		/**
		 *  The current card layout helper
		 * 
		 *  @return a CardLayoutHelper component object
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get cardLayoutHelper():CardLayoutHelper;
		
		/**
		 *  The width and height for small thumbnail images to fit in the MiniNavLayout.
		 *  This is given as an ordered pair of numbers: (width, height)
		 * 
		 *  @param value
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function set thumbFitSize(value:Point):void;
		/**
		 *  @private
		 */	
		function get thumbFitSize():Point;

		/**
		 *  The target component 
		 *  @See spark.components.supportClasses.GroupBase
		 * 
		 *  @return The target component that displays visual elements
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get target():GroupBase;
		
		/**
		 *  Indicates whether a virtual layout should be used or not
		 * 
		 *  @return <code>true</code> if a virtual layout should be used, 
		 *  returns <code>false</code> if it should not be used.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get useVirtualLayout():Boolean;
	}
}