/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.layouts.supportClasses
{
	import com.adobe.portfolio.api.IAttachment;
	
	import flash.geom.Point;

	/**
	 *  The ICardProperties interface is used to abstract card functionality required by the Freeform navigator.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */	
	public interface ICardProperties
	{
		/**
		 *  The width and height of the target
		 * 
		 *  @return An ordered pair (width, height)
		 *  representing the width and height of the target in pixels
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		function get targetSize():Point;
	
		/**
		 *  Indicates whether the location of
		 *  the card for the specified IAttachment has been set or not.
		 * 
		 *  @param attachment The IAttachment for which you want to know
		 *  if a location has been set for its card.
		 *  @return <code>true</code> if the location of the card for the
		 *  specified IAttachment has been set; returns <code>false</code>
		 *  if it has not been set.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		function isLocationSet(attachment:IAttachment):Boolean;
			
		/**
		 *  Sets the card layout location for the specified attachment.
		 * 
		 *  @param attachment The IAttachment for which you want to set
		 *  the layout location
		 *  @param value The layout location of the center of the card
		 *  given as an ordered pair: (x,y)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		function setLocation(attachment:IAttachment, value:Point):void;
		
		/**
		 *  Get the card layout location for the specified attachment.
		 * 
		 *  @param attachment The IAttachment for which you want to find
		 *  the layout location
		 *  @return The layout location of the center of the card
		 *  given as an ordered pair: (x,y)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		function getLocation(attachment:IAttachment):Point;
		
		/**
		 *  Sets the rotation of the card layout
		 *  for the specified attachment
		 * 
		 *  @param attachment The IAttachment for which you want to set
		 *  the rotation of its card layout
		 *  @param value The rotation of the card layout in degrees
		 *  of rotation from the horizontal
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		function setRotation(attachment:IAttachment, value:Number):void;
	}
}