/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.lists
{
	import com.adobe.portfolio.api.IAttachment;
	import com.adobe.portfolio.cards.ISimpleCard;
	import com.adobe.portfolio.components.supportClasses.CustomizableTextArea;
	import com.adobe.portfolio.events.LayoutEvent;
	import com.adobe.portfolio.events.MarqueeSelectionEvent;
	import com.adobe.portfolio.layouts.FreeFormLayout;
	
	import flash.display.DisplayObject;
	import flash.events.IEventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.utils.Dictionary;
	
	import mx.controls.Image;
	import mx.core.IFlexDisplayObject;
	import mx.core.IVisualElement;
	import mx.events.DragEvent;
	import mx.logging.ILogger;
	import mx.logging.Log;
	
	import spark.components.IItemRenderer;
	import spark.core.NavigationUnit;
	import spark.events.IndexChangeEvent;
	import spark.events.RendererExistenceEvent;
	
	/**
	 *  The FreeFormList class extends CardContainer to provide functionality specific to the Freeform navigator.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class FreeFormList extends CardContainer
	{
		protected static const logger:ILogger = Log.getLogger("FreeFormList");
		
		protected var mouseDownPoint:Point;
		protected var selectedItemLocations:Dictionary; // key:ISimpleCard: value:Point
		protected var selectionExtensionAnchorIndex:int = -1;
		
		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function FreeFormList()
		{
			useDragDropReordering = false;
			useDragDropOnFolders = true;
		}
		
		private function get ffLayout():FreeFormLayout
		{
			return layout as FreeFormLayout;
		}

		/** Called when a skin has added a skin part for this instance.
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, true);
		}
		
		/** Called when a skin has removed a skin part for this instance.
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, false);
		}
		
		/** Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if (instance && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (instance) {
					case dataGroup:
						types = [ RendererExistenceEvent.RENDERER_ADD, RendererExistenceEvent.RENDERER_REMOVE ];
						listeners = [ rendererAddHandler, rendererRemoveHandler ];
						layout.addEventListener(LayoutEvent.HANDLED_ATTACHMENT_FOCUS, onHandledFocus);
						break;
					
					default:
						break;
				}
	
				// Add or remove all the relevant event listeners
				var count:uint = listeners ? listeners.length : 0;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
									activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
		
		/**
		 *  @private
		 *  Called when an item has been added to this component.
		 */
		private function rendererAddHandler(event:RendererExistenceEvent):void
		{
			var renderer:IVisualElement = event.renderer;
			if (!renderer)
				return;
			
			renderer.addEventListener(MouseEvent.ROLL_OVER, onMouseRollOver);
			renderer.addEventListener(MouseEvent.ROLL_OUT, onMouseRollOut);
			rendererWasAdded(renderer);
		}
		
		/**
		 *  @private
		 *  Called when an item has been removed from this component.
		 */
		private function rendererRemoveHandler(event:RendererExistenceEvent):void
		{
			var renderer:IVisualElement = event.renderer;
			if (!renderer)
				return;
			
			renderer.removeEventListener(MouseEvent.ROLL_OVER, onMouseRollOver);
			renderer.removeEventListener(MouseEvent.ROLL_OUT, onMouseRollOut);
			rendererWasRemoved(renderer);
		}
		
		/**
		 *  Sets the state of the skin as well as the state of this instance.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void
		{
			super.setCurrentState(stateName, playTransition);
			invalidateSkinState();
		}
		
		/** Returns the current state of this instance, since its skins should have a matching
		 *  set of states.
		 */
		override protected function getCurrentSkinState():String
		{
			return currentState;
		}

		/**
		 *  @inheritDoc
		 */
		override public function createDragIndicator():IFlexDisplayObject
		{
			// Returning null will result in the dragmanager creating a drag image based on the component
			// Returning a null image ensures that we send a null image to the platform which in turn
			// results in much faster drag initiation. The actual drop feedback is handled using createDropIndicator
			// When dragging over external views, the drag cursor provides the feedback
			return new Image();
		}
		
		/**
		 *  @inheritDoc
		 */
		override public function createDropIndicator():DisplayObject
		{
			// Set it in the layout
			layout.dropIndicator = null; // dropIndicatorInstance;
			
			return null;
		}
		
		override protected function dragStartHandler(event:DragEvent):void
		{
			super.dragStartHandler(event);
			if (isDragAndDropDisabled || event.isDefaultPrevented())
				return;

			var dragPoint:Point = new Point(event.localX, event.localY);
			ffLayout.dragStartPoint = dragPoint;
		}
		
		override protected function dragExitHandler(event:DragEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			// Do not destroy the drop indicator for drags started from within as we move in and out of folders
			// It is destroyed in drag complete anyway
			if (event.dragInitiator != this)
			{
				layout.hideDropIndicator();
				destroyDropIndicator();
			}
			dragStarted = false;
		}
		
		override protected function dragDropHandler(event:DragEvent):void
		{
			CONFIG::logging{
				logger.info("Freeform List dragDropHandler: ");
			}
			super.dragDropHandler(event);
			if (event.isDefaultPrevented())
				return;
			
			// Avoid deref'ing null
			if (!ffLayout || !ffLayout.target || !ffLayout.freeFormFields)
				return;
			
			// If user is not in authoring mode, don't let them finish the drag/drop/move
			if (!collectionDropEnabled(event)) 
			{
				// Take the card out of hover state when drag start is cancelled.
				// Passing null means that if any card was in hover state, take it out.
				ffLayout.hoveringOverElement(null, false);
				return;
			}
			
			// The design states that the items are randomly arranged when dragging from
			// the platform and mouse position based for internal drags
			// Get the card(s) that was(were) moved
			var attachments:Vector.<Object> = event.dragSource.dataForFormat(DRAG_DROP_ATTACHMENTS) as Vector.<Object>;
			// Do nada if there weren't any (although there should be)
			if (!attachments) 
			{
				CONFIG::logging{
					logger.warn("Expected, but did not get attachment data with drop event");
				}
				return;				
			}
			if (event.dragInitiator == this)
			{
				// Calculate the delta between the drag start and drag drop points
				var offset:Point = new Point(event.localX, event.localY);
				var delta:Point = offset.subtract(ffLayout.dragStartPoint);
				
				// Adjust the position of each of the moved cards
				var offsetForPlatformDrops:Number = 0;
				for each (var attachment:IAttachment in attachments) 
				{
					// Get the current card position
					 var x:Number = ffLayout.freeFormFields.getX(attachment);
					 var y:Number = ffLayout.freeFormFields.getY(attachment);
						
					// If none existed, default to upper left
					if (isNaN(x))
						x = 0;
					if (isNaN(y))
						y = 0;
					
					// Offset the card's position by the delta
					x += delta.x;
					y += delta.y;
					
					// Write the new card position back
					// Note: setX and setY ensure that the new card position
					// is still on the canvas, in case a group of cards moved
					ffLayout.freeFormFields.setX(attachment, x);
					ffLayout.freeFormFields.setY(attachment, y);
				}
			}
			else
			{
				// If there is only one attachment from the platform then position
				// it at the cursor location. If not use the scatter algorithm
				if (attachments.length == 1)
				{
					ffLayout.freeFormFields.setX(attachments[0] as IAttachment, event.localX);
					ffLayout.freeFormFields.setY(attachments[0] as IAttachment, event.localY);
				}
			}
			
			// Update the display after moving the cards
			layout.target.invalidateDisplayList();
		}
		
		override protected function keyDownHandler(event:KeyboardEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			if (event.altKey) {
				// We don't handle any alt events from here
				return;
			}
			
			// Is the user entering a navigation key (up/down/left/right/home/end/etc)?
			if (NavigationUnit.isNavigationUnit(event.keyCode)) {
				if (!event.shiftKey) {
					// Reset the selection extension anchor
					selectionExtensionAnchorIndex = -1;
					// All non-shift navigation movement is handled by the superclass
					super.keyDownHandler(event);
				}
				else if (ffLayout) {
					var navigationUnit:uint = event.keyCode;
					// If shift (no control), extend the selection
					if (!event.ctrlKey) {
						// Find new item that we navigated to and...
						var newIndex:int = ffLayout.getNavigationDestinationIndex(selectedIndex, navigationUnit, false);
						// Extend selection to there
						extendSelectionToHere(newIndex);
					}
					// If ctrl-shift, use arrow keys to rotate the selected cards
					else {
						ffLayout.adjustAngleOfCards(selectedIndices, navigationUnit);
					}
				}
				event.preventDefault();
				return;
			}

			super.keyDownHandler(event);
		}
		
		override protected function item_mouseDownHandler(event:MouseEvent):void
		{
			if (event.isDefaultPrevented())
				return;

			// Don't initiate drag or extending selection if editing the name of the card
			if (CustomizableTextArea.textAreaHasFocus(focusManager))
				return;
			
			extendSelection(event);
			saveSelectedItemLocations(event);
		}
		
		
		protected function extendSelection(event:MouseEvent):void
		{
			// If modifiers other than shift in use or shift modifier is not in use
			if (modifiersActive(event, true) || !event.shiftKey) {
				// Let the superclass handler it
				super.item_mouseDownHandler(event);
				// Reset the selection extension anchor
				selectionExtensionAnchorIndex = -1;
				return;
			}
			if (!ffLayout)
				return;
			// Find which item was selected
			var newIndex:int = ffLayout.getVisualElementIndex(event.currentTarget as IVisualElement);
			// Extend selection to there
			extendSelectionToHere(newIndex);
		}
		
		protected function saveSelectedItemLocations(event:MouseEvent):void
		{
			if (!collectionMgr.authoringMode)
				return;

			if (selectedItems.length <= 0 ) {
				selectedItemLocations = null;
				removeDragListeners();
				return;
			}
			
			addDragListeners();
			ffLayout.isDragging = true;
			mouseDownPoint = new Point(event.stageX, event.stageY);
			selectedItemLocations = new Dictionary();
			for each (var attachment:IAttachment in selectedItems) {
				var card:ISimpleCard = cardFromAttachment(dataGroup, attachment);
				if (!card)
					continue;
				selectedItemLocations[card] = {location: new Point(card.x, card.y), card:card};
			}
		}
		
		protected function moveSelectedItemLocations(event:MouseEvent):void
		{
			if (!collectionMgr.authoringMode)
				return;
			if (selectedItems.length < 1 ) 
				return;
			
			var mousePoint:Point = new Point(event.stageX, event.stageY);
			var delta:Point = mousePoint.subtract(mouseDownPoint);

//			logger.debug("moveSelectedItemLocations: delta= " + delta);
			for each (var obj:Object in selectedItemLocations) {

				var origLocation:Point = obj.location;
				if (!origLocation)
					continue;
				
				var card:ISimpleCard = obj.card;
				if (!card)
					continue;
				
				card.x = origLocation.x + delta.x;
				card.y = origLocation.y + delta.y;
			}
			layout.target.invalidateDisplayList();
			layout.target.validateNow();
		}
		
		protected function onMouseMove(event:MouseEvent):void {
			if (isDragAndDropDisabled) {
				// Don't handle a drag if something else (like ScrollPad) has disabled it
				return;
			}
			moveSelectedItemLocations(event);
		}
		
		protected function onMouseUp(event:MouseEvent):void {
			ffLayout.isDragging = false;
			removeDragListeners();
		}
		
		protected function addDragListeners():void
		{
			systemManager.getSandboxRoot().addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove, false, 0, true);
			systemManager.getSandboxRoot().addEventListener(MouseEvent.MOUSE_UP, onMouseUp, false, 0, true);
		}
		
		protected function removeDragListeners():void
		{
			systemManager.getSandboxRoot().removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			systemManager.getSandboxRoot().removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		protected function extendSelectionToHere(newIndex:int):void
		{
			if (newIndex < 0)
				return;
			// If this is the initial extend selection, set the anchor
			// ToDo: What happens if selectionExtensionAnchorIndex and selectedIndex are both -1?
			// ToDo: Should selectionExtensionAnchorIndex get set to newIndex?
			if ((selectionExtensionAnchorIndex == -1) && (selectedIndex != -1))
				selectionExtensionAnchorIndex = selectedIndex;
			// Update the selected item (the one with "focus")
			setSelectedIndexWithChangeDispatch(newIndex);
			// Find all items that are within the extended selection boundary
			var selectedItems:Vector.<int> = ffLayout.getExtendedSelection(selectionExtensionAnchorIndex, selectedIndex);
			// Update the multi selection
			updateMultiSelection(selectedItems);
		}
		
		override protected function updateMultiSelection(selectedItems:Vector.<int>):void
		{
			// The order of where the selected item is with selected items
			// is significant for free form, because the new (extended) selection
			// is relative to the most recently selected item.
			if (!selectedItems || selectedItems.length <= 0)
				return;
			
			// Add each item to the new selected items list, but add the currently
			// selected item last (so it remains as the selected item)
			var newSelectedItems:Vector.<int> = new Vector.<int>();
			var theSelectedItem:int = -1;
			for (var index:int = 0; index < selectedItems.length; index++) {
				// The selected item should be in the vector of items
				if (selectedItems[index] == selectedIndex) {
					// Save it to be added last
					theSelectedItem = selectedIndex;
					continue;
				}
				// Add the new item at the beginning (same thing that List does)
				newSelectedItems.splice(0, 0, selectedItems[index]);
			}
			// Add the selected item last (so it stays as the selected item)
			if (theSelectedItem != -1)
				newSelectedItems.splice(0, 0, theSelectedItem);
			// If we have selected items (which we should), update the selected indices
			if (newSelectedItems.length > 0)
				super.updateMultiSelection(newSelectedItems);
		}
		
		protected function onMouseRollOver(event:MouseEvent):void
		{
			if (event.buttonDown || modifiersActive(event) || !ffLayout)
				return;
			
			// Don't change card depth for selected cards
			var renderer:IItemRenderer = event.currentTarget as IItemRenderer;
			if (renderer && renderer.selected)
				return;
			ffLayout.hoveringOverElement(event.currentTarget as IVisualElement, true);
		}
		
		protected function onMouseRollOut(event:MouseEvent):void
		{
			if (!ffLayout)
				return;
			
			// Don't change card depth for selected cards
			var renderer:IItemRenderer = event.currentTarget as IItemRenderer;
			if (renderer && renderer.selected)
				return;
			ffLayout.hoveringOverElement(event.currentTarget as IVisualElement, false);
		}
		
		protected function rendererWasRemoved(visualElement:IVisualElement):void
		{
			if (!ffLayout)
				return;
			ffLayout.elementWasRemoved(visualElement);
		}
		
		protected function rendererWasAdded(visualElement:IVisualElement):void
		{
			if (!ffLayout)
				return;
			ffLayout.elementWasAdded(visualElement);
		}
		
		override protected function onMarqueeSelectionBegin(event:MarqueeSelectionEvent):void
		{
			super.onMarqueeSelectionBegin(event);

			// Reset the selection extension anchor
			selectionExtensionAnchorIndex = -1;
		}
		
		override protected function resetCardSizes():void {
			if (!dataGroup || !dataProvider)
				return;
			
			if (cardSize == FreeFormLayout.AUTO_CARD_SIZE)
				return;
			
			super.resetCardSizes();
		}
		
		override protected function onIndexChangeEvent(event:IndexChangeEvent):void
		{
			super.onIndexChangeEvent(event);
			ffLayout.selectionChanged(selectedItems);
		}

	}
}
