/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.lists
{
	import com.adobe.portfolio.cards.CardFactory;
	import com.adobe.portfolio.layouts.MiniNavLayout;
	
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import mx.collections.IList;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.events.DragEvent;
	
	import spark.components.ToggleButton;
	import spark.events.IndexChangeEvent;

	[SkinState("normal")]
	[SkinState("hidden")]

	
	/**
	 * The MiniNavList class extends CardList to provide functionality specific to the MiniNav component.
	 * This component is used to aid with navigation between files. It is usually added as an additional component to a navigator. 
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class MiniNavList extends CardList
	{
		/**
		 *  String literal used to represent the skin part for slammer exposed by MiniNavList
		 *  to its skins.
		 * 
		 *  @langversion 3.0
	 	 *  @playerversion Flash 10.1
	 	 *  @playerversion AIR 2.0
	 	 *  @productversion Portfolio 10.0
	 	 */
		public static const PART_SLAMMER:String = "slammer";
		
		/** 
		 *  The "normal" state of both the miniNav and its skin.
		 * 
		 *  @langversion 3.0
	 	 *  @playerversion Flash 10.1
	 	 *  @playerversion AIR 2.0
	 	 *  @productversion Portfolio 10.0
	 	 */
		public static const SKIN_STATE_NORMAL:String = "normal";
		
		/**
		 *  The "disabled" state of both the miniNav and its skin.
		 *  
		 *  @langversion 3.0
	 	 *  @playerversion Flash 10.1
	 	 *  @playerversion AIR 2.0
	 	 *  @productversion Portfolio 10.0
	 	 */
		public static const SKIN_STATE_DISABLED:String = "disabled";
		
		/**
		 *  The "hidden" state of both the miniNav and its skin. 
		 *  The miniNav can be hidden using the slammer.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const SKIN_STATE_HIDDEN:String = "hidden";

		/**
		 *  The actual skin part, "slammer", a toggle button, that is used to hide and show the miniNav.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[SkinPart(required="false")]
		public var slammer:ToggleButton;
		
		/**
		 *  A boolean used by the component's skin to determine the visibility of the slammer
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		[Bindable]
		public var showSlammer:Boolean = true;

		/** 
		 *  Pending index change event that was initiated along with a drag. 
		 *  This is handled after the drag and drop gesture is completed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private var pendingIndexChangeEvent:IndexChangeEvent = null;

		/**
		 *  @private
		 */ 
		private var mouseDown:Boolean = false;
		private var _heightIfPresent:Number = 0;
		[Bindable]
		/**
		 *  @return The height of the miniNav component if present
		 *
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function get heightIfPresent():Number
		{
			return _heightIfPresent;
		}
		public function set heightIfPresent(value:Number):void
		{
			_heightIfPresent = value;
		}
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		public function MiniNavList()
		{
			super();
			doubleClickEnabled = true;
			useVirtualLayout = false;
			enabled = true;
			useDragDropReordering = true;
			focusCardDelta = 5;
			
			// Set default item renderer; this can be overriden in mxml or AS
			itemRenderer = new CardFactory(CardFactory.SIMPLE_CARD);
			
			// Set default layout; this can be overriden in mxml or AS
			layout = new MiniNavLayout();
			
			addListenAndWatch();
		}
		
		/**
		 *  Adds listeners and watchers that are automatically unwatched and unlistened.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		override public function addListenAndWatch():void
		{
			super.addListenAndWatch();
			addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 *  Function to cleanup the event listeners required by miniNav list. This should be
		 *  called when the component is no longer needed, to prevent memory leaks.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function removeListenAndWatch():void
		{
			super.removeListenAndWatch();
			removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * @inheritDoc 
		 */
		override protected function item_mouseDownHandler(event:MouseEvent):void
		{
			mouseDown = true;
			super.item_mouseDownHandler(event);
		}
		
		/**
		 *  Event handler to handle mouse up events on miniNav.
		 *  Calls for pending updates deferred by a drag.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		protected function onMouseUp(evt:MouseEvent = null):void
		{
			mouseDown = false;
			if (!evt)
			{
				if (!collectionMgr.newOrderedList)
					performPendingUpdate();
				return;
			}
			
			// Run a small timer to see if this is a double click. Don't perform the prending update if it is.
			var timer:Timer = new Timer(200, 1);
			var timerComplete:Function = function(event:TimerEvent):void
			{
				timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
				if (!mouseDown)
				{
					performPendingUpdate();
				}
			};
			timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
			timer.start();
		}
		
		/**
		 *  This function performs any pending updates that were deferred due to a drag and mouse down occuring at the same time.
		 *  Drag and drop gesture is completed first.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */	
		protected function performPendingUpdate():void
		{
			if (!pendingIndexChangeEvent)
				return;
		
			super.onIndexChangeEvent(pendingIndexChangeEvent);
			pendingIndexChangeEvent = null;
		}
		
		/**
		 *  This function moves focus to the item with the new index and then calls the super's onIndexChangeEvent. 
		 *  If mouse is down, or drag has started, index change is delayed until mouse is up and drag is complete. 
		 *  @inheritDoc
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override protected function onIndexChangeEvent(event:IndexChangeEvent):void
		{
			if (event.isDefaultPrevented())
				return;
			
			if (dragStarted || mouseDown)
				pendingIndexChangeEvent = event;
			else
				super.onIndexChangeEvent(event);
		}
		
		/**
		 *  Overriding to prevent the scrolling of items according to the default list behavior.
		 *  @inheritDoc 
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function ensureIndexIsVisible(index:int):void
		{
			if (!layout)
				return;
		}
		
		override protected function onCollectionChange(event:CollectionEvent):void
		{
			super.onCollectionChange(event);
			if ((event != null) && (event.kind == CollectionEventKind.REFRESH) && (layout is MiniNavLayout)) {
				(layout as MiniNavLayout).ensureFocusCentered(collectionMgr.focusAttachment);
			}
		}
		
		/**
 		  * @inheritDoc
		  * 
		  */
		override protected function dragCompleteHandler(event:DragEvent):void
		{
			// A mouse up event is not triggered after a drag.Enforce it.
			onMouseUp();
			
			super.dragCompleteHandler(event);

			// Move scrollposition back to where it was.
			if (layout is MiniNavLayout)
				(layout as MiniNavLayout).ensureFocusCentered(collectionMgr.focusAttachment);
			
		}
		
		/**
		 *  Calls the super function in <code>CardList</code> to reorder the given list of attachments. 
		 *  Also, performs pending update after reordering has completed successfully.
		 *  @inheritDoc
		 *    
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override protected function reorderAttachments(source:IList):Boolean
		{
			var result:Boolean = super.reorderAttachments(source);
			
			if (result)
				performPendingUpdate();
			
			return result;	
		}
		
		/** 
		 *  Listener to handle the click of the slammer. 
		 *  It hides or shows the mini Nav.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onSlammerClick(evt:Event):void
		{
			invalidateSkinState();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, true);
		}
		
		/**
		 *  @inheritDoc
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, false);
			collectionMgr.dispatchPartRemoved(instance);
		}
		
		/** 
		 *  @private
		 *  Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 *  
		 *  @param partName The name of the part being added.
		 *  @param instance The instance of the object to which the part is added or removed.
		 *  @param adding Whether the part is being added or removed.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array;
			var types:Array;
			
			if ((instance == null) || !(instance is IEventDispatcher))
				return;
			
			var activePart:IEventDispatcher = instance as IEventDispatcher;
			
			// Build up an array of event types and listeners that apply to this object
			switch (partName) {
				case PART_SLAMMER:
					types = [ MouseEvent.CLICK ];
					listeners = [ onSlammerClick ];
					break;
			}
			
			// Add or remove all the relevant event listeners
			var count:uint = (listeners == null) ? 0 : listeners.length;
			if (count > 0) {
				for (var i:uint = 0; i < count; ++i) {
					adding ?  activePart.addEventListener(types[i], listeners[i]) :
						activePart.removeEventListener(types[i], listeners[i]);
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}
		
		/**
		 *  Returns the current state of this instance, which is a variant
		 *  of the component state depending on
		 *  whether or not the component is selected.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function getCurrentSkinState():String
		{
			if ((slammer == null) || !slammer.selected) {
				return SKIN_STATE_NORMAL;
			} else {
				return SKIN_STATE_HIDDEN;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function resizeCachedThumbs():void 
		{
			// do nothing by design
		}

	}
}