/*************************************************************************
 *
 * ADOBE SYSTEMS INCORPORATED
 *  Copyright 2010 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  Adobe permits you to use, modify, and distribute this 
 * file in accordance with the terms of the Adobe license agreement 
 * accompanying it.  If you have received this file from a source 
 * other than Adobe, then your use, modification, or distribution 
 * of it requires the prior written permission of Adobe.
 **************************************************************************/

package com.adobe.portfolio.lists
{
	import com.adobe.portfolio.layouts.OneUpLayout;
	
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	
	import mx.collections.IList;
	import mx.core.IButton;
	import mx.events.CollectionEvent;
	import mx.states.State;

	[SkinState("empty")]
	[SkinState("normal")]
	[SkinState("preview")]
	
	
	/**
	 * The OneUpList class extends CardList to implement functionality specific
	 * to the BasicClickThrough navigator.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class OneUpList extends CardList
	{
		// String literals used to represent the skin parts exposed by OneUpList to its skins.
		
		/**
		 * A string literal representing the name of the previous button skin part.
		 */
		static public const PART_PREV_BUTTON:String = "prevButton";
		
		/**
		 * A string literal representing the name of the next button skin part.
		 */
		static public const PART_NEXT_BUTTON:String = "nextButton";
		
		// String literals used to represent the states of OneUp and its skins.
		
		/**
		 * A string literal representing the normal state.
		 */
		static public const STATE_NORMAL:String = "normal";
		
		/**
		 * A string literal representing the preview state.
		 */
		static public const STATE_PREVIEW:String = "preview";
		
		/**
		 * A string literal representing the empty skin state.
		 */
		static public const SKIN_STATE_EMPTY:String = "empty";
		
		/**
		 * A string literal representing the normal skin state.
		 */
		static public const SKIN_STATE_NORMAL:String = "normal";
		
		/**
		 * A string literal representing the preview skin state.
		 */
		static public const SKIN_STATE_PREVIEW:String = "preview";
		
		/**
		 * Default constructor
		 */
		public function OneUpList()
		{
			super();
			
			// Setup the states for this instance. They should match the skin states.
			states = [new State({name: STATE_NORMAL}), new State({name: STATE_PREVIEW})];
			currentState = STATE_NORMAL;
			
			tabEnabled = true;
			layout = new OneUpLayout();
		}

		/**
		 * A button for navigating to the previous attachment, optionally provided by this instance's skin.
		 */
		[SkinPart(required="false")]
		public var prevButton:IButton;
		
		/**
		 * A button for navigating to the next attachment, optionally provided by this instance's skin.
		 */
		[SkinPart(required="false")]
		public var nextButton:IButton;

		/**
		 * An empty implementation to prevent selected items from appear selected.
		 * 
		 * @inheritDoc
		 */
		override protected function itemSelected(index:int, selected:Boolean):void
		{
			// do nothing by design
		}
		
		/**
		 * @inheritDoc
		 */
		override public function set dataProvider(value:IList):void
		{
			super.dataProvider = value;
			invalidateSkinState();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function onCollectionChange(e:CollectionEvent):void
		{
			super.onCollectionChange(e);
			
			if (lastSkinState == SKIN_STATE_EMPTY) {
				if ((dataProvider != null) && (dataProvider.length > 0)) {
					invalidateSkinState();
				}
			} else if (lastSkinState == SKIN_STATE_NORMAL) {
				if ((dataProvider == null) || (dataProvider.length <= 0)) {
					invalidateSkinState();
				}
			}
		}

		/**
		 * @inheritDoc
		 */
		override protected function partAdded(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, true);
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function partRemoved(partName:String, instance:Object):void
		{
			handlePartChange(partName, instance, false);
		}
		
		/** Internal function for adding and removing event listeners for skin parts, depending on
		 *  whether the part is being added or removed for this instance.
		 */
		private function handlePartChange(partName:String, instance:Object, adding:Boolean):void
		{
			var listeners:Array = null;
			var types:Array = null;
			if ((instance != null) && (instance is IEventDispatcher)) {
				var activePart:IEventDispatcher = instance as IEventDispatcher;
				
				// Build up an array of event types and listeners that apply to this object
				switch (partName) {
					case PART_NEXT_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ collectionMgr.onGotoNextAttachment ];
						break;
					case PART_PREV_BUTTON:
						types = [ MouseEvent.CLICK ];
						listeners = [ collectionMgr.onGotoPreviousAttachment ];
						break;
				}
				
				// Add or remove all the relevant event listeners
				var count:uint = (listeners == null) ? 0 : listeners.length;
				if (count > 0) {
					for (var i:uint = 0; i < count; ++i) {
						adding ? 	activePart.addEventListener(types[i], listeners[i]) :
							activePart.removeEventListener(types[i], listeners[i]);
					}
				}
			}
			adding ? super.partAdded(partName, instance) : super.partRemoved(partName, instance);
		}

		/**
		 * @inheritDoc
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void
		{
			super.setCurrentState(stateName, playTransition);
			
			// We used to call skin.currentState = stateName; here, but simply invalidating
			// plays nicer with the Flex model.
			invalidateSkinState();
		}
		
		private var lastSkinState:String;
		
		/**
		 * @inheritDoc
		 */
		override protected function getCurrentSkinState():String
		{
			if ((dataProvider == null) || (dataProvider.length <= 0)) {
				lastSkinState = SKIN_STATE_EMPTY;
				return lastSkinState;
			}
			if (currentState == STATE_PREVIEW) {
				lastSkinState = SKIN_STATE_PREVIEW;
				return lastSkinState;
			}
			lastSkinState = SKIN_STATE_NORMAL;
			return lastSkinState;
		}
	}
}