/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.lists
{
	import com.adobe.portfolio.cards.BasicCard;
	import com.adobe.portfolio.cards.CardFactory;
	import com.adobe.portfolio.layouts.WaveLayout;
	
	import flash.events.MouseEvent;
	
	import mx.events.FlexEvent;
	
	import spark.components.RichEditableText;
	import spark.components.supportClasses.SkinnableTextBase;
	import spark.events.IndexChangeEvent;

	/**
	 * The WaveContainer class extends CardContainer to provide functionality specific to the Wave navigator.
	 * 
  	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class WaveContainer extends CardContainer
	{
		protected var waveLayout:WaveLayout;
		
		private var mouseDownX:Number;
		private var mouseDownEvent:MouseEvent;
		private var mouseDowncardPosition:Number;
		private var mouseMoved:Boolean;
		private var deltaX:Number;
		
		/**
		 * Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function WaveContainer()
		{
			super();
			useDragDropReordering = false;
			dragEnabled = false;
			
			// WaveLayout does not yet support useVirtualLayout, so turn this off by default
			useVirtualLayout = false;
			
			// Set default item renderer; this can be overriden in mxml or AS
			const alpha:Number = 0;
			itemRenderer = new CardFactory(CardFactory.BASIC_CARD, BasicCard.STATE_LARGE, CardFactory.DROP_SHADOW_BOTTOM, null, alpha);
			focusCardDelta = 4;
		}
		
		/**
		 * Adds listeners and watchers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function addListenAndWatch():void
		{
			super.addListenAndWatch();
			addListener(this, IndexChangeEvent.CARET_CHANGE, caretChangeHandler);
		}

		override protected function onCreationComplete(event:FlexEvent):void 
		{
			super.onCreationComplete(event);
			waveLayout = layout as WaveLayout;
		}
		
		protected function caretChangeHandler (event:IndexChangeEvent) : void
		{
			if (!waveLayout)
				return;
			waveLayout.scroll();
		}
		
		override protected function itemSelected(index:int, selected:Boolean):void
		{
			// do nothing by design
		}

		override protected function item_mouseDownHandler(event:MouseEvent):void 
		{
			if (event.isDefaultPrevented())
				return;
			
			// Don't handle mouseDown events if the user is selecting text
			var richTarget:RichEditableText = event.target as RichEditableText;
			if ((richTarget != null) && richTarget.enabled && richTarget.selectable) {
				return;
			}
			var textTarget:SkinnableTextBase = event.target as SkinnableTextBase;
			if ((textTarget != null) && textTarget.enabled && textTarget.selectable) {
				return;
			}
			
			mouseDownEvent = event;
			mouseDownEvent.stopImmediatePropagation();
			mouseDownX = mouseDownEvent.stageX;
			mouseMoved = false;
			mouseDowncardPosition = waveLayout.cardPosition;		
			
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
		}
		
		protected function onMouseUp(event:MouseEvent):void 
		{
			if (!mouseDownEvent) // how can this be? it's happening...
				return;
			
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			if (!mouseMoved) {
				super.item_mouseDownHandler(mouseDownEvent);
			} else if (waveLayout) {
				waveLayout.nudgeCard(getDeltaX(event));		
			}
			mouseDownEvent = null;
		}
		
		protected function getDeltaX(event:MouseEvent):Number 
		{
			if (event.stageX != 0) // strange 0 input sometimes
				deltaX = mouseDownX - event.stageX;
			return deltaX;
		}
		
		protected function onMouseMove(event:MouseEvent):void 
		{
			if (!waveLayout)
				return;
			
			mouseMoved = true;
			waveLayout.cardPosition = mouseDowncardPosition + getDeltaX(event);
		}
	}
}
