/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.managers
{
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.system.LoaderContext;
	import flash.utils.ByteArray;
	
	import mx.binding.utils.BindingUtils;
	import mx.controls.SWFLoader;
	
	/**
	 * The FlashSkinManager dynamically loads skin SWF files and creates object instances from
	 * them to be used as visual assets by Flex components and skins.
	 *    
     *  @langversion 3.0
     *  @playerversion Flash 10
     *  @playerversion AIR 1.5
     *  @productversion Acrobat 10
	 */
	public class FlashSkinManager extends EventDispatcher
	{
		/** Internal representation of the FlashSkinManager referenced by the instance getter.
		 */
		private static var _instance:FlashSkinManager = null;
		
		/** 
		 *  An array of the known visual assests for a Navigator: "IconBackground", 
		 *  "IconBackgroundOver", "FolderSmall", "FolderMedium", 
		 *  "FolderLarge", "HeaderBackground", "NavBackground", 
		 *  "CardTitleBar", "FolderTitleBar", "MiniNavSlammerBackground", 
		 *  "MiniNavSlammerBackgroundOver", "MiniNavBackground", "MiniNavCard", 
		 *  "MiniNavCardOver", "MiniNavCardSelected", "NavCard", "NavCardOver", 
		 *  "NavCardSelected", "NextButton", "NextButtonOver", "PreviousButton", 
		 *  "PreviousButtonOver", "CloseButton", and "CloseButtonOver"
		 *    
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
    	 */
		public static const knownAssetNames:Array = [
			"IconBackground",
			"IconBackgroundOver",
			"FolderSmall",
			"FolderMedium",
			"FolderLarge",
			"HeaderBackground",
			"NavBackground",
			"CardTitleBar",
			"FolderTitleBar",
			"MiniNavSlammerBackground",
			"MiniNavSlammerBackgroundOver",
			"MiniNavBackground",
			"MiniNavCard",
			"MiniNavCardOver",
			"MiniNavCardSelected",
			"NavCard",
			"NavCardOver",
			"NavCardSelected",
			"NextButton",
			"NextButtonOver",
			"PreviousButton",
			"PreviousButtonOver",
			"CloseButton",
			"CloseButtonOver"
			];
		
		/**
		 * Returns a FlashSkinManager static instance that can be shared by Flex components.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function get instance():FlashSkinManager
		{
			if (_instance == null) {
				_instance = new FlashSkinManager();
			}
			return _instance;
		}
		
		/** 
		 *  Constructor
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function FlashSkinManager(target:IEventDispatcher=null)
		{
			super(target);
		}
		
		/**
		 *  Returns <code>true</code> if the specified SWF file has been loaded by the manager;
		 *  <code>false</code> otherwise.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function skinsAreLoaded(swf:String = ""):Boolean
		{
			if (swf == "") {
				swf = activeSWF;
			}
			return (libs.propertyIsEnumerable(swf) && (libs[swf] != null));
		}
	
		/**
		 * Returns the specified skin from the most recently loaded SWF. If non-null, the
		 * returned object is the actual stage instance of a DisplayObject, not a copy or clone.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function getSkin(skin:String):DisplayObject
		{
			var library:Object = null;
			if ((activeSWF != "") && libs.propertyIsEnumerable(activeSWF)) {
				library = libs[activeSWF];
			}
			if (library != null) {
				if (library.hasOwnProperty(skin)) {
					return library[skin];
				} else {
					return null;
				}
			}
			return null;
		}
		
		/**
		 * Causes the FlashSkinManager to load the specified skin SWF.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function loadFlashSkins(swf:String):void
		{
			var swfLoader:SWFLoader = null;
			if (loaders.propertyIsEnumerable(swf)) {
				swfLoader = loaders[swf] as SWFLoader;
			}
			if (swfLoader == null) {
				swfLoader = new SWFLoader();
				swfLoader.loaderContext = new LoaderContext();
				swfLoader.loaderContext.allowCodeImport = true;
				loaders[swf] = swfLoader;
			}
			activeSWF = swf;
			addLoadEvents(swfLoader);
			
			BindingUtils.bindSetter(
				function (bytes:ByteArray):void {
					if (bytes != null) {
						swfLoader.load(bytes);
					}
				}, ResourceManager.instance.getResource(swf, true), "dataBytes");
		}
		
		/**
		 * Causes the FlashSkinManager to unload the specified SWF.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function unloadFlashSkins(swf:String):void
		{
			var shouldDispatch:Boolean = false;
			if (libs[swf] != null) {
				libs[swf] = null;
				shouldDispatch = true;
			}
			if (loaders[swf] != null) {
				// TODO we're loading, but we're not done yet
				loaders[swf] = null;
			}
			if (shouldDispatch) {
				dispatchEvent(new Event(Event.COMPLETE));
			}
		}
		
		/** The current SWF being used for Flash skins.
		 *     
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected var activeSWF:String = "";
		
		/**
		 *  A mapping of SWF file names to their corresponding loaders.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected var loaders:Object = new Object();
		
		/**
		 *  Adds the required event listeners to a swfLoader prior to the call to load the skin SWF.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function addLoadEvents(swfLoader:SWFLoader):void
		{
			swfLoader.addEventListener(Event.COMPLETE, onLoadComplete);
			swfLoader.addEventListener(IOErrorEvent.IO_ERROR, onLoadIOError);
			swfLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onLoadSecurityError);
		}
		
		/**
		 *  This function removes the required event listeners from a swfLoader
		 *  once a call to load has been resolved.
		 *  This is important to avoid orphaned objects and to properly handle
		 *  garbage collection dependencies.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function cleanLoadEvents(swfLoader:SWFLoader):void
		{
			swfLoader.removeEventListener(Event.COMPLETE, onLoadComplete);
			swfLoader.removeEventListener(IOErrorEvent.IO_ERROR, onLoadIOError);
			swfLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, onLoadSecurityError);
		}
		
		/**
		 *  EventHandler for successful load completion of a skin SWF.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onLoadComplete(evt:Event):void
		{
			var swfLoader:SWFLoader = evt.target as SWFLoader;
			cleanLoadEvents(swfLoader);
			
			libs[activeSWF] = swfLoader.content;
			dispatchEvent(new Event(Event.COMPLETE));
			CONFIG::logging{
				LogHelper.logger.debug("onLoadComplete: {0}", activeSWF);
			}
		}
		
		/** Event Handler for any IO errors loading a skin SWF.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onLoadIOError(evt:IOErrorEvent):void
		{
			var swfLoader:SWFLoader = evt.target as SWFLoader;
			cleanLoadEvents(swfLoader);
			LogHelper.logger.error("onLoadIOError: {0}", evt.text);
		}

		/** Event handler for security errors when loading a skin SWF.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected function onLoadSecurityError(evt:SecurityErrorEvent):void
		{
			var swfLoader:SWFLoader = evt.target as SWFLoader;
			cleanLoadEvents(swfLoader);
			LogHelper.logger.error("onLoadSecurityError: {0}", evt.text);
		}
		
		
		/** A mapping of SWF file names to their corresponding loaded instances.
		 *      
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		protected var libs:Object = new Object();
	}
}
