/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import com.adobe.portfolio.components.NavSkinnableComponent;
	import com.adobe.portfolio.utils.DoubleClickHandler;
	
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.NetStatusEvent;
	import flash.events.TimerEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundTransform;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	import flash.net.URLRequest;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.events.FlexEvent;
	import mx.logging.ILogger;
	import mx.logging.Log;
	import mx.states.State;
	
	import spark.components.Button;
	import spark.components.Group;
	import spark.components.Label;
	import spark.components.ToggleButton;
	import spark.components.mediaClasses.MuteButton;
	import spark.components.mediaClasses.ScrubBar;
	import spark.events.TrackBaseEvent;

	/**
	 * 
	 * The AudioPreview class extends NavSkinnableComponent to play audio content.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class AudioPreview extends NavSkinnableComponent
	{
		CONFIG::logging{
			private static const logger:ILogger = Log.getLogger("AudioPreview");
		}
		
		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the play/pause button.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var playPauseButton:ToggleButton;
		
		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the current time
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var currentTimeDisplay:Label;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the duration
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var durationDisplay:Label;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the hover play button.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var hoverPlay:Button;
		
		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the scrubBar.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var scrubBar:ScrubBar;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the player controls.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var playerControls:Group;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part to display the mute button.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public var muteButton:MuteButton;

		/**
		 *  The "stopped" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_STOPPED:String = "stopped";

		/**
		 *  The "playing" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_PLAYING:String = "playing";

		/**
		 *  The "paused" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_PAUSED:String = "paused";

		/**
		 *  The "stoppedHover" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_STOPPED_HOVER:String = "stoppedHover";

		/**
		 *  The "playingHover" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_PLAYING_HOVER:String = "playingHover";

		/**
		 *  The "pausedHover" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public static var STATE_PAUSED_HOVER:String = "pausedHover";
		
		/**
		 *  The "mediaError" state for AudioPreview and its skin
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.1
		 */
		public static var STATE_MEDIA_ERROR:String = "mediaError";

		protected static const MP4_MIMETYPE:String = "audio/mp4";
		protected static const DELTA_TO_END:Number = 100; // msec
		protected static const DONE_DELTA:int = 50; // (msec) for streams there may a gap between the stream duration according to the metadata and the actual duration
		
		protected static var toHover:Object;
		protected static var fromHover:Object;
		
		protected var doubleClickHandler:DoubleClickHandler;
		
		protected var sound:Sound;
		protected var soundChannel:SoundChannel;
		protected var scrubBarMouseCaptured:Boolean;
		protected var wasPlayingBeforeSeeking:Boolean;
		protected var scrubBarChanging:Boolean;
		protected var stream:NetStream;
		protected var lastStreamTime:Number;
		protected var updateTimer:Timer;

		private var _model:IPreviewModel;
		private var _source:Object;
		private var _position:Number;
		private var _volume:Number;
		private var _duration:Number;
		
		
		
		/**
		 *  Constructor
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public function AudioPreview()
		{
			_volume = 1;
			_position = NaN;
		}
		
		override protected function initStates():void 
		{
			initLookups();
			
			states = [
				new State({name: STATE_STOPPED}), 
				new State({name: STATE_PLAYING}),
				new State({name: STATE_PAUSED}),
				new State({name: STATE_STOPPED_HOVER}), 
				new State({name: STATE_PLAYING_HOVER}),
				new State({name: STATE_PAUSED_HOVER}),
				new State({name: STATE_MEDIA_ERROR})
			];
			
			currentState = STATE_STOPPED;
		}
		
		protected function initLookups():void {
			if (toHover && fromHover)
				return;
			
			toHover = new Object();
			toHover[STATE_STOPPED] = STATE_STOPPED_HOVER;
			toHover[STATE_PLAYING] = STATE_PLAYING_HOVER;
			toHover[STATE_PAUSED] = STATE_PAUSED_HOVER;

			fromHover = new Object();
			fromHover[STATE_STOPPED_HOVER] = STATE_STOPPED;
			fromHover[STATE_PLAYING_HOVER] = STATE_PLAYING;
			fromHover[STATE_PAUSED_HOVER] = STATE_PAUSED;
}
		
		override protected function attachSkin():void {
			super.attachSkin();
			addListener(this, MouseEvent.ROLL_OVER, rollOverHandler);
			addListener(this, MouseEvent.ROLL_OUT, rollOutHandler);
			doubleClickHandler = new DoubleClickHandler(this, mouseClickHandler);
		}
		
		override protected function detachSkin():void {
			super.detachSkin();
			if (doubleClickHandler)
				doubleClickHandler.removeAllListeners();
			removeListenAndWatch();
		}

		override protected function partAdded(partName:String, instance:Object):void 
		{
			super.partAdded(partName, instance);
			switch (instance) {
				case playPauseButton:
					addListener(playPauseButton, MouseEvent.CLICK, playPauseHandler);
					addListener(systemManager.getSandboxRoot(), KeyboardEvent.KEY_DOWN, keyboardHandler);
					break;
				
				case playerControls:
					addListener(playerControls, MouseEvent.CLICK, playerControlsMouseClickHandler);
					break;
				
				case scrubBar:
					addListener(scrubBar,TrackBaseEvent.THUMB_PRESS, scrubBar_thumbPressHandler);
					addListener(scrubBar,TrackBaseEvent.THUMB_RELEASE, scrubBar_thumbReleaseHandler);
					addListener(scrubBar,Event.CHANGE, scrubBar_changeHandler);
					addListener(scrubBar,FlexEvent.CHANGE_END, scrubBar_changeEndHandler);
					addListener(scrubBar,FlexEvent.CHANGE_START, scrubBar_changeStartHandler);
					addListener(scrubBar,MouseEvent.MOUSE_DOWN, scrubBar_mouseDownHandler);
					break;
				
				case muteButton:
					addListener(muteButton, FlexEvent.MUTED_CHANGE, muteButton_changeHandler);
					break;
				
				default:
					break;
			}
		}
		
		override public function set visible(value:Boolean):void
		{
			super.visible = value;
			if (!value)
				pause();
		}
		
		protected function mouseClickHandler(event:MouseEvent):void 
		{
			togglePlay();
		}

		protected function playerControlsMouseClickHandler(event:MouseEvent):void 
		{
			event.stopPropagation();
		}
		
		protected function scrubBar_changeStartHandler(event:Event):void
		{
			scrubBarChanging = true;
		}
		
		
		protected function scrubBar_mouseDownHandler(event:Event):void
		{
			// prevent card from being draggable when scrubbing the scrub bar
			event.stopImmediatePropagation();
		}
		
		protected function scrubBar_thumbPressHandler(event:TrackBaseEvent):void
		{
			scrubBarMouseCaptured = true;
			if (playing)
			{
				pause();
				wasPlayingBeforeSeeking = true;
			}
		}
		
		protected function scrubBar_thumbReleaseHandler(event:TrackBaseEvent):void
		{
			scrubBarMouseCaptured = false;
			if (wasPlayingBeforeSeeking)
			{
				play();
				wasPlayingBeforeSeeking = false;
			}
		}
		
		/**
		 *  @private
		 */
		protected function scrubBar_changeHandler(event:Event):void
		{
			seek(scrubBar.value * 1000);
		}
		
		protected function seek(value:Number):void 
		{
			position = value;
			updateCurrentTime();
			if (playing) {
				currentState = STATE_STOPPED;  // otherwise the play method returns early w/o changing the position
				play();
			}
		}
		
		protected function scrubBar_changeEndHandler(event:Event):void
		{      
			scrubBarChanging = false;
		}
	
		protected function muteButton_changeHandler(event:Event):void
		{ 
			volume = muteButton.muted ? 0 : 1;
		}
		
		protected function togglePlay():void {
			if (playing) {
				pause();
			} else {
				play();
			}
		}
		
		protected function keyboardHandler(event:KeyboardEvent):void 
		{
			if (event.isDefaultPrevented())
				return;
			if (event.keyCode != Keyboard.SPACE)
				return;
			togglePlay();
 		}
		
		protected function rollOverHandler(e:Event):void 
		{
			if (!doubleClickEnabled)
				return;
			
			var state:String = toHover[currentState];
			if (!state)
				return;
			currentState = state;
		}

		protected function rollOutHandler(e:Event):void 
		{
			if (scrubBarMouseCaptured)
				return;
			
			var state:String = fromHover[currentState];
			if (!state)
				return;
			currentState = state;
		}
		
		protected function playPauseHandler(event:Event):void 
		{
			if (!doubleClickEnabled)
				return;
			
			event.stopPropagation();
			togglePlay();
		}
		
		/**
		 *  Pause the playback of audio.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public function pause():void {
			if (soundChannel)
				soundChannel.stop();
			else if (stream) {
				stream.pause();
			}
			var paused:Boolean = position > 0;
			if (inHoverState())
				currentState = paused ? STATE_PAUSED_HOVER : STATE_STOPPED_HOVER;
			else
				currentState = paused ? STATE_PAUSED : STATE_STOPPED;
		}
		
		protected function inHoverState():Boolean
		{
			return fromHover.hasOwnProperty(currentState);
		}
		
		/**
		 *  Start the playback of audio.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10
		 *  @playerversion AIR 1.5
		 *  @productversion Portfolio 10.0
		 */
		public function play():void 
		{
			if (playing)
				return;
			
			if (soundChannel) {
				soundChannel.stop();
			} else if (stream) {
				stream.pause();
			}
			
			if (!sound && !stream)
				return;
			
			if (soundChannel) {
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, onTunePlaybackComplete);
			} 
			
			if (Math.abs(position - duration) < DELTA_TO_END) // reset to the beginning if we're asked to start near the end
				position = 0;
			
			if (sound) {
				soundChannel = sound.play(position, 0, new SoundTransform(volume));
			} else if (stream) {
				var soundTransform:SoundTransform = stream.soundTransform;
				soundTransform.volume = volume;
				stream.soundTransform = soundTransform;
				stream.seek(position/1000);
				stream.resume();
			}
			position = NaN;
			
			if (!soundChannel && !stream)
				return;
			
			if (soundChannel)
				soundChannel.addEventListener(Event.SOUND_COMPLETE, onTunePlaybackComplete, false, 0, true);
			
			if (!updateTimer) {
				updateTimer = new Timer(500); // update every 500 ms
				updateTimer.addEventListener(TimerEvent.TIMER, handleUpdateTimer, false, 0, true);
			}
			
			updateTimer.reset();
			updateTimer.start();
			currentState = STATE_PLAYING_HOVER;
			setFocus();
		}
		
		protected function onTunePlaybackComplete(event:Event = null):void 
		{
			if (soundChannel) {
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, onTunePlaybackComplete);
				soundChannel = null;
			}
			currentState = STATE_STOPPED;
		}
		
		protected function handleUpdateTimer(e:Event):void 
		{
			if (stream) {
				// check for playback complete.  if NetStreams start calling playStatus on done for audio, then use that instead
				if (lastStreamTime == stream.time && Math.abs(1000*stream.time-duration) < DONE_DELTA) {
					onTunePlaybackComplete();
				}
				lastStreamTime = stream.time;
			}
			if (!playing) {
				updateTimer.reset();
			}
			updateCurrentTime();
		}
		
		[Bindable]
		/**
		 * 
		 *  The current preview model.
		 * 
		 *  @see IPreviewModel
		 *  @see com.adobe.portfolio.events.PreviewEvent#MODEL_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get model():IPreviewModel
		{
			return _model;
		}
		
		public function set model(value:IPreviewModel):void
		{
			_model = value;
			autoWatch.removeAllWatchers();
			autoWatch.addWatcher(BindingUtils.bindProperty(this, "source", model, "source"));
		}
		
		[Bindable]
		/**
		 * The source data of an audio preview.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get source():Object
		{
			return _source;
		}

		protected function set source(value:Object):void
		{
			var mediaModel:IPreviewMediaModel = _model as IPreviewMediaModel;
			if (!mediaModel) {
				pause();
				_source = null;
				return;
			}
			
			if (!value || _source != value) {
				pause();
				sound = null;
				soundChannel = null;
				if (stream) {
					stream.close();
					stream.removeEventListener(NetStatusEvent.NET_STATUS, netStatusHandler);
					stream.removeEventListener(IOErrorEvent.IO_ERROR, onIOError);
				}
				stream = null;
				updateCurrentTime();
			}

			_source = value;

			if (value == "") {
				currentState = STATE_MEDIA_ERROR;
				return;
			}
			
			currentState = null;
			loadSound(mediaModel);
		}
		
		protected function loadSound(mediaModel:IPreviewMediaModel):void
		{
			if (!source) {
				return;
			}
			
			var url:String = source as String;
			if (!url) {
				CONFIG::logging{
					logger.warn("AudioPreview.loadSound: expected value to be a String");
				}
				return;
			}
			
			if (mediaModel.mimeType == MP4_MIMETYPE) {
				var connection:NetConnection = new NetConnection();
				connection.connect(null);
				stream = new NetStream(connection);
				stream.addEventListener(NetStatusEvent.NET_STATUS, netStatusHandler);
				stream.addEventListener(IOErrorEvent.IO_ERROR, onIOError);
				stream.client = new CustomClient(this);
				
				// for some reason you have to kick off the stream load (play) w/ a timer
				var t:Timer = new Timer(1);
				t.addEventListener(TimerEvent.TIMER,
					function (e:Event):void {
						t.removeEventListener(TimerEvent.TIMER, arguments.callee);
						t.stop();
						stream.play(url);
						stream.pause();
					}
				);
				t.start();
				
			} else {
				sound = new Sound();
				sound.addEventListener(Event.COMPLETE, 
					function (e:Event):void {
						sound.removeEventListener(Event.COMPLETE, arguments.callee);
						duration = sound.length;
					},
					false, 0, true);
				sound.addEventListener(IOErrorEvent.IO_ERROR,
					function (e:Event):void {
						sound.removeEventListener(IOErrorEvent.IO_ERROR, arguments.callee);
						currentState = STATE_MEDIA_ERROR;
					},
					false, 0, true);
				sound.load(new URLRequest(url));
			}
		}

		protected function netStatusHandler(event:NetStatusEvent):void {
//			CONFIG::logging{
//				logger.info("netStatus: " + event.info.code);
//			}
			switch (event.info.code) {
				case "NetConnection.Connect.Success":					
					break;
				case "NetStream.Play.StreamNotFound":
					CONFIG::logging{
						logger.info("Stream not found: " + event);
					}
					currentState = STATE_MEDIA_ERROR;
					break;
			}
		}

		protected function onIOError(event:Error):void 
		{
			currentState = STATE_MEDIA_ERROR;
			CONFIG::logging{
				logger.info("ioError: " + event);
			}
		}

		/**
		 *  @private
		 */
		protected function updateCurrentTime():void
		{
			var value:Number = position / 1000;
			if (currentTimeDisplay)
				currentTimeDisplay.text = formatTimeValue(value);
			
			if (scrubBar && !scrubBarChanging)
				scrubBar.value = value;
		} 
		
		protected function set position(val:Number):void 
		{
			_position = val;
		}
	
		protected function get position():Number 
		{
			var value:Number = _position;
			if (isNaN(value)) {
				if (soundChannel)
					value = soundChannel.position;
				else if (stream)
					value = stream.time * 1000;
				else
					value = 0;
			}
		
			return value;
		}

		/**
		 * The volume of the audio playback.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get volume():Number
		{
			return _volume;
		}
		
		public function set volume(value:Number):void
		{
			if (_volume == value)
				return;
			
			_volume = value;
			
			if (playing) {
				currentState = STATE_STOPPED;  // otherwise the play method returns early w/o changing the volume
				play();
			}
		}
		
		/**
		 * Whether the audio is playing or not.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get playing():Boolean
		{
			return currentState == STATE_PLAYING || currentState == STATE_PLAYING_HOVER;
		}
		
		/**
		 * Sets the current state of AudioPreview and its skin
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void 
		{
			if (currentState == STATE_MEDIA_ERROR && stateName != null)
				return;
			
			super.setCurrentState(stateName, playTransition);
			if (playPauseButton)
				playPauseButton.selected = playing;
		}

		
		protected function updateDuration():void
		{
			var durationInSecs:Number = duration/1000;
			
			if (durationDisplay)
				durationDisplay.text = formatTimeValue(durationInSecs);

			if (scrubBar)
				scrubBar.maximum = durationInSecs;
		}
		
		/**
		 *  Formats a time value, specified in seconds, into a String that 
		 *  gets used for <code>currentTime</code> and the <code>duration</code>.
		 * 
		 *  @param value Value in seconds of the time to format
		 * 
		 *  @return Formatted time value
		 */
		protected function formatTimeValue(value:Number):String
		{
			// default format: hours:minutes:seconds
			value = Math.round(value);
			
			var hours:uint = Math.floor(value/3600) % 24;
			var minutes:uint = Math.floor(value/60) % 60;
			var seconds:uint = value % 60;
			
			var result:String = "";
			if (hours != 0)
				result = hours + ":";
			
			if (result && minutes < 10)
				result += "0" + minutes + ":";
			else
				result += minutes + ":";
			
			if (seconds < 10)
				result += "0" + seconds;
			else
				result += seconds;
			
			return result;
		}

		public function get duration():Number
		{
			return _duration;
		}

		public function set duration(value:Number):void
		{
			_duration = value;
			updateDuration();
		}
	}
}
import com.adobe.portfolio.previews.AudioPreview;

import mx.logging.ILogger;
import mx.logging.Log;

class CustomClient {
	
	CONFIG::logging{
		private static const logger:ILogger = Log.getLogger("AudioPreview");
	}
	private var audioPreview:AudioPreview;
	
	public function CustomClient(audioPreview:AudioPreview):void {
		this.audioPreview = audioPreview;
	}
	
	public function onMetaData(info:Object):void {
//		CONFIG::logging{
//			logger.info("onMetaData");
//		}
		if (info.hasOwnProperty("duration"))
			audioPreview.duration = info.duration * 1000;
	}
	
	public function onPlayStatus(info:Object):void {
		CONFIG::logging{
			logger.info("onPlayStatus");	
		}
	}

}

