/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import mx.core.IVisualElement;

	/**
	 *  A PreviewEvent that is dispatched for a change to the preview model (IPreviewModel).
	 * 
	 *  @see IPreviewModel
	 *  @see com.adobe.portfolio.events.PreviewEvent#MODEL_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="modelChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 *  A PreviewEvent that is dispatched for a change to the aspect ratio of the preview.
	 * 
	 *  @see IPreviewModel
	 *  @see com.adobe.portfolio.events.PreviewEvent#ASPECT_RATIO_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="aspectRatioChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 
	 * The IPreview interface extends IVisualElement and defines the
	 * base functionality of Preview that can be accessed by other components.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public interface IPreview extends IVisualElement
	{
		[Bindable(name="modelChanged")]
		/**
		 * 
		 *  The current preview model.
		 * 
		 *  @see IPreviewModel
	 	 *  @see com.adobe.portfolio.events.PreviewEvent#MODEL_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get model():IPreviewModel;
		function set model(value:IPreviewModel):void;
		
		[Bindable(name="aspectRatioChanged")]
		/**
		 * 
		 *  The aspect ratio of the preview.
		 * 
	 	 *  @see com.adobe.portfolio.events.PreviewEvent#ASPECT_RATIO_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get aspectRatio():Number;
		function set aspectRatio(value:Number):void;
		
		/**
		 * 
		 * Whether this preview should use platform preview (as opposed to navigator preview).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get usePlatformPreview():Boolean;
		
		/**
		 * 
		 * Stop the playback of media content in this preview.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */		
		function stopPlayback():void;
		
		/**
		 * @inheritDoc
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.1
		 */
		function get enabled():Boolean;
		
		/**
		 *  Whether page navigation is enabled (via keyboard, mousewheel)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.1
		 */	
		function get pageNavigationEnabled():Boolean;

	}
}