/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import com.adobe.portfolio.events.PreviewEvent;
	
	import flash.display.Bitmap;
	import flash.events.IEventDispatcher;

	/**
	 * An event that is dispatched when the thumbnail changes.
	 * 
	 * @see com.adobe.portfolio.events.PreviewEvent.THUMBNAIL_CHANGED
	 * 
	 * @productversion Acrobat 10.0
	 */
	[Event(name="thumbnailChanged", type="com.adobe.portfolio.events.PreviewEvent")]
	
	/**
	 * An event that is dispatched when the source changes.
	 * 
	 * @see com.adobe.portfolio.events.PreviewEvent.SOURCE_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="sourceChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 
	 * The IPreviewModel interface extends IEventDispatcher and defines the
	 * base functionality of the model used by the Preview component.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public interface IPreviewModel extends IEventDispatcher
	{
		[Bindable (name="thumbnailChanged")]
		/**
		 * A thumbnail of the associated source object.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get thumbnail():Bitmap;
		function set thumbnail(value:Bitmap):void;

		[Bindable (name="sourceChanged")]
		/**
		 * The source data of a preview. The data may take a variety of forms, depending on the model type.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get source():Object;
		function set source(value:Object):void;
		
		/**
		 * The mimeType of the source data.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get mimeType():String;
	}
}