/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import flash.display.Bitmap;
	
	import mx.collections.ArrayList;
	import mx.collections.IList;

	/**
	 * 	An event that is dispatched when the document page count changes.
	 * 
	 * 	@see com.adobe.portfolio.events.PreviewEvent#PAGE_COUNT_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="pageCountChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 	An event that is dispatched when the user pages up or down in a multi-page document.
	 * 
	 * 	@see com.adobe.portfolio.events.PreviewEvent#PAGE_NUMBER_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="pageNumberChanged", type="com.adobe.portfolio.events.PreviewEvent")]
	
	/**
	 * 	An event that is dispatched when the thumbnail changes.
	 * 
	 *  @see com.adobe.portfolio.events.PreviewEvent#THUMBNAIL_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="thumbnailChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 	An event that is dispatched when the list of thumbnails for a multipage preview changes.
	 * 
	 *  @see com.adobe.portfolio.events.PreviewEvent#THUMBNAILS_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="thumbnailsChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 
	 * The IPreviewMultiPageModel interface extends IPreviewModel and defines the
	 * base functionality of PreviewMultiPageModel.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public interface IPreviewMultiPageModel extends IPreviewModel
	{
		[Bindable (name="pageCountChanged")]
		/**
		 *  The total page count of the preview (0 if unknown or not applicable)
		 * 
		 *  @see com.adobe.portfolio.events.PreviewEvent#PAGE_COUNT_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get pageCount():uint;
		function set pageCount(value:uint):void

		[Bindable (name="pageNumberChanged")]
		/**
		 *  The pageNumber of the preview (0 if unknown or not applicable)
		 * 
		 *  @see com.adobe.portfolio.events.PreviewEvent#PAGE_NUMBER_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get pageNumber():uint;
		function set pageNumber(value:uint):void;

		[Bindable (name="thumbnailsChanged")]
		/**
		 *  A list of thumbnails indexed to the page number (zero-based)
		 * 
	 	 *  @see com.adobe.portfolio.events.PreviewEvent#THUMBNAILS_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		function get thumbnails():ArrayList;		
		function set thumbnails(value:ArrayList):void;
	}
}