/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

/*
InternalAdobePatentID="B1281"
*/

package com.adobe.portfolio.previews
{
	import com.adobe.portfolio.components.NavSkinnableComponent;
	import com.adobe.portfolio.components.supportClasses.IconButton;
	import com.adobe.portfolio.events.PreviewEvent;
	
	import flash.display.DisplayObject;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.core.EventPriority;
	import mx.events.FlexEvent;
	import mx.states.State;
	
	import spark.components.TextInput;
	import spark.components.supportClasses.TextBase;
	
	/**
	 *  The PreviewHUD class extends NavSkinnableComponent to display the floating HUD (head-up-display) associated with a Preview.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class PreviewHUD extends NavSkinnableComponent
	{
		/**
		 *  The "singlePage" state of a PreviewHUD and its skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_SINGLE_PAGE:String = "singlePage";

		/**
		 *  The "multiPage" state of a PreviewHUD and its skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_MULTI_PAGE:String = "multiPage";
	
		/**
		 *  The "disabled" state of a PreviewHUD and its skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static const STATE_DISABLED: String = "disabled";
		
		/**
		 *  The Preview component associated with this PreviewHUD.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var preview:Preview;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part that shows the extract file button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var extractBtn:IconButton;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part that shows the info file button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var infoBtn:IconButton;
		
		[SkinPart(required="false")]
		/**
		 *  An optional skin part that shows the pageUp button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var pageUp:IconButton;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part that shows the pageDown button.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var pageDown:IconButton;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part used for entering page numbers.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var pageInput:TextInput;

		[SkinPart(required="false")]
		/**
		 *  An optional skin part used for displaying the total number of pages.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public var pageTotal:TextBase;
		
		protected var fadeTimer:Timer;
		protected var rollOver:Boolean = false;
		protected var previewModel:IPreviewModel;
		protected var xMouseDown:Number;
		protected var yMouseDown:Number;
		protected var xOrig:Number;
		protected var yOrig:Number;
		protected var isDraggingHUD: Boolean;
		protected var pageInputActive:Boolean;

		private var _pageTotalNumber:uint;
		private var _pageTotalString:String;
		private var _pageNumberString:String;
		private var _previewMultiPageModel:IPreviewMultiPageModel;

		/**
		 *  Constructor.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function PreviewHUD()
		{
			super();
			fadeTimer = new Timer(2000);
			addListener(fadeTimer, TimerEvent.TIMER, fadeOut);
		}
		
		override protected function initStates():void {
			states = [
				new State({name: STATE_DISABLED}),
				new State({name: STATE_SINGLE_PAGE}), 
				new State({name: STATE_MULTI_PAGE})
			];
			currentState = STATE_DISABLED;
		}

		override protected function partAdded(partName:String, instance:Object):void {
			super.partAdded(partName, instance);
			var eventDispatcher:IEventDispatcher = instance as IEventDispatcher;
			switch (eventDispatcher) {
				case extractBtn:
					addListener(eventDispatcher, MouseEvent.CLICK, handleFileExtract);
					break;
				case infoBtn:
					addListener(eventDispatcher, MouseEvent.CLICK, handleInfo);
					break;
				case pageUp:
					addListener(eventDispatcher, MouseEvent.CLICK, handlePageUp);
					break;
				case pageDown:
					addListener(eventDispatcher, MouseEvent.CLICK, handlePageDown);
					break;
				case pageInput:
					addListener(eventDispatcher, FlexEvent.ENTER, handlePageInput);
					addListener(eventDispatcher, MouseEvent.MOUSE_DOWN, handlePageInputMouseDown);
					break;
			}
		}
		
		override protected function attachSkin():void {
			super.attachSkin();
			addListener(this, MouseEvent.ROLL_OVER, handleRollOver);
			addListener(this, MouseEvent.ROLL_OUT, handleRollOut);
			addListener(this, MouseEvent.MOUSE_DOWN, handleMouseDown, false, EventPriority.DEFAULT_HANDLER);
			addListener(parent, MouseEvent.ROLL_OVER, handleParentRollOver);
			addListener(parent, MouseEvent.MOUSE_MOVE, handleParentMouseMove);
			addListener(parent, MouseEvent.ROLL_OUT, handleParentRollOut);
			addWatcher(BindingUtils.bindSetter(pageTotalChanged, this, ["previewMultiPageModel", "pageCount"]));
			addWatcher(BindingUtils.bindSetter(pageNumberChanged, this, ["previewMultiPageModel", "pageNumber"]));
			addWatcher(BindingUtils.bindSetter(modelChanged, preview, "model"));
		}
		
		/**
		 *  Sets the current state of the HUD and its skin.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		override public function setCurrentState(stateName:String, playTransition:Boolean = true):void {
			
			if (stateName != STATE_DISABLED) {
				startFadeOutTimer();
			}
			if (stateName == currentState) {
//				LogHelper.logger.debug("setCurrentState: stateName= " + stateName + " : early return");
				return;
			}
//			LogHelper.logger.debug("setCurrentState: stateName= " + stateName);
			
			super.setCurrentState(stateName, playTransition);
		}
		
		/**
		 *  Updates the current state of the HUD.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function updateCurrentState():void {
			setCurrentState(visibleState);
		}
		
		protected function pageNumberChanged(val:uint):void {
			pageNumberString = val.toString();
		}

		protected function pageTotalChanged(val:uint):void {
			pageTotalNumber = val;
			pageTotalString = "/ " + val;
		}
		
		protected function modelChanged(val:IPreviewModel):void {
			previewModel = val;
			previewMultiPageModel = val as IPreviewMultiPageModel;
			updateCurrentState();
		}
		
		protected function handleFileExtract(event:MouseEvent):void {
			event.preventDefault();
			preview.dispatchEvent(new PreviewEvent(PreviewEvent.HANDLE_FILE_EXTRACT));
		}
		
		protected function handleInfo(event:MouseEvent):void {
			event.preventDefault();
			preview.dispatchEvent(new PreviewEvent(PreviewEvent.HANDLE_FILE_INFO));
		}
		
		protected function handlePageUp(event:MouseEvent):void {
			event.preventDefault();
			if (previewMultiPageModel)
				previewMultiPageModel.pageNumber--;
		}
		
		protected function handlePageDown(event:MouseEvent):void {
			event.preventDefault();
			if (previewMultiPageModel)
				previewMultiPageModel.pageNumber++;
		}
		
		protected function handlePageInput(event:Event):void {
			event.preventDefault();
			if (previewMultiPageModel) {
				previewMultiPageModel.pageNumber = parseInt(pageInput.text);
				preview.setFocus();
				pageInputActive = false;
			}
		}
		
		protected function handlePageInputMouseDown(event:Event):void {
			event.stopPropagation();
			pageInputActive = true;
		}
		
		protected function handleRollOver(event:MouseEvent):void {
//			LogHelper.logger.debug("handleRollOver");
			rollOver = true;
			updateCurrentState();
		}

		protected function handleParentRollOver(event:MouseEvent):void {
//			LogHelper.logger.debug("handlePrevewRollOver");
			updateCurrentState();
		}
		
		protected function handleRollOut(event:MouseEvent):void {
//			LogHelper.logger.debug("handleRollOut");
			rollOver = false;
			startFadeOutTimer(300);
		}
		
		protected function handleParentRollOut(event:MouseEvent):void {
//			LogHelper.logger.debug("handleParentRollOut");
			startFadeOutTimer(300);
		}

		protected function handleParentMouseMove(event:MouseEvent):void {
//			LogHelper.logger.debug("handleParentMouseMove");
			updateCurrentState();
		}
		
		protected function handleMouseDown(event:MouseEvent):void {
			if (event.isDefaultPrevented())
				return;
			isDraggingHUD = true;
			xMouseDown = event.stageX;
			yMouseDown = event.stageY;
			xOrig = x;
			yOrig = y;
			var root:DisplayObject = systemManager.getSandboxRoot();
			if (root) {
				root.addEventListener(MouseEvent.MOUSE_MOVE, handleMouseMove, false, 0, true);
				root.addEventListener(MouseEvent.MOUSE_UP, handleMouseUp, false, 0, true);
			}
		}
		
		protected function handleMouseMove(event:MouseEvent):void {
			var deltaX:Number = event.stageX - xMouseDown;
			var deltaY:Number = event.stageY - yMouseDown;
			bottom = null;
			horizontalCenter = null;
			
			var newX:Number = Math.max(xOrig + deltaX, 0);
			newX = Math.min(newX, parent.width - width);
			x = newX;
			
			var newY:Number = Math.max(yOrig + deltaY, 0);
			newY = Math.min(newY, parent.height - height);
			y = newY;
		}
		
		protected function handleMouseUp(event:MouseEvent):void
		{
			var root:DisplayObject = systemManager.getSandboxRoot();
			if (root) {
				root.removeEventListener(MouseEvent.MOUSE_MOVE, handleMouseMove);
				root.removeEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
			}
			isDraggingHUD = false;
		}
		
		protected function globalToParentLocal(x:Number, y:Number):Point {
			return parent.globalToLocal(new Point(x,y));
		}
		
		protected function get visibleState():String {
			return previewMultiPageModel ? STATE_MULTI_PAGE : 
				previewModel ? STATE_SINGLE_PAGE : STATE_DISABLED;
		}
		
		[Bindable]
		/**
		 *  The current previewMultiPageModel (may return null).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get previewMultiPageModel():IPreviewMultiPageModel
		{
			return _previewMultiPageModel;
		}

		public function set previewMultiPageModel(value:IPreviewMultiPageModel):void
		{
			_previewMultiPageModel = value;
		}
		
		[Bindable]
		/**
		 *  The current pageNumber in String form.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function set pageNumberString(val:String):void {
			_pageNumberString = val;
		}
		public function get pageNumberString():String {
			return _pageNumberString;
		}

		[Bindable]
		/**
		 *  The current page total (count) in String form.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function set pageTotalString(val:String):void {
			_pageTotalString = val;
		}
		public function get pageTotalString():String {
			return _pageTotalString;
		}
		
		[Bindable]
		/**
		 *  The current page total (count).
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get pageTotalNumber():uint
		{
			return _pageTotalNumber;
		}
		
		public function set pageTotalNumber(value:uint):void
		{
			_pageTotalNumber = value;
		}
		
		protected function startFadeOutTimer(delay:int = 3000):void {
			
			if (isDraggingHUD)
				return;
			
//			LogHelper.logger.debug("startFadeOutTimer: delay= " + delay);
			fadeTimer.delay = delay;
			fadeTimer.reset();
			fadeTimer.start();
		}
		
		protected function fadeOut(e:TimerEvent):void {
			var t:Timer = e.target as Timer;
			t.stop();
//			LogHelper.logger.debug("fadeOut: rollOver= " + rollOver);
			if (rollOver || pageInputActive)
				return;

			// start fade effect
			currentState = "disabled";
		}
	}
}