/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import com.adobe.portfolio.events.PreviewEvent;
	
	import flash.display.Bitmap;
	import flash.events.EventDispatcher;
	
	import mx.logging.ILogger;
	import mx.logging.Log;
	import mx.utils.UIDUtil;
	
	/**
	 * An event that is dispatched when the thumbnail changes.
	 * 
	 * @see com.adobe.portfolio.events.PreviewEvent.THUMBNAIL_CHANGED
	 * 
	 * @productversion Acrobat 10.0
	 */
	[Event(name="thumbnailChanged", type="com.adobe.portfolio.events.PreviewEvent")]
	
	/**
	 * An event that is dispatched when the source changes.
	 * 
	 * @see com.adobe.portfolio.events.PreviewEvent.SOURCE_CHANGED
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	[Event(name="sourceChanged", type="com.adobe.portfolio.events.PreviewEvent")]

	/**
	 * 
	 * The PreviewModel class implements the IPreviewModel interface and is required for the Preview component.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PreviewModel extends EventDispatcher implements IPreviewModel
	{
		protected static const logger:ILogger = Log.getLogger("PreviewModel");

		private var _thumbnail:Bitmap;
		private var _source:Object;
		private var _mimeType:String;
		
		public function PreviewModel()
		{
			super();
//			logger.debug("ctor: " + UID);
		}
		
		
		[Bindable(name="thumbnailChanged")]
		/**
		 * A thumbnail of the associated source object.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get thumbnail():Bitmap
		{
			return _thumbnail;
		}
		
		public function set thumbnail(value:Bitmap):void
		{
			if (_thumbnail == value)
				return;
			
//			logger.debug("set thumbnail: " + this + " source: " + source);
//			logger.debug("set thumbnail: " + this + " value: " + (value ? (value.width + " x " + value.height) : "null"));
			_thumbnail = value;
			dispatchEvent(new PreviewEvent(PreviewEvent.THUMBNAIL_CHANGED));
		}
		
		[Bindable(name="sourceChanged")]
		/**
		 * The source data of a preview. The data may take a variety of forms, depending on the model type.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get source():Object
		{
			return _source;
		}

		public function set source(value:Object):void
		{
			if (_source == value)
				return;
			
//			logger.debug(UID + "PreviewModel: set source: value: " + value);
			_source = value;
			dispatchEvent(new PreviewEvent(PreviewEvent.SOURCE_CHANGED));
		}
		
		protected function get UID():String {
			var s:String = UIDUtil.getUID(this);
			s = s.slice(s.length-40);
			return s;
		}
		
		public function set mimeType(value:String):void
		{
			_mimeType = value;
		}
		
		public function get mimeType():String {
			return _mimeType;
		}
		
		override public function toString():String
		{
			return super.toString() + ": " + UID;
		}
	}
}