/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.previews
{
	import com.adobe.portfolio.events.PreviewEvent;
	
	import flash.display.Bitmap;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import mx.collections.ArrayList;
	import mx.collections.IList;
	import mx.collections.ListCollectionView;
	
	/**
	 * 
	 * The PreviewMultiPageModel class implements the IPreviewMultiPageModel interface and is required for the Preview component.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PreviewMultiPageModel extends PreviewModel implements IPreviewMultiPageModel
	{

		private var _pageCount:uint;
		private var _pageNumber:uint;
		private var _thumbnails:ArrayList;
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function PreviewMultiPageModel()
		{
			_thumbnails = new ArrayList(new Array(2));
		}
		
		[Bindable (name="pageCountChanged")]
		/**
		 *  The total page count of the preview (0 if unknown or not applicable)
		 * 
		 *  @see com.adobe.portfolio.events.PreviewEvent#PAGE_COUNT_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get pageCount():uint
		{
			return _pageCount;
		}
		
		public function set pageCount(value:uint):void
		{
			if (_pageCount == value)
				return;
			
			_pageCount = value;
			dispatchEvent(new PreviewEvent(PreviewEvent.PAGE_COUNT_CHANGED));
		}
		
		[Bindable (name="pageNumberChanged")]
		/**
		 *  The pageNumber of the preview (0 if unknown or not applicable)
		 * 
		 *  @see com.adobe.portfolio.events.PreviewEvent#PAGE_NUMBER_CHANGED
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get pageNumber():uint
		{
			return _pageNumber;
		}
		
		public function set pageNumber(value:uint):void
		{
			if (value < 1 || value > pageCount)
				return;
			if (_pageNumber == value)
				return;
			
			_pageNumber = value;
			dispatchEvent(new PreviewEvent(PreviewEvent.PAGE_NUMBER_CHANGED));
		}
		
		[Bindable(name="thumbnailsChanged")]
		/**
		 *  A list of thumbnails indexed to the page number (zero-based)
		 * 
		 *  @see com.adobe.portfolio.events.PreviewEvent#THUMBNAILS_CHANGED
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function get thumbnails():ArrayList
		{
			return _thumbnails;
		}
		
		public function set thumbnails(value:ArrayList):void
		{
			if (_thumbnails == value)
				return;
			
			_thumbnails = value;
			dispatchEvent(new PreviewEvent(PreviewEvent.THUMBNAILS_CHANGED));
		}
		
	}
}
