/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.properties
{
	import com.adobe.portfolio.components.supportClasses.CanvasItemData;
	
	import flash.events.IEventDispatcher;
	
	/**
	 *  This class implements the properties for a preset layout to be used by
	 * 	CanvasList.
	 * 
	 * 	@see CanvasList
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class CanvasTemplate extends PropertyValue
	{
		/**
		 *  The XML name used for the canvas template element
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const XML_ELEMENT_NAME:String = "CanvasTemplate";

		/**
		 *  Constructor
		 * 
		 *  @param target The Navigator's event dispatcher
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function CanvasTemplate(target:IEventDispatcher=null)
		{
			super(target);
		}
		
		/**
		 *  A vector of canvas items to be included by this preset
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public var canvasItems:Vector.<CanvasItemData>;
		
		/**
		 *  Loads the specified values from XML and creates
		 *  canvasItems for the elements in the XML value.
		 * 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function loadFromXML(valueXML:XML):Boolean
		{
			if (super.loadFromXML(valueXML)) {
				canvasItems = new Vector.<CanvasItemData>();
				var items:XMLList = valueXML.elements(CanvasItemData.XML_ELEMENT_NAME);
				if (items != null) {
					var subElements:XMLList = valueXML.elements();
					for each (var subElement:XML in subElements) {
						var canvasItemData:CanvasItemData = new CanvasItemData();
						if (!canvasItemData.loadFromXML(subElement)) {
							continue;
						} 
						canvasItems.push(canvasItemData);
					}
				} 
				return true;
			} else {
				return false;
			}
		}
		
		/**
		 *  Returns the XML element name for the canvas template.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function getXMLElementName():String
		{
			return XML_ELEMENT_NAME;
		}
		
		/**
		 *  Creates XML items for each canvas item in canvasItems
		 *  and appends these to the specified propertyXML
		 * 
		 *  @param propertyXML the XML property that is to contain
		 *  the list of canvas items.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function setXMLElements(propertyXML:XML):void
		{
			if (propertyXML == null) {
				return;
			}
			super.setXMLElements(propertyXML);
			for each (var canvasItemData:CanvasItemData in canvasItems) {
				var itemXML:XML = canvasItemData.generateXML();
				propertyXML.appendChild(itemXML);
			}
		}
	}
}