/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.properties
{
	import com.adobe.portfolio.api.PropertyType;
	
	import flash.events.IEventDispatcher;
	
	/**
	 *  This class handles property values that are actions.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PropertyActionValue extends PropertyValue
	{
		/**
		 *  The XML element name for property actions
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const XML_ELEMENT_NAME:String = "PropertyAction";
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function PropertyActionValue(target:IEventDispatcher=null)
		{
			super(target);
		}
		
		/**
		 *  @inheritDoc
		 * 
		 * 	@see PropertyType#ACTION
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function get type():String
		{
			return PropertyType.ACTION;
		}

		/**
		 *  Set the property's action value. Changing the value to a non-zero uint will generally
		 * 	trigger the action, as well as any select effects on the property.
		 * 
		 * 	@see PropertyValue#playSelectEffects
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function set value(newValue:Object):void
		{
			super.value = newValue;
			
			if ((value as uint) != 0) {
				// Play any select effects when the user invokes an action
				playSelectEffects();
			}
		}

		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function loadFromXML(valueXML:XML):Boolean
		{
			if (super.loadFromXML(valueXML)) {
				// Actions don't store values; always initialize to 0
				value = 0;
				return true;
			} else {
				return false;
			}
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function getXMLElementName():String
		{
			return XML_ELEMENT_NAME;
		}
		
		/**
		 *  Overrides the normal function to set the dirty flag.
		 *  Action values aren't stored, so there is nothing to mark as dirty.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function set dirty(value:Boolean):void
		{
			// Action values aren't stored, and shouldn't dirty the environment
			return;
		}
	}
}