/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.properties
{
	import com.adobe.portfolio.api.PropertyType;
	
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.events.IEventDispatcher;
	
	import mx.logging.ILogger;
	
	/**
	 *  This class handles property values that are Booleans.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class PropertyBooleanValue extends PropertyValue
	{
		/**
		 *  The XML element name for Boolean property values
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public static const XML_ELEMENT_NAME:String = "PropertyBoolean";
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function PropertyBooleanValue(target:IEventDispatcher=null)
		{
			super(target);
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function get type():String
		{
			return PropertyType.BOOLEAN;
		}
		
		[Bindable]
		/**
		 *  The Boolean property value
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function get booleanValue():Boolean
		{
			return value as Boolean;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		public function set booleanValue(newValue:Boolean):void
		{
			super.value = newValue;
			if (newValue) {
				// Play any select effects when the user sets a boolean property to true
				playSelectEffects();
			}
		}
		
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function set value(newValue:Object):void
		{
			// Force all value assignments to trigger a booleanValue PropertyChangeEvent as well
			booleanValue = newValue as Boolean;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function loadFromXML(valueXML:XML):Boolean
		{
			if (super.loadFromXML(valueXML)) {
				++dontPlayEffectsCounter;
//				booleanValue = ((valueXML.@value as String) == "true");
				var xmlBooleanValue:String = valueXML.@value;
				booleanValue = (xmlBooleanValue.toLowerCase() == "true");
				--dontPlayEffectsCounter;
				return true;
			} else {
				return false;
			}
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function getXMLElementName():String
		{
			return XML_ELEMENT_NAME;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override protected function setXMLElements(propertyXML:XML):void
		{
			if (propertyXML == null) {
				return;
			}
			super.setXMLElements(propertyXML);
			propertyXML.@value = (value as Boolean) ? "true" : "false";
		}
		
		/**
		 *  Sets the propertyBooleanValue to the value represented by the specified string.
		 * 
		 *  @param valueString This string should be either "true" or "false" (case insensitive)
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */
		override public function setValueFromString(valueString:String):Boolean
		{
			if (valueString == null) {
				return false;
			}
			switch (valueString.toLowerCase()) {
				case "true":
					value = true;
					return true;
				case "false":
					value = false;
					return true;
				default:
					CONFIG::logging{
						LogHelper.logger.warn("Unexpected string value for a boolean: "+valueString);
					}
					return false;
			}
		}
	}
}