/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.properties
{
	import com.adobe.portfolio.api.PropertyType;
	
	import flash.events.IEventDispatcher;
	
	/**
	 *  This class handles property values that are character strings.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public class PropertyStringValue extends PropertyValue
	{
		/**
		 *  The XML element name for string property values
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public static const XML_ELEMENT_NAME:String = "PropertyString";
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function PropertyStringValue(target:IEventDispatcher=null)
		{
			super(target);
		}

		/**
		 *  @inheritDoc
		 * 
		 * 	@see PropertyType#STRING
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function get type():String
		{
			return PropertyType.STRING;
		}
		
		[Bindable]
		/**
		 *  The value of the property as a string.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get stringValue():String
		{
			return value as String;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function set stringValue(newValue:String):void
		{
			super.value = newValue;
		}
		
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function set value(newValue:Object):void
		{
			// Force all value assignments to trigger a stringValue PropertyChangeEvent as well
			stringValue = newValue as String;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function loadFromXML(valueXML:XML):Boolean
		{
			if (super.loadFromXML(valueXML)) {
				stringValue = valueXML.@value;
				return true;
			} else {
				return false;
			}
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override protected function getXMLElementName():String
		{
			return XML_ELEMENT_NAME;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override protected function setXMLElements(propertyXML:XML):void
		{
			if (propertyXML == null) {
				return;
			}
			super.setXMLElements(propertyXML);
			propertyXML.@value = stringValue;
		}
		
		/**
		 *  Set the property string value equal to the specified string.
		 * 
		 *  @param valueString  The string you want to use as the value.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		override public function setValueFromString(valueString:String):Boolean
		{
			if (valueString == null) {
				return false;
			}
			stringValue = valueString;
			return true;
		}
	}
}