/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.properties
{
	import com.adobe.portfolio.api.ICollection;
	import com.adobe.portfolio.api.IProperty;
	import com.adobe.portfolio.api.PropertyType;
	import com.adobe.portfolio.managers.CollectionManager;
	import com.adobe.portfolio.managers.PropertyManager;
	import com.adobe.portfolio.utils.LogHelper;
	
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.xml.XMLNode;
	
	import mx.events.PropertyChangeEvent;
	import mx.logging.ILogger;
	
	/**
	 *  This interface implements IProperty and provides support
	 *  for properties displayed in the user interface.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */	
	public class PropertyValue extends EventDispatcher implements IProperty
	{
		/**
		 *  The XML element name for generic property values
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public static const XML_ELEMENT_NAME:String = "PropertyValue";
		
		/**
		 *  Constructor
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function PropertyValue(target:IEventDispatcher=null)
		{
			super(target);
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get type():String
		{
			return PropertyType.UNDEFINED;
		}
		
		private var _name:String = "";
		[Bindable]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get name():String
		{
			return _name;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set name(value:String):void
		{
			_name = value;
		}
		
		private var _value:Object;
		[Bindable (event="propertyChange")]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get value():Object
		{
			return _value;
		}
		
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function set value(newValue:Object):void
		{
			if (_value != newValue) {
				var oldValue:Object = _value;
				_value = newValue;
				if (!_initializing) {
					// If we're not initializing this property, dirty it when its value changes
					dirty = true;
				}
				dispatchEvent(PropertyChangeEvent.createUpdateEvent(this, "value", oldValue, _value));
			}
		}
		
		private var _label:String = "";
		[Bindable]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get label():String
		{
			return _label;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set label(value:String):void
		{
			_label = value;
		}
		
		/**
		 *  The string used as a key to request a localized label string.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var labelKey:String = "";
		
		/**
		 *  The default label string to be used when the localized label is unavailable.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var labelDefault:String = "";
		
		private var _description:String = "";
		[Bindable]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get description():String
		{
			return _description;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set description(value:String):void
		{
			_description = value;
		}
		
		/**
		 *  The string used as a key to request a localized description string.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var descriptionKey:String = "";
		
		/**
		 *  The default description string to be used when the localized description is unavailable.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var descriptionDefault:String = "";
		
		private var _enabled:Boolean = true;
		[Bindable]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get enabled():Boolean
		{
			return _enabled;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set enabled(value:Boolean):void
		{
			_enabled = value;
		}
		
		private var _visible:Boolean = true;
		[Bindable]
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get visible():Boolean
		{
			return _visible;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set visible(value:Boolean):void
		{
			_visible = value;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get subvalues():Array
		{
			return null;
		}
		
		/** 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get subvaluesType():String
		{
			return PropertyType.UNDEFINED;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get subvaluesLocked():Boolean
		{
			return true;
		}
		
		/** 
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get hasMinMax():Boolean
		{
			return false;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get minValue():Number
		{
			return Number.NEGATIVE_INFINITY;
		}
		
		/**
		 *  @inheritDoc
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get maxValue():Number
		{
			return Number.POSITIVE_INFINITY;
		}

		/**
		 *  The parent object of this property.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var parent:Object;
		
		private var _initializing:Boolean = false;
		/**
		 *  Returns <code>true</code> if the property is in an initializing state,
		 *  <code>false</code> otherwise. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function get initializing():Boolean
		{
			return _initializing;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function set initializing(value:Boolean):void
		{
			if (value == _initializing) {
				return;
			}
			_initializing = value;
		}
		
		private var _dirty:Boolean = false;
		/**
		 *  Returns <code>true</code> if the property has been modified and needs to be saved,
		 *  <code>false</code> otherwise. 
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function get dirty():Boolean
		{
			return _dirty;
		}
		/**
		 *  @private
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function set dirty(value:Boolean):void
		{
			if (_dirty != value) {
				CONFIG::logging{
					if (initializing)
						LogHelper.logger.warn("Properties shouldn't be dirtied during init time");
				}
				_dirty = value;
				if (_dirty) {
					if (parent is PropertyValue) {
						(parent as PropertyValue).dirty = true;
					} else if (parent is PropertyContext) {
						(parent as PropertyContext).dirty = true;
					}
				}
			}
		}

		/**
		 *  A counter used to keep track of when to run the side effects.
		 *  There are two types of effects:
		 *  <ol>
		 *     <li>Changes in properties</li>
		 *     <li>Changes in color values for the color palette</li>
		 *  </ol>
		 *  <p>As long as dontPlayEffectsCounter is not 0
		 *  playSelectEffects() will not run the effects.</p>
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected var dontPlayEffectsCounter:uint = 0;
		
		private var selectEffectArray:Array = new Array();
		/**
		 *  Run all the effects in the selectEffectArray
		 *  If dontPlayEffectsCounter is false (not 0)
		 *  playSelectEffects() will not run the effects.
		 *  There are two types of effects:
		 *  <ol>
		 *     <li>Changes in properties</li>
		 *     <li>Changes in color values for the color palette</li>
		 *  </ol>
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function playSelectEffects():void
		{
			if (dontPlayEffectsCounter) {
				return;
			}
			++dontPlayEffectsCounter;
			var paletteUpdates:Object = null;
			for each (var sideEffect:Object in selectEffectArray) {
				switch (sideEffect.type) {
					case "propertyValue":
						var property:PropertyValue = PropertyManager.instance.getNavProperty(sideEffect.context, sideEffect.property);
						if (property != null) {
							property.setValueFromString(sideEffect.value);
						}
						break;
					case "colorValue":
						var numberValue:Number = parseInt(sideEffect.value);
						if (isNaN(numberValue)) {
							CONFIG::logging{
								LogHelper.logger.warn("Invalid color value for side effect");
							}
							continue;
						}
						if (paletteUpdates == null) {
							paletteUpdates = new Object();
						}
						paletteUpdates[sideEffect.property] = numberValue;
						break;
					default:
						CONFIG::logging{
							LogHelper.logger.warn("Unrecognized side effect type: "+sideEffect.type);
						}
						break
				}
			}
			if (paletteUpdates != null) {
				CollectionManager.instance.updateColorPalette(paletteUpdates);
			}
			--dontPlayEffectsCounter;
		}
		
		/**
		 *  Load the specified value from an XML object.
		 *  If possible, the localized label and description will be used.
		 * 
		 *  @return <code>true</code> if the property was successfully loaded,
		 *  <code>false</code> otherwise.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function loadFromXML(valueXML:XML):Boolean
		{
			name = valueXML.@name;
			
			var xmlBooleanValue:String = valueXML.@visible;
			visible = (xmlBooleanValue.toLowerCase() != "false");
			xmlBooleanValue = valueXML.@enabled;
			enabled = (xmlBooleanValue.toLowerCase() != "false");
			
			var hostCollection:ICollection = (CollectionManager.instance.host != null) ? CollectionManager.instance.host.collection : null;
			// Try to get a localized label
			labelKey = valueXML.@labelKey;
			if ((labelKey != null) &&
				(labelKey != "") &&
				(hostCollection != null))
			{
				labelDefault = valueXML.@label;
				label = hostCollection.getLocalizedString(labelKey, labelDefault);
			} else {
				labelDefault = "";
				label = valueXML.@label;
			}
			
			// Try to get a localized description
			descriptionKey = valueXML.@descriptionKey;
			if ((descriptionKey != null) &&
				(descriptionKey != "") &&
				(hostCollection != null))
			{
				descriptionDefault = valueXML.@description;
				description = hostCollection.getLocalizedString(descriptionKey, descriptionDefault);
			} else {
				descriptionDefault = "";
				description = valueXML.@description;
			}
			
			var selectEffects:XMLList = valueXML.SelectEffect;
			for each (var sideEffect:XML in selectEffects) {
				var typeName:String = sideEffect.@type;
				var contextName:String = sideEffect.@context;
				var propertyName:String = sideEffect.@property;
				var propertyValue:String = sideEffect.@value;
				selectEffectArray.push({type: typeName, context: contextName, property: propertyName, value: propertyValue});
			}
			
			return true;
		}
		
		/**
		 *  Create an XML object with all its XML elements for this property.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function generateXML():XML
		{
			var newXML:XML = new XML("<" + getXMLElementName()+ "/>");
			setXMLElements(newXML);
			
			dirty = false;
			
			return newXML;
		}
		
		/**
		 *  Returns the XML element name for this property.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function getXMLElementName():String
		{
			return XML_ELEMENT_NAME;
		}
		
		/**
		 *  Sets all the XML elements for this property, such as
		 *  enabled, visible, label, and description.  Also, set any
		 *  SelectEffect XML objects in the SelectEffectsArray.
		 *  
		 *  @param the XML object representing the desired property
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		protected function setXMLElements(propertyXML:XML):void
		{
			if (propertyXML == null) {
				return;
			}
			propertyXML.@name = name;
			propertyXML.@enabled = enabled ? "true" : "false";
			propertyXML.@visible = visible ? "true" : "false";
			
			if ((labelKey != null) && (labelKey != "")) {
				propertyXML.@label = labelDefault;
				propertyXML.@labelKey = labelKey;	
			} else {
				propertyXML.@label = label;
			}
			if ((descriptionKey != null) && (descriptionKey != "")) {
				propertyXML.@description = descriptionDefault;
				propertyXML.@descriptionKey = descriptionKey;	
			} else {
				propertyXML.@description = description;
			}
			
			for each (var selectEffect:Object in selectEffectArray) {
				var selectEffectXML:XML = <SelectEffect/>;
				selectEffectXML.@type = selectEffect.type;
				selectEffectXML.@property = selectEffect.property;
				selectEffectXML.@value = selectEffect.value;
				if (selectEffect.type == "propertyValue") {
					selectEffectXML.@context = selectEffect.context;
				}
				
				propertyXML.appendChild(selectEffectXML);
			}
		}
		
		/**
		 *  Sets the value of this property to a string-based representation of the
		 * 	desired property value.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public function setValueFromString(valueString:String):Boolean
		{
			return false;
		}
		
		/**
		 *  Create a new PropertyValue instance of the correct type and
		 *  assign <code>parent</code> as its parent value.
		 * 
		 *  @param valueXML the value to set
		 *  @param parent the parent object for the property
		 * 
		 *  @return the property value created
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 *
		 */	
		public static function createPropertyValue(valueXML:XML, parent:Object):PropertyValue
		{
			var newValue:PropertyValue = null;
			var valueType:String = valueXML.localName();
			var newName:String = valueXML.@name;
			if ((valueType != "") && (newName != "")) {
				switch (valueType) {
					case PropertyBooleanValue.XML_ELEMENT_NAME:
						newValue = new PropertyBooleanValue();
						break;
					case PropertyListValue.XML_ELEMENT_NAME:
						newValue = new PropertyListValue();
						break;
					case PropertyNumberValue.XML_ELEMENT_NAME:
						newValue = new PropertyNumberValue();
						break;
					case PropertyColorValue.XML_ELEMENT_NAME:
						newValue = new PropertyColorValue();
						break;
					case PropertyBinaryValue.XML_ELEMENT_NAME:
						newValue = new PropertyBinaryValue();
						break;
					case PropertyStringValue.XML_ELEMENT_NAME:
						newValue = new PropertyStringValue();
						break;
					case SkinValue.XML_ELEMENT_NAME:
						newValue = new SkinValue();
						break;
					case CanvasTemplate.XML_ELEMENT_NAME:
						newValue = new CanvasTemplate();
						break;
					case PropertyActionValue.XML_ELEMENT_NAME:
						newValue = new PropertyActionValue();
						break;
					case FontList.XML_ELEMENT_NAME:
						newValue = new FontList();
						break;
					case ThemeFilteredList.XML_ELEMENT_NAME:
						newValue = new ThemeFilteredList();
						break;
					default:
						// TODO unidentified value
						newValue = new PropertyValue();
				}
				if (newValue != null) {
					newValue.initializing = true;
					newValue.parent = parent;
					newValue.loadFromXML(valueXML);
					newValue.initializing = false;
				}
			}
			return newValue;
		}
	}
}
