package com.adobe.portfolio.utils
{
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import mx.core.UIComponent;

	/**
	 *  The DoubleClickHandler class is a utility class that allows components to dispatch both single and double-click events.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class DoubleClickHandler
	{
//		private static const logger:ILogger = Log.getLogger("DoubleClickHandler");

		protected static const DOUBLE_CLICK_TIME:int = 300;
		
		protected var component:UIComponent;
		protected var singleClickHandler:Function;
		protected var doubleClickHandler:Function;
		protected var singleClickEvent:MouseEvent;
		protected var startTimer:Timer;
		
		/**
		 *  Constructor.
		 * 
		 * 	@param component The host component (listen to its events)
		 * 	@param singleClickHandler The function to call if a single click is detected.  The function takes the form: singleClickHandler(e:MouseEvent):void.
		 * 	@param doubleClickHandler An optional function to call if a double click is detected.  The function takes the form: doubleClickHandler(e:MouseEvent):void.
		 * 	@param priority The event priority to use for the mouse event listeners on component.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		// Note: We'd like to use EventPriority.DEFAULT as a default for that last param, but it breaks PyFlexBuild
		public function DoubleClickHandler(component:UIComponent, singleClickHandler:Function, doubleClickHandler:Function = null, priority:int = -50):void
		{
			startTimer = new Timer(DOUBLE_CLICK_TIME, 1);
			startTimer.addEventListener(TimerEvent.TIMER, delayedClick, false, 0, true);
			this.component = component;
			this.singleClickHandler = singleClickHandler;
			this.doubleClickHandler = doubleClickHandler;
			component.addEventListener(MouseEvent.CLICK, mouseClickHandler, false, priority, true);
			component.doubleClickEnabled = true;
			component.addEventListener(MouseEvent.DOUBLE_CLICK, mouseDoubleClickHandler, false, priority, true);
		}
		
		/**
		 *  Removes all event listeners.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public function removeAllListeners():void
		{
			component.removeEventListener(MouseEvent.CLICK, mouseClickHandler);
			component.removeEventListener(MouseEvent.DOUBLE_CLICK, mouseDoubleClickHandler);
			startTimer.removeEventListener(TimerEvent.TIMER, delayedClick);
		}
		
		protected function mouseClickHandler(event:MouseEvent):void 
		{
//			logger.debug(event.type);
			singleClickEvent = event;
			startTimer.reset(); 
			startTimer.start();
		}
		
		protected function mouseDoubleClickHandler(event:MouseEvent):void 
		{
			startTimer.reset();
//			logger.debug(event.type);
			if (doubleClickHandler != null)
				doubleClickHandler(event);
		}
		
		protected function delayedClick(e:TimerEvent):void {
//			logger.debug("delayed click");
			singleClickHandler(singleClickEvent);
		}
	}
}