/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.utils
{
	import mx.utils.ColorUtil;

	/**
	 *  The GradientUtil class implements a function which computes a shadow color from a primary color.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class GradientUtil
	{
		/**
		 *  Performs a scaled brightness adjustment to an RGB color to get a secondary color used for simple gradients.
		 *
		 *  @param color Original RGB color.
		 *
		 *  @param brite The percentage to brighten or darken the original color.
		 *  If positive, the original color is brightened toward white
		 *  by this percentage. If negative, it is darkened toward black
		 *  by this percentage.
		 *  The range for this parameter is -100 to 100;
		 *  -100 produces black while 100 produces white.
		 *  If this parameter is 0, the RGB color returned
		 *  is the same as the original color.
		 *
		 *  @return New RGB color.
		 *  
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function secondaryColor(color:uint, brite:Number):uint
		{
			return ColorUtil.adjustBrightness2(color, brite);
		}
	}
}