/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.utils
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Matrix;

	/**
	 *  The ImageUtil class implements a function that fits a Bitmap to specified dimensions.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 */
	public class ImageUtil
	{
		/**
		 *  Fits a Bitmap to specified dimensions.
		 * 
		 * 	@param bitmap The source bitmap
		 * 	@param fitWidth The desired width
		 * 	@param fitHeight The desired height
		 * 	@param smoothing An optional parameter to control smoothing of the new bitmap
		 * 	@return The new bitmap which will have dimensions such that newWidth &lt;= fitWidth and newHeight &lt;= fitHeight, 
		 * 			and either newWidth == fitWidth or newHeight == fitHeight
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function fitBitmap(bitmap:Bitmap, fitWidth:Number, fitHeight:Number, smoothing:Boolean = true):Bitmap {
			const contentWidth:Number = bitmap.bitmapData.width;
			const contentHeight:Number = bitmap.bitmapData.height;
			var inverseScale:Number = Math.max(contentWidth/fitWidth, contentHeight/fitHeight);
			var scale:Number = 1 / inverseScale;
			var bmData:BitmapData = new BitmapData(contentWidth*scale, contentHeight*scale);
			var matrix:Matrix = new Matrix();
			matrix.scale(scale, scale);
			bmData.draw(bitmap, matrix, null, null, null, true);
			
			return new Bitmap(bmData, "auto", smoothing);
		}
		
	}
}