/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  Adobe permits you to use, modify, and distribute this 
* file in accordance with the terms of the Adobe license agreement 
* accompanying it.  If you have received this file from a source 
* other than Adobe, then your use, modification, or distribution 
* of it requires the prior written permission of Adobe.
**************************************************************************/

package com.adobe.portfolio.utils
{
	import flash.utils.Dictionary;

	/**
	 * The StringUtils class provides a set of String utility functions.
	 * 
	 *  @langversion 3.0
	 *  @playerversion Flash 10.1
	 *  @playerversion AIR 2.0
	 *  @productversion Portfolio 10.0
	 *
	 */
	public class StringUtils
	{
		/** Get the stack trace as a string
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function getStackTrace():String {
			try { throw new Error(); } catch (e:Error) {
				// strip off the first few items -- we don't care how we got the stack trace
				var traceLines:Array = e.getStackTrace().split('\n');
				traceLines.shift();
				traceLines.shift();
				return traceLines.join('\n');
			}
			return null;
		}
		
		/** Converts an array to a string with delimiters.
		 * @param array The array that is to be represented as a string.
		 * @param delimiter The delimiter used in the string.
		 * @return A string representing the given array.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function arrayToString(array:Array, delimeter:String = " "):String {
			if (array == null)
				return "null";
			
			var s:String = "[" + delimeter;
			for (var i:int = 0; i < array.length; i++) {
				var o:Object = array[i];
				if (o == null){
					s += "null" + delimeter;
					continue;
				}
				s += (o.toString() + delimeter);
				s += "{";
				for (var prop:Object in o) {
					s += (prop + ": " + o[prop] + " ");
				}
				s += "}" + delimeter
			}
			s += "]";
			
			return s;
		}
		
		/** Converts an int vector to a string with delimiters
		 * @param array The int vector that is to be represented as a string.
		 * @param delimiter The delimiter used in the string.
		 * @return A string representing the given vector.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function intVectorToString(array:Vector.<int>, delimeter:String = " "):String {
			if (array == null)
				return "null";
			
			var s:String = "[" + delimeter;
			for (var i:int = 0; i < array.length; i++) {
				var o:Object = array[i];
				if (o == null){
					s += "null" + delimeter;
					continue;
				}
				s += (o.toString() + delimeter);
			}
			s += "]";
			
			return s;
		}
		
		/** Converts a dictionary to a string with delimiters.
		 * @param dict The dictionary that is to be represented as a string.
		 * @param delimiter The delimiter used in the string.
		 * @return A string representing the given dictionary.
		 * 
		 *  @langversion 3.0
		 *  @playerversion Flash 10.1
		 *  @playerversion AIR 2.0
		 *  @productversion Portfolio 10.0
		 */
		public static function dictToString(dict:Dictionary, delimeter:String = " "):String {
			if (dict == null)
				return "null";
			
			var s:String = "[" + delimeter;
			for each (var o:Object in dict) {
				if (o == null){
					s += "null" + delimeter;
					continue;
				}
				s += (o.toString() + delimeter);
			}
			s += "]";
			
			return s;
		}
	}
}