/*************************************************************************
*
* ADOBE CONFIDENTIAL
* ___________________
* 
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
* ___________________
* 
*   DRAFT: subject to change pending API review and approval 
* ___________________
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/

package com.adobe.portfolio.launching;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.debug.core.DebugPlugin;

import java.io.File;

import com.adobe.flexbuilder.project.actionscript.IActionScriptProject;
import com.adobe.flexbuilder.project.actionscript.IActionScriptProjectSettings;
import com.adobe.flexbuilder.project.air.IApolloConstants;
import com.adobe.flexbuilder.project.sdks.IFlexSDK;
import com.adobe.flexbuilder.project.LaunchMode;
import com.adobe.flexbuilder.util.logging.GlobalLog;
import com.adobe.flexide.launching.IFlexLaunchConfiguration;
import com.adobe.flexide.launching.ILaunchHook;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.ui.DebugUITools;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import flash.tools.debugger.AIRLaunchInfo;

import java.util.prefs.Preferences; 

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

public class NavLaunchHook implements ILaunchHook {

	public boolean launch(String url, AIRLaunchInfo airLaunchInfo,
			LaunchMode launchMode,
			IFlexLaunchConfiguration flexLaunchConfiguration,
			IProgressMonitor monitor) {
		
		if ( flexLaunchConfiguration ==  null )
		{
			GlobalLog.log("flexLaunchConfiguration is null", null);
			return false;
		}
		// get the standard Eclipse configuration
		
		ILaunchConfiguration configuration = flexLaunchConfiguration.getLaunchConfiguration();
		if ( configuration ==  null )
		{
			GlobalLog.log("launchConfiguration is null", null);
			return false;
		}

		// Get the Host Launch flag
		// this is set by NavMainTab.setDefaults() so that NavLaunchHook only works for NavLaunchDelegate
		
		boolean theHostLaunchFlag = false;
		try {
			theHostLaunchFlag = configuration.getAttribute(NavLaunchDelegate.ATTR_LAUNCH_ENABLED, false);
		} catch (CoreException e) {
			GlobalLog.log("ATTR_LAUNCH_ENABLED not found", null);
		}

		// this is not a NavLaunchDelegate so do not launch
		
		if ( theHostLaunchFlag ==  false )
		{
			return false;
		}

		// Get the Use Previewer flag
		
		boolean theADLFlag = false;
		try {
			theADLFlag = configuration.getAttribute(NavLaunchDelegate.ATTR_USE_ADL, false);
		} catch (CoreException e) {
			GlobalLog.log("ATTR_USE_ADL not found", null);
		}

		// get the ADL application path
		
		String theADLAppPath = getADLPath(flexLaunchConfiguration);
		
		// check if the ADL application file exists
		
		if ( theADLFlag ) {
			if ( theADLAppPath == "" ) {
				GlobalLog.log("ADL application not found", null);
				return false;
			}

			File theADLAppFile = new File( theADLAppPath );
			if ( theADLAppFile == null ) {
				GlobalLog.log("ADL application not found", null);
				return false;
			}
			if ( theADLAppFile.exists() == false ) {
				GlobalLog.log("ADL application not found", null);
				return false;
			}
		}
		
		// get the host application path
		
		String theHostAppPath = new String(""); //$NON-NLS-1$
		try {
			theHostAppPath = configuration.getAttribute(NavLaunchDelegate.ATTR_HOST_APP, ""); //$NON-NLS-1$
		} catch (CoreException e) {
			GlobalLog.log("ATTR_HOST_APP not found", null);
		}
		if ( theHostAppPath ==  null )
		{
			GlobalLog.log("ATTR_HOST_APP was null", null);
			return false;
		}
		
		// check if the host application file exists
		
		File theHostAppFile = new File( theHostAppPath );
		if ( theHostAppFile == null ) {
			GlobalLog.log("Host application not found", null);
			return false;
		}
		if ( theHostAppFile.exists() == false ) {
			GlobalLog.log("Host application not found", null);
			return false;
		}

		// get the Target Navigator
		
		String theCMDLineArg = new String("");
		try {
			theCMDLineArg = configuration.getAttribute(NavLaunchDelegate.ATTR_CMDLINE_ARG, ""); //$NON-NLS-1$
		} catch (CoreException e) {
			GlobalLog.log("ATTR_CMDLINE_ARG not found", null);
		}
		if ( theCMDLineArg ==  null )
		{
			GlobalLog.log("ATTR_CMDLINE_ARG was null", null);
			return false;
		}
		
		// get the Navigator bin-debug directory
		
		String theDebugDir = getDebugDir(flexLaunchConfiguration);
		
		// get the Navigator ID
		
		String theNavigatorID = getNavigatorID(flexLaunchConfiguration);

		// Get the flex launch path from the Main flex configuration tab.
		// This is for Acrobat. We can not put it on the command line because
		// Acrobat will choke.

		String theFlexLaunchPath = flexLaunchConfiguration.getLaunchURL(LaunchMode.RUN, true);

		// build the command line
		
		try {
			String[] cmdLine;
			
			// this first case is for when ADL is used
			
			if (theADLFlag) {
				if (theDebugDir.length() > 0) {
					String[] theStrArr = {theADLAppPath, theHostAppPath, "--", theDebugDir};
					cmdLine = theStrArr;
				} else {
					
					// this is for when there is no cmd line (swf) file
					// the user could drag and drop the swf onto Previewer 
					
					String[] theStrArr = {theADLAppPath, theHostAppPath};
					cmdLine = theStrArr;
				}
				
			// this is the non-ADL case (ie. Acrobat)
				
			} else {
				// tell Acrobat that it was launched by Flash Builder
				
				WriteLaunchPrefs( "true", theNavigatorID, theFlexLaunchPath );
				
				if (theCMDLineArg.length() > 0) {
					String[] theStrArr = {theHostAppPath, theCMDLineArg};
					cmdLine = theStrArr;
				} else {
					
					// this is for when there is no cmd line (swf) file
					// Acrobat does not like a swf file on the command line
					// the user can enter a pdf file here
										
					String[] theStrArr = {theHostAppPath};
					cmdLine = theStrArr;
				}
			}
			
			Process p = DebugPlugin.exec(cmdLine, null); 

			/* 
			Associate the java.lang.Process of the executing program executing program
			with the ILaunch that represents the launched program. A utility method 
			in DebugPlugin defines a wrapper for a system process, IProcess, that 
			allows debug plug-in to manage the process.
			*/			
			
			if (p != null && NavLaunchDelegate.sLastLaunch != null) { 
				IProcess pr = DebugPlugin.newProcess(NavLaunchDelegate.sLastLaunch, p, cmdLine[0]); 
				NavLaunchDelegate.sLastLaunch.addProcess(pr); 
				NavLaunchDelegate.sLastLaunch = null;
			} 

		}
		catch(CoreException e)
		{
			GlobalLog.log("Host application failed to launch", e);
		}

		return true; // false = nothing launched
	}

	public String getADLPath(IFlexLaunchConfiguration flexLaunchConfiguration) {
		
		String theADLpathStr = "";

		IActionScriptProject asProject = flexLaunchConfiguration.getProject();
		IActionScriptProjectSettings projectSettings = asProject.getProjectSettings();
		IFlexSDK theSDK = projectSettings.getFlexSDK();

		if (theSDK != null && theSDK.isValid()) {
			IPath thePath = theSDK.getLocation();		
			thePath = thePath.append(IApolloConstants.AIR_SDK_BIN_DIRECTORY);
	        if (Platform.OS_WIN32.equals(Platform.getOS()))
	        {
				thePath = thePath.append(IApolloConstants.WIN_ADL_NAME);
	        }
	        else if (Platform.OS_MACOSX.equals(Platform.getOS()))
	        {
				thePath = thePath.append(IApolloConstants.MAC_ADL_NAME);
	        }
	        theADLpathStr = thePath.toOSString();
		}
		
		return theADLpathStr;
	}

	public String getNavigatorID(IFlexLaunchConfiguration flexLaunchConfiguration) {
		
		String theNavigatorIDStr = "";
		
		IResource selectedResource = DebugUITools.getSelectedResource();
		if (selectedResource != null) {
			IActionScriptProject asProject = flexLaunchConfiguration.getProject();
			IPath thePath = asProject.getOutputContainer().getLocation();
			thePath = thePath.append("Navigator.xml"); //$NON-NLS-1$
			String theNavXMLPathStr = thePath.toOSString();

			//File theNavXMLFile = new File( thePath );
			File theNavXMLFile = new File( theNavXMLPathStr );
			if ( theNavXMLFile == null ) {
				GlobalLog.log("Navigator.xml application not found", null);
				return "";
			}
			if ( theNavXMLFile.exists() == false ) {
				GlobalLog.log("Navigator.xml application not found", null);
				return "";
			}

			try {
				DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
				Document doc = docBuilder.parse(theNavXMLFile);
	
				Element root = doc.getDocumentElement();
				theNavigatorIDStr = root.getAttribute("id");
				
			} catch (Exception e) {
				GlobalLog.log("Unable to parse Navigator.xml", e);
			}
			
		}

		return theNavigatorIDStr;
	}

	public String getDebugDir(IFlexLaunchConfiguration flexLaunchConfiguration) {
		
		String theTargetPathStr = "";
		
		IActionScriptProject asProject = flexLaunchConfiguration.getProject();
		IPath thePath = asProject.getOutputContainer().getLocation();
		theTargetPathStr = thePath.toOSString();
		
		return theTargetPathStr;
	}
	
	public static final String PREF_KEY_FLAG = "com.adobe.portfolio.launching.flag"; 
	public static final String PREF_KEY_NAV_ID = "com.adobe.portfolio.launching.navid"; 
	public static final String PREF_KEY_SWF_PATH = "com.adobe.portfolio.launching.swfpath"; 
	
	public static void WriteLaunchPrefs(String theLaunchFlag, String theNavID, String theSWFPath) { 
	        
		// 
		// Write Preferences information to HKCU (HKEY_CURRENT_USER), 
		// 
		Preferences userPref = Preferences.userRoot(); 
		
		// Write out the launch flag
		userPref.put(PREF_KEY_FLAG, theLaunchFlag);
		System.out.println("theLaunchFlag = " + theLaunchFlag); 
		
		// Write out the navigator ID
		userPref.put(PREF_KEY_NAV_ID, theNavID);
		System.out.println("theNavID = " + theNavID); 
		
		// Write out the SWF file path
		userPref.put(PREF_KEY_SWF_PATH, theSWFPath);
		System.out.println("theSWFPath = " + theSWFPath); 
		
		try {
			userPref.flush();
		} catch (Exception e) {
			GlobalLog.log("Backing store failure", null);
		}
		
	} 
	 
}

