/*************************************************************************
*
* ADOBE CONFIDENTIAL
* ___________________
* 
*  Copyright 2010 Adobe Systems Incorporated
*  All Rights Reserved.
* ___________________
* 
*   DRAFT: subject to change pending API review and approval 
* ___________________
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/

package com.adobe.portfolio.launching;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.IDebugEventSetListener;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;

import com.adobe.flexbuilder.debug.model.FlexDebugTarget;

public class NavLaunchListener implements IDebugEventSetListener {

	public NavLaunchListener() {
		DebugPlugin.getDefault().addDebugEventListener(this);
	}

	public void dispose() {
		DebugPlugin.getDefault().removeDebugEventListener(this);
	}

	public void handleDebugEvents(DebugEvent[] events) {

		for (DebugEvent event: events) {
			int debugEventKind = event.getKind();
			switch (debugEventKind) {
			
			case DebugEvent.TERMINATE:
				Object source = event.getSource();
				if (source instanceof FlexDebugTarget) {
					FlexDebugTarget flexDebugTarget = (FlexDebugTarget) source;
					ILaunch launch = flexDebugTarget.getLaunch(); 
					ILaunchConfiguration configuration = launch.getLaunchConfiguration();
					if ( configuration !=  null )
					{
						boolean theHostLaunchFlag = false;
						try {
							theHostLaunchFlag = configuration.getAttribute(NavLaunchDelegate.ATTR_LAUNCH_ENABLED, false);
						} catch (CoreException e) {
						}

						if ( theHostLaunchFlag )
						{
							// Clear launch prefs at the end of the debug session
							
							System.out.println("NavLaunchListener - DebugEvent.TERMINATE"); 
							NavLaunchHook.WriteLaunchPrefs( "false", "undefined", "undefined" );
						}
					}
				}
				break;

			default:
				// ignore other events
			}
		}

	}

}
